@@ -27,7 +27,9 @@
 ;;; 4) FIXMEs
 
 (defpackage :sb-introspect
-  (:use "CL" "SB-KERNEL")
+  (:use "CL" "SB-KERNEL" "SB-INT")
+  (:import-from "SB-VM" "PRIMITIVE-OBJECT-SIZE")
+  (:shadow "VALID-FUNCTION-NAME-P")
   (:export "ALLOCATION-INFORMATION"
            "FUNCTION-ARGLIST"
            "FUNCTION-LAMBDA-LIST"
@@ -78,21 +80,21 @@ include the pathname of the file and the position of the definition."
   "Debug function represent static compile-time information about a function."
   'sb-c::compiled-debug-fun)
 
-(declaim (ftype (sb-int:sfunction (function) debug-info) function-debug-info))
+(declaim (ftype (sfunction (function) debug-info) function-debug-info))
 (defun function-debug-info (function)
   (let* ((function-object (%fun-fun function))
          (function-header (fun-code-header function-object)))
     (%code-debug-info function-header)))
 
-(declaim (ftype (sb-int:sfunction (function) debug-source) function-debug-source))
+(declaim (ftype (sfunction (function) debug-source) function-debug-source))
 (defun function-debug-source (function)
   (debug-info-source (function-debug-info function)))
 
-(declaim (ftype (sb-int:sfunction (debug-info) debug-source) debug-info-source))
+(declaim (ftype (sfunction (debug-info) debug-source) debug-info-source))
 (defun debug-info-source (debug-info)
   (sb-c::debug-info-source debug-info))
 
-(declaim (ftype (sb-int:sfunction (t debug-info) debug-function) debug-info-debug-function))
+(declaim (ftype (sfunction (t debug-info) debug-function) debug-info-debug-function))
 (defun debug-info-debug-function (function debug-info)
   (sb-di::compiled-debug-fun-from-pc debug-info
                                      (sb-di::function-start-pc-offset function)))
@@ -117,7 +119,7 @@ FBOUNDP."
 receives the object and its size as arguments.  SPACES should be a
 list of the symbols :dynamic, :static, :read-only, or :immobile on
 #+immobile-space"
-  (apply #'sb-vm::map-allocated-objects
+  (apply #'sb-vm:map-allocated-objects
      (lambda (obj header size)
        (when (= sb-vm:code-header-widetag header)
          (funcall fn obj size)))
@@ -170,25 +172,28 @@ constant pool."
   (description nil :type list))
 
 (defun vops-translating-fun (name)
-  (let ((fun-info (sb-int:info :function :info name)))
+  (let ((fun-info (info :function :info name)))
     (when fun-info
       (sb-c::fun-info-templates fun-info))))
 
 (defun find-vop-source (name)
-  (let* ((vop (gethash name sb-c::*backend-template-names*))
+  (let* ((vop (gethash name sb-c::*backend-parsed-vops*))
          (translating (vops-translating-fun name))
          (vops (if vop
                    (cons vop (remove vop translating))
                    translating)))
     (loop for vop in vops
-          for name = (sb-c::vop-info-name vop)
-          for loc = (sb-c::vop-parse-source-location
-                     (gethash name sb-c::*backend-parsed-vops*))
+          for vop-parse = (if (typep vop 'sb-c::vop-parse)
+                              vop
+                              (gethash (sb-c::vop-info-name vop)
+                                       sb-c::*backend-parsed-vops*))
+          for name = (sb-c::vop-parse-name vop-parse)
+          for loc = (sb-c::vop-parse-source-location vop-parse)
           when loc
           collect (let ((source (translate-source-location loc)))
                     (setf (definition-source-description source)
-                          (if (sb-c::vop-info-note vop)
-                              (list name (sb-c::vop-info-note vop))
+                          (if (sb-c::vop-parse-note vop-parse)
+                              (list name (sb-c::vop-parse-note vop-parse))
                               (list name)))
                     source))))
 
@@ -235,21 +240,21 @@ If an unsupported TYPE is requested, the function will return NIL.
              (if profile-info
                  (sb-profile::profile-info-encapsulated-fun profile-info)
                  (fdefinition name)))))
-    (sb-int:ensure-list
+    (ensure-list
      (case type
        ((:variable)
         (when (and (symbolp name)
-                   (member (sb-int:info :variable :kind name)
+                   (member (info :variable :kind name)
                            '(:global :special :alien)))
-          (translate-source-location (sb-int:info :source-location type name))))
+          (translate-source-location (info :source-location type name))))
        ((:constant)
         (when (and (symbolp name)
-                   (eq (sb-int:info :variable :kind name) :constant))
-          (translate-source-location (sb-int:info :source-location type name))))
+                   (eq (info :variable :kind name) :constant))
+          (translate-source-location (info :source-location type name))))
        ((:symbol-macro)
         (when (and (symbolp name)
-                   (eq (sb-int:info :variable :kind name) :macro))
-          (translate-source-location (sb-int:info :source-location type name))))
+                   (eq (info :variable :kind name) :macro))
+          (translate-source-location (info :source-location type name))))
        ((:macro)
         (when (and (symbolp name)
                    (macro-function name))
@@ -258,7 +263,7 @@ If an unsupported TYPE is requested, the function will return NIL.
         (when (compiler-macro-function name)
           (find-definition-source (compiler-macro-function name))))
        (:ir1-convert
-        (let ((converter (sb-int:info :function :ir1-convert name)))
+        (let ((converter (info :function :ir1-convert name)))
           (and converter
            (find-definition-source converter))))
        ((:function :generic-function)
@@ -274,10 +279,10 @@ If an unsupported TYPE is requested, the function will return NIL.
        ((:type)
         ;; Source locations for types are saved separately when the expander
         ;; is a closure without a good source-location.
-        (let ((loc (sb-int:info :type :source-location name)))
+        (let ((loc (info :type :source-location name)))
           (if loc
               (translate-source-location loc)
-              (let ((expander-fun (sb-int:info :type :expander name)))
+              (let ((expander-fun (info :type :expander name)))
                 (when (functionp expander-fun)
                   (find-definition-source expander-fun))))))
        ((:method)
@@ -292,7 +297,7 @@ If an unsupported TYPE is requested, the function will return NIL.
         (when (and (consp name)
                    (eq (car name) 'setf))
           (setf name (cadr name)))
-        (let ((expander (sb-int:info :setf :expander name)))
+        (let ((expander (info :setf :expander name)))
           (when expander
             (find-definition-source
              (cond ((symbolp expander) (symbol-function expander))
@@ -303,9 +308,9 @@ If an unsupported TYPE is requested, the function will return NIL.
           (if class
               (when (typep class 'sb-pcl::structure-class)
                 (find-definition-source class))
-              (when (sb-int:info :typed-structure :info name)
+              (when (info :typed-structure :info name)
                 (translate-source-location
-                 (sb-int:info :source-location :typed-structure name))))))
+                 (info :source-location :typed-structure name))))))
        ((:condition :class)
         (let ((class (get-class name)))
           (when (and class
@@ -325,23 +330,22 @@ If an unsupported TYPE is requested, the function will return NIL.
               (find-definition-source package)))))
        ;; TRANSFORM and OPTIMIZER handling from swank-sbcl
        ((:transform)
-        (when (symbolp name)
-          (let ((fun-info (sb-int:info :function :info name)))
-            (when fun-info
-              (loop for xform in (sb-c::fun-info-transforms fun-info)
-                    for source = (find-definition-source
-                                  (sb-c::transform-function xform))
-                    for typespec = (type-specifier
-                                    (sb-c::transform-type xform))
-                    for note = (sb-c::transform-note xform)
-                    do (setf (definition-source-description source)
-                             (if (consp typespec)
-                                 (list (second typespec) note)
-                                 (list note)))
-                    collect source)))))
+        (let ((fun-info (info :function :info name)))
+          (when fun-info
+            (loop for xform in (sb-c::fun-info-transforms fun-info)
+                  for source = (find-definition-source
+                                (sb-c::transform-function xform))
+                  for typespec = (type-specifier
+                                  (sb-c::transform-type xform))
+                  for note = (sb-c::transform-note xform)
+                  do (setf (definition-source-description source)
+                           (if (consp typespec)
+                               (list (second typespec) note)
+                               (list note)))
+                  collect source))))
        ((:optimizer)
         (let ((fun-info (and (symbolp name)
-                             (sb-int:info :function :info name))))
+                             (info :function :info name))))
           (when fun-info
             (let ((otypes '((sb-c:fun-info-derive-type . sb-c:derive-type)
                             (sb-c:fun-info-ltn-annotate . sb-c:ltn-annotate)
@@ -365,14 +369,14 @@ If an unsupported TYPE is requested, the function will return NIL.
        (:vop
         (find-vop-source name))
        (:alien-type
-        (let ((loc (sb-int:info :source-location type name)))
+        (let ((loc (info :source-location type name)))
           (and loc
                (translate-source-location loc))))
        ((:source-transform)
         (let* ((transform-fun
-                (or (sb-int:info :function :source-transform name)
+                (or (info :function :source-transform name)
                     (and (typep name '(cons (eql setf) (cons symbol null)))
-                         (sb-int:info :function :source-transform
+                         (info :function :source-transform
                                       (second name)))))
                ;; A cons for the :source-transform is essentially the same
                ;; info that was formerly in :structure-accessor.
@@ -384,14 +388,14 @@ If an unsupported TYPE is requested, the function will return NIL.
                      (not accessor))
             (find-definition-source transform-fun))))
        (:declaration
-        (let ((locations (sb-int:info :source-location :declaration name)))
+        (let ((locations (info :source-location :declaration name)))
           (loop for (kind loc) on locations by #'cddr
                 when loc
                 collect (let ((loc (translate-source-location loc)))
                           (setf (definition-source-description loc)
                                 ;; Copy list to ensure that user code
                                 ;; cannot mutate the original.
-                                (copy-list (sb-int:ensure-list kind)))
+                                (copy-list (ensure-list kind)))
                           loc))))
        (t
         nil)))))
@@ -399,12 +403,10 @@ If an unsupported TYPE is requested, the function will return NIL.
 (defun find-definition-source (object)
   (typecase object
     ((or sb-pcl::condition-class sb-pcl::structure-class)
-     (let ((classoid (sb-impl::find-classoid (class-name object))))
+     (let ((classoid (sb-pcl::class-classoid object)))
        (when classoid
-         (let ((layout (sb-impl::classoid-layout classoid)))
-           (when layout
-             (translate-source-location
-              (sb-kernel::layout-source-location layout)))))))
+         (translate-source-location
+          (sb-kernel::classoid-source-location classoid)))))
     (method-combination
      (car
       (find-definition-sources-by-name
@@ -459,13 +461,13 @@ If an unsupported TYPE is requested, the function will return NIL.
          (tlf (sb-c::compiled-debug-info-tlf-number debug-info)))
     (make-definition-source
      :pathname
-     (when (stringp (sb-c::debug-source-namestring debug-source))
-       (parse-namestring (sb-c::debug-source-namestring debug-source)))
+     (when (stringp (debug-source-namestring debug-source))
+       (parse-namestring (debug-source-namestring debug-source)))
      :character-offset
      (sb-c::compiled-debug-info-char-offset debug-info)
      :form-path (if tlf (list tlf))
      :form-number (sb-c::compiled-debug-fun-form-number debug-fun)
-     :file-write-date (sb-c::debug-source-created debug-source)
+     :file-write-date (debug-source-created debug-source)
      :plist (sb-c::debug-source-plist debug-source))))
 
 (defun translate-source-location (location)
@@ -484,7 +486,7 @@ If an unsupported TYPE is requested, the function will return NIL.
        :plist (sb-c:definition-source-location-plist location))
       (make-definition-source)))
 
-(sb-int:define-deprecated-function :late "1.0.24.5" function-arglist function-lambda-list
+(define-deprecated-function :late "1.0.24.5" function-arglist function-lambda-list
     (function)
   (function-lambda-list function))
 
@@ -495,7 +497,7 @@ and generic functions. Signals an error if FUNCTION is not a valid extended
 function designator."
   ;; FIXME: sink this logic into SB-KERNEL:%FUN-LAMBDA-LIST and just call that?
   (cond ((and (symbolp function) (special-operator-p function))
-         (function-lambda-list (sb-int:info :function :ir1-convert function)))
+         (function-lambda-list (info :function :ir1-convert function)))
         ((valid-function-name-p function)
          (function-lambda-list (or (and (symbolp function)
                                         (macro-function function))
@@ -511,8 +513,8 @@ value, and a flag whether the arglist could be found as second
 value."
   (check-type typespec-operator symbol)
   ;; Don't return a lambda-list for combinators AND,OR,NOT.
-  (let* ((f (and (sb-int:info :type :kind typespec-operator)
-                 (sb-int:info :type :expander typespec-operator)))
+  (let* ((f (and (info :type :kind typespec-operator)
+                 (info :type :expander typespec-operator)))
          (f (if (listp f) (car f) f)))
     (if (functionp f)
         (values (%fun-lambda-list f) t)
@@ -522,7 +524,7 @@ value."
   "Returns the ftype of FUNCTION-DESIGNATOR, or NIL."
   (flet ((ftype-of (function-designator)
            (type-specifier
-            (sb-int:proclaimed-ftype function-designator))))
+            (global-ftype function-designator))))
     (etypecase function-designator
       (symbol
        (when (and (fboundp function-designator)
@@ -530,7 +532,7 @@ value."
                   (not (special-operator-p function-designator)))
          (ftype-of function-designator)))
       (cons
-       (when (and (sb-int:legal-fun-name-p function-designator)
+       (when (and (legal-fun-name-p function-designator)
                   (fboundp function-designator))
          (ftype-of function-designator)))
       (generic-function
@@ -588,7 +590,7 @@ value."
   (let ((result '()))
     (sb-c:map-simple-funs
      (lambda (name fun)
-       (sb-int:binding* ((xrefs (%simple-fun-xrefs fun) :exit-if-null))
+       (binding* ((xrefs (%simple-fun-xrefs fun) :exit-if-null))
          (sb-c:map-packed-xref-data
           (lambda (xref-kind xref-name xref-form-number)
             (when (and (eq xref-kind wanted-kind)
@@ -773,29 +775,11 @@ Experimental: interface subject to change."
   ;; for mapping over parts of arbitrary objects so users can get "deep sizes"
   ;; as well if they want to.
   ;;
-  ;; FIXME: For the memoization use-case possibly we should also provide a
-  ;; simpler HEAP-ALLOCATED-P, since that doesn't require disabling the GC
-  ;; scanning threads for negative answers? Similarly, STACK-ALLOCATED-P for
-  ;; checking if an object has been stack-allocated by a given thread for
-  ;; testing purposes might not come amiss.
   (if (not (sb-vm:is-lisp-pointer (get-lisp-obj-address object)))
       (values :immediate nil)
       (let ((plist
              (sb-sys:with-pinned-objects (object)
-               (let* ((addr (get-lisp-obj-address object))
-                      (space
-                       (cond ((< (current-dynamic-space-start) addr
-                                 (sb-sys:sap-int (dynamic-space-free-pointer)))
-                              :dynamic)
-                             #+immobile-space
-                             ((immobile-space-addr-p addr)
-                              :immobile)
-                             ((< sb-vm:read-only-space-start addr
-                                 (sb-sys:sap-int sb-vm:*read-only-space-free-pointer*))
-                              :read-only)
-                             ((< sb-vm:static-space-start addr
-                                 (sb-sys:sap-int sb-vm:*static-space-free-pointer*))
-                              :static))))
+               (let ((space (sb-ext:heap-allocated-p object)))
                  (when space
                    #+gencgc
                    (if (eq :dynamic space)
@@ -804,7 +788,8 @@ Experimental: interface subject to change."
                          ;; bits are packed in the opposite order. And thankfully,
                          ;; this fix seems not to depend on whether the numbering
                          ;; scheme is MSB 0 or LSB 0, afaict.
-                         (let* ((index (sb-vm::find-page-index addr))
+                         (let* ((index (sb-vm:find-page-index
+                                        (get-lisp-obj-address object)))
                                 (flags (sb-alien:slot page 'sb-vm::flags))
                                 .
                                 ;; The unused WP-CLR is for ease of counting
@@ -832,27 +817,16 @@ Experimental: interface subject to change."
         (cond (plist
                (values :heap plist))
               (t
-               (let ((sap (sb-sys:int-sap (get-lisp-obj-address object))))
-                 ;; FIXME: Check other stacks as well.
-                 #+sb-thread
-                 (dolist (thread (sb-thread:list-all-threads))
-                   (let ((c-start (sb-int:descriptor-sap
-                                   (sb-thread::%symbol-value-in-thread
-                                    'sb-vm:*control-stack-start*
-                                    thread)))
-                         (c-end (sb-int:descriptor-sap
-                                 (sb-thread::%symbol-value-in-thread
-                                  'sb-vm:*control-stack-end*
-                                  thread))))
-                     (when (and c-start c-end)
-                       (when (and (sb-sys:sap<= c-start sap)
-                                  (sb-sys:sap< sap c-end))
-                         (return-from allocation-information
-                           (values :stack thread))))))
-                 #-sb-thread
-                 (when (sb-vm:control-stack-pointer-valid-p sap nil)
+               #+sb-thread
+               (let ((thread (sb-ext:stack-allocated-p object t)))
+                 (when thread
                    (return-from allocation-information
-                     (values :stack sb-thread::*current-thread*))))
+                     (values :stack thread))))
+               #-sb-thread
+               (when (sb-vm:control-stack-pointer-valid-p
+                      (sb-sys:int-sap (get-lisp-obj-address object)) nil)
+                 (return-from allocation-information
+                   (values :stack sb-thread::*current-thread*)))
                :foreign)))))
 
 (defun map-root (function object &key simple (ext t))
@@ -873,36 +847,28 @@ NOTE: calling MAP-ROOT with a THREAD does not currently map over
 conservative roots from the thread registers and interrupt contexts.
 
 Experimental: interface subject to change."
+  (when (typep object '(or bignum float sb-sys:system-area-pointer
+                           fixnum character))
+    (return-from map-root object))
   (let ((fun (coerce function 'function))
-        (seen (sb-int:alloc-xset)))
+        (seen (alloc-xset)))
     (flet ((call (part)
              (when (and (sb-vm:is-lisp-pointer (get-lisp-obj-address part))
-                        (not (sb-int:xset-member-p part seen)))
-                 (sb-int:add-to-xset part seen)
+                        (not (xset-member-p part seen)))
+                 (add-to-xset part seen)
                  (funcall fun part))))
       (when ext
         (let ((table sb-pcl::*eql-specializer-table*))
-          (call (sb-int:with-locked-system-table (table)
-                  (gethash object table)))))
-      (etypecase object
-        ((or bignum float sb-sys:system-area-pointer fixnum))
-        (sb-ext:weak-pointer
-         (call (sb-ext:weak-pointer-value object)))
+          (multiple-value-bind (value foundp)
+              (with-locked-system-table (table) (gethash object table))
+            (when foundp (call value)))))
+      (sb-vm:do-referenced-object (object call)
         (cons
-         (call (car object))
-         (call (cdr object))
+         :extend
          (when (and ext (ignore-errors (fboundp object)))
            (call (fdefinition object))))
-        (ratio
-         (call (numerator object))
-         (call (denominator object)))
-        (complex
-         (call (realpart object))
-         (call (realpart object)))
-        (sb-vm::instance
-         (call (%instance-layout object))
-         (do-instance-tagged-slot (i object)
-           (call (%instance-ref object i)))
+        (instance
+         :extend
          #+sb-thread
          (when (typep object 'sb-thread:thread)
            (cond ((eq object sb-thread:*current-thread*)
@@ -932,49 +898,39 @@ Experimental: interface subject to change."
                     ;; that map onto the 'struct thread', which is just as well
                     ;; since they're either fixnums or dynamic-extent objects.
                     (mapc #'call refs))))))
-        (array
-         (if (simple-vector-p object)
-             (dotimes (i (length object))
-               (call (aref object i)))
-             (when (array-header-p object)
-               (call (%array-data object))
-               (call (%array-displaced-p object))
-               (unless simple
-                 (call (%array-displaced-from object))))))
+        ((satisfies array-header-p)
+         :override
+         ;; The default implementation always scans %array-displaced-from
+         (call (%array-data object))
+         (call (%array-displaced-p object))
+         (unless simple
+           (call (%array-displaced-from object))))
         (code-component
-         (call (%code-debug-info object))
-         (loop for i from sb-vm:code-constants-offset below (code-header-words object)
-               do (call (code-header-ref object i)))
+         :extend
          (loop for i below (code-n-entries object)
                do (call (%code-entry-point object i))))
-        (fdefn
-         (call (fdefn-name object))
-         (call (fdefn-fun object)))
-        (simple-fun
-         (call (fun-code-header object))
+        (function ; excluding CLOSURE and FUNCALLABLE-INSTANCE
+         :override
+         (unless simple
+           (call (fun-code-header object)))
          (call (%simple-fun-name object))
          (call (%simple-fun-arglist object))
          (call (%simple-fun-type object))
          (call (%simple-fun-info object)))
-        (closure
-         (call (%closure-fun object))
-         (do-closure-values (x object)
-           (call x)))
-        (funcallable-instance
-         ;; FIXME: layout ?
-         (call (%funcallable-instance-function object))
-         (loop for i from sb-vm:instance-data-start
-               below (- (1+ (get-closure-length object))
-                        sb-vm:funcallable-instance-info-offset)
-               do (call (%funcallable-instance-info object i))))
         (symbol
+         ;; We use :override here because (apparently) the intent is
+         ;; to avoid calling FUNCTION on the SYMBOL-PACKAGE
+         ;; when SIMPLE is NIL (the default). And we skip SYMBOL-EXTRA for
+         ;; the same reason that we don't call FUNCTION on SYMBOL-INFO
+         ;; (logically it's "system" data, not for user consumption).
+         ;; Frankly this entire function is a confusing mishmash that is not
+         ;; accurate for computing a true graph of objects starting from a
+         ;; certain point, given all the special cases that it implements.
+         :override
          (when ext
            (dolist (thread (sb-thread:list-all-threads))
              (call (sb-thread:symbol-value-in-thread object thread nil))))
-         (handler-case
-             ;; We don't have GLOBAL-BOUNDP, and there's no ERRORP arg.
-             (call (sb-ext:symbol-global-value object))
-           (unbound-variable ()))
+         (call (sb-sys:%primitive sb-c:fast-symbol-global-value object))
          ;; These first two are probably unnecessary.
          ;; The functoid values, if present, are in SYMBOL-INFO
          ;; which is traversed whether or not EXT was true.
@@ -989,7 +945,8 @@ Experimental: interface subject to change."
          (call (symbol-name object))
          (unless simple
            (call (symbol-package object))))
-        (sb-kernel::random-class
+        (t
+         :extend
          (case (widetag-of object)
            (#.sb-vm:value-cell-widetag
             (call (value-cell-ref object)))
@@ -999,12 +956,97 @@ Experimental: interface subject to change."
   object)
 
 (defun object-size (object)
-  (+ (sb-vm::primitive-object-size object)
+  (+ (primitive-object-size object)
      (typecase object
        (sb-mop:funcallable-standard-object
-        (sb-vm::primitive-object-size
-         (sb-pcl::standard-funcallable-instance-clos-slots object)))
+        (primitive-object-size (sb-pcl::fsc-instance-slots object)))
        (standard-object
-        (sb-vm::primitive-object-size
-         (sb-pcl::standard-instance-slots object)))
+        (primitive-object-size (sb-pcl::std-instance-slots object)))
        (t 0))))
+
+;;; Print a distribution of object sizes in SPACE.
+;;; There are two bins for cons-sized objects: conses and anything else,
+;;; the latter including SAPs, value cells, 0-length simple-vectors,
+;;; and a bunch of other things.
+(defun object-size-histogram (&optional
+                              (space :dynamic)
+                              (size-bins ; objects whose size in words is <= this
+                               `#(2 4 6 8 10 16 20 24 32 64 128 256 512 1024
+                                  2048 4096 8192 16384 32768 131072 524288
+                                  ,(ash 1 20) ,(ash 1 21) ,(ash 1 23))))
+  (declare (simple-vector size-bins))
+  (let* ((n-bins (+ (length size-bins) 2))
+         (counts (make-array n-bins :initial-element 0))
+         (size-totals (make-array n-bins :initial-element 0)))
+    (sb-vm:map-allocated-objects
+     (lambda (obj type size)
+       (declare (ignore type))
+       (cond ((consp obj)
+              (incf (aref counts 0)))
+             (t
+              (let* ((words (ash size (- sb-vm:word-shift)))
+                     (bin
+                      (let ((i (position words size-bins :test #'<=)))
+                        (if i (1+ i) (1- n-bins)))))
+                (incf (aref counts bin))
+                (incf (aref size-totals bin) words)))))
+     space)
+    (format t "     Freq     Tot Words~% =========    =========~%")
+    (dotimes (i n-bins)
+      (format t " ~9d  ~11d  ~a~%"
+              (aref counts i)
+              (if (eql i 0) ; cons bin
+                  (* 2 (aref counts i))
+                  (aref size-totals i))
+              (cond ((zerop i) "cons")
+                    ((eql i (1- n-bins))
+                     (format nil " > ~D" (aref size-bins (- n-bins 3))))
+                    (t
+                     (let ((this-bin-size (aref size-bins (1- i)))
+                           (prev-bin-size (when (>= i 2) (aref size-bins (- i 2)))))
+                       (format nil "~:[<=~;=~] ~D"
+                               (or (not prev-bin-size)
+                                   (= this-bin-size (+ prev-bin-size 2)))
+                               this-bin-size))))))))
+
+(defun largest-objects (&key (threshold #+gencgc sb-vm:gencgc-card-bytes
+                                        #-gencgc sb-c:+backend-page-bytes+)
+                             (sort :size))
+  (declare (type (member :address :size) sort))
+  (flet ((show-obj (obj)
+           #-gencgc
+           (format t "~10x ~7x ~s~%"
+                     (get-lisp-obj-address obj)
+                     (primitive-object-size obj)
+                     (type-of obj))
+           #+gencgc
+           (let* ((gen (generation-of obj))
+                  (page (sb-vm::find-page-index (sb-kernel:get-lisp-obj-address obj)))
+                  (flags (if (>= page 0)
+                             (sb-alien:slot (sb-alien:deref sb-vm:page-table page)
+                                            'sb-vm::flags))))
+             (format t "~10x ~7x ~a ~:[        ~;~:*~8b~] ~s~%"
+                     (get-lisp-obj-address obj)
+                     (primitive-object-size obj)
+                     (if gen gen #\?)
+                     flags
+                     (type-of obj)))))
+    (case sort
+     (:address
+      (sb-vm:map-allocated-objects
+       (lambda (obj widetag size)
+         (declare (ignore widetag))
+         (when (>= size threshold)
+           (show-obj obj)))
+       :all))
+     (:size
+      (let (list)
+         (sb-vm:map-allocated-objects
+          (lambda (obj widetag size)
+            (declare (ignore widetag))
+            (when (>= size threshold)
+              (push obj list)))
+          :all)
+         (mapc #'show-obj
+               (stable-sort list #'> :key #'primitive-object-size))
+         nil)))))
