@@ -137,15 +137,15 @@
                        (open-with-mode pathname flags mode)
                        (open-without-mode pathname flags))))))
     (def #-win32 "open" #+win32 "_open"))
-(define-call* "read" int minusp
-    (fd file-descriptor) (buf (* t)) (count int))
 (define-call "rename" int minusp (oldpath filename) (newpath filename))
 (define-call* "rmdir" int minusp (pathname filename))
 (define-call* "unlink" int minusp (pathname filename))
 (define-call #-netbsd "opendir" #+netbsd "_opendir"
     (* t) null-alien (pathname filename))
-(define-call* "write" int minusp
-  (fd file-descriptor) (buf (* t)) (count int))
+(define-call* "read" ssize-t minusp
+    (fd file-descriptor) (buf (* t)) (count size-t))
+(define-call* "write" ssize-t minusp
+  (fd file-descriptor) (buf (* t)) (count size-t))
 
 ;;; FIXME: to detect errors in readdir errno needs to be set to 0 and
 ;;; then checked, like it's done in sb-unix:readdir.
@@ -231,11 +231,11 @@
     ;; FIXME: What about Windows?
     (def-mk*temp mkdtemp "mkdtemp" (* char) null-alien t nil))
   (define-call-internally ioctl-without-arg "ioctl" int minusp
-                          (fd file-descriptor) (cmd int))
+                          (fd file-descriptor) (cmd unsigned-long))
   (define-call-internally ioctl-with-int-arg "ioctl" int minusp
-                          (fd file-descriptor) (cmd int) (arg int))
+                          (fd file-descriptor) (cmd unsigned-long) (arg int))
   (define-call-internally ioctl-with-pointer-arg "ioctl" int minusp
-                          (fd file-descriptor) (cmd int)
+                          (fd file-descriptor) (cmd unsigned-long)
                           (arg alien-pointer-to-anything-or-nil))
   (define-entry-point "ioctl" (fd cmd &optional (arg nil argp))
     (if argp
@@ -290,7 +290,7 @@
   (define-call "setuid" int minusp (uid uid-t))
   (define-call "getegid" gid-t never-fails)
   (define-call "getgid" gid-t never-fails)
-  #-sunos
+  #-(or sunos darwin)
   (progn
     (export '(getresgid getresuid) :sb-posix)
     (declaim (inline getresgid getresuid))
@@ -334,22 +334,29 @@
 
   ;; FIXME this is a lie, of course this can fail, but there's no
   ;; error handling here yet!
-  #+mach-exception-handler
-  (define-call "setup_mach_exceptions" void never-fails)
+  #+darwin
+  (define-call "darwin_reinit" void never-fails)
   (define-call ("posix_fork" :c-name "fork") pid-t minusp)
   (defun fork ()
     "Forks the current process, returning 0 in the new process and the PID of
 the child process in the parent. Forking while multiple threads are running is
 not supported."
     (tagbody
-       (sb-thread::with-all-threads-lock
-         (when (cdr sb-thread::*all-threads*)
-           (go :error))
-         (let ((pid (posix-fork)))
-           #+mach-exception-handler
-           (when (= pid 0)
-             (setup-mach-exceptions))
-           (return-from fork pid)))
+       (let ()
+         #+sb-thread
+         (sb-impl::finalizer-thread-stop)
+         (sb-thread::with-all-threads-lock
+           (when (let ((avltree sb-thread::*all-threads*))
+                   (or (sb-thread::avlnode-left avltree)
+                       (sb-thread::avlnode-right avltree)))
+             (go :error))
+           (let ((pid (posix-fork)))
+             #+darwin
+             (when (= pid 0)
+               (darwin-reinit))
+             #+sb-thread
+             (setf sb-impl::*finalizer-thread* t)
+             (return-from fork pid))))
      :error
        (error "Cannot fork with multiple threads running.")))
   (export 'fork :sb-posix)
@@ -457,7 +464,6 @@ not supported."
  (define-call "wtermsig" int never-fails (status int))
  (define-call "wifstopped" boolean never-fails (status int))
  (define-call "wstopsig" int never-fails (status int))
- #+nil ; see alien/waitpid-macros.c
  (define-call "wifcontinued" boolean never-fails (status int)))
 
 ;;; mmap, msync
@@ -817,10 +823,6 @@ not supported."
 
 ;;; environment
 
-(eval-when (:compile-toplevel :load-toplevel)
-  ;; Do this at compile-time as Win32 code below refers to it as
-  ;; sb-posix:getenv.
-  (export 'getenv :sb-posix))
 (defun getenv (name)
   (let ((r (alien-funcall
             (extern-alien "getenv" (function (* char) (c-string :not-null t)))
