@@ -10,9 +10,9 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 
-#!-sb-fluid (declaim (inline store-word))
+#-sb-fluid (declaim (inline store-word))
 (defun store-word (word base &optional (offset 0) (lowtag 0))
   (declare (type (unsigned-byte #.n-word-bits) word base offset)
            (type (unsigned-byte #.n-lowtag-bits) lowtag))
@@ -41,14 +41,22 @@
           :format-control "Not enough memory left in static space to ~
                            allocate vector.")))
 
-#!+immobile-space
+#+immobile-space
 (progn
 
-(define-load-time-global *immobile-space-mutex*
-    (sb!thread:make-mutex :name "Immobile space"))
+(define-alien-variable varyobj-space-size (unsigned 32))
+(define-alien-variable ("FIXEDOBJ_SPACE_START" fixedobj-space-start) unsigned-long)
+(define-alien-variable ("VARYOBJ_SPACE_START" varyobj-space-start) unsigned-long)
+(define-alien-variable ("varyobj_free_pointer" *varyobj-space-free-pointer*)
+  system-area-pointer)
+(define-alien-variable ("fixedobj_free_pointer" *fixedobj-space-free-pointer*)
+  system-area-pointer)
+
+(!define-load-time-global *immobile-space-mutex*
+    #.(sb-thread:make-mutex :name "Immobile space"))
 
 (eval-when (:compile-toplevel)
-  (assert (eql code-code-size-slot 1))
+  (assert (eql code-boxed-size-slot 1))
   (assert (eql code-debug-info-slot 2)))
 
 (define-alien-variable "varyobj_holes" long)
@@ -81,8 +89,9 @@
 
 (declaim (inline hole-p))
 (defun hole-p (raw-address)
-  (eql (sap-ref-32 (int-sap raw-address) 0)
-       (logior (ash 2 n-widetag-bits) code-header-widetag)))
+  ;; A code header with 0 boxed words is a hole.
+  ;; See also CODE-OBJ-IS-FILLER-P
+  (eql (sap-ref-64 (int-sap raw-address) 0) code-header-widetag))
 
 (defun freed-hole-p (address)
   (and (hole-p address)
@@ -95,13 +104,12 @@
 
 (declaim (inline hole-size))
 (defun hole-size (hole-address) ; in bytes
-  (+ (sap-ref-lispobj (int-sap hole-address) (ash code-code-size-slot word-shift))
-     (ash 2 word-shift))) ; add 2 boxed words
+  (ash (sap-ref-32 (int-sap hole-address) 4) word-shift))
 
 (declaim (inline (setf hole-size)))
 (defun (setf hole-size) (new-size hole) ; NEW-SIZE is in bytes
-  (setf (sap-ref-lispobj (int-sap hole) (ash code-code-size-slot word-shift))
-        (- new-size (ash 2 word-shift)))) ; account for 2 boxed words
+  (setf (sap-ref-32 (int-sap hole) 4) (ash new-size (- word-shift)))
+  new-size)
 
 (declaim (inline hole-end-address))
 (defun hole-end-address (hole-address)
@@ -130,7 +138,7 @@
          (table (cdr freelist))
          (old (gethash (hole-size hole) table)))
     ;; Check for double-free error
-    #!+immobile-space-debug (aver (not (member hole (gethash size table))))
+    #+immobile-space-debug (aver (not (member hole (gethash size table))))
     (unless old
       (setf (car freelist)
             (sorted-list-insert size (car freelist) #'identity)))
@@ -144,7 +152,7 @@
          (old-length (length list))
          (new (delete hole list :count 1)))
     (declare (ignorable old-length))
-    #!+immobile-space-debug (aver (= (length new) (1- old-length)))
+    #+immobile-space-debug (aver (= (length new) (1- old-length)))
     (cond (new
            (setf (gethash key table) new))
           (t
@@ -182,7 +190,7 @@
                  n-fixnum-tag-bits)))))
 
 (defun unallocate (hole)
-  #!+immobile-space-debug
+  #+immobile-space-debug
   (awhen *in-use-bits* (mark-range it hole (hole-size hole) nil))
   (let* ((hole-end (hole-end-address hole))
          (end-is-free-ptr (eql hole-end (sap-int *varyobj-space-free-pointer*))))
@@ -249,17 +257,16 @@
           (setf hole-end (hole-end-address successor))
           (remove-from-freelist successor)))
       ;; The hole must be an integral number of doublewords.
-      (aver (zerop (rem (- hole-end hole) 16)))
+      (aver (not (logtest (- hole-end hole) lowtag-mask)))
       (setf (hole-size hole) (- hole-end hole))))
   (add-to-freelist hole))
 
-(defun allocate-immobile-bytes (n-bytes word0 word1 lowtag)
+(defun allocate-immobile-obj (n-bytes word0 word1 lowtag errorp)
   (declare (type (and fixnum unsigned-byte) n-bytes))
-  (setq n-bytes (logandc2 (+ n-bytes (1- (* 2 n-word-bytes)))
-                          (1- (* 2 n-word-bytes))))
+  (setq n-bytes (align-up n-bytes (* 2 n-word-bytes)))
   ;; Can't allocate fewer than 4 words due to min hole size.
   (aver (>= n-bytes (* 4 n-word-bytes)))
-  (sb!thread::with-system-mutex (*immobile-space-mutex* :without-gcing t)
+  (sb-thread::with-system-mutex (*immobile-space-mutex* :without-gcing t)
    (unless (zerop varyobj-holes)
      ;; If deferred sweep needs to happen, do so now.
      ;; Concurrency could potentially be improved here: at most one thread
@@ -275,28 +282,34 @@
                 nil-value)
           (unallocate hole-addr)
           (if (eql (setq hole-addr next) 0) (return))))))
-   (let ((addr
-          (or (and *immobile-freelist*
-                   (or (find-in-freelist n-bytes '=) ; 1. Exact match?
-                       ;; 2. Try splitting a hole, adding some slack so that
-                       ;;    both pieces can potentially be used.
-                       (let ((found (find-in-freelist (+ n-bytes 192) '<=)))
-                         (when found
-                           (let* ((actual-size (hole-size found))
-                                  (remaining (- actual-size n-bytes)))
-                             (aver (zerop (rem actual-size 16)))
-                             (setf (hole-size found) remaining) ; Shorten the lower piece
-                             (add-to-freelist found)
-                             (+ found remaining)))))) ; Consume the upper piece
-              ;; 3. Extend the frontier.
-              (let* ((addr (sap-int *varyobj-space-free-pointer*))
-                     (free-ptr (+ addr n-bytes))
-                     (limit (+ varyobj-space-start varyobj-space-size)))
-                (when (> free-ptr limit)
-                  (format t "~&Immobile space exhausted~%")
-                  (sb!impl::%halt))
-                (set-varyobj-space-free-pointer free-ptr)
-                addr))))
+   (let* ((residual)
+          (shrunk-size)
+          (addr
+           (or (and *immobile-freelist*
+                    (or (find-in-freelist n-bytes '=) ; 1. Exact match?
+                        ;; 2. Try splitting a hole, adding some slack so that
+                        ;;    both pieces can potentially be used.
+                        (let ((found (find-in-freelist (+ n-bytes 192) '<=)))
+                          (when found
+                            (let* ((actual-size (hole-size found))
+                                   (remaining (- actual-size n-bytes)))
+                              (aver (not (logtest actual-size lowtag-mask)))
+                              (setq residual found ; Shorten the lower piece
+                                    shrunk-size remaining)
+                              (+ found remaining)))))) ; Consume the upper piece
+               ;; 3. Extend the frontier.
+               (let* ((addr (sap-int *varyobj-space-free-pointer*))
+                      (free-ptr (+ addr n-bytes))
+                      (limit (+ varyobj-space-start varyobj-space-size)))
+                 (when (> free-ptr limit)
+                   (cond (errorp
+                          (format t "~&Immobile space exhausted~%")
+                          (sb-debug:print-backtrace)
+                          (sb-impl::%halt))
+                         (t
+                          (return-from allocate-immobile-obj nil))))
+                 (set-varyobj-space-free-pointer free-ptr)
+                 addr))))
      (aver (not (logtest addr lowtag-mask))) ; Assert proper alignment
      ;; Compute the start and end of the first page consumed.
      (let* ((page-start (logandc2 addr (1- immobile-card-bytes)))
@@ -323,21 +336,36 @@
         (when (>= page-start obj-end) (return))
         (setf (deref varyobj-page-scan-start-offset index)
               (ash (- page-end addr) (- (1+ word-shift))))))
-     #!+immobile-space-debug ; "address sanitizer"
+     #+immobile-space-debug ; "address sanitizer"
      (awhen *in-use-bits* (mark-range it addr n-bytes t))
      (setf (sap-ref-word (int-sap addr) 0) word0
            (sap-ref-word (int-sap addr) n-word-bytes) word1)
      ;; 0-fill the remainder of the object
-     (#!+64-bit system-area-ub64-fill
-      #!-64-bit system-area-ub32-fill
+     (#+64-bit system-area-ub64-fill
+      #-64-bit system-area-ub32-fill
       0 (int-sap addr) 2 (- (ash n-bytes (- word-shift)) 2))
+     ;; Only after making the new object can we reduce the size of the hole
+     ;; that contained the new allocation (if it entailed chopping a hole
+     ;; into parts). In this way, heap scans do not read junk.
+     (when residual
+       (setf (hole-size residual) shrunk-size)
+       (add-to-freelist residual))
+     ;; The object is live despite not having a tagged pointer yet nor
+     ;; this code being pseudoatomic, because the mutex acquire has
+     ;; :WITHOUT-GCING. Ideally we'd have some notion of hazard pointers
+     ;; that could prevent GC from evicting objects from pointed-to pages
+     ;; so that we needn't inhibit GC.
      (%make-lisp-obj (logior addr lowtag)))))
 
+;;; I don't know whether immobile space vectors actually work.
+;;; I think the idea was to use them as fd-stream buffers when applicable
+;;; instead of having a finalizer on OS-allocated memory.
 (defun allocate-immobile-vector (widetag length words)
-  (allocate-immobile-bytes (pad-data-block (+ words vector-data-offset))
-                           widetag
-                           (fixnumize length)
-                           other-pointer-lowtag))
+  (allocate-immobile-obj (pad-data-block (+ words vector-data-offset))
+                         widetag
+                         (fixnumize length)
+                         other-pointer-lowtag
+                         t))
 
 (defun allocate-immobile-simple-vector (n-elements)
   (allocate-immobile-vector simple-vector-widetag n-elements n-elements))
@@ -348,28 +376,13 @@
   (allocate-immobile-vector simple-array-unsigned-byte-8-widetag n-elements
                             (ceiling n-elements n-word-bytes)))
 (defun allocate-immobile-word-vector (n-elements)
-  (allocate-immobile-vector #!+64-bit simple-array-unsigned-byte-64-widetag
-                            #!-64-bit simple-array-unsigned-byte-32-widetag
+  (allocate-immobile-vector #+64-bit simple-array-unsigned-byte-64-widetag
+                            #-64-bit simple-array-unsigned-byte-32-widetag
                             n-elements n-elements))
 
-;;; This is called when we're already inside WITHOUT-GCing
-(defun allocate-immobile-code (n-boxed-words n-unboxed-bytes)
-  (let* ((unrounded-header-n-words (+ code-constants-offset n-boxed-words))
-         (rounded-header-words (* 2 (ceiling unrounded-header-n-words 2)))
-         (total-bytes (+ (* rounded-header-words n-word-bytes)
-                         (logandc2 (+ n-unboxed-bytes lowtag-mask) lowtag-mask)))
-         (code (allocate-immobile-bytes
-                total-bytes
-                (logior (ash rounded-header-words n-widetag-bits) code-header-widetag)
-                (ash n-unboxed-bytes n-fixnum-tag-bits)
-                other-pointer-lowtag)))
-    (setf (%code-debug-info code) nil)
-    code))
-
 (defun alloc-immobile-symbol ()
   (values (%primitive alloc-immobile-fixedobj other-pointer-lowtag symbol-size
-                      (logior (ash (1- symbol-size) n-widetag-bits) symbol-widetag)
-                      nil)))
+                      (logior (ash (1- symbol-size) n-widetag-bits) symbol-widetag))))
 (defun make-immobile-symbol (name)
   (let ((symbol (truly-the symbol (alloc-immobile-symbol))))
     ;; no pin, it's immobile (and obviously live)
@@ -383,30 +396,134 @@
     symbol))
 
 (defun alloc-immobile-fdefn ()
-  (values (%primitive alloc-immobile-fixedobj other-pointer-lowtag fdefn-size
-                      (logior (ash (1- fdefn-size) n-widetag-bits) fdefn-widetag)
-                      nil)))
+  (if (= (alien-funcall (extern-alien "lisp_code_in_elf" (function int))) 1)
+      (allocate-immobile-obj (* fdefn-size n-word-bytes)
+                             (logior (ash (1- fdefn-size) n-widetag-bits) ; word 0
+                                     fdefn-widetag)
+                             0 other-pointer-lowtag t) ; word 1, lowtag, errorp
+      (values (%primitive alloc-immobile-fixedobj other-pointer-lowtag
+                          fdefn-size
+                          (logior (ash (1- fdefn-size) n-widetag-bits) ; word 0
+                                  fdefn-widetag)))))
 
+#+immobile-code
+(progn
 (defun alloc-immobile-gf ()
   (values (%primitive alloc-immobile-fixedobj fun-pointer-lowtag 6 ; kludge
-                      (logior (ash 5 n-widetag-bits) funcallable-instance-widetag)
-                      nil)))
+                      (logior (ash 5 n-widetag-bits) funcallable-instance-widetag))))
 (defun make-immobile-gf (layout slot-vector)
   (let ((gf (truly-the funcallable-instance (alloc-immobile-gf))))
     ;; Set layout prior to writing raw slots
     (setf (%funcallable-instance-layout gf) layout)
-    (setf (sap-ref-word (int-sap (get-lisp-obj-address gf))
-                        (- (ash funcallable-instance-trampoline-slot word-shift)
-                           fun-pointer-lowtag))
-          (truly-the word
-           (+ (get-lisp-obj-address gf) (- fun-pointer-lowtag) (ash 4 word-shift))))
+    ;; just being pedantic - liveness is preserved by the stack reference,
+    ;; and address is fixed, by definition.
+    (with-pinned-objects (gf)
+      (let ((addr (logandc2 (get-lisp-obj-address gf) lowtag-mask)))
+        (setf (sap-ref-word (int-sap addr)
+                            (ash funcallable-instance-trampoline-slot word-shift))
+              (truly-the word (+ addr (ash 4 word-shift))))))
     (%set-funcallable-instance-info gf 0 slot-vector)
-    (sb!vm::!set-fin-trampoline gf)
+    (!set-fin-trampoline gf)
     gf))
 
 (defun alloc-immobile-trampoline ()
-  (values (%primitive alloc-immobile-fixedobj other-pointer-lowtag 6
-                      (logior (ash 4 n-widetag-bits) code-header-widetag)
-                      (ash (* 2 n-word-bytes) n-fixnum-tag-bits))))
+  (let ((obj (%primitive alloc-immobile-fixedobj other-pointer-lowtag 6
+                         ;; total word count
+                         (logior (ash 6 code-header-size-shift) code-header-widetag))))
+    (with-pinned-objects (obj)
+      (setf (sap-ref-word (int-sap (get-lisp-obj-address obj))
+                          (- (ash code-boxed-size-slot word-shift)
+                             other-pointer-lowtag))
+            ;; The 'fixups' slot permanently contains 0. Therefore we can say that
+            ;; there are only 3 boxed header words instead of the usual 4 words.
+            (* 3 n-word-bytes))) ; boxed size in bytes, untagged
+    obj))
 
 ) ; end PROGN
+) ; end PROGN
+
+(declaim (inline immobile-space-addr-p))
+(defun immobile-space-addr-p (addr)
+  (declare (type word addr) (ignorable addr))
+  #+immobile-space
+  (or (let ((start fixedobj-space-start))
+        (<= start addr (truly-the word (+ start (1- fixedobj-space-size)))))
+      (let ((start varyobj-space-start))
+        (<= start addr (truly-the word (+ start (1- varyobj-space-size)))))))
+
+(defun immobile-space-obj-p (obj)
+  (immobile-space-addr-p (get-lisp-obj-address obj)))
+
+;;; Enforce limit on boxed words based on maximum total number of words
+;;; that can be indicated in the header for 32-bit words.
+;;; 22 bits = 4MiB, quite generous for one code object.
+;;; 25 bits is the maximum unboxed size expressed in bytes,
+;;; if n-word-bytes = 8 and almost the entire code object is unboxed
+;;; which is, practically speaking, not really possible.
+(declaim (ftype (sfunction (t (unsigned-byte 22) (unsigned-byte 25))
+                           code-component)
+                allocate-code-object))
+;;; Allocate a code component with BOXED words in the header
+;;; followed by UNBOXED bytes of raw data.
+;;; BOXED must be the exact count of boxed words desired. No adjustments
+;;; are made for alignment considerations or the fixed slots.
+(defun allocate-code-object (space boxed unboxed)
+  (declare (ignorable space))
+  (let* ((total-words
+          (the (unsigned-byte 22) ; Enforce limit on total words as well
+               (align-up (+ boxed (ceiling unboxed n-word-bytes)) 2)))
+         (code
+          #+gencgc
+          (or #+immobile-code
+              (when (member space '(:immobile :auto))
+                ;; We don't need to inhibit GC here - ALLOCATE-IMMOBILE-OBJ does it.
+                ;; Indicate that there are initially 2 boxed words, otherwise
+                ;; immobile space GC thinks this object is freeable.
+                (allocate-immobile-obj (ash total-words word-shift)
+                                       (logior (ash total-words code-header-size-shift)
+                                               code-header-widetag)
+                                       (ash 2 n-fixnum-tag-bits)
+                                       other-pointer-lowtag
+                                       (eq space :immobile)))
+              ;;; x86-64 has a vop which is nothing more than wrapping
+              ;; pseudo-atomic around a call to alloc_code_object() in the C runtime.
+              ;; The vop is defined in such a way that it can't be inserted into
+              ;; this fuction, but instead needs an out-of-line call to a helper function
+              ;; (because it clobbers all registers and doesn't indicate that)
+              #+(and x86-64 (not win32))
+              (alloc-dynamic-space-code total-words)
+              #-(and x86-64 (not win32))
+              (without-gcing
+               (%make-lisp-obj
+                (alien-funcall (extern-alien "alloc_code_object"
+                                             (function unsigned (unsigned 32)))
+                               total-words))))
+          #+cheneygc
+          (%primitive var-alloc total-words 'alloc-code
+                      ;; subtract 1 because var-alloc always adds 1 word
+                      ;; for the header, which is not right for code objects.
+                      -1 code-header-widetag other-pointer-lowtag)))
+    ;; The 1st slot beyond the header stores the boxed header size in bytes
+    ;; as an untagged number, which has the same representation as a tagged
+    ;; value denoting a word count if WORD-SHIFT = N-FIXNUM-TAG-BITS.
+    ;; This slot is allowed to be 0 prior to writing any pointer descriptors
+    ;; into the object.
+    ;; Use ATOMIC-INCF on the serialno to get automatic wraparound,
+    ;; and not because atomicity makes things deterministic, which it doesn't
+    ;; if there are several threads allocating code.
+    ;; TODO: this unnecessarily calls CODE-HEADER-SET if code cards use soft
+    ;; marking. Maybe pin the object and use (SETF SAP-REF-WORD) instead.
+    (setf (code-header-ref code code-boxed-size-slot)
+          (%make-lisp-obj
+           (logior (ash boxed word-shift)
+                   #+64-bit
+                   (logand (ash (atomic-incf sb-fasl::*code-serialno*) 32)
+                           most-positive-word))))
+    ;; Is this slot assignment necessary?  Both the C and Lisp backtrace logic
+    ;; check whether debug-info is an INSTANCE before operating with the value,
+    ;; and GC doesn't care what it is. Since stores to the code go through an
+    ;; assembly routine if soft marking is enabled, it's a spurious call
+    ;; to write a value that needn't add this object to a remembered set.
+    ;; FIXME: find out what breaks if this is removed.
+    (setf (%code-debug-info code) nil)
+    code))
