@@ -9,9 +9,9 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 
-#!-sb-fluid
+#-sb-fluid
 (declaim (inline adjustable-array-p
                  array-displacement))
 
@@ -138,6 +138,8 @@
                         (when ,type-sym
                           (ill-type))))
                     ,@body)))
+             (ill-type ()
+               `(go fastidiously-parse))
              (result (widetag)
                (let ((value (symbol-value widetag)))
                  `(values ,value
@@ -145,11 +147,7 @@
                             (find value
                                   *specialized-array-element-type-properties*
                                   :key #'saetp-typecode))))))
-    (flet ((ill-type ()
-             (declare (optimize allow-non-returning-tail-call))
-             (error "Invalid type specifier: ~/sb!impl:print-type-specifier/"
-                    type))
-           (integer-interval-widetag (low high)
+    (flet ((integer-interval-widetag (low high)
              (if (minusp low)
                  (%integer-vector-widetag-and-n-bits-shift
                   t
@@ -157,166 +155,188 @@
                  (%integer-vector-widetag-and-n-bits-shift
                   nil
                   (max (integer-length low) (integer-length high))))))
-      (let* ((consp (consp type))
-             (type-name (if consp
-                            (car type)
-                            type)))
-        (case type-name
-          ((t)
-           (when consp
-             (ill-type))
-           (result simple-vector-widetag))
-          ((base-char standard-char #!-sb-unicode character)
-           (when consp
-             (ill-type))
-           (result simple-base-string-widetag))
-          #!+sb-unicode
-          ((character extended-char)
-           (when consp
-             (ill-type))
-           (result simple-character-string-widetag))
-          (bit
-           (when consp
-             (ill-type))
-           (result simple-bit-vector-widetag))
-          (fixnum
-           (when consp
-             (ill-type))
-           (result simple-array-fixnum-widetag))
-          (unsigned-byte
-           (with-parameters ((integer 1)) (high)
-             (if (eq high '*)
-                 (result simple-vector-widetag)
-                 (%integer-vector-widetag-and-n-bits-shift nil high))))
-          (signed-byte
-           (with-parameters ((integer 1)) (high)
-             (if (eq high '*)
-                 (result simple-vector-widetag)
-                 (%integer-vector-widetag-and-n-bits-shift t high))))
-          (double-float
-           (with-parameters (double-float :intervals t) (low high)
-             (if (and (not (eq low '*))
-                      (not (eq high '*))
-                      (if (or (consp low) (consp high))
-                          (>= (type-bound-number low) (type-bound-number high))
-                          (> low high)))
-                 (result simple-array-nil-widetag)
-                 (result simple-array-double-float-widetag))))
-          (single-float
-           (with-parameters (single-float :intervals t) (low high)
-             (if (and (not (eq low '*))
-                      (not (eq high '*))
-                      (if (or (consp low) (consp high))
-                          (>= (type-bound-number low) (type-bound-number high))
-                          (> low high)))
-                 (result simple-array-nil-widetag)
-                 (result simple-array-single-float-widetag))))
-          (mod
-           (if (and (consp type)
-                    (consp (cdr type))
-                    (null (cddr type))
-                    (typep (cadr type) '(integer 1)))
-               (%integer-vector-widetag-and-n-bits-shift
-                nil (integer-length (1- (cadr type))))
-               (ill-type)))
-          #!+long-float
-          (long-float
-           (with-parameters (long-float :intervals t) (low high)
-             (if (and (not (eq low '*))
-                      (not (eq high '*))
-                      (if (or (consp low) (consp high))
-                          (>= (type-bound-number low) (type-bound-number high))
-                          (> low high)))
-                 (result simple-array-nil-widetag)
-                 (result simple-array-long-float-widetag))))
-          (integer
-           (with-parameters (integer :intervals t) (low high)
-             (let ((low (if (consp low)
-                            (1+ (car low))
-                            low))
-                   (high (if (consp high)
-                             (1- (car high))
-                             high)))
-               (cond ((or (eq high '*)
-                          (eq low '*))
-                      (result simple-vector-widetag))
-                     ((> low high)
-                      (result simple-array-nil-widetag))
-                     (t
-                      (integer-interval-widetag low high))))))
-          (complex
-           (with-parameters (t) (subtype)
-             (if (eq subtype '*)
-                 (result simple-vector-widetag)
-                 (let ((ctype (specifier-type type)))
-                   (cond ((eq ctype *empty-type*)
-                          (result simple-array-nil-widetag))
-                         ((union-type-p ctype)
-                          (cond ((csubtypep ctype (specifier-type '(complex double-float)))
-                                 (result
-                                  simple-array-complex-double-float-widetag))
-                                ((csubtypep ctype (specifier-type '(complex single-float)))
-                                 (result
-                                  simple-array-complex-single-float-widetag))
-                                #!+long-float
-                                ((csubtypep ctype (specifier-type '(complex long-float)))
-                                 (result
-                                  simple-array-complex-long-float-widetag))
-                                (t
-                                 (result simple-vector-widetag))))
-                         (t
-                          (case (numeric-type-format ctype)
-                            (double-float
-                             (result
-                              simple-array-complex-double-float-widetag))
-                            (single-float
-                             (result
-                              simple-array-complex-single-float-widetag))
-                            #!+long-float
-                            (long-float
-                             (result
-                              simple-array-complex-long-float-widetag))
-                            (t
-                             (result simple-vector-widetag)))))))))
-          ((nil)
-           (result simple-array-nil-widetag))
-          (t
-           (block nil
-             (let ((ctype (type-or-nil-if-unknown type)))
-               (unless ctype
-                 (return (result simple-vector-widetag)))
-               (typecase ctype
-                 (union-type
-                  (let ((types (union-type-types ctype)))
-                    (cond ((not (every #'numeric-type-p types))
-                           (result simple-vector-widetag))
-                          ((csubtypep ctype (specifier-type 'integer))
-                           (integer-interval-widetag
-                            (reduce #'min types :key #'numeric-type-low)
-                            (reduce #'max types :key #'numeric-type-high)))
-                          ((csubtypep ctype (specifier-type 'double-float))
-                           (result simple-array-double-float-widetag))
-                          ((csubtypep ctype (specifier-type 'single-float))
-                           (result simple-array-single-float-widetag))
-                          #!+long-float
-                          ((csubtypep ctype (specifier-type 'long-float))
-                           (result simple-array-long-float-widetag))
-                          (t
-                           (result simple-vector-widetag)))))
-                 (character-set-type
-                  #!-sb-unicode (result simple-base-string-widetag)
-                  #!+sb-unicode
-                  (if (loop for (start . end)
-                            in (character-set-type-pairs ctype)
-                            always (and (< start base-char-code-limit)
-                                        (< end base-char-code-limit)))
-                      (result simple-base-string-widetag)
-                      (result simple-character-string-widetag)))
-                 (t
-                  (let ((expansion (type-specifier ctype)))
-                    (if (equal expansion type)
+      (tagbody
+         (binding*
+             ((consp (consp type))
+              (type-name (if consp (car type) type))
+              ((widetag n-bits-shift)
+               (case type-name
+                 ((t)
+                  (when consp
+                    (ill-type))
+                  (result simple-vector-widetag))
+                 ((base-char standard-char #-sb-unicode character)
+                  (when consp
+                    (ill-type))
+                  (result simple-base-string-widetag))
+                 #+sb-unicode
+                 ((character extended-char)
+                  (when consp
+                    (ill-type))
+                  (result simple-character-string-widetag))
+                 (bit
+                  (when consp
+                    (ill-type))
+                  (result simple-bit-vector-widetag))
+                 (fixnum
+                  (when consp
+                    (ill-type))
+                  (result simple-array-fixnum-widetag))
+                 (unsigned-byte
+                  (with-parameters ((integer 1)) (high)
+                    (if (eq high '*)
                         (result simple-vector-widetag)
-                        (%vector-widetag-and-n-bits-shift expansion)))))))))))))
+                        (%integer-vector-widetag-and-n-bits-shift nil high))))
+                 (signed-byte
+                  (with-parameters ((integer 1)) (high)
+                    (if (eq high '*)
+                        (result simple-vector-widetag)
+                        (%integer-vector-widetag-and-n-bits-shift t high))))
+                 (double-float
+                  (with-parameters (double-float :intervals t) (low high)
+                    (if (and (not (eq low '*))
+                             (not (eq high '*))
+                             (if (or (consp low) (consp high))
+                                 (>= (type-bound-number low) (type-bound-number high))
+                                 (> low high)))
+                        (result simple-array-nil-widetag)
+                        (result simple-array-double-float-widetag))))
+                 (single-float
+                  (with-parameters (single-float :intervals t) (low high)
+                    (if (and (not (eq low '*))
+                             (not (eq high '*))
+                             (if (or (consp low) (consp high))
+                                 (>= (type-bound-number low) (type-bound-number high))
+                                 (> low high)))
+                        (result simple-array-nil-widetag)
+                        (result simple-array-single-float-widetag))))
+                 (mod
+                  (if (and (consp type)
+                           (consp (cdr type))
+                           (null (cddr type))
+                           (typep (cadr type) '(integer 1)))
+                      (%integer-vector-widetag-and-n-bits-shift
+                       nil (integer-length (1- (cadr type))))
+                      (ill-type)))
+                 #+long-float
+                 (long-float
+                  (with-parameters (long-float :intervals t) (low high)
+                    (if (and (not (eq low '*))
+                             (not (eq high '*))
+                             (if (or (consp low) (consp high))
+                                 (>= (type-bound-number low) (type-bound-number high))
+                                 (> low high)))
+                        (result simple-array-nil-widetag)
+                        (result simple-array-long-float-widetag))))
+                 (integer
+                  (with-parameters (integer :intervals t) (low high)
+                    (let ((low (if (consp low)
+                                   (1+ (car low))
+                                   low))
+                          (high (if (consp high)
+                                    (1- (car high))
+                                    high)))
+                      (cond ((or (eq high '*)
+                                 (eq low '*))
+                             (result simple-vector-widetag))
+                            ((> low high)
+                             (result simple-array-nil-widetag))
+                            (t
+                             (integer-interval-widetag low high))))))
+                 (complex
+                  (with-parameters (t) (subtype)
+                    (if (eq subtype '*)
+                        (result simple-vector-widetag)
+                        (let ((ctype (specifier-type type)))
+                          (cond ((eq ctype *empty-type*)
+                                 (result simple-array-nil-widetag))
+                                ((union-type-p ctype)
+                                 (cond ((csubtypep ctype (specifier-type '(complex double-float)))
+                                        (result
+                                         simple-array-complex-double-float-widetag))
+                                       ((csubtypep ctype (specifier-type '(complex single-float)))
+                                        (result
+                                         simple-array-complex-single-float-widetag))
+                                       #+long-float
+                                       ((csubtypep ctype (specifier-type '(complex long-float)))
+                                        (result
+                                         simple-array-complex-long-float-widetag))
+                                       (t
+                                        (result simple-vector-widetag))))
+                                (t
+                                 (case (numeric-type-format ctype)
+                                   (double-float
+                                    (result
+                                     simple-array-complex-double-float-widetag))
+                                   (single-float
+                                    (result
+                                     simple-array-complex-single-float-widetag))
+                                   #+long-float
+                                   (long-float
+                                    (result
+                                     simple-array-complex-long-float-widetag))
+                                   (t
+                                    (result simple-vector-widetag)))))))))
+                 ((nil)
+                  (result simple-array-nil-widetag))
+                 (t
+                  (go fastidiously-parse)))))
+           (return-from %vector-widetag-and-n-bits-shift
+             (values widetag n-bits-shift)))
+       fastidiously-parse)
+      ;; Do things the hard way after falling through the tagbody.
+      (let* ((ctype (type-or-nil-if-unknown type))
+             (ctype (and ctype
+                         (sb-kernel::replace-hairy-type ctype))))
+        (typecase ctype
+          (null (result simple-vector-widetag))
+          (union-type
+           (let ((types (union-type-types ctype)))
+             (cond ((not (every #'numeric-type-p types))
+                    (result simple-vector-widetag))
+                   ((csubtypep ctype (specifier-type 'integer))
+                    (block nil
+                      (integer-interval-widetag
+                       (dx-flet ((low (x)
+                                      (or (numeric-type-low x)
+                                          (return (result simple-vector-widetag)))))
+                         (reduce #'min types :key #'low))
+                       (dx-flet ((high (x)
+                                       (or (numeric-type-high x)
+                                           (return (result simple-vector-widetag)))))
+                         (reduce #'max types :key #'high)))))
+                   ((csubtypep ctype (specifier-type 'double-float))
+                    (result simple-array-double-float-widetag))
+                   ((csubtypep ctype (specifier-type 'single-float))
+                    (result simple-array-single-float-widetag))
+                   #+long-float
+                   ((csubtypep ctype (specifier-type 'long-float))
+                    (result simple-array-long-float-widetag))
+                   ((csubtypep ctype (specifier-type 'complex-double-float))
+                    (result simple-array-complex-double-float-widetag))
+                   ((csubtypep ctype (specifier-type 'complex-single-float))
+                    (result simple-array-complex-single-float-widetag))
+                   (t
+                    (result simple-vector-widetag)))))
+          (intersection-type
+           (let ((types (intersection-type-types ctype)))
+             (loop for type in types
+                   unless (hairy-type-p type)
+                   return (%vector-widetag-and-n-bits-shift (type-specifier type)))))
+          (character-set-type
+           #-sb-unicode (result simple-base-string-widetag)
+           #+sb-unicode
+           (if (loop for (start . end)
+                     in (character-set-type-pairs ctype)
+                     always (and (< start base-char-code-limit)
+                                 (< end base-char-code-limit)))
+               (result simple-base-string-widetag)
+               (result simple-character-string-widetag)))
+          (t
+           (let ((expansion (type-specifier ctype)))
+             (if (equal expansion type)
+                 (result simple-vector-widetag)
+                 (%vector-widetag-and-n-bits-shift expansion)))))))))
 
 (defun %complex-vector-widetag (widetag)
   (macrolet ((make-case ()
@@ -354,7 +374,7 @@
   (let* ((n-bits-shift (or n-bits-shift
                            (aref %%simple-array-n-bits-shifts%% widetag)))
          (full-length (if (or (= widetag simple-base-string-widetag)
-                              #!+sb-unicode
+                              #+sb-unicode
                               (= widetag
                                  simple-character-string-widetag))
                           (1+ length)
@@ -385,7 +405,7 @@
     (let ((widetag (%other-pointer-widetag array)))
       (make-case))))
 
-(defun sb!impl::make-vector-like (vector length)
+(defun sb-impl::make-vector-like (vector length)
   (allocate-vector-with-widetag (array-underlying-widetag vector) length nil))
 
 ;; Complain in various ways about wrong :INITIAL-foo arguments,
@@ -541,7 +561,7 @@ of specialized arrays is supported."
   ;; is expensive, but probably worth the trouble as once we've allocated
   ;; the vector we have no way to get rid of it anymore...
   (when (eq t (upgraded-array-element-type element-type))
-    (error "Static arrays of type ~/sb!impl:print-type-specifier/ not supported."
+    (error "Static arrays of type ~/sb-impl:print-type-specifier/ not supported."
            element-type))
   (validate-array-initargs initial-element-p initial-element
                            initial-contents-p initial-contents nil) ; for effect
@@ -553,22 +573,29 @@ of specialized arrays is supported."
              length))
     (unless (every (lambda (x) (typep x element-type)) initial-contents)
       (error ":INITIAL-CONTENTS contains elements not of type ~
-               ~/sb!impl:print-type-specifier/."
+               ~/sb-impl:print-type-specifier/."
              element-type)))
   (when initial-element-p
     (unless (typep initial-element element-type)
       (error ":INITIAL-ELEMENT ~S is not of type ~
-               ~/sb!impl:print-type-specifier/."
+               ~/sb-impl:print-type-specifier/."
              initial-element element-type)))
   ;; STEP 2
   ;;
   ;; Allocate and possibly initialize the vector.
   (multiple-value-bind (type n-bits-shift)
       (%vector-widetag-and-n-bits-shift element-type)
-    (let ((vector
-            (allocate-static-vector type length
-                                    (vector-length-in-words length
-                                                            n-bits-shift))))
+    (let* ((full-length
+             (if (or (= type simple-base-string-widetag)
+                     #+sb-unicode
+                     (= type
+                        simple-character-string-widetag))
+                 (1+ length)
+                 length))
+           (vector
+             (allocate-static-vector type length
+                                     (vector-length-in-words full-length
+                                                             n-bits-shift))))
       (cond (initial-element-p
              (fill vector initial-element))
             (initial-contents-p
@@ -629,6 +656,19 @@ of specialized arrays is supported."
                                ,table-name))
                 (defmacro ,name (array-var)
                   `(the function
+                     ;; Assigning TAG to 0 initially produces slightly better
+                     ;; code than would be generated by the more natural expression
+                     ;;   (let ((tag (if (%other-ptr ...) (widetag ...) 0)))
+                     ;; but either way is suboptimal. As expressed, if the array-var
+                     ;; is known to satisfy %other-pointer-p, then it performs a
+                     ;; move-immediate-to-register which is clobbered right away
+                     ;; by a zero-extending load. A peephole pass could eliminate
+                     ;; the first move as effectless.  If expressed the other way,
+                     ;; it would produce a jump around a jump because the compiler
+                     ;; is unwilling to *unconditionally* assign 0 into a register
+                     ;; to begin with. It actually wants to guard an immediate load
+                     ;; when it doesn't need to, as if both consequents of the IF
+                     ;; have side-effects that should not happen.
                      (let ((tag 0))
                        (when (%other-pointer-p ,array-var)
                          (setf tag (%other-pointer-widetag ,array-var)))
@@ -641,7 +681,7 @@ of specialized arrays is supported."
   (def !find-data-vector-reffer/check-bounds %%data-vector-reffers/check-bounds%%))
 
 ;;; Like DOVECTOR, but more magical -- can't use this on host.
-(defmacro sb!impl::do-vector-data ((elt vector &optional result) &body body)
+(defmacro sb-impl::do-vector-data ((elt vector &optional result) &body body)
   (multiple-value-bind (forms decls) (parse-body body nil)
     (with-unique-names (index vec start end ref)
       `(with-array-data ((,vec ,vector)
@@ -653,7 +693,7 @@ of specialized arrays is supported."
            (do ((,index ,start (1+ ,index)))
                ((>= ,index ,end)
                 (let ((,elt nil))
-                  ,@(sb!impl::filter-dolist-declarations decls)
+                  ,@(sb-impl::filter-dolist-declarations decls)
                   ,elt
                   ,result))
              (let ((,elt (funcall ,ref ,vec ,index)))
@@ -765,7 +805,7 @@ of specialized arrays is supported."
                 ,@(loop for widetag in '(complex-vector-widetag
                                          complex-vector-nil-widetag
                                          complex-bit-vector-widetag
-                                         #!+sb-unicode complex-character-string-widetag
+                                         #+sb-unicode complex-character-string-widetag
                                          complex-base-string-widetag
                                          simple-array-widetag
                                          complex-array-widetag)
@@ -1016,7 +1056,7 @@ of specialized arrays is supported."
         ((typep array 'vector)
          (list (length array)))
         (t
-         (sb!c::%type-check-error/c array 'object-not-array-error nil))))
+         (sb-c::%type-check-error/c array 'object-not-array-error nil))))
 
 (defun array-total-size (array)
   "Return the total number of elements in the Array."
@@ -1026,7 +1066,7 @@ of specialized arrays is supported."
         ((typep array 'vector)
          (length array))
         (t
-         (sb!c::%type-check-error/c array 'object-not-array-error nil))))
+         (sb-c::%type-check-error/c array 'object-not-array-error nil))))
 
 (defun array-displacement (array)
   "Return the values of :DISPLACED-TO and :DISPLACED-INDEX-offset
@@ -1146,7 +1186,7 @@ of specialized arrays is supported."
   (let* ((old-length (length vector))
          (min-extension (or min-extension
                             (min old-length
-                                 (- array-dimension-limit old-length))))
+                                 (- sb-xc:array-dimension-limit old-length))))
          (new-length (the index (+ old-length
                                    (max 1 min-extension))))
          (fill-pointer (1+ old-length)))
@@ -1256,8 +1296,8 @@ of specialized arrays is supported."
              (when (/= (array-underlying-widetag displaced-to) widetag)
                ;; See lp#1331299 again. Require exact match on upgraded type?
                (error "can't displace an array of type ~
-                        ~/sb!impl:print-type-specifier/ into another ~
-                        of type ~/sb!impl:print-type-specifier/"
+                        ~/sb-impl:print-type-specifier/ into another ~
+                        of type ~/sb-impl:print-type-specifier/"
                       element-type (array-element-type displaced-to)))
              (let ((displacement (or displaced-index-offset 0))
                    (array-size  (if (listp dimensions)
@@ -1394,7 +1434,7 @@ of specialized arrays is supported."
                  (lambda (saetp)
                    `((simple-array ,(saetp-specifier saetp) (*))
                      ,(if (or (eq (saetp-specifier saetp) 'character)
-                              #!+sb-unicode
+                              #+sb-unicode
                               (eq (saetp-specifier saetp) 'base-char))
                           '(code-char 0)
                           (saetp-initial-element-default saetp))))
@@ -1535,7 +1575,7 @@ function to be removed without further warning."
          (unless (or (not initial-element-p)
                      (typep initial-element element-type))
            (error "~S can't be used to initialize an array of type ~
-                    ~/sb!impl:print-type-specifier/."
+                    ~/sb-impl:print-type-specifier/."
                   initial-element element-type))
          (let ((temp (if initial-element-p
                          (make-array new-length :initial-element initial-element)
@@ -1641,7 +1681,12 @@ function to be removed without further warning."
               bit-array-1 result-bit-array))
      result-bit-array)))
 
-(defmacro def-bit-array-op (name function)
+;;; This used to be a DEFMACRO, but depending on the target's support for Unicode,
+;;; it got a constant-folding-error in the FORMAT call when producing the load-time
+;;; macro. CONCATENATE-FORMAT-P returns true, so then we want to know whether the
+;;; result is a base-string which entails calling SB-KERNEL:SIMPLE-BASE-STRING-P
+;;; which has no definition in the cross-compiler. (We could add one of course)
+(macrolet ((def-bit-array-op (name function)
   `(defun ,name (bit-array-1 bit-array-2 &optional result-bit-array)
      ,(format nil
               "Perform a bit-wise ~A on the elements of BIT-ARRAY-1 and ~
@@ -1681,7 +1726,7 @@ function to be removed without further warning."
                              (logand (,function (sbit data1 index-1)
                                                 (sbit data2 index-2))
                                      1))))
-                 result-bit-array)))))))
+                 result-bit-array))))))))
 
 (def-bit-array-op bit-and logand)
 (def-bit-array-op bit-ior logior)
@@ -1693,6 +1738,7 @@ function to be removed without further warning."
 (def-bit-array-op bit-andc2 logandc2)
 (def-bit-array-op bit-orc1 logorc1)
 (def-bit-array-op bit-orc2 logorc2)
+) ; end MACROLET
 
 (defun bit-not (bit-array &optional result-bit-array)
   "Performs a bit-wise logical NOT on the elements of BIT-ARRAY,
@@ -1744,7 +1790,7 @@ function to be removed without further warning."
 ;;; Finally, the DISPATCH-FOO macro is defined which does the actual
 ;;; dispatching when called. It expects arguments that match PARAMS.
 ;;;
-(defmacro sb!impl::!define-array-dispatch (dispatch-name params &body body)
+(defmacro sb-impl::!define-array-dispatch (dispatch-name params &body body)
   (let ((table-name (symbolicate "%%" dispatch-name "-FUNS%%"))
         (error-name (symbolicate "HAIRY-" dispatch-name "-ERROR")))
     `(progn
@@ -1789,7 +1835,7 @@ function to be removed without further warning."
                  (svref ,',table-name ,tag)))
              ,@args))))))
 
-(defun sb!kernel::check-array-shape (array dimensions)
+(defun sb-kernel::check-array-shape (array dimensions)
   (when (let ((dimensions dimensions))
           (dotimes (i (array-rank array))
             (unless (eql (array-dimension array i) (pop dimensions))
@@ -1806,4 +1852,23 @@ function to be removed without further warning."
 
 ;;; Horrible kludge for the "static-vectors" system
 ;;; which uses an internal symbol in SB-IMPL.
-(import '%vector-widetag-and-n-bits-shift 'sb!impl)
+(import '%vector-widetag-and-n-bits-shift 'sb-impl)
+
+(defun make-weak-vector (length &key (initial-contents nil contents-p)
+                                     (initial-element nil element-p))
+  (declare (index length))
+  (when (and element-p contents-p)
+    (error "Can't specify both :INITIAL-ELEMENT and :INITIAL-CONTENTS"))
+  (let ((v (if contents-p
+               (make-array length :initial-contents initial-contents)
+               (make-array length :initial-element
+                           ;; 0 is the usual default, but NIL makes more sense
+                           ;; for weak vectors because it's the value assigned
+                           ;; when a pointer is broken by GC.
+                           (if element-p initial-element nil)))))
+    (set-header-data v vector-weak-subtype)
+    v))
+
+(defun weak-vector-p (x)
+  (and (simple-vector-p x)
+       (eql (get-header-data x) vector-weak-subtype)))
