@@ -10,7 +10,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!IMPL")
+(in-package "SB-IMPL")
 
 ;;; Forward declarations
 
@@ -47,10 +47,21 @@
 ;;; CL:STREAM.
 (deftype ansi-stream () 'stream)
 
+;;; In the target SBCL, the INSTANCE type refers to a base
+;;; implementation for compound types with lowtag
+;;; INSTANCE-POINTER-LOWTAG. There's no way to express exactly that
+;;; concept portably, but we can get essentially the same effect by
+;;; testing for any of the standard types which would, in the target
+;;; SBCL, be derived from INSTANCE:
 (deftype instance ()
   '(or condition structure-object standard-object))
+(defun %instancep (x)
+  (typep x 'instance))
+
 (deftype funcallable-instance ()
   (error "not clear how to represent FUNCALLABLE-INSTANCE type"))
+(defun funcallable-instance-p (x)
+  (error "Called FUNCALLABLE-INSTANCE-P ~s" x))
 
 ;; The definition of TYPE-SPECIFIER for the target appears in the file
 ;; 'deftypes-for-target' - it allows CLASSes and CLASOIDs as specifiers.
@@ -59,28 +70,6 @@
 ;; define the types CLASS, CLASSOID, and TYPE-SPECIFIER.
 (deftype type-specifier () '(or list symbol))
 
-;;; In the target SBCL, the INSTANCE type refers to a base
-;;; implementation for compound types with lowtag
-;;; INSTANCE-POINTER-LOWTAG. There's no way to express exactly that
-;;; concept portably, but we can get essentially the same effect by
-;;; testing for any of the standard types which would, in the target
-;;; SBCL, be derived from INSTANCE:
-(defun %instancep (x)
-  (typep x '(or condition structure-object standard-object)))
-
-;;; There aren't any FUNCALLABLE-INSTANCEs in the cross-compilation
-;;; host Common Lisp.
-(defun funcallable-instance-p (x)
-  (if (typep x 'generic-function)
-    ;; In the target SBCL, FUNCALLABLE-INSTANCEs are used to implement
-    ;; generic functions, so any case which tests for this might in
-    ;; fact be trying to test for generic functions. My (WHN 19990313)
-    ;; expectation is that this case won't arise in the
-    ;; cross-compiler, but if it does, it deserves a little thought,
-    ;; rather than reflexively returning NIL.
-    (error "not clear how to handle GENERIC-FUNCTION")
-    nil))
-
 ;;; This seems to be the portable Common Lisp type test which
 ;;; corresponds to the effect of the target SBCL implementation test...
 (defun array-header-p (x)
@@ -88,14 +77,14 @@
        (or (not (typep x 'simple-array))
            (/= (array-rank x) 1))))
 
-(defvar sb!xc:*gensym-counter* 0)
+(defvar sb-xc:*gensym-counter* 0)
 
-(defun sb!xc:gensym (&optional (thing "G"))
+(defun sb-xc:gensym (&optional (thing "G"))
   (declare (type string thing))
-  (let ((n sb!xc:*gensym-counter*))
+  (let ((n sb-xc:*gensym-counter*))
     (prog1
         (make-symbol (concatenate 'string thing (write-to-string n :base 10 :radix nil :pretty nil)))
-      (incf sb!xc:*gensym-counter*))))
+      (incf sb-xc:*gensym-counter*))))
 
 ;;; These functions are needed for constant-folding.
 (defun simple-array-nil-p (object)
@@ -126,16 +115,6 @@
   (assert (typep array '(simple-array * (*))))
   (values array start end 0))
 
-(defun signed-byte-32-p (number)
-  (typep number '(signed-byte 32)))
-
-;; This has an obvious portable implementation
-;; as (typep number 'ratio), but apparently we
-;; expect never to need it.
-(defun ratiop (number)
-  (declare (ignore number))
-  (error "Should not call RATIOP"))
-
 (defun make-value-cell (value)
   (declare (ignore value))
   (error "cross-compiler can not make value cells"))
@@ -147,19 +126,15 @@
 
 (defmacro with-single-package-locked-error ((&optional kind thing &rest format)
                                             &body body)
-  ;; FIXME: perhaps this should touch THING to make it used?
-  (declare (ignore kind thing format))
-  `(progn ,@body))
+  (declare (ignore kind format))
+  `(let ((.dummy. ,thing))
+     (declare (ignore .dummy.))
+     ,@body))
 
 (defun program-assert-symbol-home-package-unlocked (context symbol control)
   (declare (ignore context control))
   symbol)
 
-(defun assert-package-unlocked (package &optional format-control
-                                &rest format-arguments)
-  (declare (ignore format-control format-arguments))
-  package)
-
 (defun assert-symbol-home-package-unlocked (name &optional format-control
                                             &rest format-arguments)
   (declare (ignore format-control format-arguments))
@@ -172,30 +147,36 @@
 ;; But portably we have to just fallback to PACKAGE-NAME.
 (defun package-%name (x) (package-name x))
 
+;;; This definition collapses SB-XC back into COMMON-LISP.
+;;; Use CL:SYMBOL-PACKAGE if that's not the behavior you want.
+;;; Notice that to determine whether a package is really supposed to be CL,
+;;; we look for the symbol in the restricted lisp package, not the real
+;;; host CL package. This works around situations where the host has *more*
+;;; symbols exported from CL than should be.
+(defun sb-xc:symbol-package (symbol)
+  (let ((p (cl:symbol-package symbol)))
+    (if (and p
+             (or (eq (find-symbol (string symbol) "XC-STRICT-CL") symbol)
+                 (eq (find-symbol (string symbol) "SB-XC") symbol)))
+        *cl-package*
+        p)))
+
 ;;; printing structures
 
 (defun default-structure-print (structure stream depth)
   (declare (ignore depth))
   (write structure :stream stream :circle t))
 
-(in-package "SB!KERNEL")
+(in-package "SB-KERNEL")
 (defun %find-position (item seq from-end start end key test)
   (let ((position (position item seq :from-end from-end
                             :start start :end end :key key :test test)))
     (values (if position (elt seq position) nil) position)))
 
-(defun sb!impl::split-seconds-for-sleep (&rest args)
+(defun sb-impl::split-seconds-for-sleep (&rest args)
   (declare (ignore args))
   (error "Can't call SPLIT-SECONDS-FOR-SLEEP"))
 
-;;; Avoid an unknown type reference from globaldb.
-(deftype fdefn () '(satisfies fdefn-p))
-
-;;; Avoid an unknown function reference from globaldb on some build
-;;; hosts.  It doesn't really matter what this function does: we don't
-;;; have FDEFN objects on the host anyway.
-(defun fdefn-p (x) (declare (ignore x)) nil)
-
 ;;; Needed for constant-folding
 (defun system-area-pointer-p (x) x nil) ; nothing is a SAP
 ;;; Needed for DEFINE-MOVE-FUN LOAD-SYSTEM-AREA-POINTER
@@ -210,12 +191,10 @@
 
 ;;;; Variables which have meaning only to the cross-compiler, defined here
 ;;;; in lieu of #+sb-xc-host elsewere which messes up toplevel form numbers.
-(in-package "SB!C")
+(in-package "SB-C")
 
-;;; Set of function names whose definition will never be seen in make-host-2,
-;;; as they are deferred until warm load.
-;;; The table is populated by compile-cold-sbcl, and not present in the target.
-(defparameter *undefined-fun-whitelist* (make-hash-table :test 'equal))
+;;; For macro lambdas that are processed by the host
+(declaim (declaration top-level-form))
 
 ;;; The opposite of the whitelist - if certain full calls are seen, it is probably
 ;;; the result of a missed transform and/or misconfiguration.
@@ -229,17 +208,104 @@
   (let* ((n (length string))
          (a (make-array n :element-type '(unsigned-byte 8))))
     (dotimes (i n a)
-      (let ((code (sb!xc:char-code (char string i))))
+      (let ((code (sb-xc:char-code (char string i))))
         (unless (<= 0 code 127)
-          (setf code (sb!xc:char-code #\?)))
+          (setf code (sb-xc:char-code #\?)))
         (setf (aref a i) code)))))
 
 ;;;; Stubs for host
-(defun sb!c:compile-in-lexenv (lambda lexenv &rest rest)
-  (declare (ignore lexenv))
-  (assert (null rest))
-  (compile nil lambda))
+(defun sb-c:compile-in-lexenv (lambda &rest rest)
+  (declare (ignore rest))
+  (compile nil (if (eq (car lambda) 'named-lambda)
+                   `(lambda ,@(cddr lambda))
+                   lambda)))
 
+(defun sb-impl::%defun (name lambda &optional inline-expansion)
+  (declare (ignore inline-expansion))
+  (proclaim `(ftype function ,name))
+  (setf (fdefinition name) (eval lambda)))
+
+(defun %svset (vector index val) ; stemming from toplevel (SETF SVREF)
+  (setf (aref vector index) val))
+(defun %puthash (key table val) ; stemming from toplevel (SETF GETHASH)
+  (setf (gethash key table) val))
+
+;;; The compiler calls this with forms in EVAL-WHEN (:COMPILE-TOPLEVEL) situations.
+;;; Since we've already performed macroexpansion using our macros, we can either
+;;; implement target-compatible functions for all things into which we might expand,
+;;; or we can un-macro-expand the form.  This does a little of both,
+;;; mainly for the sake of showing that it's quite easily done.
+;;; Truth be told I'd have preferred to use the anti-expansion technique consistently,
+;;; however occasionally we see things like (LET ((V FROB)) (%SVSET *THING* X V))
+;;; which means that the host is going to do the LET and the call %SVSET.
 (defun eval-tlf (form index &optional lexenv)
   (declare (ignore index lexenv))
-  (eval form))
+  (flet ((matchp (template form &aux results)
+           (if (named-let recurse ((form form) (template template))
+                 (typecase template
+                   (null (null form))
+                   ((eql ?) (push form results) t) ; match and store anything
+                   ((eql :ignore) t) ; match anything and disregard
+                   (cons (and (consp form)
+                              (and (recurse (car form) (car template))
+                                   (recurse (cdr form) (cdr template)))))
+                   (t (eql template form)))) ; match template exactly
+               (nreverse results)
+               (error "Pattern match failure: ~S~% ~S~%" template form))))
+    ;; Note that in all cases below, the package lock on CL prevents
+    ;; accidental appearance of a CL symbol as a thing being defined.
+    (named-let recurse ((form form))
+      (case (car form)
+       (progn (mapc #'recurse (cdr form))) ; compiler doesn't care about return value
+       (t
+        (eval
+         (case (car form)
+           (sb-impl::%defglobal
+            (destructuring-bind (symbol value)
+                (matchp '((quote ?) (if (%boundp :ignore) :ignore ?)) (cdr form))
+              `(defvar ,symbol ,value)))
+           (sb-impl::%defparameter
+            (destructuring-bind (symbol value)
+                (matchp '((quote ?) ? :ignore) (cdr form))
+              `(defparameter ,symbol ,value)))
+           (sb-impl::%defvar
+            (destructuring-bind (symbol value) ; always occurs with a value
+                (matchp '((quote ?) (source-location) (unless (%boundp :ignore) ?))
+                        (cdr form))
+              `(defvar ,symbol ,value)))
+           (sb-c::%defconstant
+            ;; There is genuinely ambiguity here - does :COMPILE-TOPLEVEL situation for
+            ;; DEFCONSTANT mean that we want the host compiler to know the constant?
+            ;; It must, because the standard specifies that defconstant need not be within
+            ;; EVAL-WHEN for the compiler to know it. But because of how our defconstant
+            ;; expands - calling %defconstant inside of an eval-when listing all 3
+            ;; situations - we can't discern whether this is our defconstant doing its
+            ;; normal thing, versus inside an explicity written eval-when with intent
+            ;; to convey the constant to the host - perhaps because of a DEFUN also inside
+            ;; an eval-when where we need the host to reference the constant.
+            ;; Therefore every constant has to be made known to the host under the
+            ;; assumption that it needs it, AND the cross-compiler under the assumption
+            ;; that this is our normal :compile-toplevel handling.
+            (destructuring-bind (symbol value) (matchp '((quote ?) ? . :ignore) (cdr form))
+              `(progn (defconstant ,symbol ,value)
+                      (sb-c::%defconstant ',symbol ,symbol nil))))
+           (sb-xc:defconstant ; we see this maro as well. The host expansion will not do,
+            ;; because it calls our %defconstant which does not assign the symbol a value.
+            ;; It might be possible to change that now that we don't use CL: symbols.
+            (destructuring-bind (symbol value) (cdr form)
+              `(progn (defconstant ,symbol ,value)
+                      (sb-c::%defconstant ',symbol ,symbol nil))))
+           (t
+            form))))))))
+
+(defmacro sb-format:tokens (string) string)
+
+;;; Used by our lockfree memoization functions (define-hash-cache)
+(defmacro sb-thread:barrier ((kind) &body body)
+  (declare (ignore kind))
+  `(progn ,@body))
+
+;;; For (eval-when (:compile-toplevel) (/show)) forms that reach the host's EVAL.
+(defmacro %primitive (name arg)
+  (ecase name
+    (print `(format t "~A~%" ,arg))))
