@@ -28,14 +28,14 @@
     (:import-from "<package-name> {symbol-name}*")
     (:shadow "{symbol-name}*")
     (:shadowing-import-from "<package-name> {symbol-name}*")
-    (:local-nicknames "{local-nickname actual-package-name}*")
-    #+sb-package-locks (:lock "boolean")
-    #+sb-package-locks (:implement "{package-name}*")
+    (:local-nicknames "{(local-nickname actual-package-name)}*")
+    (:lock "boolean")
+    (:implement "{package-name}*")
     (:documentation "doc-string")
     (:intern "{symbol-name}*")
     (:size "<integer>")
     (:nicknames "{package-name}*"))
-  '(:size #+sb-package-locks :lock)))
+  '(:size :lock)))
   (let ((nicknames nil)
         (local-nicknames nil)
         (size nil)
@@ -56,20 +56,15 @@
         (seen nil))
     (dolist (option options)
       (unless (consp option)
-        (error 'simple-program-error
-               :format-control "bogus DEFPACKAGE option: ~S"
-               :format-arguments (list option)))
+        (%program-error "bogus DEFPACKAGE option: ~S" option))
       (setq optname (car option) optval (cdr option))
       (case optname
-        ((:documentation :size #+sb-package-locks :lock)
+        ((:documentation :size :lock)
          (when (memq optname seen)
-           (error 'simple-program-error
-                  :format-control "can't specify ~S more than once."
-                  :format-arguments (list optname)))
+           (%program-error "can't specify ~S more than once." optname))
          (unless (typep optval '(cons t null))
-           (error 'simple-program-error
-                  :format-control "~S expects a single argument. Got ~S"
-                  :format-arguments (list (cdr option))))
+           (%program-error "~S expects a single argument. Got ~S"
+                          (car option) (cdr option)))
          (push optname seen)
          (setq optval (car optval))))
       (case optname
@@ -87,9 +82,7 @@
         (:size
          (if (typep optval 'unsigned-byte)
              (setf size optval)
-             (error 'simple-program-error
-                    :format-control ":SIZE is not a positive integer: ~S"
-                    :format-arguments (cdr option))))
+             (%program-error ":SIZE is not a positive integer: ~S" option)))
         (:shadow
          (setf shadows (append shadows (stringify-string-designators optval))))
         (:shadowing-import-from
@@ -114,19 +107,15 @@
          (setf interns (append interns (stringify-string-designators optval))))
         (:export
          (setf exports (append exports (stringify-string-designators optval))))
-        #+sb-package-locks
         (:implement
          (setf implement (append implement (stringify-package-designators optval))
                implement-p t))
-        #+sb-package-locks
         (:lock
          (setf lock (coerce optval 'boolean)))
         (:documentation
          (setf doc (possibly-base-stringize optval)))
         (t
-         (error 'simple-program-error
-                :format-control "bogus DEFPACKAGE option: ~S"
-                :format-arguments (list option)))))
+         (%program-error "bogus DEFPACKAGE option: ~S" option))))
     (check-disjoint `(:intern ,@interns) `(:export  ,@exports))
     (check-disjoint `(:intern ,@interns)
                     `(:import-from
@@ -157,10 +146,9 @@
       with x = (car list)
       for y in (rest list)
       for z = (remove-duplicates (intersection (cdr x)(cdr y) :test #'string=))
-      when z do (error 'simple-program-error
-                       :format-control "Parameters ~S and ~S must be disjoint ~
-                                        but have common elements ~%   ~S"
-                       :format-arguments (list (car x)(car y) z)))))
+      when z do (%program-error "Parameters ~S and ~S must be disjoint ~
+                                 but have common elements ~%   ~S"
+                                (car x) (car y) z))))
 
 (flet ((designator-to-string (kind designator)
          (cond ((and (eq kind 'package) (packagep designator))
@@ -214,7 +202,6 @@
                        imports interns
                        exports implement local-nicknames
                        lock doc-string)
-  (declare #-sb-package-locks (ignore implement lock))
   (%enter-new-nicknames package nicknames)
   ;; 1. :shadow and :shadowing-import-from
   ;;
@@ -238,20 +225,21 @@
                   exports)
           package)
   ;; 5. :local-nicknames
+  ;; FIXME: See bug at PACKAGE-LOCALLY-NICKNAMED-BY-LIST
   (setf (package-%local-nicknames package) nil) ; throw out the old ones.
   (loop :for (nickname . nickname-package) :in local-nicknames :do
-     (add-package-local-nickname nickname nickname-package package))
+     (%add-package-local-nickname nickname nickname-package package))
   ;; Everything was created: update metadata
   (when source-location
     (setf (package-source-location package) source-location))
   (setf (package-doc-string package) doc-string)
-  #+sb-package-locks
-  (progn
-    ;; Handle packages this is an implementation package of
-    (dolist (p implement)
+  ;; Handle packages this is an implementation package of
+  (dolist (p implement)
       (add-implementation-package package p))
-    ;; Handle lock
-    (setf (package-lock package) lock))
+  ;; Handle lock
+  (setf (package-lock package) lock)
+  ;; Flush cached FIND-PACKAGE values
+  (atomic-incf *package-names-cookie*)
   package)
 
 (declaim (type list *on-package-variance*))
@@ -350,7 +338,6 @@ specifies to signal a warning if SWANK package is in variance, and an error othe
            (unexport no-longer-exported package))
          (keep-them ()
            :report "Keep exporting them.")))))
-  #+sb-package-locks
   (let ((old-implements
           (set-difference (package-implements-list package)
                           (mapcar #'find-undeleted-package-or-lose implement))))
