@@ -9,7 +9,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-;;; SB-IMPL, not SB!IMPL, since we're built in warm load.
+;;; SB-IMPL, not SB-IMPL, since we're built in warm load.
 (in-package "SB-IMPL")
 
 ;;;; Utils, move elsewhere.
@@ -548,7 +548,7 @@
      (binding* ((code (fun-code-header (%fun-fun function)))
                 (info (sb-kernel:%code-debug-info code) :exit-if-null)
                 (source (sb-c::debug-info-source info) :exit-if-null))
-       (acond ((sb-c::debug-source-namestring source)
+       (acond ((debug-source-namestring source)
                (format stream "~@:_Source file: ~A" it))
               ((%simple-fun-lexpr (%fun-fun function))
                (format stream "~@:_Source form:~@:_  ~S" it)))))
@@ -576,7 +576,7 @@
             (when (or (eq :declared from) (and sure (eq :assumed from)))
               (describe-block (stream "~A names an undefined function" name)
                 (format stream "~@:_~:(~A~) type: ~/sb-impl:print-type/"
-                        from (proclaimed-ftype name))))))
+                        from (global-ftype name))))))
         ;; Defined.
         (multiple-value-bind (fun what lambda-list derived-type declared-type
                               inline methods argument-precedence-order)
@@ -594,7 +594,7 @@
                                              (%fun-type function)))
                           (legal-name-p (legal-fun-name-p name))
                           (ctype (and legal-name-p
-                                      (proclaimed-ftype name)))
+                                      (global-ftype name)))
                           (type (and ctype (type-specifier ctype)))
                           (from (and legal-name-p
                                      (info :function :where-from name)))
