@@ -11,7 +11,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!KERNEL")
+(in-package "SB-KERNEL")
 
 ;;;; DEFSTRUCT-DESCRIPTION
 
@@ -21,6 +21,7 @@
 ;;; is declared to hold a DEFSTRUCT-DESCRIPTION.
 (def!struct (defstruct-description
              (:conc-name dd-)
+             (:copier nil)
              #-sb-xc-host (:pure t)
              (:constructor make-defstruct-description (null-lexenv-p name)))
   ;; name of the structure
@@ -73,10 +74,14 @@
   ;; NIL if the option was given with no argument.
   (printer-fname nil :type (or cons symbol))
 
-  ;; the value of the :PURE option, or :UNSPECIFIED. This is only
-  ;; meaningful if DD-CLASS-P = T.
-  (pure :unspecified :type (member t nil :unspecified)))
-#!-sb-fluid (declaim (freeze-type defstruct-description))
+  ;; the value of the :PURE option, used by cheneygc when purifying.
+  ;; This is true if objects of this class are never modified to
+  ;; contain dynamic pointers in their slots or constant-like
+  ;; substructure (and hence can be copied into read-only space by
+  ;; PURIFY).
+  ;; This is only meaningful if DD-CLASS-P = T.
+  (pure nil :type (member t nil)))
+#-sb-fluid (declaim (freeze-type defstruct-description))
 (!set-load-form-method defstruct-description (:host :xc :target))
 
 ;;;; basic LAYOUT stuff
@@ -84,8 +89,7 @@
 ;;; Note: This bound is set somewhat less than MOST-POSITIVE-FIXNUM
 ;;; in order to guarantee that several hash values can be added without
 ;;; overflowing into a bignum.
-(defconstant layout-clos-hash-limit (1+ (ash sb!xc:most-positive-fixnum -3))
-  #!+sb-doc
+(defconstant layout-clos-hash-limit (1+ (ash sb-xc:most-positive-fixnum -3))
   "the exclusive upper bound on LAYOUT-CLOS-HASH values")
 ;; This must be DEF!TYPE and not just DEFTYPE because access to slots
 ;; of a layout occur "before" the structure definition is made in the
@@ -95,6 +99,10 @@
 (def!type layout-clos-hash () `(integer 0 ,layout-clos-hash-limit))
 (declaim (ftype (sfunction () layout-clos-hash) random-layout-clos-hash))
 
+(defconstant +structure-layout-flag+  #b001)
+(defconstant +pcl-object-layout-flag+ #b010)
+(defconstant +condition-layout-flag+  #b100)
+
 ;;; The LAYOUT structure is pointed to by the first cell of instance
 ;;; (or structure) objects. It represents what we need to know for
 ;;; type checking and garbage collection. Whenever a class is
@@ -107,7 +115,14 @@
 ;;; well, since the initialization of layout slots is hardcoded there.
 ;;;
 ;;; FIXME: ...it would be better to automate this, of course...
-(def!struct (layout)
+(def!struct (layout (:constructor make-layout
+                        ;; Accept a specific subset of keywords
+                        (&key %flags clos-hash classoid invalid
+                              inherits depthoid length info bitmap
+                              depth2-ancestor depth3-ancestor depth4-ancestor))
+                    (:copier nil))
+  ;; one +something-LAYOUT-FLAG+ bit or none of them
+  (%flags 0 :type fixnum :read-only nil)
   ;; a pseudo-random hash value for use by CLOS.
   (clos-hash (random-layout-clos-hash) :type layout-clos-hash)
   ;; the class that this is a layout for
@@ -140,23 +155,19 @@
   ;;      a depth when it isn't quite.
   (depthoid -1 :type layout-depthoid)
   ;; the number of top level descriptor cells in each instance
-  (length 0 :type index)
+  ;; For [FUNCALLABLE-]STANDARD-OBJECT instances, this is the slot vector
+  ;; length, not the primitive object length.
+  ;; I tried making a structure of this many slots, and the compiler blew up;
+  ;; so it's fair to say this limit is sufficient for practical purposes,
+  ;; Let's be consistent here between the two choices of word size.
+  (length 0 :type (unsigned-byte 28)) ; smaller than SB-INT:INDEX
   ;; If this layout has some kind of compiler meta-info, then this is
   ;; it. If a structure, then we store the DEFSTRUCT-DESCRIPTION here.
   (info nil :type (or null defstruct-description))
-  ;; This is true if objects of this class are never modified to
-  ;; contain dynamic pointers in their slots or constant-like
-  ;; substructure (and hence can be copied into read-only space by
-  ;; PURIFY).
-  ;;
-  ;; This slot is known to the C runtime support code.
-  (pure nil :type (member t nil 0))
   ;; Map of raw slot indices.
   (bitmap +layout-all-tagged+ :type layout-bitmap)
   ;; Per-slot comparator for implementing EQUALP.
   (equalp-tests #() :type simple-vector)
-  ;; Definition location
-  (source-location nil)
   ;; If this layout is for an object of metatype STANDARD-CLASS,
   ;; these are the EFFECTIVE-SLOT-DEFINITION metaobjects.
   (slot-list nil :type list)
@@ -164,18 +175,21 @@
   ;; access to slot-definitions and locations by name, etc.
   ;; See MAKE-SLOT-TABLE in pcl/slots-boot.lisp for further details.
   (slot-table #(1 nil) :type simple-vector)
-  ;; True IFF the layout belongs to a standand-instance or a
-  ;; standard-funcallable-instance.
-  ;; Old comment was:
-  ;;   FIXME: If we unify wrappers and layouts this can go away, since
-  ;;   it is only used in SB-PCL::EMIT-FETCH-WRAPPERS, which can then
-  ;;   use INSTANCE-SLOTS-LAYOUT instead (if there is are no slot
-  ;;   layouts, there are no slots for it to pull.)
-  ;; But while that's conceivable, it still seems advantageous to have
-  ;; a single bit that decides whether something is STANDARD-OBJECT.
-  (%for-std-class-b 0 :type bit :read-only t))
+  ;; inherited layouts or 0, only pertinent to structure classoids.
+  ;; There is no need to store the layout at depths 0 or 1
+  ;; since they're predetermined to be T and STRUCTURE-OBJECT.
+  ;; These 3 slots cause the length of a layout to be exactly 16 words
+  ;; with compact-instance-header, which avoids having to change some
+  ;; logic in 'immobile-space' that assumes this alignment constraint.
+  (depth2-ancestor 0)
+  (depth3-ancestor 0)
+  (depth4-ancestor 0))
 (declaim (freeze-type layout))
 
+(declaim (inline layout-for-std-class-p))
+(defun layout-for-std-class-p (x)
+  (logtest (layout-%flags x) +pcl-object-layout-flag+))
+
 ;;; The CLASSOID structure is a supertype of all classoid types.  A
 ;;; CLASSOID is also a CTYPE structure as recognized by the type
 ;;; system.  (FIXME: It's also a type specifier, though this might go
@@ -185,6 +199,7 @@
              (:include ctype
                        (class-info (type-class-or-lose 'classoid)))
              (:constructor nil)
+             (:copier nil)
              #-no-ansi-print-object
              (:print-object
               (lambda (class stream)
@@ -211,6 +226,9 @@
   (state nil :type (member nil :read-only :sealed))
   ;; direct superclasses of this class. Always NIL for CLOS classes.
   (direct-superclasses () :type list)
+  ;; Definition location
+  ;; Not used for standard-classoid, because pcl has its own mechanism.
+  (source-location nil)
   ;; representation of all of the subclasses (direct or indirect) of
   ;; this class. This is NIL if no subclasses or not initalized yet;
   ;; otherwise, it's an EQ hash-table mapping CLASSOID objects to the
@@ -222,12 +240,39 @@
   ;; NIL if none assigned yet
   (pcl-class nil))
 
+;;; A helper to make classoid (and named-type) hash values stable.
+;;; For other ctypes, generally improve the randomness of the hash.
+;;; (The host uses at most 21 bits of randomness. See CTYPE-RANDOM)
+#+sb-xc
+(defun !improve-ctype-hash (obj type-class-name)
+  (let ((hash (case type-class-name
+                (named
+                 (interned-type-hash (named-type-name obj) 'named))
+                (classoid
+                 (interned-type-hash (classoid-name obj)))
+                (t
+                 (interned-type-hash))))
+        ;; Preserve the interned-p and type=-optimization bits
+        ;; by affecting only bits under the ctype-hash-mask.
+        ;; Upper 5 hash bits might be an index into SAETP array
+        ;; (if this ctype is exactly a type to which upgrade occurs)
+        (nbits (- (integer-length +ctype-hash-mask+)
+                  +ctype-saetp-index-bits+)))
+    (macrolet ((slot-index ()
+                 (let* ((dd (find-defstruct-description 'ctype))
+                        (dsd (find 'hash-value (dd-slots dd) :key #'dsd-name)))
+                   (dsd-index dsd))))
+      (setf (%instance-ref obj (slot-index))
+            (dpb hash (byte nbits 0) (type-hash-value obj)))))
+  obj)
+
 ;;;; object types to represent classes
 
 ;;; An UNDEFINED-CLASSOID is a cookie we make up to stick in forward
 ;;; referenced layouts. Users should never see them.
 (def!struct (undefined-classoid
              (:include classoid)
+             (:copier nil)
              (:constructor make-undefined-classoid (name))))
 
 ;;; BUILT-IN-CLASS is used to represent the standard classes that
@@ -241,14 +286,17 @@
 ;;; system operations (union, subtypep, etc.) should never encounter
 ;;; translated classes, only their translation.
 (def!struct (built-in-classoid (:include classoid)
-                               (:constructor make-built-in-classoid))
+                               (:copier nil)
+                               (:constructor !make-built-in-classoid))
   ;; the type we translate to on parsing. If NIL, then this class
   ;; stands on its own; or it can be set to :INITIALIZING for a period
   ;; during cold-load.
   (translation nil :type (or ctype (member nil :initializing))))
 
 (def!struct (condition-classoid (:include classoid)
-                                (:constructor make-condition-classoid))
+                                (:copier nil)
+                                (:constructor %make-condition-classoid
+                                    (hash-value name)))
   ;; list of CONDITION-SLOT structures for the direct slots of this
   ;; class
   (slots nil :type list)
@@ -272,12 +320,15 @@
   ;; Values for these slots must be computed in the dynamic
   ;; environment of MAKE-CONDITION.
   (hairy-slots nil :type list))
+(defun make-condition-classoid (&key name)
+  (%make-condition-classoid (interned-type-hash name) name))
 
 ;;;; classoid namespace
 
 ;;; We use an indirection to allow forward referencing of class
 ;;; definitions with load-time resolution.
 (def!struct (classoid-cell
+             (:copier nil)
              (:constructor make-classoid-cell (name &optional classoid))
              #-no-ansi-print-object
              (:print-object (lambda (s stream)
@@ -322,23 +373,25 @@
 ;;; side does not need to distinguish between STANDARD-CLASS and
 ;;; FUNCALLABLE-STANDARD-CLASS.
 (def!struct (standard-classoid (:include classoid)
-                               (:constructor make-standard-classoid)))
+                               (:copier nil)
+                               (:constructor make-standard-classoid))
+  old-layouts)
 ;;; a metaclass for classes which aren't standardlike but will never
 ;;; change either.
 (def!struct (static-classoid (:include classoid)
+                             (:copier nil)
                              (:constructor make-static-classoid)))
 
 (declaim (freeze-type built-in-classoid condition-classoid
                       standard-classoid static-classoid))
 
-(in-package "SB!C")
+(in-package "SB-C")
 
 ;;; layout for this type being used by the compiler
 (define-info-type (:type :compiler-layout)
   :type-spec (or layout null)
   :default (lambda (name)
-             (let ((class (find-classoid name nil)))
-               (when class (classoid-layout class)))))
+             (awhen (find-classoid name nil) (classoid-layout it))))
 
 (eval-when (#-sb-xc :compile-toplevel :load-toplevel :execute)
 (defun ftype-from-fdefn (name)
@@ -349,22 +402,19 @@
   #+sb-xc-host
   (specifier-type 'function)
   #-sb-xc-host
-  (let* ((fdefn (sb!kernel::find-fdefn name))
+  (let* ((fdefn (sb-kernel::find-fdefn name))
          (fun (and fdefn (fdefn-fun fdefn))))
     (if fun
         (handler-bind ((style-warning #'muffle-warning))
-          (specifier-type (sb!impl::%fun-type fun)))
+          (specifier-type (sb-impl::%fun-type fun)))
         (specifier-type 'function)))))
 
-;;; The type specifier for this function, or a DEFSTRUCT-DESCRIPTION
-;;; or the symbol :GENERIC-FUNTION.
-;;; If a DD, it must contain a constructor whose name is
-;;; the one being sought in globaldb, which is used to derive the type.
-;;; If :GENERIC-FUNCTION, the info is recomputed from existing methods
-;;; and stored back into globaldb.
+;;; The parsed or unparsed type for this function, or the symbol :GENERIC-FUNCTION.
+;;; Ordinarily a parsed type is stored. Only if the parsed type contains
+;;; an unknown type will the original specifier be stored; we attempt to reparse
+;;; on each lookup, in the hope that the type becomes known at some point.
+;;; If :GENERIC-FUNCTION, the info is recomputed from methods at the time of lookup
+;;; and stored back. Method redefinition resets the value to :GENERIC-FUNCTION.
 (define-info-type (:function :type)
-  :type-spec (or ctype defstruct-description (member :generic-function))
+  :type-spec (or ctype (cons (eql function)) (member :generic-function))
   :default #'ftype-from-fdefn)
-
-;;; A random place for this :-(
-#+sb-xc-host (setq *info-environment* (make-info-hashtable))
