@@ -10,44 +10,45 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!KERNEL")
+(in-package "SB-KERNEL")
 
 ;;;; miscellaneous constants, utility functions, and macros
 
-(defconstant pi
-  #!+long-float 3.14159265358979323846264338327950288419716939937511l0
-  #!-long-float 3.14159265358979323846264338327950288419716939937511d0)
+(defconstant sb-xc:pi
+  #+long-float 3.14159265358979323846264338327950288419716939937511l0
+  #-long-float 3.14159265358979323846264338327950288419716939937511d0)
+
+(eval-when (:compile-toplevel :load-toplevel :execute)
+  (defun handle-reals (function var)
+    `((((foreach fixnum single-float bignum ratio))
+       (coerce (,function (coerce ,var 'double-float)) 'single-float))
+      ((double-float)
+       (,function ,var))))
+
+  (defun handle-complex (form)
+    `((((foreach (complex double-float) (complex single-float) (complex rational)))
+       ,form))))
 
 ;;; Make these INLINE, since the call to C is at least as compact as a
 ;;; Lisp call, and saves number consing to boot.
-(eval-when (:compile-toplevel :execute)
-
-(sb!xc:defmacro def-math-rtn (name num-args &optional wrapper)
+(defmacro def-math-rtn (name num-args &optional wrapper)
   (let ((function (symbolicate "%" (string-upcase name)))
         (args (loop for i below num-args
                     collect (intern (format nil "ARG~D" i)))))
     `(progn
        (declaim (inline ,function))
        (defun ,function ,args
-         (alien-funcall
-          (extern-alien ,(format nil "~:[~;sb_~]~a" wrapper name)
-                        (function double-float
-                                  ,@(loop repeat num-args
-                                          collect 'double-float)))
-          ,@args)))))
+         (truly-the ;; avoid checking the result
+          ,(type-specifier (fun-type-returns (info :function :type function)))
+          (alien-funcall
+           (extern-alien ,(format nil "~:[~;sb_~]~a" wrapper name)
+                         (function double-float
+                                   ,@(loop repeat num-args
+                                           collect 'double-float)))
+           ,@args))))))
 
-(defun handle-reals (function var)
-  `((((foreach fixnum single-float bignum ratio))
-     (coerce (,function (coerce ,var 'double-float)) 'single-float))
-    ((double-float)
-     (,function ,var))))
-
-(defun handle-complex (form)
-  `((((foreach (complex double-float) (complex single-float) (complex rational)))
-     ,form)))
-) ; EVAL-WHEN
 
-#!+x86 ;; for constant folding
+#+x86 ;; for constant folding
 (macrolet ((def (name ll)
              `(defun ,name ,ll (,name ,@ll))))
   (def %atan2 (x y))
@@ -62,7 +63,7 @@
   (def %log (x))
   (def %exp (x)))
 
-#!+(or x86-64 arm-vfp arm64) ;; for constant folding
+#+(or x86-64 arm-vfp arm64) ;; for constant folding
 (macrolet ((def (name ll)
              `(defun ,name ,ll (,name ,@ll))))
   (def %sqrt (x)))
@@ -73,35 +74,34 @@
 ;;;; into the FPU.
 
 ;;; trigonometric
-#!-x86 (def-math-rtn "sin" 1)
-#!-x86 (def-math-rtn "cos" 1)
-#!-x86 (def-math-rtn "tan" 1)
-#!-x86 (def-math-rtn "atan" 1)
-#!-x86 (def-math-rtn "atan2" 2)
+#-x86 (def-math-rtn "sin" 1)
+#-x86 (def-math-rtn "cos" 1)
+#-x86 (def-math-rtn "tan" 1)
+#-x86 (def-math-rtn "atan" 1)
+#-x86 (def-math-rtn "atan2" 2)
 
-(def-math-rtn "acos" 1 #!+win32 t)
-(def-math-rtn "asin" 1 #!+win32 t)
-(def-math-rtn "cosh" 1 #!+win32 t)
-(def-math-rtn "sinh" 1 #!+win32 t)
-(def-math-rtn "tanh" 1 #!+win32 t)
-(def-math-rtn "asinh" 1 #!+win32 t)
-(def-math-rtn "acosh" 1 #!+win32 t)
-(def-math-rtn "atanh" 1 #!+win32 t)
+(def-math-rtn "acos" 1 #+win32 t)
+(def-math-rtn "asin" 1 #+win32 t)
+(def-math-rtn "cosh" 1 #+win32 t)
+(def-math-rtn "sinh" 1 #+win32 t)
+(def-math-rtn "tanh" 1 #+win32 t)
+(def-math-rtn "asinh" 1 #+win32 t)
+(def-math-rtn "acosh" 1 #+win32 t)
+(def-math-rtn "atanh" 1 #+win32 t)
 
 ;;; exponential and logarithmic
-(def-math-rtn "hypot" 2 #!+win32 t)
-#!-x86 (def-math-rtn "exp" 1)
-#!-x86 (def-math-rtn "log" 1)
-#!-x86 (def-math-rtn "log10" 1)
+(def-math-rtn "hypot" 2 #+win32 t)
+#-x86 (def-math-rtn "exp" 1)
+#-x86 (def-math-rtn "log" 1)
+#-x86 (def-math-rtn "log10" 1)
 (def-math-rtn "pow" 2)
-#!-(or x86 x86-64 arm-vfp arm64) (def-math-rtn "sqrt" 1)
-#!-x86 (def-math-rtn "log1p" 1)
+#-(or x86 x86-64 arm-vfp arm64) (def-math-rtn "sqrt" 1)
+#-x86 (def-math-rtn "log1p" 1)
 
 
 ;;;; power functions
 
 (defun exp (number)
-  #!+sb-doc
   "Return e raised to the power NUMBER."
   (declare (explicit-check))
   (number-dispatch ((number number))
@@ -111,21 +111,33 @@
         (cis (imagpart number))))))
 
 ;;; INTEXP -- Handle the rational base, integer power case.
-
-(declaim (type (or integer null) *intexp-maximum-exponent*))
-(defparameter *intexp-maximum-exponent* nil)
-
 ;;; This function precisely calculates base raised to an integral
 ;;; power. It separates the cases by the sign of power, for efficiency
 ;;; reasons, as powers can be calculated more efficiently if power is
 ;;; a positive integer. Values of power are calculated as positive
 ;;; integers, and inverted if negative.
 (defun intexp (base power)
-  (when (and *intexp-maximum-exponent*
-             (> (abs power) *intexp-maximum-exponent*))
-    (error "The absolute value of ~S exceeds ~S."
-            power '*intexp-maximum-exponent*))
-  (cond ((minusp power)
+  (cond ((eql base 1)
+         base)
+        ((eql base -1)
+         (if (evenp power)
+             1
+             base))
+        ((ratiop base)
+         (let ((den (denominator base))
+               (num (numerator base)))
+           (if (minusp power)
+               (let ((negated (- power)))
+                 (cond ((eql num 1)
+                        (intexp den negated))
+                       ((eql num -1)
+                        (intexp (- den) negated))
+                       (t
+                        (build-ratio (intexp den negated)
+                                     (intexp num negated)))))
+               (build-ratio (intexp num power)
+                            (intexp den power)))))
+        ((minusp power)
          (/ (intexp base (- power))))
         ((eql base 2)
          (ash 1 power))
@@ -144,7 +156,6 @@
 ;;; result. We also separate the complex-real and real-complex cases
 ;;; from the general complex case.
 (defun expt (base power)
-  #!+sb-doc
   "Return BASE raised to the POWER."
   (declare (explicit-check))
   (if (zerop power)
@@ -152,7 +163,7 @@
         (error 'arguments-out-of-domain-error
                :operands (list base power)
                :operation 'expt
-               :references (list '(:ansi-cl :function expt)))
+               :references '((:ansi-cl :function expt)))
         (let ((result (1+ (* base power))))
           (if (and (floatp result) (float-nan-p result))
               (float 1 result)
@@ -246,7 +257,7 @@
                            (when (< x-hi 0)
                              (cond ((and (= x-ihi #x3ff00000) (zerop yisint))
                                     ;; (-1)**non-int
-                                    (let ((y*pi (* y pi)))
+                                    (let ((y*pi (* y sb-xc:pi)))
                                       (declare (double-float y*pi))
                                       (return-from real-expt
                                         (complex
@@ -269,7 +280,7 @@
                                (2 ; even
                                 (coerce pow rtype))
                                (t ; non-integer
-                                (let ((y*pi (* y pi)))
+                                (let ((y*pi (* y sb-xc:pi)))
                                   (declare (double-float y*pi))
                                   (complex
                                    (coerce (* pow (%cos y*pi))
@@ -337,17 +348,16 @@
   ;;
   ;; Motivated by an attempt to get LOG to work better on bignums.
   (let ((n (integer-length x)))
-    (if (< n sb!vm:double-float-digits)
+    (if (< n sb-vm:double-float-digits)
         (log (coerce x 'double-float) 2.0d0)
-        (let ((f (ldb (byte sb!vm:double-float-digits
-                            (- n sb!vm:double-float-digits))
+        (let ((f (ldb (byte sb-vm:double-float-digits
+                            (- n sb-vm:double-float-digits))
                       x)))
           (+ n (log (scale-float (coerce f 'double-float)
-                                 (- sb!vm:double-float-digits))
+                                 (- sb-vm:double-float-digits))
                     2.0d0))))))
 
 (defun log (number &optional (base nil base-p))
-  #!+sb-doc
   "Return the logarithm of NUMBER in the base BASE, which defaults to e."
   (declare (explicit-check))
   (if base-p
@@ -369,11 +379,11 @@
       (number-dispatch ((number number))
         (((foreach fixnum bignum))
          (if (minusp number)
-             (complex (log (- number)) (coerce pi 'single-float))
+             (complex (log (- number)) (coerce sb-xc:pi 'single-float))
              (coerce (/ (log2 number) (log (exp 1.0d0) 2.0d0)) 'single-float)))
         ((ratio)
          (if (minusp number)
-             (complex (log (- number)) (coerce pi 'single-float))
+             (complex (log (- number)) (coerce sb-xc:pi 'single-float))
              (let ((numerator (numerator number))
                    (denominator (denominator number)))
                (if (= (integer-length numerator)
@@ -389,33 +399,34 @@
          ;; I (pw) take the Kahan result.
          (if (< (float-sign number)
                 (coerce 0 '(dispatch-type number)))
-             (complex (log (- number)) (coerce pi '(dispatch-type number)))
+             (complex (log (- number)) (coerce sb-xc:pi '(dispatch-type number)))
              (coerce (%log (coerce number 'double-float))
                      '(dispatch-type number))))
         ((complex)
          (complex-log number)))))
 
 (defun sqrt (number)
-  #!+sb-doc
   "Return the square root of NUMBER."
   (declare (explicit-check))
   (number-dispatch ((number number))
     (((foreach fixnum bignum ratio))
      (if (minusp number)
-         (complex-sqrt number)
+         (complex 0f0
+                  (coerce (%sqrt (- (coerce number 'double-float))) 'single-float))
          (coerce (%sqrt (coerce number 'double-float)) 'single-float)))
     (((foreach single-float double-float))
      (if (minusp number)
-         (complex-sqrt (complex number))
+         (complex (coerce 0.0 '(dispatch-type number))
+                  (coerce (%sqrt (- (coerce number 'double-float)))
+                          '(dispatch-type number)))
          (coerce (%sqrt (coerce number 'double-float))
                  '(dispatch-type number))))
-     ((complex)
-      (complex-sqrt number))))
+    ((complex)
+     (complex-sqrt number))))
 
 ;;;; trigonometic and related functions
 
 (defun abs (number)
-  #!+sb-doc
   "Return the absolute value of the number."
   (declare (explicit-check))
   (number-dispatch ((number number))
@@ -435,7 +446,6 @@
           (%hypot rx (truly-the double-float ix))))))))
 
 (defun phase (number)
-  #!+sb-doc
   "Return the angle part of the polar representation of a complex number.
   For complex numbers, this is (atan (imagpart number) (realpart number)).
   For non-complex positive numbers, this is 0. For non-complex negative
@@ -444,21 +454,20 @@
   (number-dispatch ((number number))
     ((rational)
      (if (minusp number)
-         (coerce pi 'single-float)
+         (coerce sb-xc:pi 'single-float)
          0.0f0))
     ((single-float)
      (if (minusp (float-sign number))
-         (coerce pi 'single-float)
+         (coerce sb-xc:pi 'single-float)
          0.0f0))
     ((double-float)
      (if (minusp (float-sign number))
-         (coerce pi 'double-float)
+         (coerce sb-xc:pi 'double-float)
          0.0d0))
     (handle-complex
      (atan (imagpart number) (realpart number)))))
 
 (defun sin (number)
-  #!+sb-doc
   "Return the sine of NUMBER."
   (declare (explicit-check))
   (number-dispatch ((number number))
@@ -470,7 +479,6 @@
                 (* (cos x) (sinh y)))))))
 
 (defun cos (number)
-  #!+sb-doc
   "Return the cosine of NUMBER."
   (declare (explicit-check))
   (number-dispatch ((number number))
@@ -482,7 +490,6 @@
                 (- (* (sin x) (sinh y))))))))
 
 (defun tan (number)
-  #!+sb-doc
   "Return the tangent of NUMBER."
   (declare (explicit-check))
   (number-dispatch ((number number))
@@ -495,7 +502,6 @@
                 (- (realpart result)))))))
 
 (defun cis (theta)
-  #!+sb-doc
   "Return cos(Theta) + i sin(Theta), i.e. exp(i Theta)."
   (declare (explicit-check ))
   (number-dispatch ((theta real))
@@ -503,7 +509,6 @@
      (complex (cos theta) (sin theta)))))
 
 (defun asin (number)
-  #!+sb-doc
   "Return the arc sine of NUMBER."
   (declare (explicit-check))
   (number-dispatch ((number number))
@@ -521,7 +526,6 @@
      (complex-asin number))))
 
 (defun acos (number)
-  #!+sb-doc
   "Return the arc cosine of NUMBER."
   (declare (explicit-check))
   (number-dispatch ((number number))
@@ -539,7 +543,6 @@
      (complex-acos number))))
 
 (defun atan (y &optional (x nil xp))
-  #!+sb-doc
   "Return the arc tangent of Y if X is omitted or Y/X if X is supplied."
   (declare (explicit-check))
   (if xp
@@ -550,8 +553,8 @@
                    (if (zerop y)
                        (if (plusp (float-sign x))
                            y
-                           (float-sign y pi))
-                       (float-sign y (/ pi 2)))
+                           (float-sign y sb-xc:pi))
+                       (float-sign y (/ sb-xc:pi 2)))
                    (%atan2 y x))))
         (number-dispatch ((y real) (x real))
           ((double-float
@@ -576,7 +579,6 @@
 ;;; complex numbers can also lose big.
 
 (defun sinh (number)
-  #!+sb-doc
   "Return the hyperbolic sine of NUMBER."
   (declare (explicit-check))
   (number-dispatch ((number number))
@@ -588,7 +590,6 @@
                 (* (cosh x) (sin y)))))))
 
 (defun cosh (number)
-  #!+sb-doc
   "Return the hyperbolic cosine of NUMBER."
   (declare (explicit-check))
   (number-dispatch ((number number))
@@ -600,7 +601,6 @@
                 (* (sinh x) (sin y)))))))
 
 (defun tanh (number)
-  #!+sb-doc
   "Return the hyperbolic tangent of NUMBER."
   (declare (explicit-check))
   (number-dispatch ((number number))
@@ -609,7 +609,6 @@
      (complex-tanh number))))
 
 (defun asinh (number)
-  #!+sb-doc
   "Return the hyperbolic arc sine of NUMBER."
   (declare (explicit-check))
   (number-dispatch ((number number))
@@ -618,7 +617,6 @@
      (complex-asinh number))))
 
 (defun acosh (number)
-  #!+sb-doc
   "Return the hyperbolic arc cosine of NUMBER."
   (declare (explicit-check))
   (number-dispatch ((number number))
@@ -636,7 +634,6 @@
      (complex-acosh number))))
 
 (defun atanh (number)
-  #!+sb-doc
   "Return the hyperbolic arc tangent of NUMBER."
   (declare (explicit-check))
   (number-dispatch ((number number))
@@ -696,7 +693,7 @@
 ;;;;   logb
 ;;;;
 ;;;; internal functions:
-;;;;    square coerce-to-complex-type cssqs complex-log-scaled
+;;;;    square coerce-to-complex-type cssqs
 ;;;;
 ;;;; references:
 ;;;;   Kahan, W. "Branch Cuts for Complex Elementary Functions, or Much
@@ -709,7 +706,7 @@
 ;;;;   Raymond Toy at <email address deleted during 2002 spam avalanche>.
 
 ;;; FIXME: In SBCL, the floating point infinity constants like
-;;; SB!EXT:DOUBLE-FLOAT-POSITIVE-INFINITY aren't available as
+;;; SB-EXT:DOUBLE-FLOAT-POSITIVE-INFINITY aren't available as
 ;;; constants at cross-compile time, because the cross-compilation
 ;;; host might not have support for floating point infinities. Thus,
 ;;; they're effectively implemented as special variable references,
@@ -767,7 +764,7 @@
          x)
         ((float-infinity-p x)
          ;; DOUBLE-FLOAT-POSITIVE-INFINITY
-         (double-from-bits 0 (1+ sb!vm:double-float-normal-exponent-max) 0))
+         (double-from-bits 0 (1+ sb-vm:double-float-normal-exponent-max) 0))
         ((zerop x)
          ;; The answer is negative infinity, but we are supposed to
           ;; signal divide-by-zero, so do the actual division
@@ -781,7 +778,7 @@
 ;;;   Create complex number with real part X and imaginary part Y
 ;;;   such that has the same type as Z.  If Z has type (complex
 ;;;   rational), the X and Y are coerced to single-float.
-#!+long-float (eval-when (:compile-toplevel :load-toplevel :execute)
+#+long-float (eval-when (:compile-toplevel :load-toplevel :execute)
                 (error "needs work for long float support"))
 (declaim (inline coerce-to-complex-type))
 (defun coerce-to-complex-type (x y z)
@@ -797,10 +794,10 @@
 
 ;;; Compute |(x+i*y)/2^k|^2 scaled to avoid over/underflow. The
 ;;; result is r + i*k, where k is an integer.
-#!+long-float (eval-when (:compile-toplevel :load-toplevel :execute)
+#+long-float (eval-when (:compile-toplevel :load-toplevel :execute)
                 (error "needs work for long float support"))
 (defun cssqs (z)
-  (declare (muffle-conditions t))
+  (declare (muffle-conditions compiler-note))
   (let ((x (float (realpart z) 1d0))
         (y (float (imagpart z) 1d0)))
     ;; Would this be better handled using an exception handler to
@@ -816,21 +813,21 @@
                       (float-infinity-p (abs y))))
              ;; DOUBLE-FLOAT-POSITIVE-INFINITY
              (values
-              (double-from-bits 0 (1+ sb!vm:double-float-normal-exponent-max) 0)
+              (double-from-bits 0 (1+ sb-vm:double-float-normal-exponent-max) 0)
               0))
             ((let ((threshold
                     ;; (/ least-positive-double-float double-float-epsilon)
                     (load-time-value
-                     #!-long-float
+                     #-long-float
                      (make-double-float #x1fffff #xfffffffe)
-                     #!+long-float
+                     #+long-float
                      (error "(/ least-positive-long-float long-float-epsilon)")))
-                   (traps (ldb sb!vm::float-sticky-bits
-                               (sb!vm:floating-point-modes))))
+                   (traps (ldb sb-vm:float-sticky-bits
+                               (sb-vm:floating-point-modes))))
                 ;; Overflow raised or (underflow raised and rho <
                 ;; lambda/eps)
-               (or (not (zerop (logand sb!vm:float-overflow-trap-bit traps)))
-                   (and (not (zerop (logand sb!vm:float-underflow-trap-bit
+               (or (not (zerop (logand sb-vm:float-overflow-trap-bit traps)))
+                   (and (not (zerop (logand sb-vm:float-underflow-trap-bit
                                             traps)))
                         (< rho threshold))))
               ;; If we're here, neither x nor y are infinity and at
@@ -887,22 +884,21 @@
           (setf nu (float-sign y rho))))
       (coerce-to-complex-type eta nu z))))
 
-;;; Compute log(2^j*z).
+;;; log of Z = log |Z| + i * arg Z
 ;;;
-;;; This is for use with J /= 0 only when |z| is huge.
-(defun complex-log-scaled (z j)
-  (declare (muffle-conditions t))
-  (declare (type (or rational complex) z)
-           (fixnum j))
+;;; Z may be any number, but the result is always a complex.
+(defun complex-log (z)
+  (declare (muffle-conditions compiler-note))
+  (declare (type (or rational complex) z))
   ;; The constants t0, t1, t2 should be evaluated to machine
   ;; precision.  In addition, Kahan says the accuracy of log1p
   ;; influences the choices of these constants but doesn't say how to
   ;; choose them.  We'll just assume his choices matches our
   ;; implementation of log1p.
   (let ((t0 (load-time-value
-             #!-long-float
+             #-long-float
              (make-double-float #x3fe6a09e #x667f3bcd)
-             #!+long-float
+             #+long-float
              (error "(/ (sqrt 2l0))")))
         ;; KLUDGE: if repeatable fasls start failing under some weird
         ;; xc host, this 1.2d0 might be a good place to examine: while
@@ -911,9 +907,9 @@
         (t1 1.2d0)
         (t2 3d0)
         (ln2 (load-time-value
-              #!-long-float
+              #-long-float
               (make-double-float #x3fe62e42 #xfefa39ef)
-              #!+long-float
+              #+long-float
               (error "(log 2l0)")))
         (x (float (realpart z) 1.0d0))
         (y (float (imagpart z) 1.0d0)))
@@ -923,37 +919,30 @@
       (let ((beta (max (abs x) (abs y)))
             (theta (min (abs x) (abs y))))
         (coerce-to-complex-type (if (and (zerop k)
-                 (< t0 beta)
-                 (or (<= beta t1)
-                     (< rho t2)))
-                                  (/ (%log1p (+ (* (- beta 1.0d0)
-                                       (+ beta 1.0d0))
-                                    (* theta theta)))
-                                     2d0)
-                                  (+ (/ (log rho) 2d0)
-                                     (* (+ k j) ln2)))
+                                         (< t0 beta)
+                                         (or (<= beta t1)
+                                             (< rho t2)))
+                                    (/ (%log1p (+ (* (- beta 1.0d0)
+                                                     (+ beta 1.0d0))
+                                                  (* theta theta)))
+                                       2d0)
+                                    (+ (/ (log rho) 2d0)
+                                       (* k ln2)))
                                 (atan y x)
                                 z)))))
 
-;;; log of Z = log |Z| + i * arg Z
-;;;
-;;; Z may be any number, but the result is always a complex.
-(defun complex-log (z)
-  (declare (type (or rational complex) z))
-  (complex-log-scaled z 0))
-
 ;;; KLUDGE: Let us note the following "strange" behavior. atanh 1.0d0
 ;;; is +infinity, but the following code returns approx 176 + i*pi/4.
 ;;; The reason for the imaginary part is caused by the fact that arg
 ;;; i*y is never 0 since we have positive and negative zeroes. -- rtoy
 ;;; Compute atanh z = (log(1+z) - log(1-z))/2.
 (defun complex-atanh (z)
-  (declare (muffle-conditions t))
+  (declare (muffle-conditions compiler-note))
   (declare (type (or rational complex) z))
   (let* (;; constants
-         (theta (/ (sqrt most-positive-double-float) 4.0d0))
-         (rho (/ 4.0d0 (sqrt most-positive-double-float)))
-         (half-pi (/ pi 2.0d0))
+         (theta (/ (sqrt sb-xc:most-positive-double-float) 4.0d0))
+         (rho (/ 4.0d0 (sqrt sb-xc:most-positive-double-float)))
+         (half-pi (/ sb-xc:pi 2.0d0))
          (rp (float (realpart z) 1.0d0))
          (beta (float-sign rp 1.0d0))
          (x (* beta rp))
@@ -1006,7 +995,7 @@
 
 ;;; Compute tanh z = sinh z / cosh z.
 (defun complex-tanh (z)
-  (declare (muffle-conditions t))
+  (declare (muffle-conditions compiler-note))
   (declare (type (or rational complex) z))
   (let ((x (float (realpart z) 1.0d0))
         (y (float (imagpart z) 1.0d0)))
@@ -1015,9 +1004,9 @@
       (declare (optimize (speed 3) (space 0)))
     (cond ((> (abs x)
               (load-time-value
-               #!-long-float
+               #-long-float
                (make-double-float #x406633ce #x8fb9f87e)
-               #!+long-float
+               #+long-float
                (error "(/ (+ (log 2l0) (log most-positive-long-float)) 4l0)")))
            (coerce-to-complex-type (float-sign x)
                                    (float-sign y) z))
