@@ -14,14 +14,8 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!FASL")
+(in-package "SB-FASL")
 
-;;;; There looks to be an exciting amount of state being modified
-;;;; here: certainly enough that I (dan, 2003.1.22) don't want to mess
-;;;; around deciding how to thread-safetify it.  So we use a Big Lock.
-;;;; Because this code is mutually recursive with the compiler, we use
-;;;; the **WORLD-LOCK**.
-
 ;;;; miscellaneous load utilities
 
 ;;; Output the current number of semicolons after a fresh-line.
@@ -49,7 +43,7 @@
 
 ;;;; utilities for reading from fasl files
 
-#!-sb-fluid (declaim (inline read-byte))
+#-sb-fluid (declaim (inline read-byte))
 
 ;;; This expands into code to read an N-byte unsigned integer using
 ;;; FAST-READ-BYTE.
@@ -105,24 +99,31 @@
       `(with-fast-read-byte ((unsigned-byte 8) ,fasl-input-stream)
          (fast-read-u-integer ,n))))
 
+(defun read-varint-arg (fasl-input)
+  (let ((accumulator 0)
+        (shift 0))
+    (declare (fixnum shift) (type word accumulator))
+    (with-fast-read-byte ((unsigned-byte 8) (%fasl-input-stream fasl-input))
+      (loop
+       (let ((octet (fast-read-byte)))
+         (setq accumulator (logior accumulator (ash (logand octet #x7F) shift)))
+         (incf shift 7)
+         (unless (logbitp 7 octet) (return accumulator)))))))
+
 ;; FIXME: on x86-64, these functions exceed 600, 900, and 1200 bytes of code
 ;; respectively. Either don't inline them, or make a "really" fast inline case
 ;; that punts if inapplicable. e.g. if the fast-read-byte buffer will not be
 ;; refilled, then SAP-REF-WORD could work to read 8 bytes.
 ;; But this would only be feasible on machines that are little-endian
 ;; and that allow unaligned reads. (like x86)
-(declaim (inline read-byte-arg read-halfword-arg read-word-arg))
+(declaim (inline read-byte-arg read-word-arg))
 (defun read-byte-arg (stream)
   (declare (optimize (speed 0)))
   (read-arg 1 stream))
 
-(defun read-halfword-arg (stream)
-  (declare (optimize (speed 0)))
-  (read-arg #.(/ sb!vm:n-word-bytes 2) stream)) ; READ-ARG doesn't eval N
-
 (defun read-word-arg (stream)
   (declare (optimize (speed 0)))
-  (read-arg #.sb!vm:n-word-bytes stream))
+  (read-arg #.sb-vm:n-word-bytes stream))
 
 (defun read-unsigned-byte-32-arg (stream)
   (declare (optimize (speed 0)))
@@ -166,10 +167,10 @@
 
 (defun nuke-fop-vector (vector)
   (declare (simple-vector vector)
-           #!-gencgc (ignore vector)
+           #-gencgc (ignore vector)
            (optimize speed))
   ;; Make sure we don't keep any garbage.
-  #!+gencgc
+  #+gencgc
   (fill vector 0))
 
 
@@ -249,72 +250,28 @@
              (invalid-fasl-expected condition)))))
 
 (define-condition invalid-fasl-features (invalid-fasl)
-  ((potential-features :reader invalid-fasl-potential-features
-                       :initarg :potential-features)
-   (features :reader invalid-fasl-features :initarg :features))
+  ((features :reader invalid-fasl-features :initarg :features))
   (:report
    (lambda (condition stream)
-     (format stream "~@<incompatible ~S in fasl file ~S: ~2I~_~
-                     Of features affecting binary compatibility, ~4I~_~S~2I~_~
-                     the fasl has ~4I~_~A,~2I~_~
-                     while the runtime expects ~4I~_~A.~:>"
-             '*features*
-             (invalid-fasl-stream condition)
-             (invalid-fasl-potential-features condition)
+     (format stream "~@<incompatible features ~A ~_in fasl file ~S: ~2I~_~
+                     Runtime expects ~A~:>"
              (invalid-fasl-features condition)
+             (invalid-fasl-stream condition)
              (invalid-fasl-expected condition)))))
 
 ;;; Skips past the shebang line on stream, if any.
 (defun maybe-skip-shebang-line (stream)
   (let ((p (file-position stream)))
-    (flet ((next () (read-byte stream nil)))
-      (unwind-protect
-           (when (and (eq (next) (char-code #\#))
-                      (eq (next) (char-code #\!)))
-             (setf p nil)
-             (loop for x = (next)
-                   until (or (not x) (eq x (char-code #\newline)))))
-        (when p
-          (file-position stream p))))
-    t))
-
-;;; Returns T if the stream is a binary input stream with a FASL header.
-#-sb-xc-host ;; FIXME: function belongs in 'target-load'
-(defun fasl-header-p (stream &key errorp)
-  (unless (and (member (stream-element-type stream) '(character base-char))
-               ;; give up if it's not a file stream, or it's an
-               ;; fd-stream but it's either not bivalent or not
-               ;; seekable (doesn't really have a file)
-               (or (not (typep stream 'file-stream))
-                   (and (typep stream 'fd-stream)
-                        (or (not (sb!impl::fd-stream-bivalent-p stream))
-                            (not (sb!impl::fd-stream-file stream))))))
-    (let ((p (file-position stream)))
-      (unwind-protect
-           (let* ((header *fasl-header-string-start-string*)
-                  (buffer (make-array (length header) :element-type '(unsigned-byte 8)))
-                  (n 0))
-             (flet ((scan ()
-                      (maybe-skip-shebang-line stream)
-                      (setf n (read-sequence buffer stream))))
-               (if errorp
-                   (scan)
-                   (or (ignore-errors (scan))
-                       ;; no a binary input stream
-                       (return-from fasl-header-p nil))))
-             (if (mismatch buffer header
-                           :test #'(lambda (code char) (= code (char-code char))))
-                 ;; Immediate EOF is valid -- we want to match what
-                 ;; CHECK-FASL-HEADER does...
-                 (or (zerop n)
-                     (when errorp
-                       (error 'fasl-header-missing
-                              :stream stream
-                              :fhsss buffer
-                              :expected header)))
-                 t))
-        (file-position stream p)))))
-
+    (when p
+      (flet ((next () (read-byte stream nil)))
+        (unwind-protect
+             (when (and (eq (next) (char-code #\#))
+                        (eq (next) (char-code #\!)))
+               (setf p nil)
+               (loop for x = (next)
+                     until (or (not x) (eq x (char-code #\newline)))))
+          (when p
+            (file-position stream p)))))))
 
 ;;;; LOAD-AS-FASL
 ;;;;
@@ -360,18 +317,21 @@
                  (read-string-as-bytes stream result)
                  result)))
         ;; Read and validate implementation and version.
-        (let ((implementation (keywordicate (string-from-stream)))
+        (let ((implementation (string-from-stream))
               (expected-implementation +backend-fasl-file-implementation+))
           (unless (string= expected-implementation implementation)
             (error 'invalid-fasl-implementation
                    :stream stream
+                   ;; This slot used to hold a symbol. Now it's a string.
+                   ;; I don't think anyone should care, but if they do,
+                   ;; then this needs a call to KEYWORDICATE.
                    :implementation implementation
                    :expected expected-implementation)))
         (let* ((fasl-version (read-word-arg stream))
                (sbcl-version (if (<= fasl-version 76)
                                  "1.0.11.18"
                                  (string-from-stream)))
-               (expected-version (sb!xc:lisp-implementation-version)))
+               (expected-version (sb-xc:lisp-implementation-version)))
           (unless (string= expected-version sbcl-version)
             (restart-case
                 (error 'invalid-fasl-version
@@ -380,19 +340,19 @@
                        :expected expected-version)
               (continue () :report "Load the fasl file anyway"))))
         ;; Read and validate *FEATURES* which affect binary compatibility.
-        (let ((faff-in-this-file (string-from-stream)))
-          (unless (string= faff-in-this-file *features-affecting-fasl-format*)
+        (let ((faff-in-this-file (string-from-stream))
+              (expected (compute-features-affecting-fasl-format)))
+          (unless (string= faff-in-this-file expected)
             (error 'invalid-fasl-features
                    :stream stream
-                   :potential-features *features-potentially-affecting-fasl-format*
-                   :expected *features-affecting-fasl-format*
+                   :expected expected
                    :features faff-in-this-file)))
         ;; success
         t))))
 
 ;; Setting this variable gives you a trace of fops as they are loaded and
 ;; executed.
-#!+sb-show
+#+sb-show
 (defvar *show-fops-p* nil)
 
 ;;;
@@ -401,7 +361,6 @@
 ;;; Return true if we successfully load a group from the stream, or
 ;;; NIL if EOF was encountered while trying to read from the stream.
 ;;; Dispatch to the right function for each fop.
-(defconstant +2-operand-fops+ #xE0) ; start of the range
 (defun load-fasl-group (fasl-input print)
   ;;
   ;; PRINT causes most tlf-equivalent forms to print their primary value.
@@ -414,45 +373,41 @@
   ;;
   (declare (ignorable print))
   (let ((stream (%fasl-input-stream fasl-input))
-        #!+sb-show (trace *show-fops-p*))
+        (trace #+sb-show *show-fops-p*))
     (unless (check-fasl-header stream)
       (return-from load-fasl-group))
     (catch 'fasl-group-end
      (setf (svref (%fasl-input-table fasl-input) 0) 0)
-     (macrolet ((tracing (&body forms)
-                  #!+sb-show `(when trace ,@forms)
-                  #!-sb-show (progn forms nil)))
+     (macrolet ((tracing (&body forms) `(when trace ,@forms)))
       (loop
-       (let ((byte (the (unsigned-byte 8) (read-byte stream))))
+       (let* ((byte (the (unsigned-byte 8) (read-byte stream)))
+              (function (svref **fop-funs** byte))
+              (n-operands (aref (car **fop-signatures**) byte)))
          ;; Do some debugging output.
          (tracing
-           (format *trace-output* "~&~6x : [~D,~D] ~2,'0x(~A)"
-                   (1- (file-position stream))
-                   (svref (%fasl-input-stack fasl-input) 0) ; stack pointer
-                   (svref (%fasl-input-table fasl-input) 0) ; table pointer
-                   byte (aref **fop-names** byte)))
+          (format *trace-output* "~&~6x : [~D,~D] ~2,'0x(~A)"
+                  (1- (file-position stream))
+                  (svref (%fasl-input-stack fasl-input) 0) ; stack pointer
+                  (svref (%fasl-input-table fasl-input) 0) ; table pointer
+                  byte (and (functionp function) (%fun-name function))))
          ;; Actually execute the fop.
          (let ((result
-                (let ((function (svref **fop-funs** byte)))
-                  (cond ((not (functionp function))
-                         (error "corrupt fasl file: FOP code #x~x" byte))
-                        ((zerop (sbit (car **fop-signatures**) (ash byte -2)))
-                         ;; takes no arguments from the fasl file
-                         (funcall function fasl-input))
-                        (t
-                         ;; one or two arguments come from the fasl file
-                         (let (arg1 arg2) ; See !%DEFINE-FOP for encoding
-                           (with-fast-read-byte ((unsigned-byte 8) stream)
-                             (setq arg1 (fast-read-var-u-integer
-                                         (ash 1 (logand byte 3))))
-                             (when (>= byte +2-operand-fops+)
-                               (setq arg2 (fast-read-var-u-integer
-                                           (ash 1 (ldb (byte 2 2) byte))))))
-                           (tracing
-                             (format *trace-output* "{~D~@[,~D~]}" arg1 arg2))
-                           (if arg2
-                               (funcall function fasl-input arg1 arg2)
-                               (funcall function fasl-input arg1))))))))
+                 (cond ((not (functionp function))
+                        (error "corrupt fasl file: FOP code #x~x" byte))
+                       ((zerop n-operands)
+                        (funcall function fasl-input))
+                       (t
+                        (let ((arg1 (read-varint-arg fasl-input))
+                              (arg2 (when (>= n-operands 2)
+                                      (read-varint-arg fasl-input)))
+                              (arg3 (when (>= n-operands 3)
+                                      (read-varint-arg fasl-input))))
+                          (tracing (format *trace-output* "{~D~@[,~D~@[,~D~]~]}"
+                                           arg1 arg2 arg3))
+                          (case n-operands
+                            (3 (funcall function fasl-input arg1 arg2 arg3))
+                            (2 (funcall function fasl-input arg1 arg2))
+                            (1 (funcall function fasl-input arg1))))))))
            (when (plusp (sbit (cdr **fop-signatures**) byte))
              (push-fop-stack result fasl-input))
            (let ((stack (%fasl-input-stack fasl-input)))
@@ -475,7 +430,7 @@
                    (setf (%fasl-input-deprecated-stuff fasl-input) nil)
                    (loader-deprecation-warn
                     it
-                    (and (eq (svref stack 1) 'sb!impl::%defun) (svref stack 2))))
+                    (and (eq (svref stack 1) 'sb-impl::%defun) (svref stack 2))))
                  (when print
                    (load-fresh-line)
                    (prin1 result))))))))))))
@@ -517,7 +472,7 @@
                    (dotimes (i 255)
                      (declare (fixnum i))
                      (let ((n (svref ,vec i)))
-                       (push (cons (svref *fop-names* i) n) ,lvar)
+                       (push (cons (%fun-name (svref **fop-funs** i)) n) ,lvar)
                        (incf ,tvar n)))
                    (setq ,lvar (subseq (sort ,lvar (lambda (x y)
                                                      (> (cdr x) (cdr y))))
