@@ -9,13 +9,13 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!IMPL")
+(in-package "SB-IMPL")
 
 ;;; Declare each of the free space pointers (except dynamic) as an alien var
 (define-alien-variable ("read_only_space_free_pointer"
-                        sb!vm:*read-only-space-free-pointer*)
+                        sb-vm:*read-only-space-free-pointer*)
     system-area-pointer)
-(define-alien-variable ("static_space_free_pointer" sb!vm:*static-space-free-pointer*)
+(define-alien-variable ("static_space_free_pointer" sb-vm:*static-space-free-pointer*)
   system-area-pointer)
 
 (declaim (inline memmove))
@@ -29,10 +29,10 @@
       "Return the value of the C library pseudo-variable named \"errno\".")
 
 ;;; Decode errno into a string.
-#!-win32
+#-win32
 (defun strerror (&optional (errno (get-errno)))
   (alien-funcall (extern-alien "strerror" (function c-string int)) errno))
 
-#!+win32
-(defun strerror (&optional (errno (sb!win32:get-last-error)))
-  (sb!win32:format-system-message errno))
+#+win32
+(defun strerror (&optional (errno (sb-win32:get-last-error)))
+  (sb-win32:format-system-message errno))
