@@ -1,5 +1,5 @@
-;;;; error-handling machinery for PARSE-DEFMACRO, separated from
-;;;; PARSE-DEFMACRO code itself because the happy path can be handled
+;;;; error-handling machinery for MAKE-MACRO-LAMBDA separated from
+;;;; that code because the happy path can be handled
 ;;;; earlier in the bootstrap sequence than DEFINE-CONDITION can be
 
 ;;;; This software is part of the SBCL system. See the README file for
@@ -11,7 +11,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!KERNEL")
+(in-package "SB-KERNEL")
 
 (define-condition defmacro-lambda-list-bind-error (error)
   ((kind :reader defmacro-lambda-list-bind-error-kind
@@ -35,25 +35,12 @@
   (declare (type stream stream) (type function fun))
   (pprint-logical-block (stream nil)
     (format stream
-            "error while parsing arguments to ~A~@[ ~S~]:~2I~:@_"
+            "Error while parsing arguments to~@[ ~A~]~@[ ~S~]:~2I~:@_"
             (defmacro-lambda-list-bind-error-kind condition)
             (defmacro-lambda-list-bind-error-name condition))
     (pprint-logical-block (stream nil)
       (funcall fun stream))))
 
-(define-condition defmacro-bogus-sublist-error
-                  (defmacro-lambda-list-bind-error)
-  ((object :reader defmacro-bogus-sublist-error-object :initarg :object)
-   (lambda-list :reader defmacro-bogus-sublist-error-lambda-list
-                :initarg :lambda-list))
-  (:report
-   (lambda (condition stream)
-     (!printing-defmacro-lambda-list-bind-error (condition stream)
-       (format stream
-               "bogus sublist ~2I~_~S ~I~_to satisfy lambda-list ~2I~_~:S"
-               (defmacro-bogus-sublist-error-object condition)
-               (defmacro-bogus-sublist-error-lambda-list condition))))))
-
 (define-condition arg-count-error (defmacro-lambda-list-bind-error)
   ((args :reader arg-count-error-args :initarg :args)
    (lambda-list :reader arg-count-error-lambda-list
@@ -62,25 +49,29 @@
    (maximum :reader arg-count-error-maximum :initarg :maximum))
   (:report
    (lambda (condition stream)
-     (!printing-defmacro-lambda-list-bind-error (condition stream)
+    (!printing-defmacro-lambda-list-bind-error (condition stream)
+      (let* ((min (arg-count-error-minimum condition))
+             (max (arg-count-error-maximum condition))
+             (actual (arg-count-error-args condition))
+             (n-actual (if (proper-list-p actual) (length actual) nil)))
        (format stream
-               "invalid number of elements in ~2I~_~:S ~
+               "~A elements in ~2I~_~:S ~
                 ~I~_to satisfy lambda list ~2I~_~:S: ~I~_"
-               (arg-count-error-args condition)
-               (arg-count-error-lambda-list condition))
-       (cond ((null (arg-count-error-maximum condition))
-              (format stream "at least ~W expected"
-                      (arg-count-error-minimum condition)))
-             ((= (arg-count-error-minimum condition)
-                 (arg-count-error-maximum condition))
-              (format stream "exactly ~W expected"
-                      (arg-count-error-minimum condition)))
+               (cond ((and n-actual (< n-actual min)) "too few")
+                     ((and n-actual max (> n-actual max)) "too many")
+                     (t "invalid number of"))
+               actual (arg-count-error-lambda-list condition))
+       (format stream
+               (cond ((null max) "at least ~W expected")
+                     ((= min max) "exactly ~W expected")
+                     (t "between ~W and ~W expected"))
+               min max)
+       (cond ((and (atom actual) actual)
+              (format stream ", but got a non-list"))
+             ((cdr (last actual))
+              (format stream ", but got an improper list"))
              (t
-              (format stream "between ~W and ~W expected"
-                      (arg-count-error-minimum condition)
-                      (arg-count-error-maximum condition))))
-       (format stream ", but ~W found"
-               (length (arg-count-error-args condition)))))))
+              (format stream ", but got ~d" n-actual))))))))
 
 (define-condition defmacro-lambda-list-broken-key-list-error
                   (defmacro-lambda-list-bind-error)
@@ -105,6 +96,9 @@
                          (:odd-length
                           "odd number of elements in keyword/value list: ~S")
                          (:unknown-keyword
+                          ;; Todo: print the keyword portion of the actual args
+                          ;;  "unknown keyword foo in (:A 1 :B ...);
+                          ;;   expected one of ..."
                           "~{unknown keyword: ~S; expected one of ~
                            ~{~S~^, ~}~}"))
                        (defmacro-lambda-list-broken-key-list-error-info
