@@ -9,50 +9,54 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!IMPL")
+(in-package "SB-IMPL")
 
 ;;;; exported printer control variables
 
-(!defvar *print-readably* nil
+;; NB: all of the following are initialized during genesis
+(defparameter *print-readably* nil
   "If true, all objects will be printed readably. If readable printing
   is impossible, an error will be signalled. This overrides the value of
   *PRINT-ESCAPE*.")
-(!defvar *print-escape* t
+(defparameter *print-escape* t
   "Should we print in a reasonably machine-readable way? (possibly
   overridden by *PRINT-READABLY*)")
-(!defvar *print-pretty* nil ; (set later when pretty-printer is initialized)
+(defparameter *print-pretty* nil ; (set later when pretty-printer is initialized)
   "Should pretty printing be used?")
-(!defvar *print-base* 10.
+(defparameter *print-base* 10.
   "The output base for RATIONALs (including integers).")
-(!defvar *print-radix* nil
+(defparameter *print-radix* nil
   "Should base be verified when printing RATIONALs?")
-(!defvar *print-level* nil
+(defparameter *print-level* nil
   "How many levels should be printed before abbreviating with \"#\"?")
-(!defvar *print-length* nil
+(defparameter *print-length* nil
   "How many elements at any level should be printed before abbreviating
   with \"...\"?")
-(!defvar *print-circle* nil
+(defparameter *print-vector-length* nil
+  "Like *PRINT-LENGTH* but works on strings and bit-vectors.
+Does not affect the cases that are already controlled by *PRINT-LENGTH*")
+(defparameter *print-circle* nil
   "Should we use #n= and #n# notation to preserve uniqueness in general (and
   circularity in particular) when printing?")
-(!defvar *print-case* :upcase
+(defparameter *print-case* :upcase
   "What case should the printer should use default?")
-(!defvar *print-array* t
+(defparameter *print-array* t
   "Should the contents of arrays be printed?")
-(!defvar *print-gensym* t
+(defparameter *print-gensym* t
   "Should #: prefixes be used when printing symbols with null SYMBOL-PACKAGE?")
-(!defvar *print-lines* nil
+(defparameter *print-lines* nil
   "The maximum number of lines to print per object.")
-(!defvar *print-right-margin* nil
+(defparameter *print-right-margin* nil
   "The position of the right margin in ems (for pretty-printing).")
-(!defvar *print-miser-width* nil
+(defparameter *print-miser-width* nil
   "If the remaining space between the current column and the right margin
    is less than this, then print using ``miser-style'' output. Miser
    style conditional newlines are turned on, and all indentations are
    turned off. If NIL, never use miser mode.")
 (defvar *print-pprint-dispatch*
-  (sb!pretty::make-pprint-dispatch-table) ; for type-correctness
+  (sb-pretty::make-pprint-dispatch-table) ; for type-correctness
   "The pprint-dispatch-table that controls how to pretty-print objects.")
-(!defvar *suppress-print-errors* nil
+(defparameter *suppress-print-errors* nil
   "Suppress printer errors when the condition is of the type designated by this
 variable: an unreadable object representing the error is printed instead.")
 
@@ -62,7 +66,7 @@ variable: an unreadable object representing the error is printed instead.")
 (defun %with-standard-io-syntax (function)
   (declare (type function function))
   (declare (dynamic-extent function))
-  (let ((*package* (load-time-value (find-package "COMMON-LISP-USER") t))
+  (let ((*package* #.(find-package "COMMON-LISP-USER"))
         (*print-array* t)
         (*print-base* 10)
         (*print-case* :upcase)
@@ -73,7 +77,7 @@ variable: an unreadable object representing the error is printed instead.")
         (*print-level* nil)
         (*print-lines* nil)
         (*print-miser-width* nil)
-        (*print-pprint-dispatch* sb!pretty::*standard-pprint-dispatch-table*)
+        (*print-pprint-dispatch* sb-pretty::*standard-pprint-dispatch-table*)
         (*print-pretty* nil)
         (*print-radix* nil)
         (*print-readably* t)
@@ -190,7 +194,7 @@ variable: an unreadable object representing the error is printed instead.")
          (and *print-pretty* (pprint-dispatch object))
        (if pretty
            (with-simple-output-to-string (stream)
-              (sb!pretty::with-pretty-stream (stream)
+              (sb-pretty::with-pretty-stream (stream)
                 (funcall fun stream object)))
            (let ((buffer-size (approx-chars-in-repr object)))
              (let* ((string (make-string buffer-size :element-type 'base-char))
@@ -225,8 +229,8 @@ variable: an unreadable object representing the error is printed instead.")
     (+ (if (minusp object) 1 0) ; leading sign
        (if *print-radix* 4 0) ; #rNN or trailing decimal
        (ceiling (if (fixnump object)
-                    sb!vm:n-positive-fixnum-bits
-                    (* (%bignum-length object) sb!bignum::digit-size))
+                    sb-vm:n-positive-fixnum-bits
+                    (* (%bignum-length object) sb-bignum::digit-size))
                 bits-per-char))))
 
 ;;;; support for the PRINT-UNREADABLE-OBJECT macro
@@ -248,11 +252,12 @@ variable: an unreadable object representing the error is printed instead.")
       o)))
 
 ;;; guts of PRINT-UNREADABLE-OBJECT
-(defun %print-unreadable-object (object stream type identity &optional body)
+(defun %print-unreadable-object (object stream flags &optional body)
   (declare (type (or null function) body))
   (if *print-readably*
       (print-not-readable-error object stream)
-      (flet ((print-description ()
+      (flet ((print-description (&aux (type (logbitp 0 (truly-the (mod 4) flags)))
+                                      (identity (logbitp 1 flags)))
                (when type
                  (write (type-of object) :stream stream :circle nil
                                          :level nil :length nil)
@@ -290,7 +295,7 @@ variable: an unreadable object representing the error is printed instead.")
   (or (numberp x)
       (characterp x)
       (and (symbolp x)
-           (symbol-package x))))
+           (sb-xc:symbol-package x))))
 
 (defvar *in-print-error* nil)
 
@@ -305,7 +310,7 @@ variable: an unreadable object representing the error is printed instead.")
              (multiple-value-bind (fun pretty)
                  (and *print-pretty* (pprint-dispatch object))
                (if pretty
-                   (sb!pretty::with-pretty-stream (stream)
+                   (sb-pretty::with-pretty-stream (stream)
                      (funcall fun stream object))
                    (output-ugly-object stream object))))
            (handle-it (stream)
@@ -371,7 +376,7 @@ variable: an unreadable object representing the error is printed instead.")
       ;; If an instance has no layout, it has no PRINT-OBJECT method.
       ;; Additionally, if the object is an obsolete CONDITION, don't crash.
       ;; (There is no update-instance protocol for conditions)
-      (when (or (sb!kernel::undefined-classoid-p classoid)
+      (when (or (sb-kernel::undefined-classoid-p classoid)
                 (and (layout-invalid layout)
                      (logtest (layout-%flags layout) +condition-layout-flag+)))
         ;; not only is this unreadable, it's unprintable too.
@@ -385,7 +390,7 @@ variable: an unreadable object representing the error is printed instead.")
 (defmethod print-object ((object symbol) stream)
   (if (or *print-escape* *print-readably*)
       ;; Write so that reading back works
-      (output-symbol object (symbol-package object) stream)
+      (output-symbol object (sb-xc:symbol-package object) stream)
       ;; Write only the characters of the name, never the package
       (let ((rt *readtable*))
         (funcall (truly-the function
@@ -399,7 +404,7 @@ variable: an unreadable object representing the error is printed instead.")
     (flet ((output-token (name)
              (declare (type simple-string name))
              (cond ((or (and (readtable-normalization readtable)
-                             (not (sb!unicode:normalized-p name :nfkc)))
+                             (not (sb-unicode:normalized-p name :nfkc)))
                         (symbol-quotep name readtable))
                     ;; Output NAME surrounded with |'s,
                     ;; and with any embedded |'s or \'s escaped.
@@ -439,12 +444,10 @@ variable: an unreadable object representing the error is printed instead.")
             ;; To preserve print-read consistency, use the local nickname if
             ;; one exists.
             (unless (and accessible (eq found symbol))
-              (let ((prefix (or (car (rassoc package (package-%local-nicknames current)))
-                                (package-name package))))
-                (output-token prefix))
-              (write-char #\: stream)
-              (when (eql (find-external-symbol name package) 0)
-                (write-char #\: stream))))))
+              (output-token (or (package-local-nickname package current)
+                                (package-name package)))
+              (write-string (if (eql (find-external-symbol name package) 0) "::" ":")
+                            stream)))))
         (output-token name)))))
 
 ;;;; escaping symbols
@@ -457,7 +460,7 @@ variable: an unreadable object representing the error is printed instead.")
 ;;; fixnum with bits set corresponding to attributes that the
 ;;; character has. At characters have at least one bit set, so we can
 ;;; search for any character with a positive test.
-(defvar *character-attributes*
+(define-load-time-global *character-attributes*
   (make-array 160 ; FIXME
               :element-type '(unsigned-byte 16)
               :initial-element 0))
@@ -475,31 +478,28 @@ variable: an unreadable object representing the error is printed instead.")
 (defconstant slash-attribute            (ash 1 7)) ; /
 (defconstant funny-attribute            (ash 1 8)) ; Anything illegal.
 
-(eval-when (:compile-toplevel :load-toplevel :execute)
-
 ;;; LETTER-ATTRIBUTE is a local of SYMBOL-QUOTEP. It matches letters
 ;;; that don't need to be escaped (according to READTABLE-CASE.)
-(defglobal *attribute-names*
-  `((number . number-attribute) (lowercase . lowercase-attribute)
+(defconstant-eqx +attribute-names+
+  '((number . number-attribute) (lowercase . lowercase-attribute)
     (uppercase . uppercase-attribute) (letter . letter-attribute)
     (sign . sign-attribute) (extension . extension-attribute)
     (dot . dot-attribute) (slash . slash-attribute)
-    (other . other-attribute) (funny . funny-attribute)))
-
-) ; EVAL-WHEN
+    (other . other-attribute) (funny . funny-attribute))
+  #'equal)
 
 ;;; For each character, the value of the corresponding element is the
 ;;; lowest base in which that character is a digit.
 (declaim (type (simple-array (unsigned-byte 8) (128)) ; FIXME: range?
                *digit-bases*))
-(defvar *digit-bases*
+(define-load-time-global *digit-bases*
   (make-array 128 ; FIXME
               :element-type '(unsigned-byte 8)))
 
 (defun !printer-cold-init ()
 ;; The dispatch table will be changed later, so this doesn't really matter
 ;; except if a full call to WRITE wants to read the current binding.
-(setq *print-pprint-dispatch* (sb!pretty::make-pprint-dispatch-table))
+(setq *print-pprint-dispatch* (sb-pretty::make-pprint-dispatch-table))
 (setq *digit-bases* (make-array 128 ; FIXME
                                 :element-type '(unsigned-byte 8)
                                 :initial-element 36)
@@ -566,7 +566,7 @@ variable: an unreadable object representing the error is printed instead.")
                              (logior ,@(mapcar
                                         (lambda (x)
                                           (or (cdr (assoc x
-                                                          *attribute-names*))
+                                                          +attribute-names+))
                                               (error "Blast!")))
                                         attributes))
                              bits)))))
@@ -1094,7 +1094,7 @@ variable: an unreadable object representing the error is printed instead.")
   ;; certainly quite platform dependent: this gives 87 for 32 bit
   ;; SBCL, which is about right at least for x86/Darwin.
   (if (or (fixnump integer)
-          (< (integer-length integer) (* 3 sb!vm:n-positive-fixnum-bits)))
+          (< (integer-length integer) (* 3 sb-vm:n-positive-fixnum-bits)))
       (%output-reasonable-integer-in-base integer base stream)
       (%output-huge-integer-in-base integer base stream)))
 
@@ -1256,10 +1256,10 @@ variable: an unreadable object representing the error is printed instead.")
 ;;; possible extension for the enthusiastic: printing floats in bases
 ;;; other than base 10.
 (defconstant single-float-min-e
-  (- 2 sb!vm:single-float-bias sb!vm:single-float-digits))
+  (- 2 sb-vm:single-float-bias sb-vm:single-float-digits))
 (defconstant double-float-min-e
-  (- 2 sb!vm:double-float-bias sb!vm:double-float-digits))
-#!+long-float
+  (- 2 sb-vm:double-float-bias sb-vm:double-float-digits))
+#+long-float
 (defconstant long-float-min-e
   (nth-value 1 (decode-float least-positive-long-float)))
 
@@ -1278,7 +1278,7 @@ variable: an unreadable object representing the error is printed instead.")
           (etypecase float
             (single-float single-float-min-e)
             (double-float double-float-min-e)
-            #!+long-float
+            #+long-float
             (long-float long-float-min-e))))
     (multiple-value-bind (f e)
         (integer-decode-float float)
@@ -1437,7 +1437,7 @@ variable: an unreadable object representing the error is printed instead.")
 
 (eval-when (:compile-toplevel :execute)
   (setf *read-default-float-format*
-        #!+long-float 'long-float #!-long-float 'double-float))
+        #+long-float 'long-float #-long-float 'double-float))
 (defun scale-exponent (original-x)
   (let* ((x (coerce original-x 'long-float)))
     (multiple-value-bind (sig exponent) (decode-float x)
@@ -1456,15 +1456,15 @@ variable: an unreadable object representing the error is printed instead.")
                                    ;; ulp in this value, which is a
                                    ;; little unfortunate.)
                                    (load-time-value
-                                    #!-long-float
+                                    #-long-float
                                     (make-double-float 1070810131 1352628735)
-                                    #!+long-float
+                                    #+long-float
                                     (error "(log 2 10) not computed")))))))
                  (x (if (minusp ex)
                         (if (float-denormalized-p x)
-                            #!-long-float
+                            #-long-float
                             (* x 1.0e16 (expt 10.0e0 (- (- ex) 16)))
-                            #!+long-float
+                            #+long-float
                             (* x 1.0e18 (expt 10.0e0 (- (- ex) 18)))
                             (* x 10.0e0 (expt 10.0e0 (- (- ex) 1))))
                         (/ x 10.0e0 (expt 10.0e0 (1- ex))))))
@@ -1512,9 +1512,9 @@ variable: an unreadable object representing the error is printed instead.")
   (cond ((case *read-default-float-format*
            ((short-float single-float)
             (typep x 'single-float))
-           ((double-float #!-long-float long-float)
+           ((double-float #-long-float long-float)
             (typep x 'double-float))
-           #!+long-float
+           #+long-float
            (long-float
             (typep x 'long-float)))
          (unless (eql exp 0)
@@ -1531,21 +1531,20 @@ variable: an unreadable object representing the error is printed instead.")
          (%output-integer-in-base exp 10 stream))))
 
 (defun output-float-infinity (x stream)
-  (declare (float x) (stream stream))
-  (cond (*read-eval*
-         (write-string "#." stream))
-        (*print-readably*
-         (return-from output-float-infinity
-           (print-not-readable-error x stream)))
-        (t
-         (write-string "#<" stream)))
-  (write-string "SB-EXT:" stream)
-  (write-string (symbol-name (float-format-name x)) stream)
-  (write-string (if (plusp x) "-POSITIVE-" "-NEGATIVE-")
-                stream)
-  (write-string "INFINITY" stream)
-  (unless *read-eval*
-    (write-string ">" stream)))
+  (declare (stream stream))
+  (let ((symbol (etypecase x
+                  (single-float (if (minusp x)
+                                    'single-float-negative-infinity
+                                    'single-float-positive-infinity))
+                  (double-float (if (minusp x)
+                                    'double-float-negative-infinity
+                                    'double-float-positive-infinity)))))
+    (cond (*read-eval*
+           (write-string "#." stream)
+           (output-symbol symbol (sb-xc:symbol-package symbol) stream))
+          (t
+           (print-unreadable-object (x stream)
+             (output-symbol symbol (sb-xc:symbol-package symbol) stream))))))
 
 (defun output-float-nan (x stream)
   (print-unreadable-object (x stream)
@@ -1614,24 +1613,29 @@ variable: an unreadable object representing the error is printed instead.")
 
 (defmethod print-object ((component code-component) stream)
   (print-unreadable-object (component stream :identity t)
-    (let ((dinfo (%code-debug-info component)))
-      (cond ((eq dinfo :bogus-lra)
+    (let (dinfo)
+      (cond ((code-obj-is-filler-p component)
+             (format stream "filler ~dw"
+                     (ash (code-object-size component) (- sb-vm:word-shift))))
+            ((eq (setq dinfo (%code-debug-info component)) :bogus-lra)
              (write-string "bogus code object" stream))
             ((functionp dinfo)
              (format stream "trampoline ~S" dinfo))
             (t
-             (format stream "code object [~D]" (code-n-entries component))
+             (format stream "code~@[ id=~x~] [~D]"
+                     (%code-serialno component)
+                     (code-n-entries component))
              (let ((fun-name (awhen (%code-entry-point component 0)
                                (%simple-fun-name it))))
                (when fun-name
                  (write-char #\Space stream)
                  (write fun-name :stream stream))
-               (cond ((not (typep dinfo 'sb!c::debug-info)))
-                     ((neq (sb!c::debug-info-name dinfo) fun-name)
+               (cond ((not (typep dinfo 'sb-c::debug-info)))
+                     ((neq (sb-c::debug-info-name dinfo) fun-name)
                       (write-string ", " stream)
-                      (output-object (sb!c::debug-info-name dinfo) stream)))))))))
+                      (output-object (sb-c::debug-info-name dinfo) stream)))))))))
 
-#!-(or x86 x86-64)
+#-(or x86 x86-64)
 (defmethod print-object ((lra lra) stream)
   (print-unreadable-object (lra stream :identity t)
     (write-string "return PC object" stream)))
@@ -1645,7 +1649,7 @@ variable: an unreadable object representing the error is printed instead.")
     (let ((*print-length* 20)) ; arbitrary
       (prin1 (fdefn-name fdefn) stream))))
 
-#!+sb-simd-pack
+#+sb-simd-pack
 (defmethod print-object ((pack simd-pack) stream)
   (cond ((and *print-readably* *read-eval*)
          (multiple-value-bind (format maker extractor)
@@ -1694,6 +1698,42 @@ variable: an unreadable object representing the error is printed instead.")
                           'simd-pack
                           (split-num low 0) (split-num low 32)
                           (split-num high 0) (split-num high 32))))))))))
+
+#+sb-simd-pack-256
+(defmethod print-object ((pack simd-pack-256) stream)
+  (cond ((and *print-readably* *read-eval*)
+         (multiple-value-bind (format maker extractor)
+             (etypecase pack
+               ((simd-pack-256 double-float)
+                (values "#.(~@{~S~^ ~})"
+                        '%make-simd-pack-256-double #'%simd-pack-256-doubles))
+               ((simd-pack-256 single-float)
+                (values "#.(~@{~S~^ ~})"
+                        '%make-simd-pack-256-single #'%simd-pack-256-singles))
+               (t
+                (values "#.(~S~@{ #X~16,'0X~})"
+                        '%make-simd-pack-256-ub64 #'%simd-pack-256-ub64s)))
+           (multiple-value-call
+               #'format stream format maker (funcall extractor pack))))
+        (*print-readably*
+         (print-not-readable-error pack stream))
+        (t
+         (print-unreadable-object (pack stream)
+           (etypecase pack
+             ((simd-pack-256 double-float)
+              (multiple-value-call #'format stream "~S~@{ ~,13E~}"
+                'simd-pack-256
+                (%simd-pack-256-doubles pack)))
+             ((simd-pack-256 single-float)
+              (multiple-value-call #'format stream "~S~@{ ~,7E~}"
+                'simd-pack-256
+                (%simd-pack-256-singles pack)))
+             (t
+              (multiple-value-bind (p0 p1 p2 p3)
+                  (%simd-pack-256-ub64s pack)
+                (format stream "~S~@{ ~16,'0X~}"
+                        'simd-pack-256
+                        p0 p1 p2 p3))))))))
 
 ;;;; functions
 
@@ -1712,47 +1752,41 @@ variable: an unreadable object representing the error is printed instead.")
 ;;;; catch-all for unknown things
 
 (declaim (inline lowtag-of))
-(defun lowtag-of (x) (logand (get-lisp-obj-address x) sb!vm:lowtag-mask))
+(defun lowtag-of (x) (logand (get-lisp-obj-address x) sb-vm:lowtag-mask))
 
 (defmethod print-object ((object t) stream)
-  (flet ((output-it (stream)
-          (when (eq object sb!pcl:+slot-unbound+)
-            (print-unreadable-object (object stream)
-              ;; If specifically the unbound marker with 0 data,
-              ;; as opposed to any other unbound marker.
-              (write-string "unbound" stream))
-            (return-from output-it))
-           (print-unreadable-object (object stream :identity t)
-             (let ((lowtag (lowtag-of object)))
-               (case lowtag
-                 (#.sb!vm:other-pointer-lowtag
-                  (let ((widetag (widetag-of object)))
-                    (case widetag
-                      (#.sb!vm:value-cell-widetag
-                       (write-string "value cell " stream)
-                       (output-object (value-cell-ref object) stream))
-                      (#.sb!vm:filler-widetag
-                       (write-string "pad " stream)
-                       (write (1+ (get-header-data object)) :stream stream)
-                       (write-string "w" stream)) ; words
-                      (t
-                       (write-string "unknown pointer object, widetag=" stream)
-                       (output-integer widetag stream 16 t)))))
-                 ((#.sb!vm:fun-pointer-lowtag
-                   #.sb!vm:instance-pointer-lowtag
-                   #.sb!vm:list-pointer-lowtag)
-                  (write-string "unknown pointer object, lowtag=" stream)
-                  (output-integer lowtag stream 16 t))
-                (t
-                 (case (widetag-of object)
-                   (#.sb!vm:unbound-marker-widetag
-                    (write-string "unbound marker" stream))
-                   (t
-                    (write-string "unknown immediate object, lowtag=" stream)
-                    (output-integer lowtag stream 2 t)
-                    (write-string ", widetag=" stream)
-                    (output-integer (widetag-of object) stream 16 t)))))))))
-    (if *print-pretty*
-      ;; This block might not be necessary. Not sure, probably can't hurt.
-        (pprint-logical-block (stream nil) (output-it stream))
-        (output-it stream))))
+  (when (eq object sb-pcl:+slot-unbound+)
+    ;; If specifically the unbound marker with 0 data,
+    ;; as opposed to any other unbound marker.
+    (print-unreadable-object (object stream) (write-string "unbound" stream))
+    (return-from print-object))
+  (print-unreadable-object (object stream :identity t)
+    (let ((lowtag (lowtag-of object)))
+      (case lowtag
+        (#.sb-vm:other-pointer-lowtag
+         (let ((widetag (widetag-of object)))
+           (case widetag
+             (#.sb-vm:value-cell-widetag
+              (write-string "value cell " stream)
+              (output-object (value-cell-ref object) stream))
+             (#.sb-vm:filler-widetag
+              (write-string "pad " stream)
+              (write (1+ (get-header-data object)) :stream stream)
+              (write-string "w" stream)) ; words
+             (t
+              (write-string "unknown pointer object, widetag=" stream)
+              (output-integer widetag stream 16 t)))))
+        ((#.sb-vm:fun-pointer-lowtag
+          #.sb-vm:instance-pointer-lowtag
+          #.sb-vm:list-pointer-lowtag)
+         (write-string "unknown pointer object, lowtag=" stream)
+         (output-integer lowtag stream 16 t))
+        (t
+         (case (widetag-of object)
+           (#.sb-vm:unbound-marker-widetag
+            (write-string "unbound marker" stream))
+           (t
+            (write-string "unknown immediate object, lowtag=" stream)
+            (output-integer lowtag stream 2 t)
+            (write-string ", widetag=" stream)
+            (output-integer (widetag-of object) stream 16 t))))))))
