@@ -9,21 +9,18 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!IMPL")
+(in-package "SB-IMPL")
 
 ;;;; miscellaneous global variables
 
 ;;; ANSI: "the floating-point format that is to be used when reading a
 ;;; floating-point number that has no exponent marker or that has e or
 ;;; E for an exponent marker"
-(!defvar *read-default-float-format* 'single-float)
-(declaim (type (member short-float single-float double-float long-float)
-               *read-default-float-format*))
+(defparameter *read-default-float-format* 'single-float)
 
 (defvar *readtable*)
-(declaim (type readtable *readtable*))
-#!+sb-doc
-(setf (fdocumentation '*readtable* 'variable)
+
+(setf (documentation '*readtable* 'variable)
       "Variable bound to current readtable.")
 
 ;;; A standard Lisp readtable (once cold-init is through). This is for
@@ -31,7 +28,7 @@
 ;;; WITH-STANDARD-IO-SYNTAX), and should not normally be user-visible.
 ;;; If the initial value is changed from NIL to something more interesting,
 ;;; be sure to update the duplicated definition in "src/code/print.lisp"
-(defglobal *standard-readtable* nil)
+(define-load-time-global *standard-readtable* nil)
 
 ;;; In case we get an error trying to parse a symbol, we want to rebind the
 ;;; above stuff so it's cool.
@@ -118,7 +115,7 @@
       function
       (cond ((functionp x) x)
             ((null x) ,fallback)
-            (t (sb!c:safe-fdefn-fun x))))))
+            (t (sb-c:safe-fdefn-fun x))))))
 
 ;; Return a function-designator given a character-macro-table entry.
 (defmacro !cmt-entry-to-fun-designator (val)
@@ -133,7 +130,7 @@
 
 ;;; predicates for testing character attributes
 
-#!-sb-fluid
+#-sb-fluid
 (progn
   (declaim (inline whitespace[1]p whitespace[2]p))
   (declaim (inline constituentp terminating-macrop))
@@ -171,47 +168,43 @@
 
 ;;; There are a number of "secondary" attributes which are constant
 ;;; properties of characters (as long as they are constituents).
-
-(declaim (type attribute-table *constituent-trait-table*))
-(defglobal *constituent-trait-table*
-  (make-array base-char-code-limit
-              :element-type '(unsigned-byte 8)
-              :initial-element +char-attr-constituent+))
-
-(defun !set-constituent-trait (char trait)
-  (aver (typep char 'base-char))
-  (setf (elt *constituent-trait-table* (char-code char))
-        trait))
-
-(defun !cold-init-constituent-trait-table ()
-  (!set-constituent-trait #\: +char-attr-package-delimiter+)
-  (!set-constituent-trait #\. +char-attr-constituent-dot+)
-  (!set-constituent-trait #\+ +char-attr-constituent-sign+)
-  (!set-constituent-trait #\- +char-attr-constituent-sign+)
-  (!set-constituent-trait #\/ +char-attr-constituent-slash+)
-  (do ((i (char-code #\0) (1+ i)))
-      ((> i (char-code #\9)))
-    (!set-constituent-trait (code-char i) +char-attr-constituent-digit+))
-  (!set-constituent-trait #\E +char-attr-constituent-expt+)
-  (!set-constituent-trait #\F +char-attr-constituent-expt+)
-  (!set-constituent-trait #\D +char-attr-constituent-expt+)
-  (!set-constituent-trait #\S +char-attr-constituent-expt+)
-  (!set-constituent-trait #\L +char-attr-constituent-expt+)
-  (!set-constituent-trait #\e +char-attr-constituent-expt+)
-  (!set-constituent-trait #\f +char-attr-constituent-expt+)
-  (!set-constituent-trait #\d +char-attr-constituent-expt+)
-  (!set-constituent-trait #\s +char-attr-constituent-expt+)
-  (!set-constituent-trait #\l +char-attr-constituent-expt+)
-  (!set-constituent-trait #\Space +char-attr-invalid+)
-  (!set-constituent-trait #\Newline +char-attr-invalid+)
-  (dolist (c (list backspace-char-code tab-char-code form-feed-char-code
-                   return-char-code rubout-char-code))
-    (!set-constituent-trait (code-char c) +char-attr-invalid+)))
+;;; FIXME: this initform is considered too hairy to assign (a constant array, really?)
+;;; if changed to DEFCONSTANT-EQX, which makes this file unslammable as-is. Oh well.
+(defconstant +constituent-trait-table+
+  #.(let ((a (!make-specialized-array base-char-code-limit '(unsigned-byte 8))))
+      (fill a +char-attr-constituent+)
+      (flet ((!set-constituent-trait (char trait)
+               (aver (typep char 'base-char))
+               (setf (elt a (char-code char)) trait)))
+        (!set-constituent-trait #\: +char-attr-package-delimiter+)
+        (!set-constituent-trait #\. +char-attr-constituent-dot+)
+        (!set-constituent-trait #\+ +char-attr-constituent-sign+)
+        (!set-constituent-trait #\- +char-attr-constituent-sign+)
+        (!set-constituent-trait #\/ +char-attr-constituent-slash+)
+        (do ((i (char-code #\0) (1+ i)))
+            ((> i (char-code #\9)))
+          (!set-constituent-trait (code-char i) +char-attr-constituent-digit+))
+        (!set-constituent-trait #\E +char-attr-constituent-expt+)
+        (!set-constituent-trait #\F +char-attr-constituent-expt+)
+        (!set-constituent-trait #\D +char-attr-constituent-expt+)
+        (!set-constituent-trait #\S +char-attr-constituent-expt+)
+        (!set-constituent-trait #\L +char-attr-constituent-expt+)
+        (!set-constituent-trait #\e +char-attr-constituent-expt+)
+        (!set-constituent-trait #\f +char-attr-constituent-expt+)
+        (!set-constituent-trait #\d +char-attr-constituent-expt+)
+        (!set-constituent-trait #\s +char-attr-constituent-expt+)
+        (!set-constituent-trait #\l +char-attr-constituent-expt+)
+        (!set-constituent-trait #\Space +char-attr-invalid+)
+        (!set-constituent-trait #\Newline +char-attr-invalid+)
+        (dolist (c (list backspace-char-code tab-char-code form-feed-char-code
+                         return-char-code rubout-char-code))
+          (!set-constituent-trait (code-char c) +char-attr-invalid+)))
+      a))
 
 (declaim (inline get-constituent-trait))
 (defun get-constituent-trait (char)
   (if (typep char 'base-char)
-      (elt *constituent-trait-table* (char-code char))
+      (elt +constituent-trait-table+ (char-code char))
       +char-attr-constituent+))
 
 ;;;; Readtable Operations
@@ -222,21 +215,21 @@
             :operation operation)))
 
 (defun readtable-case (readtable)
-  (%readtable-case readtable))
+  (aref #(:upcase :downcase :preserve :invert) (%readtable-case readtable)))
 
 (defun (setf readtable-case) (case readtable)
   ;; This function does not accept a readtable designator, only a readtable.
   (assert-not-standard-readtable readtable '(setf readtable-case))
-  (setf (%readtable-case readtable) case))
+  (setf (%readtable-case readtable)
+        (ecase case (:upcase 0) (:downcase 1) (:preserve 2) (:invert 3)))
+  case)
 
 (defun readtable-normalization (readtable)
-  #!+sb-doc
   "Returns T if READTABLE normalizes strings to NFKC, and NIL otherwise.
 The READTABLE-NORMALIZATION of the standard readtable is T."
   (%readtable-normalization readtable))
 
 (defun (setf readtable-normalization) (new-value readtable)
-  #!+sb-doc
   "Sets the READTABLE-NORMALIZATION of the given READTABLE to NEW-VALUE.
 Pass T to make READTABLE normalize symbols to NFKC (the default behavior),
 and NIL to suppress normalization."
@@ -244,6 +237,29 @@ and NIL to suppress normalization."
   (assert-not-standard-readtable readtable '(setf readtable-normalization))
   (setf (%readtable-normalization readtable) new-value))
 
+(defun readtable-base-char-preference (readtable)
+  "Returns :SYMBOLS, :STRINGS, :BOTH, or NIL, depending on whether the
+reader should try to intern a base-string when reading a symbol name,
+respectively produce a base-string when reading a quoted string, or in both
+cases, or neither. The preference applies when a symbol-name or string
+contains only BASE-CHAR characters. An (ARRAY CHARACTER (*)) can always
+be interned (returned, respectively) as required. The default is :SYMBOLS."
+  ;; For efficiency the single preference occupies two slots internally.
+  (let ((symbols (eq (%readtable-symbol-preference readtable) 'base-char))
+        (strings (eq (%readtable-string-preference readtable) 'base-char)))
+    (cond ((and strings symbols) :both)
+          (symbols :symbols)
+          (strings :strings))))
+
+(defun (setf readtable-base-char-preference) (new-value readtable)
+  (declare (type (member :symbols :strings :both nil) new-value))
+  "Sets the READTABLE-BASE-CHAR-PREFERENCE of the given READTABLE."
+  (setf (%readtable-symbol-preference readtable)
+        (if (member new-value '(:symbols :both)) 'base-char 'character)
+        (%readtable-string-preference readtable)
+        (if (member new-value '(:strings :both)) 'base-char 'character))
+  new-value)
+
 (defun replace/eql-hash-table (to from &optional (transform #'identity))
   (maphash (lambda (k v) (setf (gethash k to) (funcall transform v))) from)
   to)
@@ -272,35 +288,37 @@ and NIL to suppress normalization."
         entry)))
 
 (defun copy-readtable (&optional (from-readtable *readtable*) to-readtable)
+  "Copies FROM-READTABLE and returns the result. Uses TO-READTABLE as a target
+for the copy when provided, otherwise a new readtable is created. The
+FROM-READTABLE defaults to the standard readtable when NIL and to the current
+readtable when not provided."
   (assert-not-standard-readtable to-readtable 'copy-readtable)
   (let ((really-from-readtable (or from-readtable *standard-readtable*))
         (really-to-readtable (or to-readtable (make-readtable))))
     (replace (character-attribute-array really-to-readtable)
              (character-attribute-array really-from-readtable))
     (replace/eql-hash-table
-     (character-attribute-hash-table really-to-readtable)
+     (clrhash (character-attribute-hash-table really-to-readtable))
      (character-attribute-hash-table really-from-readtable))
-    ;; CLHS says that when TO-READTABLE is non-nil "... the readtable specified
-    ;; ... is modified and returned." Is that to imply making TO-READTABLE look
-    ;; exactly like FROM-READTABLE, or does it mean to augment it?
-    ;; We have conflicting behaviors - everything in the base-char range,
-    ;; is overwritten, but above that range it's additive.
     (map-into (character-macro-array really-to-readtable)
               #'copy-cmt-entry
               (character-macro-array really-from-readtable))
     (replace/eql-hash-table
-     (character-macro-hash-table really-to-readtable)
+     (clrhash (character-macro-hash-table really-to-readtable))
      (character-macro-hash-table really-from-readtable)
      #'copy-cmt-entry)
     (setf (readtable-case really-to-readtable)
           (readtable-case really-from-readtable))
+    (setf (%readtable-string-preference really-to-readtable)
+          (%readtable-string-preference really-from-readtable)
+          (%readtable-symbol-preference really-to-readtable)
+          (%readtable-symbol-preference really-from-readtable))
     (setf (readtable-normalization really-to-readtable)
           (readtable-normalization really-from-readtable))
     really-to-readtable))
 
 (defun set-syntax-from-char (to-char from-char &optional
                              (to-readtable *readtable*) (from-readtable nil))
-  #!+sb-doc
   "Causes the syntax of TO-CHAR to be the same as FROM-CHAR in the optional
 readtable (defaults to the current readtable). The FROM-TABLE defaults to the
 standard Lisp readtable when NIL."
@@ -316,7 +334,6 @@ standard Lisp readtable when NIL."
 (defun set-macro-character (char function &optional
                                  (non-terminatingp nil)
                                  (rt-designator *readtable*))
-  #!+sb-doc
   "Causes CHAR to be a macro character which invokes FUNCTION when seen
    by the reader. The NON-TERMINATINGP flag can be used to make the macro
    character non-terminating, i.e. embeddable in a symbol name."
@@ -330,7 +347,6 @@ standard Lisp readtable when NIL."
     t)) ; (ANSI-specified return value)
 
 (defun get-macro-character (char &optional (rt-designator *readtable*))
-  #!+sb-doc
   "Return the function associated with the specified CHAR which is a macro
   character, or NIL if there is no such function. As a second value, return
   T if CHAR is a macro character which is non-terminating, i.e. which can
@@ -358,7 +374,6 @@ standard Lisp readtable when NIL."
 (defun make-dispatch-macro-character (char &optional
                                       (non-terminating-p nil)
                                       (rt *readtable*))
-  #!+sb-doc
   "Cause CHAR to become a dispatching macro character in readtable (which
    defaults to the current readtable). If NON-TERMINATING-P, the char will
    be non-terminating."
@@ -380,7 +395,6 @@ standard Lisp readtable when NIL."
 
 (defun set-dispatch-macro-character (disp-char sub-char function
                                      &optional (rt-designator *readtable*))
-  #!+sb-doc
   "Cause FUNCTION to be called whenever the reader reads DISP-CHAR
    followed by SUB-CHAR."
   ;; Get the dispatch char for macro (error if not there), diddle
@@ -408,7 +422,6 @@ standard Lisp readtable when NIL."
 
 (defun get-dispatch-macro-character (disp-char sub-char
                                      &optional (rt-designator *readtable*))
-  #!+sb-doc
   "Return the macro character function for SUB-CHAR under DISP-CHAR
    or NIL if there is no associated function."
   (let ((dtable (get-dispatch-macro-char-table
@@ -424,7 +437,7 @@ standard Lisp readtable when NIL."
 (defun flush-whitespace (stream)
   ;; This flushes whitespace chars, returning the last char it read (a
   ;; non-white one). It always gets an error on end-of-file.
-  (let* ((stream (in-synonym-of stream))
+  (let* ((stream (in-stream-from-designator stream))
          (rt *readtable*)
          (attribute-array (character-attribute-array rt))
          (attribute-hash-table (character-attribute-hash-table rt)))
@@ -536,11 +549,8 @@ standard Lisp readtable when NIL."
 (defvar *read-buffer*)
 
 ;; A list of available TOKEN-BUFs
-;; Should need no toplevel binding if multi-threaded,
-;; but doesn't really matter, as INITIAL-THREAD-FUNCTION-TRAMPOLINE
-;; rebinds to NIL.
 (declaim (type (or null token-buf) *token-buf-pool*))
-(defvar *token-buf-pool* nil)
+(!define-thread-local *token-buf-pool* nil)
 
 (defun reset-read-buffer (buffer)
   ;; Turn BUFFER into an empty read buffer.
@@ -555,7 +565,7 @@ standard Lisp readtable when NIL."
 (defun ouch-read-buffer (char buffer)
   ;; When buffer overflow
   (let ((op (token-buf-fill-ptr buffer)))
-    (declare (optimize (sb!c::insert-array-bounds-checks 0)))
+    (declare (optimize (sb-c::insert-array-bounds-checks 0)))
     (when (>= op (length (token-buf-string buffer)))
     ;; an out-of-line call for the uncommon case avoids bloat.
     ;; Size should be doubled.
@@ -578,7 +588,7 @@ standard Lisp readtable when NIL."
 ;; Retun the next character from the buffered token, or NIL.
 (declaim (maybe-inline token-buf-getchar))
 (defun token-buf-getchar (b)
-  (declare (optimize (sb!c::insert-array-bounds-checks 0)))
+  (declare (optimize (sb-c::insert-array-bounds-checks 0)))
   (let ((i (token-buf-cursor (truly-the token-buf b))))
     (and (< i (token-buf-fill-ptr b))
          (prog1 (elt (token-buf-string b) i)
@@ -603,7 +613,7 @@ standard Lisp readtable when NIL."
   (named-let free ((buffer chain))
     ;; If 'adjustable-string' was displaced to 'string',
     ;; adjust it back down to allow GC of the abnormally large string.
-    (unless (eq (%array-data-vector (token-buf-adjustable-string buffer))
+    (unless (eq (%array-data (token-buf-adjustable-string buffer))
                 (token-buf-initial-string buffer))
       (adjust-array (token-buf-adjustable-string buffer) '(0)
                     :displaced-to (token-buf-initial-string buffer)))
@@ -668,14 +678,14 @@ standard Lisp readtable when NIL."
 ;;; On the other hand, it's probably very very seldom a problem in practice.
 ;;; On the third hand, it might be just as easy to use a hash table,
 ;;; so maybe we should. -- WHN 19991202
-(defvar *sharp-equal* ())
+(defvar *sharp-equal*)
 
 (declaim (ftype (sfunction (t t) (values bit t)) read-maybe-nothing))
 
 ;;; Like READ-PRESERVING-WHITESPACE, but doesn't check the read buffer
 ;;; for being set up properly.
 (defun %read-preserving-whitespace (stream eof-error-p eof-value recursive-p)
-  (declare (optimize (sb!c::check-tag-existence 0)))
+  (declare (optimize (sb-c::check-tag-existence 0)))
   (if recursive-p
       ;; a loop for repeating when a macro returns nothing
       (let* ((tracking-p (form-tracking-stream-p stream))
@@ -717,7 +727,6 @@ standard Lisp readtable when NIL."
                                              (eof-error-p t)
                                              (eof-value nil)
                                              (recursive-p nil))
-  #!+sb-doc
   "Read from STREAM and return the value read, preserving any whitespace
    that followed the object."
   (declare (explicit-check))
@@ -729,30 +738,27 @@ standard Lisp readtable when NIL."
 ;;; for functions that want comments to return so that they can look
 ;;; past them. CHAR must not be whitespace.
 (defun read-maybe-nothing (stream char)
-  (truly-the
-   (values bit t) ; avoid a type-check. M-V-CALL is lame
-   (multiple-value-call
-       (lambda (stream start-pos &optional (result nil supplied-p) &rest junk)
-         (declare (ignore junk)) ; is this ANSI-specified?
-         (when (and supplied-p start-pos)
-           (funcall (form-tracking-stream-observer stream)
-                    start-pos
-                    (form-tracking-stream-input-char-pos stream) result))
-         (values (if supplied-p 1 0) result))
-     ;; KLUDGE: not capturing anything in the lambda avoids closure consing
-     stream
-     (and (form-tracking-stream-p stream)
-          ;; Subtract 1 because the position points _after_ CHAR.
-          (1- (form-tracking-stream-input-char-pos stream)))
-     (funcall (!cmt-entry-to-function
-               (get-raw-cmt-entry char *readtable*) #'read-token)
-              stream char))))
+  (multiple-value-call
+      (lambda (stream start-pos &optional (result nil supplied-p) &rest junk)
+        (declare (ignore junk))         ; is this ANSI-specified?
+        (when (and supplied-p start-pos)
+          (funcall (form-tracking-stream-observer stream)
+                   start-pos
+                   (form-tracking-stream-input-char-pos stream) result))
+        (values (if supplied-p 1 0) result))
+    ;; KLUDGE: not capturing anything in the lambda avoids closure consing
+    stream
+    (and (form-tracking-stream-p stream)
+         ;; Subtract 1 because the position points _after_ CHAR.
+         (1- (form-tracking-stream-input-char-pos stream)))
+    (funcall (!cmt-entry-to-function
+              (get-raw-cmt-entry char *readtable*) #'read-token)
+             stream char)))
 
 (defun read (&optional (stream *standard-input*)
                        (eof-error-p t)
                        (eof-value nil)
                        (recursive-p nil))
-  #!+sb-doc
   "Read the next Lisp value from STREAM, and return it."
   (declare (explicit-check))
   (check-for-recursive-read stream recursive-p 'read)
@@ -786,10 +792,10 @@ standard Lisp readtable when NIL."
         #'(lambda (decoding-error)
             (declare (ignorable decoding-error))
             (style-warn
-             'sb!kernel::character-decoding-error-in-macro-char-comment
+             'sb-kernel::character-decoding-error-in-macro-char-comment
              :position (file-position stream) :stream stream)
             (invoke-restart 'attempt-resync))))
-    (let ((stream (in-synonym-of stream)))
+    (let ((stream (in-stream-from-designator stream)))
       (if (ansi-stream-p stream)
           (prepare-for-fast-read-char stream
            (loop (let ((char (fast-read-char nil +EOF+)))
@@ -859,7 +865,6 @@ standard Lisp readtable when NIL."
   (defun read-delimited-list (endchar &optional
                                       (input-stream *standard-input*)
                                       recursive-p)
-  #!+sb-doc
   "Read Lisp values from INPUT-STREAM until the next character after a
    value's representation is ENDCHAR, and return the objects as a list."
     (declare (explicit-check))
@@ -893,21 +898,6 @@ standard Lisp readtable when NIL."
               (simple-reader-error
                stream "More than one object follows . in list.")))))))
 
-;;; Whether it is permissible to read strings as base-string
-;;; if no extended-chars are present. The system itself prefers this, but
-;;; otherwise it is a contentious issue. We don't (by default) use base-strings,
-;;; so that people can dubiously write:
-;;;   (SETF (CHAR (READ-STRING S) 0) #\PILE_OF_POO),
-;;; which is stupid because it makes an assumption about what READ does.
-#!+sb-unicode
-(defvar *read-prefer-base-string* t)
-(eval-when (:compile-toplevel :execute)
-  (sb!xc:defmacro token-elt-type (flag)
-    (declare (ignorable flag))
-    `(if (and ,flag #!+sb-unicode *read-prefer-base-string*)
-         'base-char
-         'character)))
-
 (defun read-string (stream closech)
   ;; This accumulates chars until it sees same char that invoked it.
   ;; We avoid copying any given input character more than twice-
@@ -920,7 +910,7 @@ standard Lisp readtable when NIL."
   (declare (character closech))
   (macrolet ((scan (read-a-char eofp &optional finish)
                `(loop (let ((char ,read-a-char))
-                        (declare (optimize (sb!c::insert-array-bounds-checks 0)))
+                        (declare (optimize (sb-c::insert-array-bounds-checks 0)))
                         (cond (,eofp (error 'end-of-file :stream stream))
                               ((eql char closech)
                                (return ,finish))
@@ -935,13 +925,13 @@ standard Lisp readtable when NIL."
                                   buf (make-array lim :element-type 'character)))
                           (setq ptr 0))
                         (setf (schar buf ptr) (truly-the character char))
-                        #!+sb-unicode ; BASE-CHAR-P does not exist if not
+                        #+sb-unicode ; BASE-CHAR-P does not exist if not
                         (unless (base-char-p char) (setq only-base-chars nil))
                         (incf ptr)))))
     (let* ((token-buf *read-buffer*)
            (buf (token-buf-string token-buf))
            (rt *readtable*)
-           (stream (in-synonym-of stream))
+           (stream (in-stream-from-designator stream))
            (suppress *read-suppress*)
            (lim (length buf))
            (ptr 0)
@@ -959,7 +949,9 @@ standard Lisp readtable when NIL."
           (let* ((sum (loop for buf in chain sum (length buf)))
                  (result
                   (make-array (+ sum ptr)
-                              :element-type (token-elt-type only-base-chars))))
+                              :element-type (if only-base-chars
+                                                (%readtable-string-preference rt)
+                                                'character))))
             (setq ptr sum)
             ;; Now work backwards from the end
             (replace result buf :start1 ptr)
@@ -1093,19 +1085,16 @@ standard Lisp readtable when NIL."
                      +char-attr-constituent-digit+)
                  +char-attr-constituent-decimal-digit+))
             ((= att +char-attr-invalid+)
-             (simple-reader-error stream "invalid constituent"))
+             (simple-reader-error stream "invalid constituent: ~s" char))
             (t att))))))
 
 ;;;; token fetching
 
 (defvar *read-suppress* nil
-  #!+sb-doc
   "Suppress most interpreting in the reader when T.")
 
 (defvar *read-base* 10
-  #!+sb-doc
   "the radix that Lisp reads numbers in")
-(declaim (type (integer 2 36) *read-base*))
 
 ;;; Normalize TOKEN-BUF to NFKC, returning a new TOKEN-BUF and the
 ;;; COLON value
@@ -1120,7 +1109,7 @@ standard Lisp readtable when NIL."
     (reset-read-buffer token-buf)
     (macrolet ((clear-str-to-normalize ()
                `(progn
-                  (loop for char across (sb!unicode:normalize-string
+                  (loop for char across (sb-unicode:normalize-string
                                          (subseq str-to-normalize 0 normalize-ptr)
                                          :nfkc) do
                        (ouch-read-buffer char token-buf))
@@ -1152,7 +1141,7 @@ standard Lisp readtable when NIL."
      ((and (zerop (length escapes)) (eq case :upcase))
       (let ((buffer (token-buf-string token-buf)))
         (dotimes (i (token-buf-fill-ptr token-buf))
-          (declare (optimize (sb!c::insert-array-bounds-checks 0)))
+          (declare (optimize (sb-c::insert-array-bounds-checks 0)))
           (setf (schar buffer i) (char-upcase (schar buffer i))))))
      ((eq case :preserve))
      (t
@@ -1163,7 +1152,7 @@ standard Lisp readtable when NIL."
                                   -1 (vector-pop escapes))))
                         ((minusp i))
                       (declare (fixnum i)
-                               (optimize (sb!c::insert-array-bounds-checks 0)))
+                               (optimize (sb-c::insert-array-bounds-checks 0)))
                       (if (< esc i)
                           (let ((ch (schar buffer i)))
                             ,@body)
@@ -1212,7 +1201,6 @@ standard Lisp readtable when NIL."
                       :format-arguments (list package-designator)))))))
 
 (defun read-token (stream firstchar)
-  #!+sb-doc
   "Default readmacro function. Handles numbers, symbols, and SBCL's
 extended <package-name>::<form-in-package> syntax."
   ;; Check explicitly whether FIRSTCHAR has an entry for
@@ -1255,7 +1243,7 @@ extended <package-name>::<form-in-package> syntax."
         (#.+char-attr-package-delimiter+ (go COLON))
         (#.+char-attr-multiple-escape+ (go MULT-ESCAPE))
         (#.+char-attr-invalid+ (simple-reader-error stream
-                                                    "invalid constituent"))
+                                                    "invalid constituent: ~s" char))
         ;; can't have eof, whitespace, or terminating macro as first char!
         (t (go SYMBOL)))
      SIGN ; saw "sign"
@@ -1455,7 +1443,7 @@ extended <package-name>::<form-in-package> syntax."
         (#.+char-attr-package-delimiter+ (go COLON))
         (t (go SYMBOL)))
      SYMBOL ; not a dot, dots, or number
-      (let ((stream (in-synonym-of stream)))
+      (let ((stream (in-stream-from-designator stream)))
         (macrolet
            ((scan (read-a-char &optional finish)
              `(prog ()
@@ -1575,7 +1563,10 @@ extended <package-name>::<form-in-package> syntax."
         (return (%intern (token-buf-string buf)
                          (token-buf-fill-ptr buf)
                          pkg
-                         (token-elt-type (token-buf-only-base-chars buf)))))))))
+                         (if (token-buf-only-base-chars buf)
+                             (%readtable-symbol-preference rt)
+                             'character)
+                         nil)))))))
 
 ;;; For semi-external use: Return 3 values: the token-buf,
 ;;; a flag for whether there was an escape char, and the position of
@@ -1607,7 +1598,7 @@ extended <package-name>::<form-in-package> syntax."
         ((> base 36) a)
       (do ((total (1- base) (+ (* total base) (1- base)))
            (n-digits 0 (1+ n-digits)))
-          ((sb!xc:typep total 'bignum)
+          ((sb-xc:typep total 'bignum)
            (setf (aref a (- base 2)) n-digits))
         ;; empty DO body
         )))
@@ -1620,14 +1611,14 @@ extended <package-name>::<form-in-package> syntax."
            (d (char (write-to-string (1- base) :base base) 0))
            (string (make-string (1+ n-digits) :initial-element d))) ; 1 extra
       (assert (not (typep (parse-integer string :radix base)
-                          `(unsigned-byte ,sb!vm:n-positive-fixnum-bits))))
+                          `(unsigned-byte ,sb-vm:n-positive-fixnum-bits))))
       (assert (typep (parse-integer string :end n-digits :radix base)
-                     `(unsigned-byte ,sb!vm:n-positive-fixnum-bits))))))
+                     `(unsigned-byte ,sb-vm:n-positive-fixnum-bits))))))
 
 (defmacro !setq-optional-leading-sign (sign-flag token-buf rewind)
   ;; guaranteed to have at least one character in buffer at the start
   ;; or immediately following [ESFDL] marker depending on 'rewind' flag.
-  `(locally (declare (optimize (sb!c::insert-array-bounds-checks 0)))
+  `(locally (declare (optimize (sb-c::insert-array-bounds-checks 0)))
      (,(if rewind 'setf 'incf)
        (token-buf-cursor ,token-buf)
        (case (elt (token-buf-string ,token-buf)
@@ -1637,7 +1628,6 @@ extended <package-name>::<form-in-package> syntax."
          (t   0)))))
 
 (defun make-integer (&optional (base *read-base*))
-  #!+sb-doc
   "Minimizes bignum-fixnum multiplies by reading a 'safe' number of digits,
   then multiplying by a power of the base and adding."
   (declare ((integer 2 36) base)
@@ -1677,15 +1667,14 @@ extended <package-name>::<form-in-package> syntax."
        (setq result (+ (* result base-power) acc))))))
 
 (defun truncate-exponent (exponent number divisor)
-  #!+sb-doc
   "Truncate exponent if it's too large for a float"
   ;; Work with base-2 logarithms to avoid conversions to floats,
   ;; and convert to base-10 conservatively at the end.
   ;; Use the least positive float, because denormalized exponent
   ;; can be larger than normalized.
   (let* ((max-exponent
-          #!-long-float
-          (+ sb!vm:double-float-digits sb!vm:double-float-bias))
+          #-long-float
+          (+ sb-vm:double-float-digits sb-vm:double-float-bias))
          (number-magnitude (integer-length number))
          (divisor-magnitude (1- (integer-length divisor)))
          (magnitude (- number-magnitude divisor-magnitude)))
@@ -1752,7 +1741,7 @@ extended <package-name>::<form-in-package> syntax."
 (defun make-float-aux (number divisor float-format stream)
   (handler-case
       (coerce (/ number divisor) float-format)
-    (type-error (c)
+    (arithmetic-error (c)
       (error 'reader-impossible-number-error
              :error c :stream stream
              :format-control "failed to build float from ~a"
@@ -1791,6 +1780,14 @@ extended <package-name>::<form-in-package> syntax."
   (declare (optimize allow-non-returning-tail-call))
   (declare (ignore ignore))
   (if *read-suppress*
+      ;; This seems dubious. For comparison's sake, other implementations
+      ;; will signal an error if the character is not a defined macro.
+      ;; Test case: (read-from-string "#+nope (#!+(or feat) a) b") => B and 25
+      ;; CLISP:
+      ;; *** - READ from #<INPUT STRING-INPUT-STREAM>: After #\# is #\! an undefined dispatch macro character
+      ;; CCL:
+      ;; Error: Reader error on #<STRING-INPUT-STREAM  #x3020004913AD>, near position 10, within "#+nope (#!+(or feat)":
+      ;;        Undefined character #\! in a #\# dispatch macro.
       (values)
       (simple-reader-error stream
                            "no dispatch function defined for ~S"
@@ -1834,7 +1831,6 @@ extended <package-name>::<form-in-package> syntax."
 (declare (muffle-conditions style-warning))
 (defun read-from-string (string &optional (eof-error-p t) eof-value
                                 &key (start 0) end preserve-whitespace)
-  #!+sb-doc
   "The characters of string are successively given to the lisp reader
    and the lisp object built by the reader is returned. Macro chars
    will take effect."
@@ -1845,7 +1841,6 @@ extended <package-name>::<form-in-package> syntax."
 ;;;; PARSE-INTEGER
 
 (defun parse-integer (string &key (start 0) end (radix 10) junk-allowed)
-  #!+sb-doc
   "Examine the substring of string delimited by start and end
   (default to the beginning and end of the string)  It skips over
   whitespace characters and then tries to parse an integer. The
@@ -1905,7 +1900,6 @@ extended <package-name>::<form-in-package> syntax."
 ;;;; reader initialization code
 
 (defun !reader-cold-init ()
-  (!cold-init-constituent-trait-table)
   (!cold-init-standard-readtable))
 
 (defmethod print-object ((readtable readtable) stream)
