@@ -48,7 +48,6 @@
   (def %byte-blt (src src-start dst dst-start dst-end))
   (def get-header-data)
   (def set-header-data (x val))
-  (def get-closure-length)
   (def widetag-of)
   (def %other-pointer-widetag)
   (def vector-sap)
@@ -68,6 +67,9 @@
   (def closurep)
   (def %closure-fun)
   (def %closure-index-ref (closure index))
+  (def fdefn-name)
+  (def fdefn-fun)
+  (def fdefn-makunbound)
   (def sb-c::vector-length)
   (def make-array-header (type rank))
   (def code-instructions)
@@ -77,15 +79,35 @@
   (def %set-vector-raw-bits (object offset value))
   (def single-float-bits)
   (def double-float-high-bits)
+  #+64-bit
+  (def double-float-bits)
   (def double-float-low-bits)
   (def value-cell-ref)
   (def %caller-frame ())
   (def %caller-pc ())
-  (def %code-code-size)
   (def %code-debug-info)
   #+(or x86 immobile-space) (def sb-vm::%code-fixups)
+
+  ;; instances
+  (def %make-instance) ; Allocate a new instance with X data slots.
+  (def %instance-length) ; Given an instance, return its length.
+  (def %instance-layout)
+  (def %set-instance-layout (instance new-value))
+  ; (def %instance-ref (instance index)) ; defined in 'target-defstruct'
+  (def %instance-set (instance index new-value))
+  ;; funcallable instances
+  (def %make-funcallable-instance)
   (def %funcallable-instance-layout)
-  (def %set-funcallable-instance-layout (x new-value))
+  (def %set-funcallable-instance-layout (fin new-value))
+  (def %funcallable-instance-fun)
+  (def (setf %funcallable-instance-fun) (fin new-value))
+  (def %funcallable-instance-info (fin i))
+  (def %set-funcallable-instance-info (fin i new-value))
+
+  #+compare-and-swap-vops
+  (def* (%array-atomic-incf/word (array index diff))
+        (%raw-instance-atomic-incf/word (instance index diff)))
+
   #+sb-simd-pack
   (def* (%make-simd-pack (tag low high))
         (%make-simd-pack-single (x y z w))
@@ -94,27 +116,33 @@
         (%simd-pack-tag)
         (%simd-pack-low)
         (%simd-pack-high))
+  #+sb-simd-pack-256
+  (def* (%make-simd-pack-256 (tag p0 p1 p2 p3))
+        (%make-simd-pack-256-single (a b c d e f g h))
+        (%make-simd-pack-256-double (a b c d))
+        (%make-simd-pack-256-ub64 (a b c d))
+        (%simd-pack-256-tag)
+        (%simd-pack-256-0)
+        (%simd-pack-256-1)
+        (%simd-pack-256-2)
+        (%simd-pack-256-3))
   #+sb-thread (def sb-vm::current-thread-offset-sap)
   (def current-sp ())
   (def current-fp ())
   (def stack-ref (s n))
   (def %set-stack-ref (s n value))
   (def fun-code-header)
+  (def sb-vm::symbol-extra)
   #-(or x86 x86-64) (def lra-code-header)
   (def %make-lisp-obj)
-  (def get-lisp-obj-address)
-  )
+  (def get-lisp-obj-address))
 
 (defun spin-loop-hint ()
   "Hints the processor that the current thread is spin-looping."
   (spin-loop-hint))
 
-;;; Same criterion as decides whether WIDETAGS-FROM-UNION-TYPE
-;;; should transform into a call to this.
-#.(let ((info (info :function :info '%other-pointer-subtype-p)))
-    (if (sb-c::fun-info-templates info)
-        '(defun %other-pointer-subtype-p (x choices)
-           (and (%other-pointer-p x)
-                (member (%other-pointer-widetag x) choices)
-                t))
-        (values)))
+
+(defun %other-pointer-subtype-p (x choices)
+  (and (%other-pointer-p x)
+       (member (%other-pointer-widetag x) choices)
+       t))
