@@ -11,12 +11,11 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!FASL")
+(in-package "SB-FASL")
 
 (defvar *load-source-default-type* "lisp"
   "The source file types which LOAD looks for by default.")
 
-(declaim (type (or pathname null) *load-truename* *load-pathname*))
 (defvar *load-truename* nil
   "the TRUENAME of the file that LOAD is currently loading")
 (defvar *load-pathname* nil
@@ -24,6 +23,9 @@
 
 ;;;; LOAD-AS-SOURCE
 
+;;; something not EQ to anything we might legitimately READ
+(define-load-time-global *eof-object* (make-symbol "EOF-OBJECT"))
+
 ;;; Load a text stream.  (Note that load-as-fasl is in another file.)
 ;; We'd like, when entering the debugger as a result of an EVAL error,
 ;; that the condition be annotated with the stream position.
@@ -66,21 +68,24 @@
                          (eval-tlf form index)))
                    (return))))))
         (if pathname
-            (let* ((info (sb!c::make-file-source-info
+            (let* ((info (sb-c::make-file-source-info
                           pathname (stream-external-format stream)))
-                   (sb!c::*source-info* info))
-              (setf (sb!c::source-info-stream info) stream)
-              (sb!c::do-forms-from-info ((form current-index) info
-                                         'sb!c::input-error-in-load)
-                (sb!c::with-source-paths
-                  (sb!c::find-source-paths form current-index)
+                   (sb-c::*source-info* info))
+              (setf (sb-c::source-info-stream info) stream)
+              (sb-c::do-forms-from-info ((form current-index) info
+                                         'sb-c::input-error-in-load)
+                (sb-c::with-source-paths
+                  (sb-c::find-source-paths form current-index)
                   (eval-form form current-index))))
-            (let ((sb!c::*source-info* nil))
-              (do ((form (read stream nil *eof-object*)
-                         (read stream nil *eof-object*)))
-                  ((eq form *eof-object*))
-                (sb!c::with-source-paths
-                  (eval-form form nil))))))))
+            (let ((sb-c::*source-info* nil))
+              (loop for form =
+                    (handler-case (read stream nil *eof-object*)
+                      ((or reader-error end-of-file) (c)
+                        (error 'sb-c::input-error-in-load :stream stream
+                                                          :condition c)))
+                    until (eq form *eof-object*)
+                    do (sb-c::with-source-paths
+                         (eval-form form nil))))))))
   t)
 
 ;;;; LOAD itself
@@ -122,36 +127,74 @@
                   (*load-depth* (1+ *load-depth*)))
              (funcall function)))
 
+;;; Returns T if the stream is a binary input stream with a FASL header.
+(defun fasl-header-p (stream &key errorp)
+  (unless (and (member (stream-element-type stream) '(character base-char))
+               ;; give up if it's not a file stream, or it's an
+               ;; fd-stream but it's either not bivalent or not
+               ;; seekable (doesn't really have a file)
+               (or (not (typep stream 'file-stream))
+                   (and (typep stream 'fd-stream)
+                        (or (not (sb-impl::fd-stream-bivalent-p stream))
+                            (not (sb-impl::fd-stream-file stream))
+                            (neq (sb-impl::fd-stream-fd-type stream) :regular)))))
+    (let ((p (file-position stream)))
+      (when p ; Can't do it non-destructively on non-seekable streams.
+        (unwind-protect
+             (let* ((header *fasl-header-string-start-string*)
+                    (buffer (make-array (length header) :element-type '(unsigned-byte 8)))
+                    (n 0))
+               (flet ((scan ()
+                        (maybe-skip-shebang-line stream)
+                        (setf n (read-sequence buffer stream))))
+                 (if errorp
+                     (scan)
+                     (or (ignore-errors (scan))
+                         ;; no a binary input stream
+                         (return-from fasl-header-p nil))))
+               (cond ((not (mismatch buffer header
+                                     :test #'(lambda (code char) (= code (char-code char))))))
+                     ((zerop n)
+                      ;; Immediate EOF is valid -- we want to match what
+                      ;; CHECK-FASL-HEADER does...
+                      nil)
+                     (errorp
+                      (error 'fasl-header-missing
+                             :stream stream
+                             :fhsss buffer
+                             :expected header))))
+          (file-position stream p))))))
+
 (defun load (pathspec &key (verbose *load-verbose*) (print *load-print*)
-             (if-does-not-exist t) (external-format :default))
+                           (if-does-not-exist t) (external-format :default))
   "Load the file given by FILESPEC into the Lisp environment, returning
    T on success."
   (flet ((load-stream (stream faslp)
            (when (and (fd-stream-p stream)
-                      (eq (sb!impl::fd-stream-fd-type stream) :directory))
+                      (eq (sb-impl::fd-stream-fd-type stream) :directory))
              (error 'simple-file-error
                     :pathname (pathname stream)
                     :format-control
                     "Can't LOAD a directory: ~s."
                     :format-arguments (list (pathname stream))))
            (dx-flet ((thunk ()
-                       (let ( ;; Bindings required by ANSI.
-                             (*readtable* *readtable*)
-                             (*package* (sane-package))
-                             ;; KLUDGE: I can't find in the ANSI spec where it says
-                             ;; that DECLAIM/PROCLAIM of optimization policy should
-                             ;; have file scope. CMU CL did this, and it seems
-                             ;; reasonable, but it might not be right; after all,
-                             ;; things like (PROCLAIM '(TYPE ..)) don't have file
-                             ;; scope, and I can't find anything under PROCLAIM or
-                             ;; COMPILE-FILE or LOAD or OPTIMIZE which justifies this
-                             ;; behavior. Hmm. -- WHN 2001-04-06
-                             (sb!c::*policy* sb!c::*policy*))
-                         (if faslp
-                             (load-as-fasl stream verbose print)
-                             (sb!c:with-compiler-error-resignalling
-                                 (load-as-source stream :verbose verbose
-                                                        :print print))))))
+                            (let (;; Bindings required by ANSI.
+                                  (*readtable* *readtable*)
+                                  (*package* (sane-package))
+                                  ;; KLUDGE: I can't find in the ANSI spec where it says
+                                  ;; that DECLAIM/PROCLAIM of optimization policy should
+                                  ;; have file scope. CMU CL did this, and it seems
+                                  ;; reasonable, but it might not be right; after all,
+                                  ;; things like (PROCLAIM '(TYPE ..)) don't have file
+                                  ;; scope, and I can't find anything under PROCLAIM or
+                                  ;; COMPILE-FILE or LOAD or OPTIMIZE which justifies this
+                                  ;; behavior. Hmm. -- WHN 2001-04-06
+                                  (sb-c::*policy* sb-c::*policy*))
+                              (if faslp
+                                  (load-as-fasl stream verbose print)
+                                  (sb-c:with-compiler-error-resignalling
+                                    (load-as-source stream :verbose verbose
+                                                           :print print))))))
              (call-with-load-bindings #'thunk stream))))
 
     ;; Case 1: stream.
@@ -181,17 +224,17 @@
                           (return-from load nil))))
         (let* ((real (probe-file stream))
                (should-be-fasl-p
-                (and real (string-equal (pathname-type real) *fasl-file-type*))))
+                 (and real (string-equal (pathname-type real) *fasl-file-type*))))
           ;; Don't allow empty .fasls, and assume other empty files
           ;; are source files.
-          (when (and (or should-be-fasl-p (not (eql 0 (file-length stream))))
+          (cond ((or (and should-be-fasl-p (eql 0 (file-length stream)))
                      (fasl-header-p stream :errorp should-be-fasl-p))
-            (return-from load (load-stream stream t)))))
-
-      ;; Case 3: Open using the given external format, process as source.
-      (with-open-file (stream pathname :external-format external-format
-                              :class 'form-tracking-stream)
-        (load-stream stream nil)))))
+                 (load-stream stream t))
+                (t
+                 ;; Case 3: Open using the given external format, process as source.
+                 (with-open-file (stream pathname :external-format external-format
+                                                  :class 'form-tracking-stream)
+                   (load-stream stream nil)))))))))
 
 ;; This implements the defaulting SBCL seems to have inherited from
 ;; CMU.  This routine does not try to perform any loading; all it does
@@ -231,54 +274,40 @@
           (defaulted-fasl-truename defaulted-fasl-pathname)
           (defaulted-source-truename defaulted-source-pathname))))
 
-;;; Load a code object. BOX-NUM objects are popped off the stack for
-;;; the boxed storage section, then CODE-LENGTH bytes of code are read in.
-(defun load-code (nfuns box-num code-length stack ptr fasl-input)
-  (declare (fixnum box-num code-length))
-  (declare (simple-vector stack) (type index ptr))
-  (let* ((debug-info-index (+ ptr box-num))
-         (immobile-p (svref stack (1+ debug-info-index)))
-         (code (sb!c:allocate-code-object immobile-p box-num code-length)))
-    (setf (%code-debug-info code) (svref stack debug-info-index))
-    (loop for i of-type index from sb!vm:code-constants-offset
-          for j of-type index from ptr below debug-info-index
-          do (setf (code-header-ref code i) (svref stack j)))
-    (with-pinned-objects (code)
-      (read-n-bytes (%fasl-input-stream fasl-input)
-                    (code-instructions code) 0 code-length)
-      (sb!c::set-code-entrypoints
-       code (loop repeat nfuns collect (read-varint-arg fasl-input)))
-      (sb!c::apply-fasl-fixups stack code))
-    code))
-
 ;;;; linkage fixups
 
+;;; Lisp assembler routines are named by Lisp symbols, not strings,
+;;; and so can be compared by EQ.
+(define-load-time-global *assembler-routines* nil)
+(declaim (code-component *assembler-routines*))
+
+(defun calc-asm-routine-bounds ()
+  (loop for v being each hash-value of (car (%code-debug-info *assembler-routines*))
+        minimize (car v) into min
+        maximize (cadr v) into max
+        ;; min/max are inclusive byte ranges, but return the answer
+        ;; using standard convention of exclusive upper bound.
+        finally (return (values min (1+ max)))))
+
 ;;; how we learn about assembler routines at startup
 (defvar *!initial-assembler-routines*)
 
 (defun get-asm-routine (name &optional indirect &aux (code *assembler-routines*))
-  (awhen (gethash (the symbol name) (car (%code-debug-info code)))
+  (awhen (the list (gethash (the symbol name) (car (%code-debug-info code))))
     (sap-int (sap+ (code-instructions code)
                    (if indirect
                        ;; Return the address containing the routine address
-                       (ash (cddr it) sb!vm:word-shift)
+                       (ash (cddr it) sb-vm:word-shift)
                        ;; Return the routine address itself
                        (car it))))))
 
 (defun !loader-cold-init ()
   (let* ((code *assembler-routines*)
-         (size (%code-code-size code))
+         (size (%code-text-size code))
          (vector (the simple-vector *!initial-assembler-routines*))
          (count (length vector))
          (ht (make-hash-table :test 'eq)))
-    ;; code-debug-info stores the name->addr hashtable, but readonly
-    ;; space can't point to dynamic space. indirect through a static cons
-    (setf (%code-debug-info code)
-          (rplaca (let ((ptr (sap-int sb!vm:*static-space-free-pointer*)))
-                    (setf sb!vm:*static-space-free-pointer*
-                          (int-sap (+ ptr (* sb!vm:n-word-bytes 2))))
-                    (%make-lisp-obj (logior ptr sb!vm:list-pointer-lowtag)))
-                  ht))
+    (rplaca (%code-debug-info code) ht)
     (dotimes (i count)
       (destructuring-bind (name . offset) (svref vector i)
         (let ((next-offset (if (< (1+ i) count) (cdr (svref vector (1+ i))) size)))
@@ -287,12 +316,12 @@
           (setf (gethash name ht) (list* offset (1- next-offset) i)))))))
 
 (defun !warm-load (file)
-  (restart-case (let ((sb!c::*source-namestring*
+  (restart-case (let ((sb-c::*source-namestring*
                        (format nil "SYS:~A" (substitute #\; #\/ file))))
                   (load file))
-    (abort ()
+    (abort-build ()
       :report "Abort building SBCL."
-      (sb!ext:exit :code 1))))
+      (sb-ext:exit :code 1))))
 
 ;;; Remember where cold artifacts went, and put the warm ones there too
 ;;; because it looks nicer not to scatter them throughout the source tree.
