@@ -10,18 +10,25 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!IMPL")
+(in-package "SB-IMPL")
 
 ;;; various environment inquiries
 
-(!defvar *features* '#.sb!xc:*features*
+(defparameter *features*
+   ;; The :SB-XC keyword indicates the build phase and is not intended
+   ;; to persist into the target.
+   ;; GCC_TLS is not a Lisp feature- it's just freeloading off the means
+   ;; by which additional #defines get into "genesis/config.h".
+   ;; Literally nothing except C code tests for it.
+   '#.(remove-if (lambda (x) (member x '(:sb-xc :gcc-tls)))
+                 sb-xc:*features*)
   "a list of symbols that describe features provided by the
    implementation")
 
 (defun machine-instance ()
   "Return a string giving the name of the local machine."
-  #!+win32 (sb!win32::get-computer-name)
-  #!-win32 (truly-the simple-string (sb!unix:unix-gethostname)))
+  #+win32 (sb-win32::get-computer-name)
+  #-win32 (truly-the simple-string (sb-unix:unix-gethostname)))
 
 (declaim (type (or null string) *machine-version*))
 (defvar *machine-version*)
@@ -40,9 +47,9 @@ are running on, or NIL if we can't find any useful information."
 ;;; for Conforming Implementations" it is kosher to add a SETF function for
 ;;; a symbol in COMMON-LISP..
 (declaim (type (or null string) *short-site-name* *long-site-name*))
-(defvar *short-site-name* nil
+(define-load-time-global *short-site-name* nil
   "The value of SHORT-SITE-NAME.")
-(defvar *long-site-name* nil
+(define-load-time-global *long-site-name* nil
   "The value of LONG-SITE-NAME.")
 (defun short-site-name ()
   "Return a string with the abbreviated site name, or NIL if not known."
@@ -129,10 +136,11 @@ the file system."
 
 ;;; DEFmumble helpers
 
-(defun %defglobal (name value boundp doc docp source-location)
-  (%compiler-defglobal name :always-bound value (not boundp))
+(defun %defglobal (name value source-location &optional (doc nil docp))
+  (%compiler-defglobal name :always-bound
+                       (not (unbound-marker-p value)) value)
   (when docp
-    (setf (fdocumentation name 'variable) doc))
+    (setf (documentation name 'variable) doc))
   (when source-location
     (setf (info :source-location :variable name) source-location))
   name)
@@ -141,7 +149,7 @@ the file system."
   (%compiler-defvar var)
   (set var val)
   (when docp
-    (setf (fdocumentation var 'variable) doc))
+    (setf (documentation var 'variable) doc))
   (when source-location
     (setf (info :source-location :variable var) source-location))
   var)
@@ -152,28 +160,30 @@ the file system."
              (not (boundp var)))
     (set var val))
   (when docp
-    (setf (fdocumentation var 'variable) doc))
+    (setf (documentation var 'variable) doc))
   (when source-location
     (setf (info :source-location :variable var) source-location))
   var)
 
-(defun %defun (name def &optional inline-lambda dxable-args)
+(defun %defun (name def &optional inline-lambda extra-info)
   (declare (type function def))
   ;; should've been checked by DEFMACRO DEFUN
   (aver (legal-fun-name-p name))
-  (sb!c:%compiler-defun name nil inline-lambda dxable-args)
+  ;; If a warning handler decides to disallow this redefinition
+  ;; by nonlocally exiting, then we'll skip the rest of this stuff.
   (when (fboundp name)
     (warn 'redefinition-with-defun :name name :new-function def))
+  (sb-c:%compiler-defun name nil inline-lambda extra-info)
   (setf (fdefinition name) def)
   ;; %COMPILER-DEFUN doesn't do this except at compile-time, when it
   ;; also checks package locks. By doing this here we let (SETF
   ;; FDEFINITION) do the load-time package lock checking before
   ;; we frob any existing inline expansions.
-  (sb!c::%set-inline-expansion name nil inline-lambda dxable-args)
-  (sb!c::note-name-defined name :function)
+  (sb-c::%set-inline-expansion name nil inline-lambda extra-info)
+  (sb-c::note-name-defined name :function)
   name)
 
-(in-package "SB!C")
+(in-package "SB-C")
 
 (defun real-function-name (name)
   ;; Resolve the actual name of the function named by NAME
@@ -240,3 +250,9 @@ version 1[.0.0...] or greater."
                or later is required)."
               (lisp-implementation-version)
               subversions))))
+
+(defparameter sb-pcl::*!docstrings* nil)
+(defun (setf documentation) (string name doc-type)
+  (declare (type (or null string) string))
+  (push (list string name doc-type) sb-pcl::*!docstrings*)
+  string)
