@@ -12,7 +12,15 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!IMPL")
+(in-package "SB-IMPL")
+
+;;; Unlike most other "Stub functions" that never called called except
+;;; by the interpreter, these two do get called, by MAKE-UNPORTABLE-FLOAT
+;;; due to the NOTINLINE declaration.
+#-sb-xc-host
+(progn
+  (defun make-single-float (x) (make-single-float x))
+  (defun make-double-float (hi lo) (make-double-float hi lo)))
 
 (defun make-unportable-float (name)
   (flet ((opaque-identity (x) x))
@@ -20,9 +28,9 @@
     (declare (notinline opaque-identity make-single-float make-double-float))
     (ecase name
       (:single-float-negative-zero (make-single-float
-				    (opaque-identity #x-80000000)))
+                                    (opaque-identity #x-80000000)))
       (:double-float-negative-zero (make-double-float
-				    (opaque-identity #x-80000000)
-				    (opaque-identity #x00000000)))
-      #!+long-float
+                                    (opaque-identity #x-80000000)
+                                    (opaque-identity #x00000000)))
+      #+long-float
       (:long-float-negative-zero (error "write LONG-FLOAT creation form")))))
