@@ -9,7 +9,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 (defun machine-type ()
   "Return a string describing the type of the local machine."
   "X86-64")
@@ -40,9 +40,9 @@
         (:relative
          ;; Fixup is the actual address wanted.
          ;; Replace word with value to add to that loc to get there.
-         ;; In the #!-immobile-code case, there's nothing to assert.
+         ;; In the #-immobile-code case, there's nothing to assert.
          ;; Relative fixups pretty much can't happen.
-         #!+immobile-code
+         #+immobile-code
          (unless (immobile-space-obj-p code)
            (error "Can't compute fixup relative to movable object ~S" code))
          (setf (signed-sap-ref-32 sap offset)
@@ -58,7 +58,7 @@
   ;;  (1) Call fixups occur in both :RELATIVE and :ABSOLUTE kinds.
   ;;      We can ignore the :RELATIVE kind, except for foreign call.
   ;;  (2) :STATIC-CALL fixups point to immobile space, not static space.
-  #!+immobile-space
+  #+immobile-space
   (return-from fixup-code-object
     (case flavor
       ((:named-call :layout :immobile-object ; -> fixedobj subspace
@@ -72,7 +72,7 @@
        (if (eq kind :relative) :relative))))
   nil) ; non-immobile-space builds never record code fixups
 
-#!+(or darwin linux win32)
+#+(or darwin linux openbsd win32)
 (define-alien-routine ("os_context_float_register_addr" context-float-register-addr)
   (* unsigned) (context (* os-context-t)) (index int))
 
@@ -81,11 +81,11 @@
 
 (defun context-float-register (context index format)
   (declare (ignorable context index))
-  #!-(or darwin linux win32)
+  #-(or darwin linux openbsd win32)
   (progn
     (warn "stub CONTEXT-FLOAT-REGISTER")
     (coerce 0 format))
-  #!+(or darwin linux win32)
+  #+(or darwin linux openbsd win32)
   (let ((sap (alien-sap (context-float-register-addr context index))))
     (ecase format
       (single-float
@@ -101,11 +101,11 @@
 
 (defun %set-context-float-register (context index format value)
   (declare (ignorable context index format))
-  #!-(or linux win32)
+  #-(or linux win32)
   (progn
     (warn "stub %SET-CONTEXT-FLOAT-REGISTER")
     value)
-  #!+(or linux win32)
+  #+(or linux win32)
   (let ((sap (alien-sap (context-float-register-addr context index))))
     (ecase format
       (single-float
@@ -125,12 +125,12 @@
 
 ;;; Given a signal context, return the floating point modes word in
 ;;; the same format as returned by FLOATING-POINT-MODES.
-#!-linux
+#-linux
 (defun context-floating-point-modes (context)
   (declare (ignore context)) ; stub!
   (warn "stub CONTEXT-FLOATING-POINT-MODES")
   0)
-#!+linux
+#+linux
 (define-alien-routine ("os_context_fp_control" context-floating-point-modes)
     (unsigned 32)
   (context (* os-context-t)))
@@ -156,16 +156,17 @@
     (if (= trap-number invalid-arg-count-trap)
         (values #.(error-number-or-lose 'invalid-arg-count-error)
                 '(#.arg-count-sc))
-        (sb!kernel::decode-internal-error-args (sap+ pc 1) trap-number))))
+        (sb-kernel::decode-internal-error-args (sap+ pc 1) trap-number))))
 
 
-#!+immobile-code
+#+immobile-code
 (progn
+(defconstant trampoline-entry-offset n-word-bytes)
 (defun fun-immobilize (fun)
   (let ((code (truly-the (values code-component &optional)
-                         (sb!vm::alloc-immobile-trampoline))))
+                         (sb-vm::alloc-immobile-trampoline))))
     (setf (%code-debug-info code) fun)
-    (let ((sap (code-instructions code))
+    (let ((sap (sap+ (code-instructions code) trampoline-entry-offset))
           (ea (+ (logandc2 (get-lisp-obj-address code) lowtag-mask)
                  (ash code-debug-info-slot word-shift))))
       ;; For a funcallable-instance, the instruction sequence is:
@@ -187,17 +188,17 @@
                      (setf (sap-ref-32 sap 7) (logior (ash disp8 24) #x408B48))
                      11))))
         (setf (sap-ref-32 sap i) #xFD60FF))) ; JMP [RAX-3]
+    ;; It is critical that there be a trailing 'uint16' of 0 in this object
+    ;; so that CODE-N-ENTRIES reports 0.  By luck, there is exactly enough
+    ;; room in the object to hold two 0 bytes. It would be easy enough to enlarge
+    ;; by 2 words if it became necessary. The assertions makes sure we stay ok.
+    (aver (zerop (code-n-entries code)))
     code))
 
 ;;; Return T if FUN can't be called without loading RAX with its descriptor.
 ;;; This is true of any funcallable instance which is not a GF, and closures.
 (defun fun-requires-simplifying-trampoline-p (fun)
   (cond ((not (immobile-space-obj-p fun)) t) ; always
-        ((funcallable-instance-p fun)
-         ;; A funcallable-instance with no raw slots has no machine
-         ;; code within it, and thus requires an external trampoline.
-         (eql (layout-bitmap (%funcallable-instance-layout fun))
-              sb!kernel::+layout-all-tagged+))
         (t
          (closurep fun))))
 
@@ -211,64 +212,61 @@
 (defun %set-fdefn-fun (fdefn fun)
   (declare (type fdefn fdefn) (type function fun)
            (values function))
-  (unless (eql (sb!vm::fdefn-has-static-callers fdefn) 0)
-    (sb!vm::remove-static-links fdefn))
+  (unless (eql (sb-vm::fdefn-has-static-callers fdefn) 0)
+    (sb-vm::remove-static-links fdefn))
   (let ((trampoline (when (fun-requires-simplifying-trampoline-p fun)
                       (fun-immobilize fun)))) ; a newly made CODE object
     (with-pinned-objects (fdefn trampoline fun)
-      (binding* (((fun-entry-addr nop-byte)
-                  ;; The NOP-BYTE is an arbitrary value used to indicate the
-                  ;; kind of callee in the FDEFN-RAW-ADDR slot.
-                  ;; Though it should never be executed, it is a valid encoding.
-                  (if trampoline
-                      (values (sap-int (code-instructions trampoline)) #x90)
-                      (values (sap-ref-word (int-sap (get-lisp-obj-address fun))
-                                            (- (ash simple-fun-self-slot word-shift)
-                                               fun-pointer-lowtag))
-                              (if (simple-fun-p fun) 0 #x48))))
-                 (fdefn-addr (- (get-lisp-obj-address fdefn) ; base of the object
-                                other-pointer-lowtag))
-                 (fdefn-entry-addr (+ fdefn-addr ; address that callers jump to
-                                      (ash fdefn-raw-addr-slot word-shift)))
-                 (displacement (the (signed-byte 32)
-                                 (- fun-entry-addr (+ fdefn-entry-addr 5)))))
-        (setf (sap-ref-word (int-sap fdefn-entry-addr) 0)
-              (logior #xE9
-                      ;; Allow negative displacement
-                      (ash (ldb (byte 32 0) displacement) 8) ; JMP opcode
-                      (ash nop-byte 40))
-              (sap-ref-lispobj (int-sap fdefn-addr) (ash fdefn-fun-slot word-shift))
-              fun)))))
+      (let* ((jmp-target
+              (if trampoline
+                  ;; Jump right to code-instructions + N. There's no simple-fun.
+                  (sap-int (sap+ (code-instructions trampoline)
+                                 trampoline-entry-offset))
+                  ;; CLOSURE-CALLEE accesses the self pointer of a funcallable
+                  ;; instance w/ builtin trampoline, or a simple-fun.
+                  ;; But the result is shifted by N-FIXNUM-TAG-BITS because
+                  ;; CELL-REF yields a descriptor-reg, not an unsigned-reg.
+                  (get-lisp-obj-address (%closure-callee fun))))
+             (tagged-ptr-bias
+              ;; compute the difference between the entry address
+              ;; and the tagged pointer to the called object.
+              (the (unsigned-byte 8)
+                   (- jmp-target (get-lisp-obj-address (or trampoline fun)))))
+             (fdefn-addr (- (get-lisp-obj-address fdefn) ; base of the object
+                            other-pointer-lowtag))
+             (jmp-origin ; 5 = instruction length
+              (+ fdefn-addr (ash fdefn-raw-addr-slot word-shift) 5))
+             (jmp-operand
+              (ldb (byte 32 0) (the (signed-byte 32) (- jmp-target jmp-origin))))
+             (instruction
+              (logior #xE9 ; JMP opcode
+                      (ash jmp-operand 8)
+                      (ash #xA890 40) ; "NOP ; TEST %AL, #xNN"
+                      (ash tagged-ptr-bias 56))))
+        (%primitive sb-vm::set-fdefn-fun fdefn fun instruction))))
+  fun)
+
 ) ; end PROGN
 
 ;;; Find an immobile FDEFN or FUNCTION given an interior pointer to it.
-#!+immobile-space
+#+immobile-space
 (defun find-called-object (address)
-  ;; The ADDRESS [sic] is actually any immediate operand to MOV,
-  ;; which in general decodes as a *signed* integer. So ignore negative values.
-  (let ((obj (if (typep address 'sb!ext:word)
-                 (alien-funcall (extern-alien "search_all_gc_spaces"
-                                              (function unsigned unsigned))
-                                address)
-                 0)))
-      (unless (eql obj 0)
-        (case (sap-ref-8 (int-sap obj) 0)
-         (#.fdefn-widetag
-          (make-lisp-obj (logior obj other-pointer-lowtag)))
-         (#.funcallable-instance-widetag
-          (make-lisp-obj (logior obj fun-pointer-lowtag)))
-         (#.code-header-widetag
-          (let ((code (make-lisp-obj (logior obj other-pointer-lowtag))))
-            (dotimes (i (code-n-entries code))
-              (let ((f (%code-entry-point code i)))
-                (if (= (+ (get-lisp-obj-address f)
-                          (ash simple-fun-code-offset word-shift)
-                          (- fun-pointer-lowtag))
-                       address)
-                    (return f))))))))))
+  (let ((obj (alien-funcall (extern-alien "search_all_gc_spaces"
+                                          (function unsigned unsigned))
+                            address)))
+    (unless (eql obj 0)
+      (case (sap-ref-8 (int-sap obj) 0)
+        (#.code-header-widetag
+         (%simple-fun-from-entrypoint
+          (make-lisp-obj (logior obj other-pointer-lowtag))
+          address))
+        (#.fdefn-widetag
+         (make-lisp-obj (logior obj other-pointer-lowtag)))
+        (#.funcallable-instance-widetag
+         (make-lisp-obj (logior obj fun-pointer-lowtag)))))))
 
 ;;; Compute the PC that FDEFN will jump to when called.
-#!+immobile-code
+#+immobile-code
 (defun fdefn-call-target (fdefn)
   (let ((pc (+ (get-lisp-obj-address fdefn)
                (- other-pointer-lowtag)
@@ -291,3 +289,69 @@
       (set-context-pc context (sap-ref-word (int-sap fun-addr)
                                             (- (ash simple-fun-self-slot word-shift)
                                                fun-pointer-lowtag))))))
+
+;;; CALL-DIRECT-P when true indicates that at the instruction level
+;;; it is fine to change the JMP or CALL instruction - i.e. the offset is
+;;; a (SIGNED-BYTE 32), and the function can be called without loading
+;;; its address into RAX, *and* there is a 1:1 relation between the fdefns
+;;; and fdefn-funs for all callees of the code that contains FUN.
+;;; The 1:1 constraint is due to a design limit - when removing static links,
+;;; it is impossible to distinguish fdefns that point to the same called function.
+;;; FIXME: It would be a nice to remove the uniqueness constraint, either
+;;; by recording ay ambiguous fdefns, or just recording all replacements.
+;;; Perhaps we could remove the static linker mutex as well?
+(defun call-direct-p (fun code-header-funs)
+  #-immobile-code (declare (ignore fun code-header-funs))
+  #+immobile-code
+  (flet ((singly-occurs-p (thing things &aux (len (length things)))
+           ;; Return T if THING occurs exactly once in vector THINGS.
+           (declare (simple-vector things))
+           (dotimes (i len)
+             (when (eq (svref things i) thing)
+               ;; re-using I as the index is OK because we leave the outer loop
+               ;; after this.
+               (return (loop (cond ((>= (incf i) len) (return t))
+                                   ((eq thing (svref things i)) (return nil)))))))))
+    (and (immobile-space-obj-p fun)
+         (not (fun-requires-simplifying-trampoline-p fun))
+         (singly-occurs-p fun code-header-funs))))
+
+;;; Allocate a code object.
+(defun alloc-dynamic-space-code (total-words)
+  (values (%primitive alloc-dynamic-space-code (the fixnum total-words))))
+
+;;; Remove calls via fdefns from CODE when compiling into memory.
+(defun statically-link-code-obj (code fixups)
+  (declare (ignorable fixups))
+  (unless (and (immobile-space-obj-p code)
+               (fboundp 'sb-vm::remove-static-links))
+    (return-from statically-link-code-obj))
+  #+immobile-code
+  (let ((insts (code-instructions code))
+        (fdefns)) ; group by fdefn
+    (loop for (offset . name) in fixups
+          do (binding* ((fdefn (find-fdefn name) :exit-if-null)
+                        (cell (assq fdefn fdefns)))
+               (if cell
+                   (push offset (cdr cell))
+                   (push (list fdefn offset) fdefns))))
+    (let ((funs (make-array (length fdefns))))
+      (sb-thread::with-system-mutex (sb-c::*static-linker-lock*)
+        (loop for i from 0 for (fdefn) in fdefns
+              do (setf (aref funs i) (fdefn-fun fdefn)))
+        (dolist (fdefn-use fdefns)
+          (let* ((fdefn (car fdefn-use))
+                 (callee (fdefn-fun fdefn)))
+            ;; Because we're holding the static linker lock, the elements of
+            ;; FUNS can not change while this test is performed.
+            (when (call-direct-p callee funs)
+              (let ((entry (sb-vm::fdefn-call-target fdefn)))
+                (dolist (offset (cdr fdefn-use))
+                  ;; Only a CALL or JMP will get statically linked.
+                  ;; A MOV will always load the address of the fdefn.
+                  (when (eql (logior (sap-ref-8 insts (1- offset)) 1) #xE9)
+                    ;; Set the statically-linked flag
+                    (setf (sb-vm::fdefn-has-static-callers fdefn) 1)
+                    ;; Change the machine instruction
+                    (setf (signed-sap-ref-32 insts offset)
+                          (- entry (+ (sap-int (sap+ insts offset)) 4)))))))))))))
