@@ -79,7 +79,7 @@
   ;; extensions, so we can use something arbitrary.
   ".lisp-obj")
 (defvar *target-assem-obj-suffix*
-  ;; Target fasl files from SB!C:ASSEMBLE-FILE are LOADed via GENESIS.
+  ;; Target fasl files from SB-C:ASSEMBLE-FILE are LOADed via GENESIS.
   ;; The source files are compiled once as assembly files and once as
   ;; normal lisp files.  In the past, they were kept separate by
   ;; clever symlinking in the source tree, but that became less clean
@@ -148,15 +148,11 @@
 ;;; Return an expression read from the file named NAMESTRING.
 ;;; For user-supplied inputs, protect against more than one expression
 ;;; appearing in the file. With trusted inputs we needn't bother.
-(defun read-from-file (namestring)
+(defun read-from-file (namestring &optional (enforce-single-expr t))
   (with-open-file (s (prepend-genfile-path namestring))
     (let* ((result (read s))
            (eof-result (cons nil nil)))
-      (when (string= (pathname-name namestring) "build-order")
-        ;; build-order uses both host and target conditionals.
-        ;; Our sanity checks during make-host-1 would complain
-        ;; about #+feature when reading the second expression in the file
-        ;; whenever such feature is also a possible target feature.
+      (unless enforce-single-expr
         (return-from read-from-file result))
       (unless (eq (read s nil eof-result) eof-result)
         (error "more than one expression in file ~S" namestring))
@@ -186,13 +182,23 @@
 ;;; When cross-compiling, the *FEATURES* set for the target Lisp is
 ;;; not in general the same as the *FEATURES* set for the host Lisp.
 ;;; In order to refer to target features specifically, we refer to
-;;; SB!XC:*FEATURES* instead of CL:*FEATURES*, and use the #!+ and #!-
-;;; readmacros instead of the ordinary #+ and #- readmacros.
-(setf sb!xc:*features*
-      (let* ((default-features
-               (funcall (compile
-                         nil
-                         (read-from-file "local-target-features.lisp-expr"))
+;;; SB-XC:*FEATURES* instead of CL:*FEATURES*.
+;;;
+;;; To support building in a read-only filesystem, the 'local-target-features'
+;;; file might not be directly located here, since it's a generated file.
+;;; In as much as we use files as the means of passing parameters to
+;;; our Lisp scripts - because we can't in general assume that we can read
+;;; the command-line arguments in any Lisp - it doesn't make sense to have
+;;; another file specifying the name of the local-target-features file.
+;;; The compromise is to examine a variable specifying a path
+;;; (and it can't go in SB-COLD because the package is not made soon enough)
+(setf sb-xc:*features*
+      (let* ((pathname (let ((var 'cl-user::*sbcl-target-features-file*))
+                         (if (boundp var)
+                             (symbol-value var)
+                             "local-target-features.lisp-expr")))
+             (default-features
+               (funcall (compile nil (read-from-file pathname))
                         (read-from-file "base-target-features.lisp-expr")))
              (customizer-file-name "customize-target-features.lisp")
              (customizer (if (probe-file customizer-file-name)
@@ -211,8 +217,8 @@
                            (when (probe-file filename)
                              (read-from-file filename))))
 (dolist (target-feature '(:sb-after-xc-core :cons-profiling))
-  (when (member target-feature sb!xc:*features*)
-    (setf sb!xc:*features* (delete target-feature sb!xc:*features*))
+  (when (member target-feature sb-xc:*features*)
+    (setf sb-xc:*features* (delete target-feature sb-xc:*features*))
     ;; If you use --fancy and --with-sb-after-xc-core you might
     ;; add the feature twice if you don't use pushnew
     (pushnew target-feature *build-features*)))
@@ -235,7 +241,7 @@
 
 ;;; You can get all the way through make-host-1 without either one of these
 ;;; features, but then 'bit-bash' will fail to cross-compile.
-(unless (intersection '(:big-endian :little-endian) sb!xc:*features*)
+(unless (intersection '(:big-endian :little-endian) sb-xc:*features*)
   (warn "You'll have bad time without either endian-ness defined"))
 
 ;;; Some feature combinations simply don't work, and sometimes don't
@@ -251,7 +257,7 @@
           ":GENCGC and :CHENEYGC are incompatible")
          ("(and cheneygc (not (or alpha arm hppa mips ppc sparc)))"
           ":CHENEYGC not supported on selected architecture")
-         ("(and gencgc (not (or sparc ppc x86 x86-64 arm arm64)))"
+         ("(and gencgc (not (or sparc ppc ppc64 x86 x86-64 arm arm64)))"
           ":GENCGC not supported on selected architecture")
          ("(not (or gencgc cheneygc))"
           "One of :GENCGC or :CHENEYGC must be enabled")
@@ -290,7 +296,7 @@
           ":SB-THREAD not supported on selected architecture")))
       (failed-test-descriptions nil))
   (dolist (test feature-compatibility-tests)
-    (let ((cl:*features* sb!xc:*features*))
+    (let ((cl:*features* sb-xc:*features*))
       (when (read-from-string (concatenate 'string "#+" (first test) "T NIL"))
         (push (second test) failed-test-descriptions))))
   (when failed-test-descriptions
@@ -304,11 +310,11 @@
 ;;; values of special variables such as *** and +, anyway). Set up
 ;;; machinery to warn us when/if we change it.
 ;;;
-;;; All code depending on this is itself dependent on #!+SB-SHOW.
-#!+sb-show
-(progn
+;;; All code depending on this is itself dependent on #+SB-SHOW.
+(defvar *cl-snapshot*)
+(when (member :sb-show sb-xc:*features*)
   (load "src/cold/snapshot.lisp")
-  (defvar *cl-snapshot* (take-snapshot "COMMON-LISP")))
+  (setq *cl-snapshot* (take-snapshot "COMMON-LISP")))
 
 ;;;; master list of source files and their properties
 
@@ -346,15 +352,27 @@
     ;; CL. Eventually, it would be great to just get rid of all
     ;; warnings and remove support for this flag. -- WHN 19990323)
     :ignore-failure-p
-    ;; meaning: Build this file, but don't put it on the list for
-    ;; genesis to include in the cold core.
-    :not-genesis))
+    ;; meaning: ignore this flag.
+    ;; This works around nonstandard behavior of "#." in certain hosts.
+    ;; When the evaluated form yields 0 values, ECL and CLISP treat it
+    ;; as though if yielded NIL:
+    ;; * (read-from-string "#(#.(cl:if (cl:eql 1 2) x (values)))")
+    ;;   => #(NIL)
+    ;; The correct value for the above expression - as obtained in SBCL,
+    ;; CCL, and ABCL - is #() because _any_ reader macro is permitted
+    ;; to produce 0 values. In fact you can demonstrate this by actually
+    ;; implementing your own "#." which conditionally returns 0 values,
+    ;; and seeing that it works in any lisp including the suspect ones.
+    ;; The oft-used idiom of "#+#.(cl:if (test) '(and) '(or)) X"
+    ;; is sufficiently unclear that its worth allowing a spurious NIL
+    ;; just to avoid that ugly mess.
+    nil))
 
 (defvar *array-to-specialization* (make-hash-table :test #'eq))
 
-(defmacro do-stems-and-flags ((stem flags) &body body)
+(defmacro do-stems-and-flags ((stem flags build-phase) &body body)
   (let ((stem-and-flags (gensym "STEM-AND-FLAGS")))
-    `(dolist (,stem-and-flags (get-stems-and-flags))
+    `(dolist (,stem-and-flags (get-stems-and-flags ,build-phase))
        (let ((,stem (first ,stem-and-flags))
              (,flags (rest ,stem-and-flags)))
          ,@body
@@ -404,13 +422,26 @@
 (compile 'stem-object-path)
 
 (defvar *stems-and-flags* nil)
-;;; Check for stupid typos in FLAGS list keywords.
-(defun get-stems-and-flags ()
- (when *stems-and-flags*
-   (return-from get-stems-and-flags *stems-and-flags*))
- (setf *stems-and-flags* (read-from-file "build-order.lisp-expr"))
- (let ((stems (make-hash-table :test 'equal)))
-  (do-stems-and-flags (stem flags)
+;;; Read the set of files to compile with respect to a build phase, 1 or 2.
+(defun get-stems-and-flags (build-phase)
+  (when (and *stems-and-flags* (eql (car *stems-and-flags*) build-phase))
+    (return-from get-stems-and-flags (cdr *stems-and-flags*)))
+  (let* ((feature (aref #(:sb-xc-host :sb-xc) (1- build-phase)))
+         (list
+          ;; The build phase feature goes into CL:*FEATURES*, not SB-XC:*FEATURES*
+          ;; because firstly we don't use feature expressions to control the set of
+          ;; files pertinent to the build phase - that is governed by :NOT-{HOST,TARGET}
+          ;; flags, and secondly we can not assume existence of the SB-XC package in
+          ;; warm build. The sole reason for this hack is to allow testing for CMU
+          ;; as the build host in make-host-1 which apparently needs to be allowed
+          ;; to produce warnings as a bug workaround.
+          (let ((cl:*features* (cons feature cl:*features*))
+                (*readtable* *xc-readtable*))
+            (read-from-file "build-order.lisp-expr" nil))))
+    (setf *stems-and-flags* (cons build-phase list)))
+  ;; Now check for duplicate stems and bogus flags.
+  (let ((stems (make-hash-table :test 'equal)))
+    (do-stems-and-flags (stem flags build-phase)
     ;; We do duplicate stem comparison based on the object path in
     ;; order to cover the case of stems with an :assem flag, which
     ;; have two entries but separate object paths for each.  KLUDGE:
@@ -418,18 +449,19 @@
     ;; set up later in the build process and we don't actually care
     ;; what it is so long as it doesn't change while we're checking
     ;; for duplicate stems.
-    (let* ((*target-obj-prefix* "")
-           (object-path (stem-object-path stem flags :target-compile)))
-      (if (gethash object-path stems)
-          (error "duplicate stem ~S in *STEMS-AND-FLAGS*" stem)
-          (setf (gethash object-path stems) t)))
+      (let* ((*target-obj-prefix* "")
+             (object-path (stem-object-path stem flags :target-compile)))
+        (if (gethash object-path stems)
+            (error "duplicate stem ~S in *STEMS-AND-FLAGS*" stem)
+            (setf (gethash object-path stems) t)))
+    ;; Check for stupid typos in FLAGS list keywords.
     ;; FIXME: We should make sure that the :assem flag is only used
     ;; when paired with :not-host.
-    (let ((set-difference (set-difference flags *expected-stem-flags*)))
-      (when set-difference
-        (error "found unexpected flag(s) in *STEMS-AND-FLAGS*: ~S"
-               set-difference)))))
-  *stems-and-flags*)
+      (let ((set-difference (set-difference flags *expected-stem-flags*)))
+        (when set-difference
+          (error "found unexpected flag(s) in *STEMS-AND-FLAGS*: ~S"
+                 set-difference)))))
+  (cdr *stems-and-flags*))
 
 ;;;; tools to compile SBCL sources to create the cross-compiler
 
@@ -560,11 +592,17 @@
     (pathname obj)))
 (compile 'compile-stem)
 
+(defparameter *host-quirks*
+  (or #+cmu  '(:host-quirks-cmu :no-ansi-print-object)
+      #+ecl  '(:host-quirks-ecl)
+      #+sbcl '(:host-quirks-sbcl))) ; not so much a "quirk", but consistent anyway
+
 ;;; Execute function FN in an environment appropriate for compiling the
 ;;; cross-compiler's source code in the cross-compilation host.
 (defun in-host-compilation-mode (fn)
   (declare (type function fn))
-  (let ((*features* (cons :sb-xc-host *features*)))
+  (let ((sb-xc:*features* (append '(:sb-xc-host) *host-quirks* sb-xc:*features*))
+        (*readtable* *xc-readtable*))
     (funcall fn)))
 (compile 'in-host-compilation-mode)
 
@@ -604,7 +642,7 @@
 (defun target-compile-stem (stem flags)
   (funcall *in-target-compilation-mode-fn*
            (lambda ()
-             (progv (list (intern "*SOURCE-NAMESTRING*" "SB!C"))
+             (progv (list (intern "*SOURCE-NAMESTRING*" "SB-C"))
                     (list (lpnify-stem stem))
                (loop
                 (with-simple-restart (recompile "Recompile")
