@@ -11,7 +11,28 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
+
+(defconstant sb-assem:assem-scheduler-p nil)
+(defconstant sb-assem:+inst-alignment-bytes+ 4)
+
+(defconstant +backend-fasl-file-implementation+ :arm)
+
+  ;; Minumum observed value, not authoritative.
+(defconstant +backend-page-bytes+ #+linux 4096 #+netbsd 8192)
+
+;;; The size in bytes of GENCGC cards, i.e. the granularity at which
+;;; writes to old generations are logged.  With mprotect-based write
+;;; barriers, this must be a multiple of the OS page size.
+(defconstant gencgc-card-bytes +backend-page-bytes+)
+;;; The minimum size of new allocation regions.  While it doesn't
+;;; currently make a lot of sense to have a card size lower than
+;;; the alloc granularity, it will, once we are smarter about finding
+;;; the start of objects.
+(defconstant gencgc-alloc-granularity 0)
+;;; The minimum size at which we release address ranges to the OS.
+;;; This must be a multiple of the OS page size.
+(defconstant gencgc-release-granularity +backend-page-bytes+)
 
 ;;; number of bits per word where a word holds one lisp descriptor
 (defconstant n-word-bits 32)
@@ -50,7 +71,7 @@
 (defconstant double-float-digits
   (+ (byte-size double-float-significand-byte) n-word-bits 1))
 
-#!+arm-vfp
+#+arm-vfp
 (progn
   (defconstant float-invalid-trap-bit (ash 1 0))
   (defconstant float-divide-by-zero-trap-bit (ash 1 1))
@@ -74,13 +95,13 @@
 ;; NOTE: As with the FLOAT-REGISTERS SB in vm.lisp, if you define this
 ;; for non-VFP systems, please use a specific positive feature
 ;; conditional.
-#!-arm-vfp
+#-arm-vfp
 (error "Don't know how to set the FPU control word layout on non-VFP systems")
 
 ;;;; Where to put the different spaces.
 
 ;;; On non-gencgc we need large dynamic and static spaces for PURIFY
-#!-gencgc
+#-gencgc
 (progn
   (defconstant read-only-space-start #x04000000)
   (defconstant read-only-space-end   #x07ff8000)
@@ -90,42 +111,29 @@
   (defconstant linkage-table-space-start #x0a000000)
   (defconstant linkage-table-space-end   #x0b000000))
 
-#!+gencgc
-(progn
-  (defconstant linkage-table-space-start #x0a000000)
-  (defconstant linkage-table-space-end   #x0b000000)
-
-  (defconstant read-only-space-start     #x04000000)
-  (defconstant read-only-space-end       #x07ff8000)
-
-  (defconstant static-space-start        #x08000000)
-  (defconstant static-space-end          #x097fff00)
-
-  (defconstant dynamic-space-start       #x4f000000)
-  (defconstant dynamic-space-end         (!configure-dynamic-space-end)))
+#+gencgc
+(!gencgc-space-setup #x04000000 :dynamic-space-start #x4f000000)
 
 (defconstant linkage-table-entry-size 16)
 
-#!+(or linux netbsd)
+#+(or linux netbsd)
 (progn
-  #!-gencgc
+  #-gencgc
   (progn
-    (defconstant dynamic-0-space-start #x4f000000)
-    (defconstant dynamic-0-space-end   #x66fff000)
-    (defconstant dynamic-1-space-start #x67000000)
-    (defconstant dynamic-1-space-end   #x7efff000)))
+    (defparameter dynamic-0-space-start #x4f000000)
+    (defparameter dynamic-0-space-end   #x66fff000)))
 
 ;;;; other miscellaneous constants
 
 (defenum (:start 8)
   halt-trap
   pending-interrupt-trap
-  error-trap
   cerror-trap
   breakpoint-trap
   fun-end-breakpoint-trap
   single-step-around-trap
-  single-step-before-trap)
+  single-step-before-trap
+  error-trap)
 
 ;;;; Static symbols.
 
@@ -138,8 +146,7 @@
 ;;; can be loaded directly out of them by indirecting relative to NIL.
 ;;;
 (defconstant-eqx +static-symbols+
-  `(,@+common-static-symbols+
-    ,@+c-callable-static-symbols+
+ `#(,@+common-static-symbols+
     *allocation-pointer*
 
      *control-stack-pointer*
@@ -148,24 +155,18 @@
 
      ;; interrupt handling
      *pseudo-atomic-atomic*
-     *pseudo-atomic-interrupted*
-
-     ;; Needed for callbacks to work across saving cores. see
-     ;; ALIEN-CALLBACK-ASSEMBLER-WRAPPER in c-call.lisp for gory
-     ;; details.
-     sb!alien::*enter-alien-callback*
-     #!+gencgc *restart-lisp-function*)
-  #'equal)
+     *pseudo-atomic-interrupted*)
+  #'equalp)
 
 (defconstant-eqx +static-fdefns+
-  '(two-arg-gcd two-arg-lcm
+  #(two-arg-gcd two-arg-lcm
     two-arg-+ two-arg-- two-arg-* two-arg-/
     two-arg-< two-arg-> two-arg-=
     two-arg-and two-arg-ior two-arg-xor two-arg-eqv
 
     eql
-    sb!kernel:%negate)
-  #'equal)
+    sb-kernel:%negate)
+  #'equalp)
 
 
 ;;;; Assembler parameters:
