@@ -9,12 +9,18 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!ASSEM")
+(in-package "SB-ASSEM")
 
 ;;;; assembly control parameters
 
-(defvar *assem-max-locations* 0)
-(declaim (type index *assem-max-locations*))
+;;; Only the scheduling assembler cares about this constant,
+;;; but it has to be defined. If ASSEM-SCHEDULER-P is true and it hasn't
+;;; been assigned, we'll get an error. It's OK if ASSEM-SCHEDULER-P is NIL
+;;; and the constant _has_ already been defined. (This is weird but harmless)
+(defconstant +assem-max-locations+
+  (if (boundp '+assem-max-locations+)
+      (symbol-value '+assem-max-locations+)
+      0))
 
 ;;;; the SEGMENT structure
 
@@ -26,6 +32,7 @@
   ;; array operations are rather slow compared to simple ones).
   (buffer (make-array 100 :element-type 'assembly-unit)
           :type (simple-array assembly-unit 1))
+  (encoder-state)
   ;; whether or not to run the scheduler. Note: if the instruction
   ;; definitions were not compiled with the scheduler turned on, this
   ;; has no effect.
@@ -51,7 +58,7 @@
   ;; number of bytes to subtract from all finalized positions such that
   ;; position 0 corresponds to CODE-INSTRUCTIONS of the code component
   ;; being assembled.
-  (header-skew 0 :type (member 0 #.sb!vm:n-word-bytes))
+  (header-skew 0 :type (member 0 #.sb-vm:n-word-bytes))
   ;; the position the last time we synchronized
   (sync-posn 0 :type index)
   ;; a label at position 0
@@ -71,9 +78,9 @@
   (inst-number 0 :type index)
   ;; SIMPLE-VECTORs mapping locations to the instruction that reads them and
   ;; instructions that write them
-  (readers (make-array *assem-max-locations* :initial-element nil)
+  (readers (make-array +assem-max-locations+ :initial-element nil)
            :type simple-vector)
-  (writers (make-array *assem-max-locations* :initial-element nil)
+  (writers (make-array +assem-max-locations+ :initial-element nil)
            :type simple-vector)
   ;; The number of additional cycles before the next control transfer,
   ;; or NIL if a control transfer hasn't been queued. When a delayed
@@ -103,17 +110,17 @@
   (fixup-notes)
   ;; Whether or not to collect dynamic statistics. This is just the same as
   ;; *COLLECT-DYNAMIC-STATISTICS* but is faster to reference.
-  #!+sb-dyncount
+  #+sb-dyncount
   (collect-dynamic-statistics nil))
 (defprinter (segment :identity t))
 
 ;;; Record a FIXUP of KIND occurring at the current position in SEGMENT
-(defun sb!c::note-fixup (segment kind fixup)
+(defun sb-c::note-fixup (segment kind fixup)
   (emit-back-patch
    segment
    0
    (lambda (segment posn)
-     (push (sb!c::make-fixup-note kind fixup
+     (push (sb-c::make-fixup-note kind fixup
                                   (- posn (segment-header-skew segment)))
            (segment-fixup-notes segment)))))
 
@@ -210,7 +217,7 @@
   ;; with the segment as its one argument.
   (emitter (missing-arg) :type (or null function))
   ;; The attributes of this instruction.
-  (attributes (instruction-attributes) :type sb!c:attributes)
+  (attributes (instruction-attributes) :type sb-c:attributes)
   ;; Number of instructions or cycles of delay before additional
   ;; instructions can read our writes.
   (delay 0 :type (and fixnum unsigned-byte))
@@ -231,15 +238,15 @@
   (write-dependents (make-sset) :type sset)
   ;; instructions which read what we write
   (read-dependents (make-sset) :type sset))
-#!+sb-show-assem (defvar *inst-ids* (make-hash-table :test 'eq))
-#!+sb-show-assem (defvar *next-inst-id* 0)
+#+sb-show-assem (defvar *inst-ids* (make-hash-table :test 'eq))
+#+sb-show-assem (defvar *next-inst-id* 0)
 (defmethod print-object ((inst instruction) stream)
   (print-unreadable-object (inst stream :type t :identity t)
-    #!+sb-show-assem
+    #+sb-show-assem
     (princ (ensure-gethash inst *inst-ids* (incf *next-inst-id*))
            stream)
     (format stream
-            #!+sb-show-assem " emitter=~S" #!-sb-show-assem "emitter=~S"
+            #+sb-show-assem " emitter=~S" #-sb-show-assem "emitter=~S"
             (let ((emitter (inst-emitter inst)))
               (if emitter
                   (multiple-value-bind (lambda lexenv-p name)
@@ -250,7 +257,7 @@
     (when (inst-depth inst)
       (format stream ", depth=~W" (inst-depth inst)))))
 
-#!+sb-show-assem
+#+sb-show-assem
 (defun reset-inst-ids ()
   (clrhash *inst-ids*)
   (setf *next-inst-id* 0))
@@ -258,43 +265,45 @@
 
 ;;; Instructions are streamed into a SECTION before (optionally combining
 ;;; sections and) assembling into a SEGMENT.
-;;; The SECTION representation is just a list: (LAST-VOP INDEX . BUFFERS).
+;;; The SECTION representation is just a list: (INDEX . BUFFERS).
 ;;; Each BUFFER is a simple-vector.
 ;;; Because the elsewhere section and unboxed data section often hold nothing,
 ;;; we start with an empty vector to avoid waste.
-(defun make-section () (list nil 0 #()))
-(defmacro section-last-vop (section) `(car ,section))
-(defmacro section-last-buf-length (section) `(cadr ,section))
-(defmacro section-last-buf (section) `(caddr ,section))
+(defun make-section () (list 0 #()))
+(defmacro section-buf-chain (section) `(cdr ,section))
+(defmacro section-buf-index (section) `(car ,section))
+(defmacro section-last-buf (section) `(cadr ,section))
 (defun emit (section &rest things)
   ;; each element of THINGS can be:
   ;; - a list (symbol . args) for a machine instruction or assembler directive
   ;; - a label
   ;; - a function to emit a postit
-  (let ((data (cdr section)))
-    (dolist (thing things)
-      (let ((index (car data))
-            (vector (the simple-vector (cadr data))))
-        (unless (< index (length vector))
-          ;; We double the size, but rather than copying the old data into the new,
-          ;; just grow the chain of vectors. No need to materialize a single vector.
-          (let ((new-vector (make-array (max 10 (* (length vector) 2)))))
-            (if (= (length vector) 0)
-                (rplaca (cdr data) new-vector)
-                (push new-vector (cdr data)))
-            (setq vector new-vector
-                  index 0)))
-        (unless (typep thing '(or function label))
-          (unless (member (car thing) '(.align .byte .skip .coverage-mark))
-            (dolist (operand (cdr thing))
+  (dolist (thing things section)
+    (let ((vector (the simple-vector (section-last-buf section)))
+          (index (section-buf-index section)))
+      (unless (< index (length vector))
+        ;; We double the size, but rather than copying the old data into the new,
+        ;; just grow the chain of vectors. No need to materialize a single vector.
+        (let ((new-vector (make-array (max 10 (* (length vector) 2)))))
+          (if (= (length vector) 0)
+              (rplaca (cdr section) new-vector)
+              (push new-vector (cdr section)))
+          (setq vector new-vector
+                index 0)))
+      (unless (typep thing '(or function label))
+        (let ((inst thing))
+          (let ((vop (car inst)))
+            (when (sb-c::vop-p vop) (pop inst)))
+          (unless (member (car inst) '(.align .byte .skip .coverage-mark))
+            (dolist (operand (cdr inst))
               (if (label-p operand)
                   (setf (label-usedp operand) t)
                   ;; backend decides what labels are used
-                  (%mark-used-labels operand)))))
-        (setf (aref vector index) thing)
-        (setf (car data) (1+ index))))))
+                  (%mark-used-labels operand))))))
+      (setf (aref vector index) thing)
+      (setf (section-buf-index section) (1+ index)))))
 
-#!-(or x86-64 x86)
+#-(or x86-64 x86)
 (defun %mark-used-labels (operand) ; default implementation
   (declare (ignore operand)))
 
@@ -303,6 +312,7 @@
   (code-section (make-section) :read-only t)
   (elsewhere-section (make-section) :read-only t)
   (elsewhere-label (gen-label) :read-only t)
+  (inter-function-padding :normal :type (member :normal :nop))
   ;; for collecting unique "unboxed constants" prior to placing them
   ;; into the data section
   (constant-table (make-hash-table :test #'equal) :read-only t)
@@ -386,12 +396,16 @@
                   `(note-write-dependency ,',segment ,',inst ,loc ,@keys)))
        ,@body)))
 
-#!+(or hppa sparc ppc mips) ; only for platforms with scheduling assembler.
-(progn
+#.(unless assem-scheduler-p
+    '(defmacro schedule-pending-instructions (segment)
+      (declare (ignore segment))))
+
+#.(when assem-scheduler-p
+'(progn
 (defun note-read-dependency (segment inst read)
   (multiple-value-bind (loc-num size)
-      (sb!c:location-number read)
-    #!+sb-show-assem (format *trace-output*
+      (sb-c:location-number read)
+    #+sb-show-assem (format *trace-output*
                              "~&~S reads ~S[~W for ~W]~%"
                              inst read loc-num size)
     (when loc-num
@@ -427,8 +441,8 @@
 
 (defun note-write-dependency (segment inst write &key partially)
   (multiple-value-bind (loc-num size)
-      (sb!c:location-number write)
-    #!+sb-show-assem (format *trace-output*
+      (sb-c:location-number write)
+    #+sb-show-assem (format *trace-output*
                              "~&~S writes ~S[~W for ~W]~%"
                              inst write loc-num size)
     (when loc-num
@@ -454,15 +468,14 @@
           (setf (svref (segment-writers segment) index) nil))
         (push inst (svref (segment-writers segment) index)))))
   (values))
-) ; end PROGN
 
 ;;; This routine is called by due to uses of the INST macro when the
 ;;; scheduler is turned on. The change to the dependency graph has
 ;;; already been computed, so we just have to check to see whether the
 ;;; basic block is terminated.
 (defun queue-inst (segment inst)
-  #!+sb-show-assem (format *trace-output* "~&queuing ~S~%" inst)
-  #!+sb-show-assem (format *trace-output*
+  #+sb-show-assem (format *trace-output* "~&queuing ~S~%" inst)
+  #+sb-show-assem (format *trace-output*
                            "  reads ~S~%  writes ~S~%"
                            (collect ((reads))
                              (do-sset-elements (read
@@ -493,9 +506,6 @@
         (schedule-pending-instructions segment))))
   (values))
 
-#!-(or mips ppc sparc) ; not defined for platforms other than these
-(defun sb!c:emit-nop (seg) seg (bug "EMIT-NOP"))
-
 ;;; Emit all the pending instructions, and reset any state. This is
 ;;; called whenever we hit a label (i.e. an entry point of some kind)
 ;;; and when the user turns the scheduler off (otherwise, the queued
@@ -510,7 +520,7 @@
     (return-from schedule-pending-instructions
                  (values)))
 
-  #!+sb-show-assem (format *trace-output*
+  #+sb-show-assem (format *trace-output*
                            "~&scheduling pending instructions..~%")
 
   ;; Note that any values live at the end of the block have to be
@@ -549,7 +559,7 @@
                (cond ((and (sset-empty (inst-read-dependents inst))
                            (instruction-attributep (inst-attributes inst)
                                                    flushable))
-                      #!+sb-show-assem (format *trace-output*
+                      #+sb-show-assem (format *trace-output*
                                                "flushing ~S~%"
                                                inst)
                       (setf (inst-emitter inst) nil)
@@ -569,13 +579,13 @@
       (setf (segment-delayed segment) delayed))
     (dolist (branch (segment-queued-branches segment))
       (grovel-inst (cdr branch))))
-  #!+sb-show-assem (format *trace-output*
+  #+sb-show-assem (format *trace-output*
                            "queued branches: ~S~%"
                            (segment-queued-branches segment))
-  #!+sb-show-assem (format *trace-output*
+  #+sb-show-assem (format *trace-output*
                            "initially emittable: ~S~%"
                            (segment-emittable-insts-queue segment))
-  #!+sb-show-assem (format *trace-output*
+  #+sb-show-assem (format *trace-output*
                            "initially delayed: ~S~%"
                            (segment-delayed segment))
 
@@ -635,7 +645,7 @@
                                (inst-write-dependents inst))
                               (schedule-one-inst segment t)
                               :nop)))
-                #!+sb-show-assem (format *trace-output*
+                #+sb-show-assem (format *trace-output*
                                          "filling branch delay slot with ~S~%"
                                          fill)
                 (push fill results)))
@@ -643,7 +653,7 @@
             (incf insts-from-end))
           (note-resolved-dependencies segment inst)
           (push inst results)
-          #!+sb-show-assem (format *trace-output* "emitting ~S~%" inst)
+          #+sb-show-assem (format *trace-output* "emitting ~S~%" inst)
           (advance-one-inst segment))))
 
     ;; Keep scheduling stuff until we run out.
@@ -659,7 +669,7 @@
     (assemble (segment)
      (dolist (inst results)
        (if (eq inst :nop)
-           (sb!c:emit-nop segment)
+           (sb-c:emit-nop segment)
            (funcall (inst-emitter inst) segment))))
     (setf (segment-run-scheduler segment) t))
 
@@ -698,7 +708,7 @@
                    (instruction-attributep (inst-attributes inst)
                                            variable-length))
         ;; We've got us a live one here. Go for it.
-        #!+sb-show-assem (format *trace-output* "emitting ~S~%" inst)
+        #+sb-show-assem (format *trace-output* "emitting ~S~%" inst)
         ;; Delete it from the list of insts.
         (if prev
             (setf (cdr prev) (cdr remaining))
@@ -720,7 +730,7 @@
   (cond ((segment-delayed segment)
          ;; No emittable instructions, but we have more work to do. Emit
          ;; a NOP to fill in a delay slot.
-         #!+sb-show-assem (format *trace-output* "emitting a NOP~%")
+         #+sb-show-assem (format *trace-output* "emitting a NOP~%")
          :nop)
         (t
          ;; All done.
@@ -781,7 +791,7 @@
 ;;; emitting code in SCHEDULE-PENDING-INSTRUCTIONS.
 (defun insert-emittable-inst (segment inst)
   (unless (instruction-attributep (inst-attributes inst) branch)
-    #!+sb-show-assem (format *trace-output* "now emittable: ~S~%" inst)
+    #+sb-show-assem (format *trace-output* "now emittable: ~S~%" inst)
     (do ((my-depth (inst-depth inst))
          (remaining (segment-emittable-insts-queue segment) (cdr remaining))
          (prev nil remaining))
@@ -791,6 +801,7 @@
              (setf (segment-emittable-insts-queue segment)
                    (cons inst remaining))))))
   (values))
+)) ; end PROGN
 
 ;;;; structure used during output emission
 
@@ -875,9 +886,9 @@
        (emit-byte segment pattern)))
     ;; EMIT-LONG-NOP does not exist for most backends.
     ;; Better to get an ECASE error than undefined-function.
-    #!+x86-64
+    #+x86-64
     ((eql :long-nop)
-     (sb!vm:emit-long-nop segment amount)))
+     (sb-vm:emit-long-nop segment amount)))
   (values))
 
 ;;; This is used to handle the common parts of annotation emission. We
@@ -1278,23 +1289,21 @@
                          (string string-designator))
                      *backend-instruction-set-package*))))
 
-;;; Join all sections into one and return only the chain of buffers,
-;;; NOT an actual section.
+;;; Append all buffer chains of all sections, placing them in forward order.
 (defun combine-sections (sections)
-  (let ((first-section (cdar sections))
-        (more-sections (cdr sections)))
+  (let ((first (car sections)))
     ;; There shouldn't be much consing due to this REVAPPEND. We're not
     ;; reversing the contents of the buffers, just the buffer chain itself.
-    (revappend (let ((last-buffer-len (car first-section))
-                     (more-buffers (cddr first-section)))
+    (revappend (let ((last-buffer-len (section-buf-index first))
+                     (more-buffers (cddr first)))
                  (if (eql last-buffer-len 0)
                      more-buffers ; Exclude empty buffer
-                     (cons (subseq (cadr first-section) 0 last-buffer-len)
+                     (cons (subseq (section-last-buf first) 0 last-buffer-len)
                            more-buffers)))
-               (when more-sections (combine-sections more-sections)))))
+               (awhen (rest sections) (combine-sections it)))))
 
 ;;; Combine INPUTS into one assembly stream and assemble into SEGMENT
-(defun assemble-sections (segment &rest inputs)
+(defun %assemble-sections (segment &rest inputs)
   (let ((**current-vop** nil)
         (sections (combine-sections inputs))
         (in-without-scheduling)
@@ -1314,67 +1323,157 @@
     (when (plusp (segment-header-skew segment))
       (%emit-skip segment (segment-header-skew segment)))
     (%emit-label segment nil (segment-origin segment))
-    #!+sb-dyncount
+    #+sb-dyncount
     (setf (segment-collect-dynamic-statistics segment) *collect-dynamic-statistics*)
-    (dolist (buffer sections segment)
+    (dolist (buffer sections)
       (dovector (operation buffer)
         (etypecase operation
           (cons
-           (let ((mnemonic (the symbol (car operation)))
-                 (operands (cdr operation)))
-             (if (char/= (char (symbol-name mnemonic) 0) #\.) ; not a pseudo-op
-                 (apply mnemonic operands segment operands)
-                 ;; potentially a pseudo-op
+           (let ((first (car operation)))
+             (when (sb-c::vop-p first)
+               (setq **current-vop** first)
+               (pop operation)))
+           (block op
+             (let ((mnemonic (the symbol (car operation)))
+                   (operands (cdr operation)))
+               (when (char= (char (symbol-name mnemonic) 0) #\.)
+                 ;; potentially a pseudo-op. Any mnemonic can start with a dot,
+                 ;; not just the ones handled here by the generic assembler.
                  (case mnemonic
                    (.align
                     (destructuring-bind (bits &optional (pattern 0)) operands
-                      (%emit-alignment segment **current-vop** bits pattern)))
+                      (%emit-alignment segment **current-vop** bits pattern))
+                    (return-from op))
                    (.byte ; takes >1 byte, unlike inst BYTE which takes only 1
-                    (dolist (byte operands)
+                    (dolist (byte operands (return-from op))
                       (emit-byte segment byte)))
                    (.skip
                     (destructuring-bind (n-bytes &optional (pattern 0)) operands
-                      (%emit-skip segment n-bytes pattern)))
+                      (%emit-skip segment n-bytes pattern))
+                    (return-from op))
                    (.begin-without-scheduling
                     (aver (not in-without-scheduling))
                     (setq in-without-scheduling t
                           was-scheduling (segment-run-scheduler segment))
                     (when was-scheduling
                       (schedule-pending-instructions segment)
-                      (setf (segment-run-scheduler segment) nil)))
+                      (setf (segment-run-scheduler segment) nil))
+                    (return-from op))
                    (.end-without-scheduling
                     (aver in-without-scheduling)
                     (setf (segment-run-scheduler segment) was-scheduling
                           in-without-scheduling nil
-                          was-scheduling nil))
-                   (.comment) ; ignore it
-                   (t
-                    ;; A strange instruction whose name starts with #\.
-                    ;; but which isn't recognized as a pseudo-op.
-                    (apply mnemonic operands segment operands))))))
+                          was-scheduling nil)
+                    (return-from op))
+                   (.comment ; ignore it
+                    (return-from op))))
+               (instruction-hooks segment)
+               (apply mnemonic segment (perform-operand-lowering operands)))))
           (label (%emit-label segment **current-vop** operation))
-          (function (%emit-postit segment operation)))))))
+          (function (%emit-postit segment operation))))))
+  (finalize-segment segment))
+
+;;; The interface to %ASSEMBLE-SECTIONS
+(defun assemble-sections (asmstream simple-fun-labels segment)
+  (let* ((n-entries (length simple-fun-labels))
+         (trailer-len (* (+ n-entries 1) 4))
+         (end-text (gen-label))
+         (octets
+          (segment-buffer
+           (%assemble-sections
+            segment
+            (asmstream-data-section asmstream)
+            (asmstream-code-section asmstream)
+            (asmstream-elsewhere-section asmstream)
+            (emit (make-section)
+                  end-text
+                  `(.align 2)
+                  `(.skip ,trailer-len)
+                  `(.align ,sb-vm:n-lowtag-bits))))))
+    (flet ((store-ub16 (index val)
+             (multiple-value-bind (b0 b1)
+                 #+little-endian
+                 (values (ldb (byte 8 0) val) (ldb (byte 8 8) val))
+                 #+big-endian
+                 (values (ldb (byte 8 8) val) (ldb (byte 8 0) val))
+               (setf (aref octets (+ index 0)) b0
+                     (aref octets (+ index 1)) b1)))
+           (store-ub32 (index val)
+             (multiple-value-bind (b0 b1 b2 b3)
+                 #+little-endian
+                 (values (ldb (byte 8  0) val) (ldb (byte 8  8) val)
+                         (ldb (byte 8 16) val) (ldb (byte 8 24) val))
+                 #+big-endian
+                 (values (ldb (byte 8 24) val) (ldb (byte 8 16) val)
+                         (ldb (byte 8  8) val) (ldb (byte 8  0) val))
+               (setf (aref octets (+ index 0)) b0
+                     (aref octets (+ index 1)) b1
+                     (aref octets (+ index 2)) b2
+                     (aref octets (+ index 3)) b3))))
+      (let ((index (length octets))
+            (fun-offsets))
+        ;; Total size of the code object must be multiple of 2 lispwords
+        (aver (not (logtest (+ (segment-header-skew segment) index)
+                            sb-vm:lowtag-mask)))
+        (let ((padding (if (eql n-entries 0)
+                           0
+                           (- index trailer-len (label-position end-text)))))
+          (unless (and (typep trailer-len '(unsigned-byte 16))
+                       (typep n-entries '(unsigned-byte 12))
+                       (typep padding '(unsigned-byte 4)))
+            (bug "Oversized code component?"))
+          (store-ub16 (- index 2) trailer-len)
+          (store-ub16 (- index 4) (logior (ash n-entries 4) padding)))
+        (decf index trailer-len)
+        ;; Iteration over label positions occurs from numerically highest
+        ;; to lowest, which is right because the 0th indexed simple-fun
+        ;; has the lowest entry offset, and is the last one written
+        ;; into the trailer table. The trailer table is indexed backwards:
+        ;;  ... simple-fun-2 | simple-fun-1 | simple-fun-0
+        ;; And because we use PUSH, we collect them in forward order
+        ;; which is the correct thing to return.
+        (dolist (label simple-fun-labels)
+          (let ((val (label-position label)))
+            (push val fun-offsets)
+            (store-ub32 index val)
+            (incf index 4)))
+        (aver (= index (- (length octets) 4)))
+        (values segment
+                (label-position end-text)
+                (segment-fixup-notes segment)
+                fun-offsets)))))
+
+;;; Most backends do not convert register TNs into a different type of
+;;; internal object prior to handing the operands off to the emitter.
+;;; x86-64 does have a different representation, which makes some of
+;;; the emitter logic easier to understand.
+#-x86-64
+(defun perform-operand-lowering (operands) operands)
+
+(defun truncate-section-to-length (section)
+  (setf (section-last-buf section)
+        (subseq (section-last-buf section) 0 (section-buf-index section))))
 
 ;;; Tack SECOND on to the end of FIRST, and reset SECOND to be empty.
 ;;; Typically for appending the elsewhere section to the regular section.
 (defun join-sections (first second)
-  ;; Truncate the first sections' last buffer to its in-use length
-  (setf (section-last-buf first)
-        (subseq (section-last-buf first) 0 (section-last-buf-length first)))
+  ;; Truncate the first section's last buffer to its in-use length
+  (truncate-section-to-length first)
   ;; Extend FIRST's chain of buffers with those in SECOND
-  (setf (cddr first) (nconc (cddr second) (cddr first)))
+  (setf (section-buf-chain first) (nconc (section-buf-chain second)
+                                         (section-buf-chain first)))
   ;; Set the in-use length of the buffer that is now the final buffer
   ;; in FIRST's chain of buffers with the in-use length from SECOND.
-  (setf (section-last-buf-length first) (section-last-buf-length second))
+  (setf (section-buf-index first) (section-buf-index second))
   ;; Clear out SECOND
-  (setf (cddr second) (list #()) (section-last-buf-length second) 0))
+  (setf (section-buf-chain second) (list #())
+        (section-buf-index second) 0))
 
 ;;; Produce a unified vector of the section contents.
 ;;; Might be useful for something, but not at present.
 (defun section-contents (section)
-  (pop section)
-  (let* ((last-buffer-length (car section))
-         (chain (cdr section))
+  (let* ((last-buffer-length (section-buf-index section))
+         (chain (section-buf-chain section))
          (total-length (+ last-buffer-length
                           ;; all but the last must be completely full
                           (reduce #'+ (cdr chain) :key #'length)))
@@ -1388,13 +1487,13 @@
       (replace contents buf :start1 start))))
 
 (defun trace-inst (section mnemonic operands)
-  (when sb!c::*compiler-trace-output*
+  (when sb-c::*compiler-trace-output*
     (let* ((asmstream *asmstream*)
            (section-name
             (if (eq section (asmstream-code-section asmstream))
                 :regular
                 :elsewhere)))
-      (sb!c::trace-instruction section-name **current-vop** mnemonic operands
+      (sb-c::trace-instruction section-name **current-vop** mnemonic operands
                                (asmstream-tracing-state asmstream)))))
 
 (defmacro inst (&whole whole instruction &rest args &environment env)
@@ -1405,7 +1504,7 @@
                 (null sym))
            (warn "Undefined instruction: ~s in~% ~s" instruction whole)
            `(error "Undefined instruction: ~s in~% ~s" ',instruction ',whole))
-          ((#-sb-xc macro-function #+sb-xc sb!xc:macro-function sym env)
+          ((#-sb-xc macro-function #+sb-xc sb-xc:macro-function sym env)
            `(,sym ,@args))
           (t
            `(%inst ,(if stringablep `',sym `(op-encoder-name ,instruction))
@@ -1428,14 +1527,13 @@
     (typecase dest
       (cons ; streaming in to the assembler
        (trace-inst dest mnemonic operands)
-       (emit dest `(,mnemonic . ,operands)))
+       (let ((v **current-vop**)
+             (inst `(,mnemonic . ,operands)))
+         (aver (typep v '(or null sb-c::vop)))
+         (emit dest (if v (cons v inst) inst))))
       (segment ; streaming out of the assembler
-       ;; Pass operands to the machine instruction encoder as a list and as
-       ;; spread arguments. The list alleviates the need for emitter to listify
-       ;; operands prior to calling any instruction hooks. The spread arguments
-       ;; allow the compiler to generate normal &OPTIONAL / &KEY parsing code
-       ;; in lieu of our generating a destructuring-bind to achieve the same.
-       (apply mnemonic operands dest operands))))
+       (instruction-hooks dest)
+       (apply mnemonic dest (perform-operand-lowering operands)))))
   (values))
 
 (defun emit-label (label)
@@ -1454,6 +1552,13 @@
     (trace-inst s :align bits)
     (emit s `(.align ,bits ,pattern))))
 
+;; ECL bug workaround: it miscompiles LABEL-POSITION with this decl
+;; This might be unnecessary now that I'm proclaiming an OPTIMIZE policy
+;; that seems to fix everything. It certainly was needed without that.
+;; At least by keeping this we might be able to report some specific bugs
+;; against ECL in the hope it gets fixed. Of course we can't actually ever
+;; remove workarounds.
+#-host-quirks-ecl
 (declaim (ftype (sfunction (label &optional t index) (or null index))
                 label-position))
 (defun label-position (label &optional if-after delta)
@@ -1486,7 +1591,8 @@
         (when (label-p note)
           (decf (label-index note) skew)
           (decf (label-posn note) skew)))))
-  (compact-segment-buffer segment))
+  (compact-segment-buffer segment)
+  segment)
 
 ;;; Return the contents of SEGMENT as a vector. We assume SEGMENT has
 ;;; been finalized so that we can simply return its buffer.
@@ -1520,12 +1626,12 @@
                                  total-bits assembly-unit-bits))
                         quo))
            (bytes (make-array num-bytes :initial-element nil))
-           (segment-arg (sb!xc:gensym "SEGMENT-")))
+           (segment-arg (sb-xc:gensym "SEGMENT-")))
       (dolist (byte-spec-expr byte-specs)
         (let* ((byte-spec (eval byte-spec-expr))
                (byte-size (byte-size byte-spec))
                (byte-posn (byte-position byte-spec))
-               (arg (sb!xc:gensym (format nil "~:@(ARG-FOR-~S-~)" byte-spec-expr))))
+               (arg (sb-xc:gensym (format nil "~:@(ARG-FOR-~S-~)" byte-spec-expr))))
           (when (ldb-test (byte byte-size byte-posn) overall-mask)
             (error "The byte spec ~S either overlaps another byte spec, or ~
                     extends past the end."
@@ -1588,7 +1694,7 @@
                   forms)))
         `(defun ,name (,segment-arg ,@(arg-names))
            (declare (type segment ,segment-arg) ,@(arg-types))
-           ,@(ecase sb!c:*backend-byte-order*
+           ,@(ecase sb-c:*backend-byte-order*
                (:little-endian (nreverse forms))
                (:big-endian forms))
            ',name)))))
@@ -1647,18 +1753,16 @@
           (t
            (error "unknown option: ~S" option)))))
     (when emitter
-      (push `(instruction-hooks ,segment-name ,sym-name .operands.)
-            emitter)
       (unless cost (setf cost 1))
-      #!+sb-dyncount
+      #+sb-dyncount
       (push `(when (segment-collect-dynamic-statistics ,segment-name)
-               (let* ((info (sb!c:ir2-component-dyncount-info
-                             (sb!c:component-info
-                              sb!c:*component-being-compiled*)))
-                      (costs (sb!c:dyncount-info-costs info))
-                      (block-number (sb!c:block-number
-                                     (sb!c:ir2-block-block
-                                      (sb!c:vop-block ,vop-name)))))
+               (let* ((info (sb-c:ir2-component-dyncount-info
+                             (sb-c:component-info
+                              sb-c:*component-being-compiled*)))
+                      (costs (sb-c:dyncount-info-costs info))
+                      (block-number (sb-c:block-number
+                                     (sb-c:ir2-block-block
+                                      (sb-c:vop-block ,vop-name)))))
                  (incf (aref costs block-number) ,cost)))
             emitter)
       (when assem-scheduler-p
@@ -1688,16 +1792,15 @@
                                     (,flet-name ,segment-name)))))))))
     `(progn
        #-sb-xc-host ; The disassembler is not used on the host.
-       (setf (get ',defun-name 'sb!disassem::instruction-flavors)
+       (setf (get ',defun-name 'sb-disassem::instruction-flavors)
              (list ,@pdefs))
        ,(when emitter
-          `(defun ,defun-name (.operands. ,segment-name ,@(cdr lambda-list))
+          `(defun ,defun-name (,segment-name ,@(cdr lambda-list))
              (declare ,@decls)
              (let ,(and vop-name `((,vop-name **current-vop**)))
                ,@emitter))))))
 
-(defun instruction-hooks (segment mnemonic operands)
-  (declare (ignore mnemonic operands))
+(defun instruction-hooks (segment)
   (let ((postits (segment-postits segment)))
     (setf (segment-postits segment) nil)
     (dolist (postit postits)
