@@ -10,7 +10,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!C")
+(in-package "SB-C")
 
 (deftype byte-buffer () '(vector (unsigned-byte 8)))
 (defvar *byte-buffer*)
@@ -75,11 +75,11 @@
    it."
   (let ((loc (note-debug-location vop nil kind)))
     (emit-postit (lambda (segment posn)
-                   (declare (ignore segment))
-                   (setf (location-info-label loc) posn))))
+                   (setf (location-info-label loc)
+                         (- posn (segment-header-skew segment))))))
   (values))
 
-#!-sb-fluid (declaim (inline ir2-block-physenv))
+#-sb-fluid (declaim (inline ir2-block-physenv))
 (defun ir2-block-physenv (2block)
   (declare (type ir2-block 2block))
   (block-physenv (ir2-block-block 2block)))
@@ -144,7 +144,7 @@
      (let ((offset (- (label-position (restart-location-label x))
                       location))
            (tn (restart-location-tn x))
-           (registers-size #.(integer-length (sb-size (sb-or-lose 'sb!vm::registers)))))
+           (registers-size #.(integer-length (sb-size (sb-or-lose 'sb-vm::registers)))))
        (if tn
            (the fixnum (logior (ash offset registers-size)
                                (tn-offset tn)))
@@ -154,8 +154,8 @@
 
 (defun decode-restart-location (x)
   (declare (fixnum x))
-  (let ((registers-size #.(integer-length (sb-size (sb-or-lose 'sb!vm::registers)))))
-    (values (make-sc+offset sb!vm:descriptor-reg-sc-number
+  (let ((registers-size #.(integer-length (sb-size (sb-or-lose 'sb-vm::registers)))))
+    (values (make-sc+offset sb-vm:descriptor-reg-sc-number
                             (ldb (byte registers-size 0) x))
             (ash x (- registers-size)))))
 
@@ -288,7 +288,8 @@
 (defun debug-source-for-info (info &key function)
   (declare (type source-info info))
   (let ((file-info (get-toplevelish-file-info info)))
-    (make-debug-source
+    (multiple-value-call
+        (if function #'sb-c::make-core-debug-source #'make-debug-source)
      :compiled (source-info-start-time info)
      :namestring (or *source-namestring*
                      (make-file-info-namestring
@@ -299,11 +300,12 @@
                         (if (pathnamep pathname) pathname))
                       file-info))
      :created (file-info-write-date file-info)
-     :form (when function
-             (let ((direct-file-info (source-info-file-info info)))
-               (when (eq :lisp (file-info-name direct-file-info))
-                 (elt (file-info-forms direct-file-info) 0))))
-     :function function)))
+     (if function
+         (values :form (let ((direct-file-info (source-info-file-info info)))
+                         (when (eq :lisp (file-info-name direct-file-info))
+                           (elt (file-info-forms direct-file-info) 0)))
+                 :function function)
+         (values)))))
 
 (defun smallest-element-type (integer negative)
   (let ((bits (max (+ (integer-length integer)
@@ -389,11 +391,10 @@
 ;;; environment live and is an argument. If a :DEBUG-ENVIRONMENT TN,
 ;;; then we also exclude set variables, since the variable is not
 ;;; guaranteed to be live everywhere in that case.
-(defun dump-1-var (fun var tn minimal buffer &optional same-name-p)
+(defun dump-1-var (fun var tn minimal buffer &optional name same-name-p)
   (declare (type lambda-var var) (type (or tn null) tn)
            (type clambda fun))
-  (let* ((name (leaf-debug-name var))
-         (save-tn (and tn (tn-save-tn tn)))
+  (let* ((save-tn (and tn (tn-save-tn tn)))
          (kind (and tn (tn-kind tn)))
          (flags 0)
          (info (lambda-var-arg-info var))
@@ -429,7 +430,7 @@
     (when (and same-name-p
                (not (or more minimal)))
       (setf flags (logior flags compiled-debug-var-same-name-p)))
-    #!+64-bit ; FIXME: fails if SB-VM:N-FIXNUM-TAG-BITS is 3
+    #+64-bit ; FIXME: fails if SB-VM:N-FIXNUM-TAG-BITS is 3
               ; which early-vm.lisp claims to work
     (cond (indirect
            (setf (ldb (byte 27 8) flags) (tn-sc+offset tn))
@@ -451,20 +452,20 @@
       ;; and PARSE-COMPILED-DEBUG-VARS can create the interned symbol.
       ;; This reduces core size by omitting zillions of symbols whose names
       ;; are spelled the same.
-      (vector-push-extend (if (symbol-package name) name (string name)) buffer))
+      (vector-push-extend (if (cl:symbol-package name) name (string name)) buffer))
 
     (cond (indirect
            ;; Indirect variables live in the parent frame, and are
            ;; accessed through a saved frame pointer.
            ;; The first one/two sc-offsets are for the frame pointer,
            ;; the third is for the stack offset.
-           #!-64-bit
+           #-64-bit
            (vector-push-extend (tn-sc+offset tn) buffer)
-           #!-64-bit
+           #-64-bit
            (when save-tn
              (vector-push-extend (tn-sc+offset save-tn) buffer))
            (vector-push-extend (tn-sc+offset (leaf-info var)) buffer))
-          #!-64-bit
+          #-64-bit
           (t
            (if (and tn (tn-offset tn))
                (vector-push-extend (tn-sc+offset tn) buffer)
@@ -473,6 +474,18 @@
              (vector-push-extend (tn-sc+offset save-tn) buffer)))))
   (values))
 
+(defun leaf-principal-name (leaf)
+  ;; If all the references are from the same substituted variable
+  ;; use its name.
+  ;; Helps with &key processing variables.
+  (let ((refs (leaf-refs leaf)))
+    (loop for ref in refs
+          for name = (ref-%source-name ref)
+          for first-name = name then first-name
+          unless (eq name first-name)
+          return (leaf-debug-name leaf)
+          finally (return name))))
+
 ;;; Return a vector suitable for use as the DEBUG-FUN-VARS
 ;;; of FUN. LEVEL is the current DEBUG-INFO quality. VAR-LOCS is a
 ;;; hash table in which we enter the translation from LAMBDA-VARS to
@@ -482,10 +495,10 @@
   (declare (type clambda fun) (type hash-table var-locs))
   (collect ((vars))
     (labels ((frob-leaf (leaf tn gensym-p)
-               (let ((name (leaf-debug-name leaf)))
+               (let ((name (leaf-principal-name leaf)))
                  (when (and name (leaf-refs leaf) (tn-offset tn)
-                            (or gensym-p (symbol-package name)))
-                   (vars (cons leaf tn)))))
+                            (or gensym-p (cl:symbol-package name)))
+                   (vars (list* name leaf tn)))))
              (frob-lambda (x gensym-p)
                (dolist (leaf (lambda-vars x))
                  (frob-leaf leaf (leaf-info leaf) gensym-p))))
@@ -501,21 +514,21 @@
 
     (let ((sorted (sort (vars) #'string<
                         :key (lambda (x)
-                               (symbol-name (leaf-debug-name (car x))))))
+                               (symbol-name (car x)))))
           (prev-name nil)
           (i 0)
           (buffer (make-array 0 :fill-pointer 0 :adjustable t))
           ;; XEPs don't have any useful variables
           (minimal (eq (functional-kind fun) :external)))
       (declare (type index i))
-      (dolist (x sorted)
-        (let* ((var (car x))
-               (name (leaf-debug-name var)))
-          (dump-1-var fun var (cdr x) minimal buffer
-                      (and prev-name (eq prev-name name)))
-          (setf prev-name name)
-          (setf (gethash var var-locs) i)
-          (incf i)))
+      (loop for (name var . tn) in sorted
+            do
+            (dump-1-var fun var tn minimal buffer
+                        name
+                        (and prev-name (eq prev-name name)))
+            (setf prev-name name)
+            (setf (gethash var var-locs) i)
+            (incf i))
       (compact-vector buffer))))
 
 ;;; Return a vector suitable for use as the DEBUG-FUN-VARS of
@@ -623,24 +636,24 @@
                    name)))
     (funcall (compiled-debug-fun-ctor kind)
              :name name
-             #!-fp-and-pc-standard-save :return-pc
-             #!-fp-and-pc-standard-save (tn-sc+offset (ir2-physenv-return-pc 2env))
-             #!-fp-and-pc-standard-save :return-pc-pass
-             #!-fp-and-pc-standard-save (tn-sc+offset (ir2-physenv-return-pc-pass 2env))
-             #!-fp-and-pc-standard-save :old-fp
-             #!-fp-and-pc-standard-save (tn-sc+offset (ir2-physenv-old-fp 2env))
+             #-fp-and-pc-standard-save :return-pc
+             #-fp-and-pc-standard-save (tn-sc+offset (ir2-physenv-return-pc 2env))
+             #-fp-and-pc-standard-save :return-pc-pass
+             #-fp-and-pc-standard-save (tn-sc+offset (ir2-physenv-return-pc-pass 2env))
+             #-fp-and-pc-standard-save :old-fp
+             #-fp-and-pc-standard-save (tn-sc+offset (ir2-physenv-old-fp 2env))
              :encoded-locs
              (cdf-encode-locs
               (label-position (ir2-physenv-environment-start 2env))
               (label-position (ir2-physenv-elsewhere-start 2env))
               (when (ir2-physenv-closure-save-tn 2env)
                 (tn-sc+offset (ir2-physenv-closure-save-tn 2env)))
-              #!+unwind-to-frame-and-call-vop
+              #+unwind-to-frame-and-call-vop
               (when (ir2-physenv-bsp-save-tn 2env)
                 (tn-sc+offset (ir2-physenv-bsp-save-tn 2env)))
-              #!-fp-and-pc-standard-save
+              #-fp-and-pc-standard-save
               (label-position (ir2-physenv-lra-saved-pc 2env))
-              #!-fp-and-pc-standard-save
+              #-fp-and-pc-standard-save
               (label-position (ir2-physenv-cfp-saved-pc 2env))))))
 
 ;;; Return a complete C-D-F structure for FUN. This involves
@@ -650,7 +663,7 @@
   (declare (type clambda fun) (type hash-table var-locs))
   (let* ((dfun (dfun-from-fun fun))
          (actual-level (policy (lambda-bind fun) compute-debug-fun))
-         (level (cond #!+sb-dyncount
+         (level (cond #+sb-dyncount
                       (*collect-dynamic-statistics*
                        (max actual-level 2))
                       (actual-level))))
@@ -733,17 +746,17 @@
        ;; so if there's only one function, pick that as the component name.
        ;; Otherwise preserve whatever crummy name was already assigned.
        :name (let* ((2comp (component-info component))
-                    (entries (sb!c::ir2-component-entries 2comp)))
+                    (entries (sb-c::ir2-component-entries 2comp)))
                (or (and (not (cdr entries))
-                        (sb!c::entry-info-name (car entries)))
+                        (sb-c::entry-info-name (car entries)))
                    (component-name component)))
        :fun-map fun-map
-       :tlf-number component-tlf-num
-       :char-offset
-       (and component-tlf-num
-            (aref (file-info-positions
-                   (source-info-file-info *source-info*))
-                  component-tlf-num))
+       :tlf-num+offset (pack-tlf-num+offset
+                        component-tlf-num
+                        (and component-tlf-num
+                             (aref (file-info-positions
+                                    (source-info-file-info *source-info*))
+                                   component-tlf-num)))
        :contexts (compact-vector *contexts*)))))
 
 ;;; Write BITS out to BYTE-BUFFER in backend byte order. The length of
