@@ -14,63 +14,55 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!C")
-
-;;; ** FIXME **: when ALLOCATE-CODE-OBJECT is expressed as in the commented-out
-;;; block below, it is miscompiled on ppc + cheneygc as evident from failure to
-;;; complete cold-init. Random experimentation showed that inserting any call in
-;;; between the AVERs fixed the miscompilation, as did changing AVER to ASSERT.
-#|
-(declaim (ftype (sfunction (boolean fixnum fixnum) code-component)
-                allocate-code-object))
-(defun allocate-code-object (immobile-p boxed unboxed)
-  (declare (ignorable immobile-p))
-  (aver (typep boxed '(unsigned-byte 22)))
-  (aver (typep unboxed '(unsigned-byte 22)))
-  (%primitive allocate-code-object boxed unboxed))
-|#
-
-;;; Enforce limit on boxed words dependent on how many bits it gets in header.
-;;; Enforce limit on unboxed size. 22 bits = 4MiB, quite generous for code.
-(declaim (ftype (sfunction (boolean (unsigned-byte #!+64-bit 32 #!-64-bit 22)
-                                    (unsigned-byte 22))
-                           code-component)
-                allocate-code-object))
-;;; Allocate a code component with BOXED words in the header
-;;; followed by UNBOXED bytes of raw data.
-;;; BOXED must be the exact count of boxed words desired. No adjustments
-;;; are made for alignment considerations or the fixed slots.
-(defun allocate-code-object (immobile-p boxed unboxed)
-  (declare (ignorable immobile-p))
-  #!+gencgc
-  (without-gcing
-      (cond #!+immobile-code
-            (immobile-p (sb!vm::allocate-immobile-code boxed unboxed))
-            (t (%make-lisp-obj
-                (alien-funcall (extern-alien "alloc_code_object"
-                                             (function unsigned
-                                                       (unsigned 32)
-                                                       (unsigned 32)))
-                               boxed unboxed)))))
-  #!-gencgc
-  (%primitive allocate-code-object boxed unboxed))
+(in-package "SB-C")
 
 ;;; Map of code-component -> list of PC offsets at which allocations occur.
 ;;; This table is needed in order to enable allocation profiling.
 (define-load-time-global *allocation-point-fixups*
   (make-hash-table :test 'eq :weakness :key :synchronized t))
 
-#!-x86-64
+#-x86-64
+(progn
 (defun convert-alloc-point-fixups (dummy1 dummy2)
   (declare (ignore dummy1 dummy2)))
+(defun sb-vm::statically-link-code-obj (code fixups)
+  (declare (ignore code fixups))))
 
-(flet ((fixup (code-obj offset sym kind flavor layout-finder preserved-lists)
+#+immobile-code
+(progn
+  ;; Use FDEFINITION because it strips encapsulations - whether that's
+  ;; the right behavior for it or not is a separate concern.
+  ;; If somebody tries (TRACE LENGTH) for example, it should not cause
+  ;; compilations to fail on account of LENGTH becoming a closure.
+  (defun sb-vm::function-raw-address (name &aux (fun (fdefinition name)))
+    (cond ((not fun)
+           (error "Can't statically link to undefined function ~S" name))
+          ((not (immobile-space-obj-p fun))
+           (error "Can't statically link to ~S: code is movable" name))
+          ((neq (fun-subtype fun) sb-vm:simple-fun-widetag)
+           (error "Can't statically link to ~S: non-simple function" name))
+          (t
+           (let ((addr (get-lisp-obj-address fun)))
+             (sap-ref-word (int-sap addr)
+                           (- (ash sb-vm:simple-fun-self-slot sb-vm:word-shift)
+                              sb-vm:fun-pointer-lowtag))))))
+
+  ;; Return the address to which to jump when calling NAME through its fdefn.
+  (defun sb-vm::fdefn-entry-address (name)
+    (let ((fdefn (find-or-create-fdefn name)))
+      (+ (get-lisp-obj-address fdefn)
+         (ash sb-vm:fdefn-raw-addr-slot sb-vm:word-shift)
+         (- sb-vm:other-pointer-lowtag)))))
+
+(flet ((fixup (code-obj offset sym kind flavor layout-finder
+               preserved-lists statically-link-p)
+         (declare (ignorable statically-link-p))
          ;; PRESERVED-LISTS is a vector of lists of locations (by kind)
          ;; at which fixup must be re-applied after code movement.
          ;; CODE-OBJ must already be pinned in order to legally call this.
          ;; One call site that reaches here is below at MAKE-CORE-COMPONENT
          ;; and the other is LOAD-CODE, both of which pin the code.
-         (when (sb!vm:fixup-code-object
+         (when (sb-vm:fixup-code-object
                  code-obj offset
                  (ecase flavor
                    ((:assembly-routine :assembly-routine*)
@@ -82,63 +74,69 @@
                    (:symbol-tls-index (ensure-symbol-tls-index sym))
                    (:layout (get-lisp-obj-address (funcall layout-finder sym)))
                    (:immobile-object (get-lisp-obj-address sym))
-                   #!+immobile-code (:named-call (sb!vm::fdefn-entry-address sym))
-                   #!+immobile-code (:static-call (sb!vm::function-raw-address sym)))
+                   #+immobile-code
+                   (:named-call
+                    (when statically-link-p
+                      (push (cons offset sym) (elt preserved-lists 3)))
+                    (sb-vm::fdefn-entry-address sym))
+                   #+immobile-code (:static-call (sb-vm::function-raw-address sym)))
                  kind flavor)
            (ecase kind
              (:relative (push offset (elt preserved-lists 0)))
              (:absolute (push offset (elt preserved-lists 1)))))
          ;; These won't exist except for x86-64, but it doesn't matter.
-         (when (member sym '(sb!vm::enable-alloc-counter
-                             sb!vm::enable-sized-alloc-counter))
+         (when (member sym '(sb-vm::enable-alloc-counter
+                             sb-vm::enable-sized-alloc-counter))
            (push offset (elt preserved-lists 2))))
 
        (finish-fixups (code-obj preserved-lists)
          (declare (ignorable code-obj preserved-lists))
-         #!+(or immobile-space x86)
+         #+(or immobile-space x86)
          (let ((rel-fixups (elt preserved-lists 0))
                (abs-fixups (elt preserved-lists 1)))
            (when (or abs-fixups rel-fixups)
-             (setf (sb!vm::%code-fixups code-obj)
-                   (sb!c::pack-code-fixup-locs abs-fixups rel-fixups))))
+             (setf (sb-vm::%code-fixups code-obj)
+                   (sb-c::pack-code-fixup-locs abs-fixups rel-fixups))))
          (awhen (elt preserved-lists 2)
            (setf (gethash code-obj *allocation-point-fixups*)
                  (convert-alloc-point-fixups code-obj it)))
+         (awhen (aref preserved-lists 3)
+           (sb-vm::statically-link-code-obj code-obj it))
          ;; Assign all SIMPLE-FUN-SELF slots
          (dotimes (i (code-n-entries code-obj))
            (let ((fun (%code-entry-point code-obj i)))
              (setf (%simple-fun-self fun)
                    ;; x86 backends store the address of the entrypoint in 'self'
-                   #!+(or x86 x86-64)
+                   #+(or x86 x86-64)
                    (%make-lisp-obj
                     (truly-the word (+ (get-lisp-obj-address fun)
-                                       (ash sb!vm:simple-fun-code-offset sb!vm:word-shift)
-                                       (- sb!vm:fun-pointer-lowtag))))
+                                       (ash sb-vm:simple-fun-code-offset sb-vm:word-shift)
+                                       (- sb-vm:fun-pointer-lowtag))))
                    ;; non-x86 backends store the function itself (what else?) in 'self'
-                   #!-(or x86 x86-64) fun)
+                   #-(or x86 x86-64) fun)
              ;; And maybe store the layout in the high half of the header
-             #!+(and compact-instance-header x86-64)
+             #+(and compact-instance-header x86-64)
              (setf (sap-ref-32 (int-sap (get-lisp-obj-address fun))
-                               (- 4 sb!vm:fun-pointer-lowtag))
+                               (- 4 sb-vm:fun-pointer-lowtag))
                    (truly-the (unsigned-byte 32)
                      (get-lisp-obj-address #.(find-layout 'function))))))
          ;; And finally, make the memory range executable
-         #!-(or x86 x86-64)
-         (sb!vm:sanctify-for-execution code-obj)))
+         #-(or x86 x86-64)
+         (sb-vm:sanctify-for-execution code-obj)))
 
   (defun apply-fasl-fixups (fop-stack code-obj &aux (top (svref fop-stack 0)))
-    (dx-let ((preserved (vector nil nil nil)))
+    (dx-let ((preserved (make-array 4 :initial-element nil)))
       (macrolet ((pop-fop-stack () `(prog1 (svref fop-stack top) (decf top))))
         (dotimes (i (pop-fop-stack) (setf (svref fop-stack 0) top))
           (multiple-value-bind (offset kind flavor)
-              (sb!fasl::!unpack-fixup-info (pop-fop-stack))
+              (sb-fasl::!unpack-fixup-info (pop-fop-stack))
             (fixup code-obj offset (pop-fop-stack) kind flavor
-                   #'find-layout preserved))))
+                   #'find-layout preserved nil))))
       (finish-fixups code-obj preserved)))
 
   (defun apply-core-fixups (fixup-notes code-obj)
     (declare (list fixup-notes))
-    (dx-let ((preserved (vector nil nil nil)))
+    (dx-let ((preserved (make-array 4 :initial-element nil)))
       (dolist (note fixup-notes)
         (let ((fixup (fixup-note-fixup note))
               (offset (fixup-note-position note)))
@@ -151,7 +149,7 @@
                ;; :IMMOBILE-OBJECT fixups. But since they're not, inform the
                ;; fixupper not to call find-layout on them.
                  #'identity
-                 preserved)))
+                 preserved t)))
       (finish-fixups code-obj preserved))))
 
 ;;; Note the existence of FUNCTION.
@@ -189,10 +187,9 @@
   (let ((debug-info (debug-info-for-component component)))
     (let* ((2comp (component-info component))
            (constants (ir2-component-constants 2comp))
+           (nboxed (align-up (length constants) sb-c::code-boxed-words-align))
            (code-obj (allocate-code-object
-                      (or #!+immobile-code (eq *compile-to-memory-space* :immobile))
-                      (align-up (length constants) sb!c::code-boxed-words-align)
-                      length)))
+                      (component-mem-space component) nboxed length)))
 
       ;; The following operations need the code pinned:
       ;; 1. copying into code-instructions (a SAP)
@@ -213,17 +210,14 @@
             (setf (%simple-fun-name fun) (entry-info-name entry-info))
             (setf (%simple-fun-arglist fun) (entry-info-arguments entry-info))
             (setf (%simple-fun-type fun) (entry-info-type entry-info))
-            ;; We can't make a decision to record the source here
-            ;; because the policy is not available in a simple way.
-            ;; (It would demand groveling through the component to find
-            ;; the lambda-bind corresponding to this entry)
-            (setf (%simple-fun-info fun) (entry-info-info entry-info))
+            (apply #'set-simple-fun-info fun
+                   (entry-info-form/doc/xrefs entry-info))
             (note-fun entry-info fun object))))
 
       (push debug-info (core-object-debug-info object))
       (setf (%code-debug-info code-obj) debug-info)
 
-      (do ((index sb!vm:code-constants-offset (1+ index)))
+      (do ((index sb-vm:code-constants-offset (1+ index)))
           ((>= index (length constants)))
         (let ((const (aref constants index)))
             (etypecase const
