@@ -7,13 +7,13 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!C")
+(in-package "SB-C")
 
-(def-alloc %make-structure-instance 1 :structure-alloc
-           sb!vm:instance-header-widetag sb!vm:instance-pointer-lowtag
+(def-alloc '%make-structure-instance 1 :structure-alloc
+           sb-vm:instance-widetag sb-vm:instance-pointer-lowtag
            nil)
 
-#!+stack-allocatable-fixed-objects
+#+stack-allocatable-fixed-objects
 (defoptimizer (%make-structure-instance stack-allocate-result)
     ((defstruct-description &rest args) node dx)
   (declare (ignore args dx))
@@ -25,16 +25,15 @@
   ;; when we're on GENCGC (since CHENEYGC doesn't have conservation)
   ;; and C-STACK-IS-CONTROL-STACK (otherwise, the C stack is the
   ;; number stack, and we precisely-scavenge the control stack).
-  #!-(and :gencgc :c-stack-is-control-stack)
+  #-(and :gencgc :c-stack-is-control-stack)
   (every (lambda (x) (eq (dsd-raw-type x) t))
          (dd-slots (lvar-value defstruct-description)))
-  #!+(and :gencgc :c-stack-is-control-stack)
+  #+(and :gencgc :c-stack-is-control-stack)
   t)
 
 (defoptimizer ir2-convert-reffer ((object) node block name offset lowtag)
   (let* ((lvar (node-lvar node))
-         (locs (lvar-result-tns lvar
-                                (list *backend-t-primitive-type*)))
+         (locs (lvar-result-tns lvar (list *universal-type*)))
          (res (first locs)))
     (vop slot node block (lvar-tn node block object)
          name offset lowtag res)
@@ -54,11 +53,11 @@
          name offset lowtag)
     (move-lvar-result node block (list value-tn) (node-lvar node))))
 
-#!+compare-and-swap-vops
+#+compare-and-swap-vops
 (defoptimizer ir2-convert-casser
     ((object old new) node block name offset lowtag)
   (let* ((lvar (node-lvar node))
-         (locs (lvar-result-tns lvar (list *backend-t-primitive-type*)))
+         (locs (lvar-result-tns lvar (list *universal-type*)))
          (res (first locs)))
     (vop compare-and-swap-slot node block
          (lvar-tn node block object)
@@ -68,20 +67,19 @@
          res)
     (move-lvar-result node block locs lvar)))
 
+(eval-when (:compile-toplevel)
+  ;; Assert correctness of build order. (Need not be exhaustive)
+  #+(and x86-64 (not (vop-named sb-vm::raw-instance-init/word)))
+  (error "Expected raw-instance-init vops"))
+
 (defun emit-inits (node block name object lowtag inits args)
   (let ((unbound-marker-tn nil)
         (funcallable-instance-tramp-tn nil)
-        (lvar (node-lvar node)))
+        (dx-p (node-stack-allocate-p node)))
     (flet ((zero-init-p (x)
              ;; dynamic-space is already zeroed
-             (and (or (not lvar)
-                      (not (lvar-dynamic-extent lvar)))
-                  ;; KLUDGE: can't ignore type-derived
-                  ;; constants since they can be closed over
-                  ;; and not using them confuses the register
-                  ;; allocator.
-                  ;; See compiler.pure/cons-zero-initialization
-                  (strictly-constant-lvar-p x)
+             (and (not dx-p)
+                  (constant-lvar-p x)
                   (eql (lvar-value x) 0))))
      (dolist (init inits)
        (let ((kind (car init))
@@ -105,19 +103,22 @@
                          `(ecase raw-type
                             ((t)
                              (vop init-slot node block object arg-tn
-                                  name (+ sb!vm:instance-slots-offset slot) lowtag))
+                                  name dx-p (+ sb-vm:instance-slots-offset slot) lowtag))
                             ,@(map 'list
                                (lambda (rsd)
-                                 `(,(sb!kernel::raw-slot-data-raw-type rsd)
-                                   (vop ,(sb!kernel::raw-slot-data-init-vop rsd)
+                                 `(,(sb-kernel::raw-slot-data-raw-type rsd)
+                                   (vop ,(sb-kernel::raw-slot-data-init-vop rsd)
                                         node block object arg-tn slot)))
                                (symbol-value rsd-list)))))
-                    (make-case #!+raw-instance-init-vops
-                               sb!kernel::*raw-slot-data*))))))
+                    (make-case #+(vop-named sb-vm::raw-instance-init/word)
+                               sb-kernel::*raw-slot-data*))))))
            (:dd
             (vop init-slot node block object
-                 (emit-constant (sb!kernel::dd-layout-or-lose slot))
-                 name sb!vm:instance-slots-offset lowtag))
+                 (emit-constant (sb-kernel::dd-layout-or-lose slot))
+                 name dx-p
+                 ;; Layout has no index if compact headers.
+                 (or #+compact-instance-header :layout sb-vm:instance-slots-offset)
+                 lowtag))
            (otherwise
             (if (and (eq kind :arg)
                      (zero-init-p (car args)))
@@ -128,11 +129,16 @@
                         (aver args)
                         (lvar-tn node block (pop args)))
                        (:unbound
+                        ;; SLOT should be the word index to alter, but with structure
+                        ;; instances, SLOT is a cons whose car is the raw-slot-type
+                        ;; since BOXED-COMBINATION-REF-P expects that #'cddr is the
+                        ;; slot index.
+                        (when (listp slot)
+                          (setq slot (+ (cdr slot) sb-vm:instance-slots-offset)))
                         (or unbound-marker-tn
                             (setf unbound-marker-tn
                                   (let ((tn (make-restricted-tn
-                                             nil
-                                             (sc-number-or-lose 'sb!vm::any-reg))))
+                                             nil sb-vm:any-reg-sc-number)))
                                     (vop make-unbound-marker node block tn)
                                     tn))))
                        (:null
@@ -141,11 +147,10 @@
                         (or funcallable-instance-tramp-tn
                             (setf funcallable-instance-tramp-tn
                                   (let ((tn (make-restricted-tn
-                                             nil
-                                             (sc-number-or-lose 'sb!vm::any-reg))))
+                                             nil sb-vm:any-reg-sc-number)))
                                     (vop make-funcallable-instance-tramp node block tn)
                                     tn)))))
-                     name slot lowtag))))))))
+                     name dx-p slot lowtag))))))))
   (unless (null args)
     (bug "Leftover args: ~S" args)))
 
@@ -159,7 +164,7 @@
 (defoptimizer ir2-convert-fixed-allocation
               ((&rest args) node block name words type lowtag inits)
   (let* ((lvar (node-lvar node))
-         (locs (lvar-result-tns lvar (list *backend-t-primitive-type*)))
+         (locs (lvar-result-tns lvar (list *universal-type*)))
          (result (first locs)))
     (emit-fixed-alloc node block name words type lowtag result lvar)
     (emit-inits node block name result lowtag inits args)
@@ -168,7 +173,7 @@
 (defoptimizer ir2-convert-variable-allocation
               ((extra &rest args) node block name words type lowtag inits)
   (let* ((lvar (node-lvar node))
-         (locs (lvar-result-tns lvar (list *backend-t-primitive-type*)))
+         (locs (lvar-result-tns lvar (list *universal-type*)))
          (result (first locs)))
     (if (constant-lvar-p extra)
         (let ((words (+ (lvar-value extra) words)))
@@ -182,14 +187,18 @@
     ((dd slot-specs &rest args) node block name words type lowtag inits)
   (declare (ignore inits))
   (let* ((lvar (node-lvar node))
-         (locs (lvar-result-tns lvar (list *backend-t-primitive-type*)))
+         (locs (lvar-result-tns lvar (list *universal-type*)))
          (result (first locs)))
     (aver (and (constant-lvar-p dd) (constant-lvar-p slot-specs) (= words 1)))
     (let* ((c-dd (lvar-value dd))
            (c-slot-specs (lvar-value slot-specs))
            (words (+ (dd-length c-dd) words)))
+      #+compact-instance-header
+      (progn (aver (= type sb-vm:instance-widetag))
+             (emit-constant (setq type (sb-kernel::dd-layout-or-lose c-dd))))
       (emit-fixed-alloc node block name words type lowtag result lvar)
-      (emit-inits node block name result lowtag `((:dd . ,c-dd) ,@c-slot-specs) args)
+      (emit-inits node block name result lowtag
+                  `(#-compact-instance-header (:dd . ,c-dd) ,@c-slot-specs) args)
       (move-lvar-result node block locs lvar))))
 
 (defoptimizer (initialize-vector ir2-convert)
@@ -201,7 +210,7 @@
                              'initialize-vector)))
          (saetp (find-saetp-by-ctype elt-ctype))
          (lvar (node-lvar node))
-         (locs (lvar-result-tns lvar (list (primitive-type vector-ctype))))
+         (locs (lvar-result-tns lvar (list vector-ctype)))
          (result (first locs))
          (elt-ptype (primitive-type elt-ctype))
          (tmp (make-normal-tn elt-ptype)))
@@ -209,35 +218,35 @@
     (flet ((compute-setter ()
              (macrolet
                  ((frob ()
-                    (let ((*package* (find-package :sb!vm))
+                    (let ((*package* (find-package :sb-vm))
                           (clauses nil))
                       (map nil (lambda (s)
-                                 (when (sb!vm:saetp-specifier s)
+                                 (when (sb-vm:saetp-specifier s)
                                    (push
-                                    `(,(sb!vm:saetp-typecode s)
+                                    `(,(sb-vm:saetp-typecode s)
                                        (lambda (index tn)
-                                         #!+x86-64
+                                         #+x86-64
                                          (vop ,(symbolicate "DATA-VECTOR-SET-WITH-OFFSET/"
-                                                            (sb!vm:saetp-primitive-type-name s)
+                                                            (sb-vm:saetp-primitive-type-name s)
                                                             "-C")
                                               node block result tn index 0 tn)
-                                         #!+x86
+                                         #+x86
                                          (vop ,(symbolicate "DATA-VECTOR-SET-WITH-OFFSET/"
-                                                            (sb!vm:saetp-primitive-type-name s))
+                                                            (sb-vm:saetp-primitive-type-name s))
                                               node block result index tn 0 tn)
-                                         #!-(or x86 x86-64)
+                                         #-(or x86 x86-64)
                                          (vop ,(symbolicate "DATA-VECTOR-SET/"
-                                                            (sb!vm:saetp-primitive-type-name s))
+                                                            (sb-vm:saetp-primitive-type-name s))
                                               node block result index tn tn)))
                                     clauses)))
-                           sb!vm:*specialized-array-element-type-properties*)
-                      `(ecase (sb!vm:saetp-typecode saetp)
+                           sb-vm:*specialized-array-element-type-properties*)
+                      `(ecase (sb-vm:saetp-typecode saetp)
                          ,@(nreverse clauses)))))
                (frob)))
            (tnify (index)
-             #!-x86-64
+             #-x86-64
              (emit-constant index)
-             #!+x86-64
+             #+x86-64
              index))
       (let ((setter (compute-setter))
             (length (length initial-contents))
@@ -249,12 +258,7 @@
           (let ((value (pop initial-contents)))
             ;; dynamic-space is already zeroed
             (unless (and (not dx-p)
-                         ;; KLUDGE: can't ignore type-derived
-                         ;; constants since they can be closed over
-                         ;; and not using them confuses the register
-                         ;; allocator.
-                         ;; See compiler.pure/vector-zero-initialization
-                         (strictly-constant-lvar-p value)
+                         (constant-lvar-p value)
                          (if character
                              (eql (char-code (lvar-value value)) 0)
                              (eql (lvar-value value) 0)))
@@ -262,24 +266,32 @@
               (funcall setter (tnify i) tmp))))))
     (move-lvar-result node block locs lvar)))
 
+(defun vector-initialized-p (call)
+  (let ((lvar (node-lvar call)))
+    (when lvar
+      (let ((dest (principal-lvar-dest lvar)))
+        (and (basic-combination-p dest)
+             (lvar-fun-is (basic-combination-fun dest) '(initialize-vector)))))))
+
 ;;; An array header for simple non-unidimensional arrays is a fixed alloc,
 ;;; because the rank has to be known.
 ;;; (There are no compile-time optimizations for unknown rank arrays)
-(defoptimizer (make-array-header* ir2-convert) ((&rest args) node block)
+(defoptimizer (make-array-header* ir2-convert) ((widetag &rest args) node block)
   (let ((n-args (length args)))
+    ;; Remove the widetag lvar
+    (pop (basic-combination-args node))
     (ir2-convert-fixed-allocation
      node block 'make-array
      ;; Each argument fills in one slot of the array header.
      ;; Add one word for the primitive object's header word.
      (1+ n-args)
-     ;; MAKE-ARRAY optimizations will not kick in for complex arrays.
-     sb!vm:simple-array-widetag
-     sb!vm:other-pointer-lowtag
+     (lvar-value widetag)
+     sb-vm:other-pointer-lowtag
      (loop for i from 1 to n-args collect `(:arg . ,i)))))
 
 ;;; :SET-TRANS (in objdef.lisp !DEFINE-PRIMITIVE-OBJECT) doesn't quite
 ;;; cut it for symbols, where under certain compilation options
-;;; (e.g. #!+SB-THREAD) we have to do something complicated, rather
+;;; (e.g. #+SB-THREAD) we have to do something complicated, rather
 ;;; than simply set the slot.  So we build the IR2 converting function
 ;;; by hand.  -- CSR, 2003-05-08
 (let ((fun-info (fun-info-or-lose '%set-symbol-value)))
@@ -294,7 +306,7 @@
                  node block (list value-tn) (node-lvar node))))))))
 
 ;;; Stack allocation optimizers per platform support
-#!+stack-allocatable-vectors
+#+stack-allocatable-vectors
 (progn
   (defoptimizer (make-array-header* stack-allocate-result) ((&rest args) node dx)
     args dx
@@ -305,32 +317,30 @@
     (and
      ;; Can't put unboxed data on the stack unless we scavenge it
      ;; conservatively.
-     #!-c-stack-is-control-stack
+     #-c-stack-is-control-stack
      (constant-lvar-p type)
-     #!-c-stack-is-control-stack
+     #-c-stack-is-control-stack
      (member (lvar-value type)
-             '#.(list (sb!vm:saetp-typecode (find-saetp 't))
-                      (sb!vm:saetp-typecode (find-saetp 'fixnum))))
+             '#.(list (sb-vm:saetp-typecode (find-saetp 't))
+                      (sb-vm:saetp-typecode (find-saetp 'fixnum))))
      (or (eq dx :always-dynamic)
          (zerop (policy node safety))
          ;; a vector object should fit in one page -- otherwise it might go past
          ;; stack guard pages.
          (values-subtypep (lvar-derived-type words)
-                          (load-time-value
-                           (specifier-type `(integer 0 ,(- (/ sb!vm::*backend-page-bytes*
-                                                              sb!vm:n-word-bytes)
-                                                           sb!vm:vector-data-offset))))))))
+                          (specifier-type
+                           `(integer 0 ,(- (/ +backend-page-bytes+ sb-vm:n-word-bytes)
+                                           sb-vm:vector-data-offset)))))))
 
   (defoptimizer (allocate-vector ltn-annotate) ((type length words) call ltn-policy)
     (declare (ignore type length words))
     (vectorish-ltn-annotate-helper call ltn-policy
-                                   'sb!vm::allocate-vector-on-stack
-                                   'sb!vm::allocate-vector-on-heap))
+                                   'sb-vm::allocate-vector-on-stack
+                                   'sb-vm::allocate-vector-on-heap))
 
   (defun vectorish-ltn-annotate-helper (call ltn-policy dx-template &optional not-dx-template)
     (let* ((args (basic-combination-args call))
-           (template-name (if (awhen (node-lvar call)
-                                (lvar-dynamic-extent it))
+           (template-name (if (node-stack-allocate-p call)
                               dx-template
                               not-dx-template))
            (template (and template-name
@@ -349,7 +359,7 @@
         (annotate-1-value-lvar arg)))))
 
 ;;; ...lists
-#!+stack-allocatable-lists
+#+stack-allocatable-lists
 (progn
   (defoptimizer (list stack-allocate-result) ((&rest args) node dx)
     (declare (ignore dx))
@@ -362,7 +372,7 @@
     t))
 
 ;;; ...conses
-#!+stack-allocatable-fixed-objects
+#+stack-allocatable-fixed-objects
 (progn
   (defoptimizer (cons stack-allocate-result) ((&rest args) node dx)
     (declare (ignore args dx))
@@ -372,7 +382,7 @@
     t))
 
 ;;; MAKE-LIST optimizations
-#!+x86-64
+#+x86-64
 (progn
   (defoptimizer (%make-list stack-allocate-result) ((length element) node dx)
     (declare (ignore element))
@@ -384,21 +394,21 @@
         ;; could be removed, because allocation would never miss the guard page
         ;; if it tries to consume too much stack space.
         (values-subtypep (lvar-derived-type length)
-                         (load-time-value
-                          (specifier-type `(integer 0 ,(/ sb!vm::*backend-page-bytes*
-                                                          sb!vm:n-word-bytes 2)))))))
+                         (specifier-type
+                          `(integer 0 ,(/ +backend-page-bytes+ sb-vm:n-word-bytes 2))))))
   (defoptimizer (%make-list ltn-annotate) ((length element) call ltn-policy)
     (declare (ignore length element))
     (vectorish-ltn-annotate-helper call ltn-policy
-                                   'sb!vm::allocate-list-on-stack)))
+                                   'sb-vm::allocate-list-on-stack)))
 
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 ;;; Return a list of parameters with which to call MAKE-ARRAY-HEADER*
 ;;; given the mandatory slots for a simple array of rank 0 or > 1.
 (defun make-array-header-inits (storage n-elements dimensions)
   (macrolet ((expand ()
-               `(list* ,@(mapcar (lambda (slot)
+               `(list* sb-vm:simple-array-widetag
+                       ,@(mapcar (lambda (slot)
                                    (ecase (slot-name slot)
                                      (data           'storage)
                                      (fill-pointer   'n-elements)
@@ -412,3 +422,23 @@
                                                  :key 'primitive-object-name))))
                        dimensions)))
     (expand)))
+
+;;; This order is used by SB-C::TRANSFORM-MAKE-ARRAY-VECTOR
+(assert (equal (mapcar #'slot-name
+                       (primitive-object-slots
+                        (find 'array *primitive-objects*
+                              :key 'primitive-object-name)))
+               '(fill-pointer fill-pointer-p elements data
+                 displacement displaced-p displaced-from dimensions)))
+
+(defun emit-code-page-write-barrier-p (fun-name)
+  (and (listp fun-name)
+       (eq (car fun-name) 'setf)
+       (member (cadr fun-name)
+               '(%code-debug-info
+                 %code-fixups
+                 %simple-fun-name
+                 %simple-fun-arglist
+                 %simple-fun-type
+                 %simple-fun-info))
+       t))
