@@ -10,7 +10,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 
 ;;;; other miscellaneous stuff
 
@@ -32,7 +32,7 @@
          (list* (car options) (cadr options)
                 (remove-keywords (cddr options) keywords)))))
 
-(def!struct (prim-object-slot
+(defstruct (prim-object-slot
              (:constructor make-slot (name rest-p offset special options))
              (:copier nil)
              (:conc-name slot-))
@@ -44,7 +44,7 @@
   ;; referenced as special variables, this slot holds the name of that variable.
   (special nil :type symbol :read-only t))
 
-(def!struct (primitive-object (:copier nil))
+(defstruct (primitive-object (:copier nil))
   (name nil :type symbol :read-only t)
   (widetag nil :type symbol :read-only t)
   (lowtag nil :type symbol :read-only t)
@@ -54,10 +54,8 @@
   (variable-length-p nil :type (member t nil) :read-only t))
 
 (declaim (freeze-type prim-object-slot primitive-object))
-(!set-load-form-method prim-object-slot (:host :xc))
-(!set-load-form-method primitive-object (:host :xc))
 
-(defvar *primitive-objects* nil)
+(define-load-time-global *primitive-objects* nil)
 
 (defun !%define-primitive-object (primobj)
   (let ((name (primitive-object-name primobj)))
@@ -70,7 +68,8 @@
 (defvar *!late-primitive-object-forms* nil)
 
 (defmacro !define-primitive-object
-          ((name &key lowtag widetag alloc-trans (type t))
+          ((name &key lowtag widetag alloc-trans (type t)
+                      (size (symbolicate name "-SIZE")))
            &rest slot-specs)
   (collect ((slots) (specials) (constants) (forms) (inits))
     (let ((offset (if widetag 1 0))
@@ -89,9 +88,9 @@
                        pointer
                        &allow-other-keys)
             (if (atom spec) (list spec) spec)
-          #!-alpha
+          #-alpha
           (declare (ignorable pointer))
-          #!+alpha
+          #+alpha
           (when pointer
             ;; Pointer values on ALPHA are 64 bits wide, and
             ;; double-word aligned.  We may also wish to have such a
@@ -103,19 +102,20 @@
               (setf length 2))
             (when (oddp offset)
               (incf offset)))
-          (slots (make-slot slot-name rest-p offset special
-                            (remove-keywords options '(:rest-p :length))))
+          (slots `(make-slot ',slot-name ,rest-p ,offset ',special
+                             ',(remove-keywords options '(:rest-p :length))))
           (let ((offset-sym (symbolicate name "-" slot-name
                                          (if rest-p "-OFFSET" "-SLOT"))))
             (constants `(defconstant ,offset-sym ,offset))
             (when special
-              (specials `(defvar ,special))))
-          #-c-headers-only
+              (specials `(progn
+                           (defvar ,special)
+                           (setf (info :variable :always-bound ',special)
+                                 :always-bound)))))
           (when ref-trans
             (when ref-known-p
               (forms `(defknown ,ref-trans (,type) ,slot-type ,ref-known)))
-            (forms `(def-reffer ,ref-trans ,offset ,lowtag)))
-          #-c-headers-only
+            (forms `(def-reffer ',ref-trans ,offset ,lowtag)))
           (when set-trans
             (when set-known-p
               (forms `(defknown ,set-trans
@@ -124,8 +124,7 @@
                                      (list type slot-type))
                                 ,slot-type
                         ,set-known)))
-            (forms `(def-setter ,set-trans ,offset ,lowtag)))
-          #-c-headers-only
+            (forms `(def-setter ',set-trans ,offset ,lowtag)))
           (when cas-trans
             (when rest-p
               (error ":REST-P and :CAS-TRANS incompatible."))
@@ -133,33 +132,31 @@
              `(progn
                 (defknown ,cas-trans (,type ,slot-type ,slot-type)
                     ,slot-type ())
-                #!+compare-and-swap-vops
-                (def-casser ,cas-trans ,offset ,lowtag))))
+                #+compare-and-swap-vops
+                (def-casser ',cas-trans ,offset ,lowtag))))
           (when init
             (inits (cons init offset)))
           (when rest-p
             (setf variable-length-p t))
           (incf offset length)))
       (unless variable-length-p
-        (constants `(defconstant ,(symbolicate name "-SIZE") ,offset)))
-      #-c-headers-only
+        (constants `(defconstant ,size ,offset)))
       (when alloc-trans
-        (forms `(def-alloc ,alloc-trans ,offset
+        (forms `(def-alloc ',alloc-trans ,offset
                   ,(if variable-length-p :var-alloc :fixed-alloc)
                   ,widetag
                   ,lowtag ',(inits))))
       `(progn
-         (eval-when (:compile-toplevel :load-toplevel :execute)
-           (setf (info :type :source-location ',name) (source-location))
-           (!%define-primitive-object
-            ',(make-primitive-object :name name
-                                     :widetag widetag
-                                     :lowtag lowtag
-                                     :slots (slots)
-                                     :length offset
-                                     :variable-length-p variable-length-p))
-           ,@(constants)
-           ,@(specials))
+         (setf (info :type :source-location ',name) (source-location))
+         (!%define-primitive-object
+            (make-primitive-object :name ',name
+                                   :widetag ',widetag
+                                   :lowtag ',lowtag
+                                   :slots (list ,@(slots))
+                                   :length ,offset
+                                   :variable-length-p ,variable-length-p))
+         ,@(constants)
+         ,@(specials)
          (setf *!late-primitive-object-forms*
                (append *!late-primitive-object-forms*
                        ',(forms)))))))
@@ -190,39 +187,66 @@
 ;;; storage class. Applies to all backends.
 (defconstant sc-number-limit 62)
 (defconstant sc-number-bits (integer-length (1- sc-number-limit)))
-(def!type sb!c::sc-number () `(integer 0 (,sc-number-limit)))
+(def!type sb-c::sc-number () `(integer 0 (,sc-number-limit)))
 
 (defconstant sc-offset-limit (ash 1 21))
 (defconstant sc-offset-bits (integer-length (1- sc-offset-limit)))
 (deftype sc-offset () `(integer 0 (,sc-offset-limit)))
 
 (defconstant finite-sc-offset-limit
-  #!-(or sparc alpha hppa) 32
-  #!+(or sparc alpha hppa) 64)
+  #-(or sparc alpha hppa) 32
+  #+(or sparc alpha hppa) 64)
 (defconstant finite-sc-offset-bits
   (integer-length (1- finite-sc-offset-limit)))
 (deftype finite-sc-offset () `(integer 0 (,finite-sc-offset-limit)))
+(deftype finite-sc-offset-map () `(unsigned-byte ,finite-sc-offset-limit))
 
 ;;;; stuff for defining reffers and setters
 
-(in-package "SB!C")
+(in-package "SB-C")
+
+(defun def-reffer (name offset lowtag)
+  (let ((fun-info (fun-info-or-lose name)))
+    (setf (fun-info-ir2-convert fun-info)
+          (lambda (node block)
+            (ir2-convert-reffer node block name offset lowtag))))
+  name)
+
+(defun def-setter (name offset lowtag)
+  (let ((fun-info (fun-info-or-lose name)))
+    (setf (fun-info-ir2-convert fun-info)
+          (if (listp name)
+              (lambda (node block)
+                (ir2-convert-setfer node block name offset lowtag))
+              (lambda (node block)
+                (ir2-convert-setter node block name offset lowtag)))))
+  name)
+
+(defun def-alloc (name words allocation-style header lowtag inits)
+  (let ((info (fun-info-or-lose name)))
+    (setf (fun-info-ir2-convert info)
+          (ecase allocation-style
+            (:var-alloc
+             (lambda (node block)
+                (ir2-convert-variable-allocation node block name words header
+                                                 lowtag inits)))
+            (:fixed-alloc
+             (lambda (node block)
+               (ir2-convert-fixed-allocation node block name words header
+                                             lowtag inits)))
+            (:structure-alloc
+             (lambda (node block)
+               (ir2-convert-structure-allocation node block name words header
+                                                 lowtag inits))))))
+  name)
+
+#+compare-and-swap-vops ; same as IR2-CONVERT-CASSER
+(defun def-casser (name offset lowtag)
+  (let ((fun-info (fun-info-or-lose name)))
+    (setf (fun-info-ir2-convert fun-info)
+          (lambda (node block)
+            (ir2-convert-casser node block name offset lowtag)))))
 
-(defmacro def-reffer (name offset lowtag)
-  `(%def-reffer ',name ,offset ,lowtag))
-(defmacro def-setter (name offset lowtag)
-  `(%def-setter ',name ,offset ,lowtag))
-(defmacro def-alloc (name words alloc-style header lowtag inits)
-  `(%def-alloc ',name ,words ,alloc-style ,header ,lowtag ,inits))
-#!+compare-and-swap-vops
-(defmacro def-casser (name offset lowtag)
-  `(%def-casser ',name ,offset ,lowtag))
-;;; KLUDGE: The %DEF-FOO functions used to implement the macros here
-;;; are defined later in another file, since they use structure slot
-;;; setters defined later, and we can't have physical forward
-;;; references to structure slot setters because ANSI in its wisdom
-;;; allows the xc host CL to implement structure slot setters as SETF
-;;; expanders instead of SETF functions. -- WHN 2002-02-09
-
 ;;; Modular functions
 
 ;;; For a documentation, see CUT-TO-WIDTH.
@@ -237,9 +261,9 @@
   (versions (make-hash-table :test 'eq))
   ;; list of increasing widths + signedps
   (widths nil))
-(defvar *untagged-unsigned-modular-class* (make-modular-class))
-(defvar *untagged-signed-modular-class* (make-modular-class))
-(defvar *tagged-modular-class* (make-modular-class))
+(define-load-time-global *untagged-unsigned-modular-class* (make-modular-class))
+(define-load-time-global *untagged-signed-modular-class* (make-modular-class))
+(define-load-time-global *tagged-modular-class* (make-modular-class))
 (defun find-modular-class (kind signedp)
   (ecase kind
     (:untagged
@@ -308,7 +332,7 @@
   (check-type kind (member :untagged :tagged))
   (when lambda-list-p
     (dolist (arg lambda-list)
-      (when (member arg sb!xc:lambda-list-keywords)
+      (when (member arg sb-xc:lambda-list-keywords)
         (error "Lambda list keyword ~S is not supported for modular ~
                 function lambda lists." arg)))))
 
