@@ -32,7 +32,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!C")
+(in-package "SB-IMPL")
 
 #-no-ansi-print-object
 (defmethod print-object ((x meta-info) stream)
@@ -40,41 +40,6 @@
     (format stream "~S ~S, ~D" (meta-info-category x) (meta-info-kind x)
             (meta-info-number x))))
 
-(!begin-collecting-cold-init-forms)
-#!+sb-show (!cold-init-forms (/show0 "early in globaldb.lisp cold init"))
-
-;;; This is sorta semantically equivalent to SXHASH, but better-behaved for
-;;; legal function names. It performs more work by not cutting off as soon
-;;; in the CDR direction, thereby improving the distribution of method names.
-;;; More work here equates to less work in the global hashtable.
-;;; To wit: (eq (sxhash '(foo a b c bar)) (sxhash '(foo a b c d))) => T
-;;; but the corresponding globaldb-sxhashoids differ.
-;;; This is no longer inline because for the cases where it is needed -
-;;; names which are not just symbols or (SETF F) - an extra call has no impact.
-(defun globaldb-sxhashoid (name)
-  ;; we can't use MIX because it's in 'target-sxhash',
-  ;; so use the host's sxhash, but ensure that the result is a target fixnum.
-  #+sb-xc-host (logand (sxhash name) sb!xc:most-positive-fixnum)
-  #-sb-xc-host
-  (locally
-      (declare (optimize (safety 0))) ; after the argc check
-    ;; TRAVERSE will walk across more cons cells than RECURSE will descend.
-    ;; That's why this isn't just one self-recursive function.
-    (labels ((traverse (accumulator x length-limit)
-               (declare (fixnum length-limit))
-               (cond ((atom x) (sb!int:mix (sxhash x) accumulator))
-                     ((zerop length-limit) accumulator)
-                     (t (traverse (sb!int:mix (recurse (car x) 4) accumulator)
-                                  (cdr x) (1- length-limit)))))
-             (recurse (x depthoid) ; depthoid = a blend of level and length
-               (declare (fixnum depthoid))
-               (cond ((atom x) (sxhash x))
-                     ((zerop depthoid)
-                      #.(logand sb!xc:most-positive-fixnum #36Rglobaldbsxhashoid))
-                     (t (sb!int:mix (recurse (car x) (1- depthoid))
-                                    (recurse (cdr x) (1- depthoid)))))))
-      (traverse 0 name 10))))
-
 ;;; Given any non-negative integer, return a prime number >= to it.
 ;;;
 ;;; FIXME: This logic should be shared with ALMOST-PRIMIFY in
@@ -101,7 +66,6 @@
 ;;; sources partway through bootstrapping, tch tch, overwriting its
 ;;; version with our version would be unlikely to help, because that
 ;;; would make the cross-compiler very confused.)
-(eval-when (#-sb-xc :compile-toplevel :load-toplevel :execute)
 (defun !register-meta-info (metainfo)
   (let* ((name (meta-info-kind metainfo))
          (list (!get-meta-infos name)))
@@ -123,31 +87,23 @@
            (!make-meta-info id category kind type-spec type-checker
                             validate-function default)))))
 
-) ; EVAL-WHEN
-
 #-sb-xc
 (setf (get '!%define-info-type :sb-cold-funcall-handler/for-effect)
       (lambda (category kind type-spec checker validator default id)
-        ;; The SB!FASL: symbols are poor style, but the lesser evil.
+        ;; The SB-FASL: symbols are poor style, but the lesser evil.
         ;; If exported, then they'll stick around in the target image.
         ;; Perhaps SB-COLD should re-export some of these.
-        (declare (special sb!fasl::*dynamic* sb!fasl::*cold-layouts*))
-        (let ((layout (gethash 'meta-info sb!fasl::*cold-layouts*)))
-          (sb!fasl::cold-svset
-           (sb!fasl::cold-symbol-value '*info-types*)
+        (declare (special sb-fasl::*dynamic* sb-fasl::*cold-layouts*))
+        (let ((layout (gethash 'meta-info sb-fasl::*cold-layouts*)))
+          (sb-fasl::cold-svset
+           (sb-fasl::cold-symbol-value '*info-types*)
            id
-           (sb!fasl::write-slots
-            (sb!fasl::allocate-struct sb!fasl::*dynamic* layout)
+           (sb-fasl::write-slots
+            (sb-fasl::allocate-struct sb-fasl::*dynamic* layout)
             'meta-info ; give the type name in lieu of layout
             :category category :kind kind :type-spec type-spec
             :type-checker checker :validate-function validator
             :default default :number id)))))
-
-(!cold-init-forms
- (dovector (x (the simple-vector *info-types*))
-   ;; Genesis writes the *INFO-TYPES* array, but setting up the mapping
-   ;; from keyword-pair to object is deferred until cold-init.
-   (when x (!register-meta-info x))))
 
 ;;;; info types, and type numbers, part II: what's
 ;;;; needed only at compile time, not at run time
@@ -168,43 +124,26 @@
 ;;;  :DEFAULT (CONSTANTLY #'<a-function-name>) to adhere to the convention
 ;;; that default objects satisfying FUNCTIONP will always be funcalled.
 ;;;
-(eval-when (:compile-toplevel :execute)
-;; This convoluted idiom creates a macro that disappears from the target,
-;; kind of an alternative to the "!" name convention.
-(#+sb-xc-host defmacro
- #-sb-xc-host sb!xc:defmacro
-    define-info-type ((category kind)
-                       &key (type-spec (missing-arg))
-                            (validate-function)
-                            default)
+(defmacro define-info-type ((category kind)
+                            &key (type-spec (missing-arg))
+                                 (validate-function)
+                                 default)
   (declare (type keyword category kind))
   ;; There was formerly a remark that (COPY-TREE TYPE-SPEC) ensures repeatable
   ;; fasls. That's not true now, probably never was. A compiler is permitted to
   ;; coalesce EQUAL quoted lists and there's no defense against it, so why try?
-  (let ((form
-         `(!%define-info-type
+  `(!%define-info-type
            ,category ,kind ',type-spec
-           ,(cond ((eq type-spec 't) '#'identity)
-                  ;; evil KLUDGE to avoid "undefined type" warnings
-                  ;; when building the cross-compiler.
-                  #+sb-xc-host
-                  ((member type-spec
-                           '((or fdefn null)
-                             (or alien-type null) (or heap-alien-info null))
-                           :test 'equal)
-                   `(lambda (x)
-                      (declare (notinline typep))
-                      (if (typep x ',type-spec)
-                          x
-                          (error "~S is not a ~S" x ',type-spec))))
-                  (t
-                   `(named-lambda "check-type" (x) (the ,type-spec x))))
+           ,(if (eq type-spec 't)
+                '#'identity
+                `(named-lambda "check-type" (x) (the ,type-spec x)))
            ,validate-function ,default
            ;; Rationale for hardcoding here is explained at INFO-VECTOR-FDEFN.
            ,(or (and (eq category :function) (eq kind :definition)
                      +fdefn-info-num+)
-                #+sb-xc (meta-info-number (meta-info category kind))))))
-    `(eval-when (#-sb-xc :compile-toplevel :load-toplevel :execute) ,form))))
+                #+sb-xc (meta-info-number (meta-info category kind)))))
+;; It's an external symbol of SB-INT so wouldn't be removed automatically
+(push '("SB-INT" define-info-type) *!removable-symbols*)
 
 
 (macrolet ((meta-info-or-lose (category kind)
@@ -269,20 +208,30 @@
 
 ;;;; *INFO-ENVIRONMENT*
 
-(!cold-init-forms
-  (setq *info-environment* (make-info-hashtable))
-  (/show0 "done setting *INFO-ENVIRONMENT*"))
+(defun !globaldb-cold-init ()
+  ;; Genesis writes the *INFO-TYPES* array, but setting up the mapping
+  ;; from keyword-pair to object is deferred until cold-init.
+  (dovector (x (the simple-vector *info-types*))
+    (when x (!register-meta-info x)))
+  #-sb-xc-host (setq *info-environment* (make-info-hashtable)))
 
 ;;;; GET-INFO-VALUE
 
 ;;; If non-nil, *GLOBALDB-OBSERVER*'s CAR is a bitmask over info numbers
 ;;; for which you'd like to call the function in the CDR whenever info
 ;;; of that number is queried.
-(!defvar *globaldb-observer* nil)
+(defparameter *globaldb-observer* nil)
 (declaim (type (or (cons (unsigned-byte #.(ash 1 info-number-bits)) function)
                    null) *globaldb-observer*))
 #-sb-xc-host (declaim (always-bound *globaldb-observer*))
 
+#+sb-xc-host
+(progn
+  (defun info-gethash (key table) (gethash key table))
+  (defun info-puthash (table key augmenter)
+    (let ((old (gethash key table)))
+      (setf (gethash key table) (funcall augmenter old)))))
+
 ;;; Return the value of NAME / INFO-NUMBER from the global environment,
 ;;; or return the default if there is no global info.
 ;;; The secondary value indicates whether info was found vs defaulted.
@@ -314,50 +263,10 @@
       (when hookp
         (funcall (truly-the function (cdr hook)) name info-number answer nil))
       (values answer nil))))
-
-;; interface to %ATOMIC-SET-INFO-VALUE
-;; GET-INFO-VALUE-INITIALIZING is a restricted case of this,
-;; and perhaps could be implemented as such.
-;; Atomic update will be important for making the fasloader threadsafe
-;; using a predominantly lock-free design, and other nice things.
-(def!macro atomic-set-info-value (category kind name lambda)
-  (with-unique-names (info-number proc)
-    `(let ((,info-number
-            ,(if (and (keywordp category) (keywordp kind))
-                 (meta-info-number (meta-info category kind))
-                 `(meta-info-number (meta-info ,category ,kind)))))
-       ,(if (and (listp lambda) (eq (car lambda) 'lambda))
-            ;; rewrite as FLET because the compiler is unable to dxify
-            ;;   (DX-LET ((x (LAMBDA <whatever>))) (F x))
-            (destructuring-bind (lambda-list . body) (cdr lambda)
-              `(dx-flet ((,proc ,lambda-list ,@body))
-                 (%atomic-set-info-value ,name ,info-number #',proc)))
-            `(%atomic-set-info-value ,name ,info-number ,lambda)))))
-
-;; Call FUNCTION once for each Name in globaldb that has information associated
-;; with it, passing the function the Name as its only argument.
-;;
-(defun call-with-each-globaldb-name (fun-designator)
-  (let ((function (coerce fun-designator 'function)))
-    (with-package-iterator (iter (list-all-packages) :internal :external)
-      (loop (multiple-value-bind (winp symbol access package) (iter)
-              (declare (ignore access))
-              (if (not winp) (return))
-              ;; Try to process each symbol at most once by associating it with
-              ;; a single package. If a symbol is apparently uninterned,
-              ;; always keep it since we can't know if it has been seen once.
-              (when (or (not (symbol-package symbol))
-                        (eq package (symbol-package symbol)))
-                (dolist (name (info-vector-name-list symbol))
-                  (funcall function name))))))
-    (info-maphash (lambda (name data)
-                    (declare (ignore data))
-                    (funcall function name))
-                  *info-environment*)))
 
 ;;;; ":FUNCTION" subsection - Data pertaining to globally known functions.
 
-(define-info-type (:function :definition) :type-spec (or fdefn null))
+(define-info-type (:function :definition) :type-spec (or #-sb-xc-host fdefn null))
 
 ;;; the kind of functional object being described. If null, NAME isn't
 ;;; a known functional object.
@@ -373,38 +282,17 @@
   #+sb-xc-host nil
   #-sb-xc-host (lambda (name) (if (fboundp name) :function nil)))
 
+;;; The deferred mode processor for fasteval special operators.
+;;; Immediate processors are hung directly off symbols in a dedicated slot.
+#+sb-fasteval
+(define-info-type (:function :interpreter) :type-spec (or function null))
+
 ;;; Indicates whether the function is deprecated.
 (define-info-type (:function :deprecated)
   :type-spec (or null deprecation-info))
 
 (declaim (ftype (sfunction (t) ctype)
-                specifier-type ctype-of sb!kernel::ctype-of-array))
-
-(eval-when (#-sb-xc :compile-toplevel :load-toplevel :execute)
-(defun ftype-from-fdefn (name)
-  (declare (ignorable name))
-  ;; Again [as in (DEFINE-INFO-TYPE (:FUNCTION :TYPE) ...)] it's
-  ;; not clear how to generalize the FBOUNDP expression to the
-  ;; cross-compiler. -- WHN 19990330
-  #+sb-xc-host
-  (specifier-type 'function)
-  #-sb-xc-host
-  (let* ((fdefn (sb!kernel::find-fdefn name))
-         (fun (and fdefn (fdefn-fun fdefn))))
-    (if fun
-        (handler-bind ((style-warning #'muffle-warning))
-          (specifier-type (sb!impl::%fun-type fun)))
-        (specifier-type 'function)))))
-
-;;; The type specifier for this function, or a DEFSTRUCT-DESCRIPTION
-;;; or the symbol :GENERIC-FUNTION.
-;;; If a DD, it must contain a constructor whose name is
-;;; the one being sought in globaldb, which is used to derive the type.
-;;; If :GENERIC-FUNCTION, the info is recomputed from existing methods
-;;; and stored back into globaldb.
-(define-info-type (:function :type)
-  :type-spec (or ctype defstruct-description (member :generic-function))
-  :default #'ftype-from-fdefn)
+                specifier-type ctype-of sb-kernel::ctype-of-array))
 
 ;;; the ASSUMED-TYPE for this function, if we have to infer the type
 ;;; due to not having a declaration or definition
@@ -435,17 +323,21 @@
   #+sb-xc-host :assumed
   #-sb-xc-host (lambda (name) (if (fboundp name) :defined :assumed)))
 
-;;; something which can be decoded into the inline expansion of the
-;;; function, or NIL if there is none
-;;;
-;;; To inline a function, we want a lambda expression, e.g.
-;;; '(LAMBDA (X) (+ X 1)).
-(define-info-type (:function :inline-expansion-designator)
-  :type-spec list)
+;;; Two kinds of hints for compiling calls as efficiently as possible:
+;;; (A) Inline expansion: To inline a function, we want a lambda
+;;; expression, e.g. '(LAMBDA (X) (+ X 1)) or a lambda-with-lexenv.
+;;; (B) List of arguments which could be dynamic-extent closures, and which
+;;; we could, under suitable compilation policy, DXify in the caller
+;;; especially when open-coding a call to this function.
+;;; If only (A) is stored, then this value is a list (the lambda expression).
+;;; If only (B) is stored, then this is a DXABLE-ARGS.
+;;; If both, this is an INLINING-DATA.
+(define-info-type (:function :inlining-data)
+    :type-spec (or list sb-c::dxable-args sb-c::inlining-data))
 
 ;;; This specifies whether this function may be expanded inline. If
 ;;; null, we don't care.
-(define-info-type (:function :inlinep) :type-spec inlinep)
+(define-info-type (:function :inlinep) :type-spec sb-c::inlinep)
 
 ;;; Track how many times IR2 converted a call to this function as a full call
 ;;; that was not in the scope of a local or global notinline declaration.
@@ -465,6 +357,8 @@
 ;;; We don't actually have anything like that any more though.
 ;;; For user-defined functions, the invariant is maintained that at most
 ;;; one of :source-transform and an inline-expansion exist.
+;;; However, there is one exception: a structure constructor can have an
+;;; inline expansion and also store (#<dd> . :constructor) here.
 (define-info-type (:function :source-transform)
   :type-spec (or function null (cons atom atom)))
 
@@ -480,7 +374,7 @@
 
 ;;; If a function is "known" to the compiler, then this is a FUN-INFO
 ;;; structure containing the info used to special-case compilation.
-(define-info-type (:function :info) :type-spec (or fun-info null))
+(define-info-type (:function :info) :type-spec (or sb-c::fun-info null))
 
 ;;; This is a type specifier <t> such that if an argument X to the function
 ;;; does not satisfy (TYPEP x <t>) then the function definitely returns NIL.
@@ -510,8 +404,10 @@
 ;;; the declared type for this variable
 (define-info-type (:variable :type)
   :type-spec ctype
-  ;; This gets set to *UNIVERSAL-TYPE* in 'late-type'
-  :default (lambda (x) (declare (ignore x)) (error "Too early for INFO")))
+  :default #+sb-xc-host (lambda (x)
+                          (declare (special *universal-type*) (ignore x))
+                          *universal-type*)
+           #-sb-xc-host *universal-type*)
 
 ;;; where this type and kind information came from
 (define-info-type (:variable :where-from)
@@ -521,15 +417,14 @@
 (define-info-type (:variable :macro-expansion) :type-spec t)
 
 (define-info-type (:variable :alien-info)
-  :type-spec (or heap-alien-info null))
+  :type-spec (or null sb-alien-internals:heap-alien-info))
 
 (define-info-type (:variable :documentation) :type-spec (or string null))
 
 ;; :WIRED-TLS describes how SYMBOL-VALUE (implicit or not) should be compiled.
-;;  - :ALWAYS-HAS-TLS means that calls to SYMBOL-VALUE should access the TLS
-;;     with a fixed offset. The index is assigned no later than load-time of
-;;     the file containing code thus compiled. Presence of an index in the
-;;     image that performed compilation is irrelevant (for now).
+;;  - T means that SYMBOL-VALUE should access the TLS with a fixed offset,
+;;     resolved at load-time. Presence of an index in the image that performed
+;;     compilation is irrelevant (for now).
 ;;  - :ALWAYS-THREAD-LOCAL implies a fixed offset, *and* that the check for
 ;;     no-tls-value may be elided. There is currently no way to set this.
 ;;     Note that this does not affect elision of the check for unbound-marker
@@ -545,24 +440,9 @@
 ;; ever referenced by SYMBOL-VALUE or SET. Depletion should occur lazily.
 ;;
 (define-info-type (:variable :wired-tls)
-    :type-spec (or (member nil :always-has-tls :always-thread-local)
-                   fixnum) ; the actual index, for thread slots (to be done)
-    :default
-    (lambda (symbol)
-      (declare (symbol symbol))
-      (and (eq (info :variable :kind symbol) :special)
-           #-sb-xc-host
-           (eq (symbol-package symbol) *cl-package*)
-           #+sb-xc-host
-           (flet ((external-in-package-p (pkg)
-                    (and (string= (package-name (symbol-package symbol)) pkg)
-                         (eq (nth-value 1 (find-symbol (string symbol) pkg))
-                             :external))))
-             ;; I'm not worried about random extra externals in some bizarro
-             ;; host lisp. TLS assignment has no bearing on semantics at all.
-             (or (external-in-package-p "COMMON-LISP")
-                 (external-in-package-p "SB-XC")))
-           :always-has-tls)))
+    :type-spec (or (member nil t :always-thread-local)
+                   fixnum) ; the actual index, for thread slots
+    :default nil)
 
 ;;;; ":TYPE" subsection - Data pertaining to globally known types.
 
@@ -600,34 +480,6 @@
 ;;; The classoid-cell for this type
 (define-info-type (:type :classoid-cell) :type-spec t)
 
-(defun find-classoid-cell (name &key create)
-  (let ((real-name (uncross name)))
-    (cond ((info :type :classoid-cell real-name))
-          (create
-           (get-info-value-initializing
-            :type :classoid-cell real-name
-            (sb!kernel::make-classoid-cell real-name))))))
-
-;;; Return the classoid with the specified NAME. If ERRORP is false,
-;;; then NIL is returned when no such class exists.
-(defun find-classoid (name &optional (errorp t))
-  (declare (type symbol name))
-  (let ((cell (find-classoid-cell name)))
-    (cond ((and cell (classoid-cell-classoid cell)))
-          (errorp
-           (error 'simple-type-error
-                  :datum nil
-                  :expected-type 'class
-                  :format-control "Class not yet defined: ~S"
-                  :format-arguments (list name))))))
-
-;;; layout for this type being used by the compiler
-(define-info-type (:type :compiler-layout)
-  :type-spec (or layout null)
-  :default (lambda (name)
-             (let ((class (find-classoid name nil)))
-               (when class (classoid-layout class)))))
-
 ;;; DEFTYPE lambda-list
 ;; FIXME: remove this after making swank-fancy-inspector not use it.
 (define-info-type (:type :lambda-list) :type-spec t)
@@ -672,16 +524,19 @@
   :type-spec (member :primitive :defined :unknown)
   :default :unknown)
 (define-info-type (:alien-type :translator) :type-spec (or function null))
-(define-info-type (:alien-type :definition) :type-spec (or alien-type null))
-(define-info-type (:alien-type :struct) :type-spec (or alien-type null))
-(define-info-type (:alien-type :union) :type-spec (or alien-type null))
-(define-info-type (:alien-type :enum) :type-spec (or alien-type null))
+(define-info-type (:alien-type :definition)
+  :type-spec (or null sb-alien-internals:alien-type))
+(define-info-type (:alien-type :struct)
+  :type-spec (or null sb-alien-internals:alien-type))
+(define-info-type (:alien-type :union)
+  :type-spec (or null sb-alien-internals:alien-type))
+(define-info-type (:alien-type :enum)
+  :type-spec (or null sb-alien-internals:alien-type))
 
 ;;;; ":SETF" subsection - Data pertaining to expansion of the omnipotent macro.
-(define-info-type (:setf :inverse) :type-spec (or symbol null))
 (define-info-type (:setf :documentation) :type-spec (or string null))
 (define-info-type (:setf :expander)
-    :type-spec (or function (cons integer function) null))
+    :type-spec (or symbol function (cons integer function) null))
 
 ;;;; ":CAS" subsection - Like SETF but there are no "inverses", just expanders
 (define-info-type (:cas :expander) :type-spec (or function null))
@@ -704,7 +559,6 @@
 (define-info-type (:source-location :constant) :type-spec t)
 (define-info-type (:source-location :typed-structure) :type-spec t)
 (define-info-type (:source-location :symbol-macro) :type-spec t)
-(define-info-type (:source-location :vop) :type-spec t)
 (define-info-type (:source-location :declaration) :type-spec t)
 (define-info-type (:source-location :alien-type) :type-spec t)
 
@@ -725,7 +579,5 @@
                  (if (not type) type-num)
                  (if type
                      (list (meta-info-category type) (meta-info-kind type))))
-         (write val :level 1)))
+         (write val :level 2)))
      sym)))
-
-(!defun-from-collected-cold-init-forms !globaldb-cold-init)
