@@ -11,7 +11,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!C")
+(in-package "SB-C")
 
 ;;;; LAMBDA hackery
 
@@ -20,57 +20,68 @@
 ;;;; function representation" before you seriously mess with this
 ;;;; stuff.
 
-;;; Verify that the NAME is a legal name for a variable and return a
-;;; VAR structure for it, filling in info if it is globally special.
-;;; If it is losing, we punt with a COMPILER-ERROR. NAMES-SO-FAR is a
-;;; list of names which have previously been bound. If the NAME is in
-;;; this list, then we error out.
-(declaim (ftype (sfunction (t list &optional t) lambda-var) varify-lambda-arg))
-(defun varify-lambda-arg (name names-so-far &optional (context "lambda list"))
-  (declare (inline member))
-  (unless (symbolp name) ;; FIXME: probably unreachable. Change to AVER?
-    (compiler-error "~S is not a symbol, and cannot be used as a variable." name))
-  (when (member name names-so-far :test #'eq)
-    (compiler-error "The variable ~S occurs more than once in the ~A."
-                    name
-                    context))
-  (let ((kind (info :variable :kind name)))
-    (cond ((keywordp name)
-           (compiler-error "~S is a keyword, and cannot be used as a local variable."
-                           name))
-          ((eq kind :constant)
-           (compiler-error "~@<~S names a defined constant, and cannot be used as a ~
-                            local variable.~:@>"
-                           name))
-          ((eq :global kind)
-           (compiler-error "~@<~S names a global lexical variable, and cannot be used ~
-                            as a local variable.~:@>"
-                           name))
-          ((eq kind :special)
-           (let ((specvar (find-free-var name)))
-             (make-lambda-var :%source-name name
-                              :type (leaf-type specvar)
-                              :where-from (leaf-where-from specvar)
-                              :specvar specvar)))
-          (t
-           (make-lambda-var :%source-name name)))))
+(declaim (ftype (sfunction * function) make-repeated-name-check))
+(defun make-repeated-name-check (&key
+                                   (kind "variable")
+                                   (context "lambda list")
+                                   (signal-via #'compiler-error))
+  (let ((seen '()))
+    (lambda (name)
+      (when (member name seen :test #'eq)
+        (funcall signal-via "~@<The ~A ~S occurs more than once in ~
+                             the ~A.~@:>"
+                 kind name context))
+      (push name seen)
+      name)))
 
-;;; Make the default keyword for a &KEY arg, checking that the keyword
-;;; isn't already used by one of the VARS.
-(declaim (ftype (sfunction (symbol list t) symbol) make-keyword-for-arg))
-(defun make-keyword-for-arg (symbol vars keywordify)
-  (let ((key (if (and keywordify (not (keywordp symbol)))
-                 (keywordicate symbol)
-                 symbol)))
-    (dolist (var vars)
-      (let ((info (lambda-var-arg-info var)))
-        (when (and info
-                   (eq (arg-info-kind info) :keyword)
-                   (eq (arg-info-key info) key))
-          (compiler-error
-           "The keyword ~S appears more than once in the lambda list."
-           key))))
-    key))
+;;; Verify that NAME is a legal name for a variable.
+(declaim (ftype (function (t &key
+                             (:context t) (:allow-special t) (:allow-symbol-macro t)
+                             (:signal-via (or symbol function)))
+                          (values symbol keyword))
+                check-variable-name-for-binding))
+(defun check-variable-name-for-binding (name
+                                        &key
+                                          context
+                                          (allow-special t)
+                                          (allow-symbol-macro t)
+                                          (signal-via #'compiler-error))
+  (check-variable-name name :signal-via signal-via)
+  (flet ((lose (kind)
+           (funcall signal-via
+                    (sb-format:tokens "~@<~/sb-ext:print-symbol-with-prefix/ names a ~
+                               ~A, and cannot be used in ~A.~:@>")
+                    name kind context)))
+    (let ((kind (info :variable :kind name)))
+      (case kind
+        (:macro
+         (unless allow-symbol-macro
+           (program-assert-symbol-home-package-unlocked
+            :compile name (format nil "lexically binding global ~
+                                       symbol-macro ~~A in ~A"
+                                  context))))
+        ((:constant)
+         (lose "defined constant"))
+        ((:global)
+         (lose "global lexical variable"))
+        (:special
+         (unless allow-special
+           (lose "special variable"))))
+      (values name kind))))
+
+;;; Return a VAR structure for NAME, filling in info if it is globally
+;;; special. If it is losing, we punt with a COMPILER-ERROR.
+(declaim (ftype (sfunction (symbol) lambda-var) varify-lambda-arg))
+(defun varify-lambda-arg (name)
+  (case (info :variable :kind name)
+    (:special
+     (let ((variable (find-free-var name)))
+       (make-lambda-var :%source-name name
+                        :type (leaf-type variable)
+                        :where-from (leaf-where-from variable)
+                        :specvar variable)))
+    (t
+     (make-lambda-var :%source-name name))))
 
 ;;; Parse a lambda list into a list of VAR structures, stripping off
 ;;; any &AUX bindings. Each arg name is checked for legality, and
@@ -88,94 +99,54 @@
                 make-lambda-vars))
 (defun make-lambda-vars (list)
   (multiple-value-bind (llks required optional rest/more keys aux)
-      (parse-lambda-list list)
+      (multiple-value-call #'check-lambda-list-names (parse-lambda-list list))
     (collect ((vars)
-              (names-so-far)
               (aux-vars)
               (aux-vals))
-      (flet (;; PARSE-DEFAULT deals with defaults and supplied-p args
-             ;; for optionals and keywords args.
-             (parse-default (spec info)
-               (when (consp (cdr spec))
-                 (setf (arg-info-default info) (second spec))
-                 (when (consp (cddr spec))
-                   (let* ((supplied-p (third spec))
-                          (supplied-var (varify-lambda-arg supplied-p
-                                                           (names-so-far))))
-                     (setf (arg-info-supplied-p info) supplied-var)
-                     (names-so-far supplied-p))))))
-
-        (dolist (name required)
-          (let ((var (varify-lambda-arg name (names-so-far))))
-            (vars var)
-            (names-so-far name)))
-
+      (flet ((add-var (name)
+               (let ((var (varify-lambda-arg name)))
+                 (vars var)
+                 var))
+             (add-info (var kind &key (default nil defaultp) suppliedp-var key)
+               (let ((info (make-arg-info :kind kind)))
+                 (when defaultp
+                   (setf (arg-info-default info) default))
+                 (when suppliedp-var
+                   (setf (arg-info-supplied-p info)
+                         (varify-lambda-arg suppliedp-var)))
+                 (when key
+                   (setf (arg-info-key info) key))
+                 (setf (lambda-var-arg-info var) info))))
+        ;; Required
+        (mapc #'add-var required)
+        ;; Optional
         (dolist (spec optional)
-          (if (atom spec)
-              (let ((var (varify-lambda-arg spec (names-so-far))))
-                (setf (lambda-var-arg-info var)
-                      (make-arg-info :kind :optional))
-                (vars var)
-                (names-so-far spec))
-              (let* ((name (first spec))
-                     (var (varify-lambda-arg name (names-so-far)))
-                     (info (make-arg-info :kind :optional)))
-                (setf (lambda-var-arg-info var) info)
-                (vars var)
-                (names-so-far name)
-                (parse-default spec info))))
-
+          (multiple-value-bind (name default suppliedp-var defaultp)
+              (parse-optional-arg-spec spec)
+            (apply #'add-info (add-var name) :optional
+                   :suppliedp-var (first suppliedp-var)
+                   (when defaultp (list :default default)))))
+        ;; Rest/more
         (when rest/more
           (mapc (lambda (name kind)
-                  (let ((var (varify-lambda-arg name (names-so-far))))
-                    (setf (lambda-var-arg-info var) (make-arg-info :kind kind))
-                    (vars var)
-                    (names-so-far name)))
+                  (add-info (add-var name) kind))
                 rest/more (let ((morep (eq (ll-kwds-restp llks) '&more)))
                             (if morep '(:more-context :more-count) '(:rest)))))
-
+        ;; Keys
         (dolist (spec keys)
-          (cond
-           ((atom spec)
-            (let ((var (varify-lambda-arg spec (names-so-far))))
-              (setf (lambda-var-arg-info var)
-                    (make-arg-info :kind :keyword
-                                   :key (make-keyword-for-arg spec
-                                                              (vars)
-                                                              t)))
-              (vars var)
-              (names-so-far spec)))
-           ((atom (first spec))
-            (let* ((name (first spec))
-                   (var (varify-lambda-arg name (names-so-far)))
-                   (info (make-arg-info
-                          :kind :keyword
-                          :key (make-keyword-for-arg name (vars) t))))
-              (setf (lambda-var-arg-info var) info)
-              (vars var)
-              (names-so-far name)
-              (parse-default spec info)))
-           (t
-            (let ((head (first spec)))
-              (let* ((name (second head))
-                     (var (varify-lambda-arg name (names-so-far)))
-                     (info (make-arg-info
-                            :kind :keyword
-                            :key (make-keyword-for-arg (first head)
-                                                       (vars)
-                                                       nil))))
-                (setf (lambda-var-arg-info var) info)
-                (vars var)
-                (names-so-far name)
-                (parse-default spec info))))))
-
+          (multiple-value-bind (keyword name default suppliedp-var defaultp)
+              (parse-key-arg-spec spec)
+            (apply #'add-info (add-var name) :keyword
+                   :suppliedp-var (first suppliedp-var)
+                   :key keyword
+                   (when defaultp (list :default default)))))
+        ;; Aux
         (dolist (spec aux)
           (multiple-value-bind (name val)
               (if (atom spec) spec (values (car spec) (cadr spec)))
-            (let ((var (varify-lambda-arg name nil)))
+            (let ((var (varify-lambda-arg name)))
               (aux-vars var)
-              (aux-vals val)
-              (names-so-far name))))
+              (aux-vals val))))
 
         (values (vars) (ll-kwds-keyp llks) (ll-kwds-allowp llks)
                 (aux-vars) (aux-vals))))))
@@ -193,7 +164,7 @@
   (declare (type ctran start next) (type (or lvar null) result)
            (list body aux-vars aux-vals))
   (if (null aux-vars)
-      (let ((*lexenv* (make-lexenv :vars (copy-list post-binding-lexenv))))
+      (let ((*lexenv* (make-lexenv :vars post-binding-lexenv)))
         (ir1-convert-progn-body start next result body))
       (let ((ctran (make-ctran))
             (fun-lvar (make-lvar))
@@ -381,9 +352,9 @@
          (fun (collect ((default-bindings)
                         (default-vals))
                 (dolist (default defaults)
-                  (if (sb!xc:constantp default)
+                  (if (sb-xc:constantp default)
                       (default-vals default)
-                      (let ((var (sb!xc:gensym)))
+                      (let ((var (sb-xc:gensym)))
                         (default-bindings `(,var ,default))
                         (default-vals var))))
                 (let ((bindings (default-bindings))
@@ -435,7 +406,7 @@
          (default (arg-info-default info))
          (supplied-p (arg-info-supplied-p info))
          (force (or force
-                    (not (sb!xc:constantp (arg-info-default info)))))
+                    (not (sb-xc:constantp (arg-info-default info)))))
          (ep (if supplied-p
                  (ir1-convert-hairy-args
                   res
@@ -468,7 +439,12 @@
                                   default-vars default-vals
                                   (if supplied-p (list default nil) (list default))
                                   name)
-          (let* ((default `',(constant-form-value default))
+          (let* ((value (constant-form-value default))
+                 ;; One-and-only-once-more: MAYBE-EMIT-MAKE-LOAD-FORMS has a similar test.
+                 (namedp-not-eql-comparable
+                  (and (symbolp default)
+                       (not (typep value '(or symbol character number)))))
+                 (default (if namedp-not-eql-comparable default `',value))
                  (defaults (if supplied-p (list default nil) (list default))))
             ;; DEFAULT can contain a reference to a
             ;; to-be-optimized-away function/block/tag, so better to
@@ -518,10 +494,10 @@
                                  :type (leaf-type var)
                                  :where-from (leaf-where-from var))))
 
-    (let* ((n-context (sb!xc:gensym "N-CONTEXT-"))
+    (let* ((n-context (sb-xc:gensym "N-CONTEXT-"))
            (context-temp (make-lambda-var :%source-name n-context
                                           :arg-info (make-arg-info :kind :more-context)))
-           (n-count (sb!xc:gensym "N-COUNT-"))
+           (n-count (sb-xc:gensym "N-COUNT-"))
            (count-temp (make-lambda-var :%source-name n-count
                                         :type (specifier-type 'index)
                                         :arg-info (make-arg-info :kind :more-count))))
@@ -541,24 +517,24 @@
       ;; and take advantage of the base+index+displacement addressing
       ;; mode on x86oids.)
       (when (optional-dispatch-keyp res)
-        (let ((n-index (sb!xc:gensym "N-INDEX-"))
-              (n-key (sb!xc:gensym "N-KEY-"))
-              (n-value-temp (sb!xc:gensym "N-VALUE-TEMP-"))
-              (n-allowp (sb!xc:gensym "N-ALLOWP-"))
-              (n-lose (sb!xc:gensym "N-LOSE-"))
-              (n-losep (sb!xc:gensym "N-LOSEP-"))
+        (let ((n-index (sb-xc:gensym "N-INDEX-"))
+              (n-key (sb-xc:gensym "N-KEY-"))
+              (n-value-temp (sb-xc:gensym "N-VALUE-TEMP-"))
+              (n-allowp (sb-xc:gensym "N-ALLOWP-"))
+              (n-lose (sb-xc:gensym "N-LOSE-"))
+              (n-losep (sb-xc:gensym "N-LOSEP-"))
               (allowp (or (optional-dispatch-allowp res)
                           (policy *lexenv* (zerop safety))))
               (found-allow-p nil))
 
-          (temps #!-stack-grows-downward-not-upward
+          (temps #-stack-grows-downward-not-upward
                  `(,n-index (1- ,n-count))
-                 #!+stack-grows-downward-not-upward
+                 #+stack-grows-downward-not-upward
                  `(,n-index (- (1- ,n-count)))
-                 #!-stack-grows-downward-not-upward n-value-temp
-                 #!-stack-grows-downward-not-upward n-key)
+                 #-stack-grows-downward-not-upward n-value-temp
+                 #-stack-grows-downward-not-upward n-key)
           (body `(declare (fixnum ,n-index)
-                          #!-stack-grows-downward-not-upward
+                          #-stack-grows-downward-not-upward
                           (ignorable ,n-value-temp ,n-key)))
 
           (collect ((tests))
@@ -568,9 +544,9 @@
                      (keyword (arg-info-key info))
                      (supplied-p (arg-info-supplied-p info))
                      (supplied-used-p (arg-info-supplied-used-p info))
-                     (n-value (sb!xc:gensym "N-VALUE-"))
+                     (n-value (sb-xc:gensym "N-VALUE-"))
                      (clause (cond (supplied-p
-                                    (let ((n-supplied (sb!xc:gensym "N-SUPPLIED-")))
+                                    (let ((n-supplied (sb-xc:gensym "N-SUPPLIED-")))
                                       (temps (list n-supplied
                                                    (if supplied-used-p
                                                        nil
@@ -609,7 +585,7 @@
                 (%odd-key-args-error)))
 
             (body
-             #!-stack-grows-downward-not-upward
+             #-stack-grows-downward-not-upward
              `(locally
                 (declare (optimize (safety 0)))
                 (loop
@@ -619,7 +595,7 @@
                   (setq ,n-key (%more-arg ,n-context ,n-index))
                   (decf ,n-index)
                   (cond ,@(tests))))
-             #!+stack-grows-downward-not-upward
+             #+stack-grows-downward-not-upward
              `(locally (declare (optimize (safety 0)))
                 (loop
                   (when (plusp ,n-index) (return))
@@ -684,10 +660,10 @@
         ;; Make up two extra variables, and squirrel them away in
         ;; ARG-INFO-DEFAULT for transforming (VALUES-LIST REST) into
         ;; (%MORE-ARG-VALUES CONTEXT 0 COUNT) when possible.
-        (let* ((context-name (sb!xc:gensym "REST-CONTEXT-"))
+        (let* ((context-name (sb-xc:gensym "REST-CONTEXT-"))
                (context (make-lambda-var :%source-name context-name
                                          :arg-info (make-arg-info :kind :more-context)))
-               (count-name (sb!xc:gensym "REST-COUNT-"))
+               (count-name (sb-xc:gensym "REST-COUNT-"))
                (count (make-lambda-var :%source-name count-name
                                        :arg-info (make-arg-info :kind :more-count)
                                        :type (specifier-type 'index))))
@@ -705,7 +681,7 @@
     (dolist (key keys)
       (let* ((info (lambda-var-arg-info key))
              (default (arg-info-default info))
-             (hairy-default (not (sb!xc:constantp default)))
+             (hairy-default (not (sb-xc:constantp default)))
              (supplied-p (arg-info-supplied-p info))
              ;; was: (format nil "~A-DEFAULTING-TEMP" (leaf-source-name key))
              (n-val (make-symbol ".DEFAULTING-TEMP."))
@@ -713,7 +689,7 @@
         (main-vars val-temp)
         (bind-vars key)
         (cond ((or hairy-default supplied-p)
-               (let* ((n-supplied (sb!xc:gensym "N-SUPPLIED-"))
+               (let* ((n-supplied (sb-xc:gensym "N-SUPPLIED-"))
                       (supplied-temp (make-lambda-var
                                       :%source-name n-supplied)))
                  (unless supplied-p
@@ -998,14 +974,14 @@
         res))))
 
 (defun wrap-forms-in-debug-catch (forms)
-  #!+unwind-to-frame-and-call-vop
+  #+unwind-to-frame-and-call-vop
   `((multiple-value-prog1
       (progn
         ,@forms)
       ;; Just ensure that there won't be any tail-calls, IR2 magic will
       ;; handle the rest.
       (values)))
-  #!-unwind-to-frame-and-call-vop
+  #-unwind-to-frame-and-call-vop
   `( ;; Normally, we'll return from this block with the below RETURN-FROM.
     (block
         return-value-tag
@@ -1028,6 +1004,14 @@
              (progn
                ,@forms))))))))
 
+;; FIXME: really should be an aspect of the lexical environment,
+;; but LEXENVs don't know whether they are toplevel or not.
+(defun has-toplevelness-decl (lambda-expr)
+  (dolist (expr (cddr lambda-expr)) ; Skip over (LAMBDA (ARGS))
+    (cond ((equal expr '(declare (top-level-form))) (return t))
+          ((typep expr '(or (cons (eql declare)) string))) ; DECL | DOCSTRING
+          (t (return nil)))))
+
 ;;; helper for LAMBDA-like things, to massage them into a form
 ;;; suitable for IR1-CONVERT-LAMBDA.
 (defun ir1-convert-lambdalike (thing
@@ -1051,6 +1035,8 @@
                                           :maybe-add-debug-catch t
                                           :source-name name))
                  (info (info :function :info name)))
+             (when (has-toplevelness-decl lambda-expression)
+               (setf (functional-top-level-defun-p res) t))
              (assert-global-function-definition-type name res)
              (push res (defined-fun-functionals defined-fun-res))
              (unless (or
@@ -1094,13 +1080,11 @@
                        (:macro
                         (let ((macros
                                (mapcar (lambda (binding)
+                                         ;; XC compile-in-lexenv ignores its second arg
+                                         #+sb-xc-host (aver (null-lexenv-p lexenv))
                                          (list* (car binding) 'macro
-                                                #-sb-xc-host
-                                                (eval-in-lexenv (cdr binding) lexenv)
-                                                #+sb-xc-host ; no EVAL-IN-LEXENV
-                                                (if (null-lexenv-p lexenv)
-                                                    (eval (cdr binding))
-                                                    (bug "inline-lexenv?"))))
+                                                (compile-in-lexenv (cdr binding) lexenv
+                                                                   nil nil nil t nil)))
                                        bindings)))
                           (recurse body
                                    (make-lexenv :default lexenv
@@ -1134,13 +1118,15 @@
                (make-lexenv
                 :default (process-inline-lexenv (second fun))
                 :handled-conditions (lexenv-handled-conditions *lexenv*)
-                :policy policy)
+                :policy policy
+                :flushable (lexenv-flushable *lexenv*))
                (make-almost-null-lexenv
                 policy
                 ;; Inherit MUFFLE-CONDITIONS from the call-site lexenv
                 ;; rather than the definition-site lexenv, since it seems
                 ;; like a much more common case.
-                (lexenv-handled-conditions *lexenv*))))
+                (lexenv-handled-conditions *lexenv*)
+                (lexenv-flushable *lexenv*))))
          (clambda (ir1-convert-lambda body
                                       :source-name source-name
                                       :debug-name debug-name
@@ -1167,7 +1153,7 @@
                                           (list (parse-key-arg-spec spec) t))
                                         key-list))))
             (allow (when (ll-kwds-allowp llks) '(&allow-other-keys))))
-        (compiler-specifier-type `(function (,@reqs ,@opts ,@rest ,@keys ,@allow) *))))))
+        (careful-specifier-type `(function (,@reqs ,@opts ,@rest ,@keys ,@allow) *))))))
 
 ;;; Get a DEFINED-FUN object for a function we are about to define. If
 ;;; the function has been forward referenced, then substitute for the
@@ -1187,8 +1173,8 @@
                                           :defined-here)
                           :type (if (eq :declared where-from)
                                     (leaf-type found)
-                                    (if lp
-                                        (ftype-from-lambda-list lambda-list)
+                                    (or (and lp
+                                             (ftype-from-lambda-list lambda-list))
                                         (specifier-type 'function))))))
                (substitute-leaf res found)
                (setf (gethash name *free-funs*) res)))
@@ -1280,56 +1266,97 @@
     fun))
 
 ;;; Store INLINE-LAMBDA as the inline expansion of NAME.
-(defun %set-inline-expansion (name defined-fun inline-lambda)
-  (cond ((member inline-lambda '(:accessor :predicate))
-         ;; Special-case that implies a structure-related source-transform.
-         (when (info :function :inline-expansion-designator name)
-           ;; Any inline expansion that existed can't be useful.
-           (warn "structure ~(~A~) ~S clobbers inline function"
-                 inline-lambda name))
-         (setq inline-lambda nil)) ; will be cleared below
+;;; EXTRA-INFO is either a keyword denoting that NAME pertains to
+;;; an auto-generated defstruct function, or else it is the list of
+;;; funargs that could be auto-dxified.
+(defun %set-inline-expansion (name defined-fun inline-lambda extra-info
+                                   &aux (defstruct-snippet
+                                          (when (keywordp extra-info)
+                                            extra-info))
+                                        (dxable-args
+                                          (unless (keywordp extra-info)
+                                            extra-info)))
+  (cond (defstruct-snippet
+         ;; In this case, NAME is a system-generated function. Warn if blowing away
+         ;; a previously existing inline expansion coming from an ordinary DEFUN.
+         ;; FIXME: It's tricky to correctly warn about stomping on a constructor
+         ;; because it might actually be the right inline lambda.
+         ;; Probably should compare with EQUALP.
+         ;; FIXME: what does the below KLUDGE mean ?
+         (unless (eq defstruct-snippet :constructor)
+           (let ((old (info :function :inlining-data name)))
+           ;; KLUDGE: This is like (NTH-VALUE 1 (FUN-NAME-INLINE-EXPANSION))
+           ;; but expressed in a way that doesn't crash in cold-init.
+             (when (or (typep old 'inlining-data) (consp old))
+             ;; Any inline expansion that existed can't be useful.
+               (warn "structure ~(~A~) ~S clobbers inline function"
+                     defstruct-snippet name)))))
         (t
-         ;; Warn if stomping on a structure predicate or accessor
-         ;; whether or not we are about to install an inline-lambda.
          (let ((info (info :function :source-transform name)))
+           ;; If NAME was a defstruct snippet, and now it isn't, then warn
+           ;; and remove the transform.
            (when (consp info)
              (clear-info :function :source-transform name)
              ;; This is serious enough that you can get two warnings:
              ;; - one because you redefined a function at all,
              ;; - and one because the source-transform is erased.
-             (warn "redefinition of ~S clobbers structure ~:[accessor~;predicate~]"
-                   name (eq (cdr info) :predicate))))))
-  (cond (inline-lambda
-         (setf (info :function :inline-expansion-designator name)
-               inline-lambda)
-         (when defined-fun
-           (setf (defined-fun-inline-expansion defined-fun)
-                 inline-lambda)))
-        (t
-         (clear-info :function :inline-expansion-designator name))))
+             (warn "redefinition of ~S clobbers structure ~:[accessor~;~(~a~)~]"
+                   name (symbolp (cdr info)) (cdr info))))))
+  ;; says CLHS: "Only an implementation that was willing to be responsible
+  ;; for recompiling f if the definition of g changed incompatibly could
+  ;; legitimately stack allocate the list argument to g in f."
+  ;; Yeah, well, we're not going be responsible for bupkis.
+  ;; If you want to do something dangerous, then do it.
+  ;; Of course it would be nice NOT to warn when we haven't actually baked-in
+  ;; any assumptions about callees, but I don't feel like adding more metadata
+  ;; to track when we assumed something.
+  (let ((old (fun-name-dx-args name)))
+    (when (and old (not (subsetp old dxable-args)))
+      (warn "redefinition of ~S with differing DYNAMIC-EXTENT declarations ~
+is potentially harmful to any already-compiled callers using (SAFETY 0)."
+            name)))
+  (if (or inline-lambda dxable-args)
+      (setf (info :function :inlining-data name)
+            (if dxable-args
+                (if inline-lambda
+                    (make-inlining-data inline-lambda dxable-args)
+                    (make-dxable-args dxable-args))
+                inline-lambda))
+      (clear-info :function :inlining-data name))
+  (when (and inline-lambda defined-fun)
+    (setf (defined-fun-inline-expansion defined-fun)
+          inline-lambda)))
 
 ;;; the even-at-compile-time part of DEFUN
 ;;;
-;;; The INLINE-LAMBDA is either the symbol :ACCESSOR, meaning that
-;;; the function is a structure accessor, or a LAMBDA-WITH-LEXENV,
-;;; or NIL if there is no inline expansion.
-(defun %compiler-defun (name inline-lambda compile-toplevel)
+;;; INLINE-LAMBDA is either (LAMBDA (...) ...) or (LAMBDA-WITH-LEXENV ...)
+;;; EXTRA-INFO is one of:
+;;; * a symbol in {:ACCESSOR, :PREDICATE, :COPIER, :CONSTRUCTOR} if the function
+;;;   came from defstruct; or
+;;; * a possibly empty list of dynamic extent arguments.
+;;; The inline lambda will be NIL for a structure accessor, predicate, or copier
+;;; since those can always be reconstructed from a defstruct description.
+(defun %compiler-defun (name compile-toplevel inline-lambda extra-info)
   (let ((defined-fun nil)) ; will be set below if we're in the compiler
     (when compile-toplevel
       (with-single-package-locked-error
           (:symbol name "defining ~S as a function")
         (setf defined-fun
-              (if (consp inline-lambda)
-                  ;; FIFTH as an accessor - how informative!
-                  ;; Obfuscation aside, I doubt this is even right.
-                  (get-defined-fun name (fifth inline-lambda))
-                  (get-defined-fun name))))
+              ;; Try to pass the lambda-list to GET-DEFINED-FUN if we can.
+              (if (atom inline-lambda)
+                  (get-defined-fun name)
+                  (get-defined-fun
+                   name (ecase (car inline-lambda)
+                         (lambda-with-lexenv (third inline-lambda))
+                         (lambda (second inline-lambda)))))))
       (when (boundp '*lexenv*)
         (aver (producing-fasl-file))
         (if (member name *fun-names-in-this-file* :test #'equal)
             (warn 'duplicate-definition :name name)
             (push name *fun-names-in-this-file*)))
-      (%set-inline-expansion name defined-fun inline-lambda))
+      ;; I don't know why this is guarded by (WHEN compile-toplevel),
+      ;; because regular old %DEFUN is going to call this anyway.
+      (%set-inline-expansion name defined-fun inline-lambda extra-info))
 
     (become-defined-fun-name name)
 
@@ -1360,14 +1387,13 @@
 ;; and not flat-out wrong, though there is indeed some waste in the fasl.
 ;;
 ;; KIND is the globaldb KIND of this NAME
-(defun %compiler-defmacro (kind name compile-toplevel)
-  (when compile-toplevel
-    (let ((name-key `(,kind ,name)))
-      (when (boundp '*lexenv*)
-        (aver (producing-fasl-file))
-        (if (member name-key *fun-names-in-this-file* :test #'equal)
-            (compiler-style-warn 'same-file-redefinition-warning :name name)
-            (push name-key *fun-names-in-this-file*))))))
+(defun %compiler-defmacro (kind name)
+  (let ((name-key `(,kind ,name)))
+    (when (boundp '*lexenv*)
+      (aver (producing-fasl-file))
+      (if (member name-key *fun-names-in-this-file* :test #'equal)
+          (compiler-style-warn 'same-file-redefinition-warning :name name)
+          (push name-key *fun-names-in-this-file*)))))
 
 
 ;;; Entry point utilities
