@@ -10,7 +10,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!C")
+(in-package "SB-C")
 
 (declaim (special *compiler-error-bailout*))
 
@@ -21,6 +21,11 @@
 (declaim (type index *current-form-number*))
 (defvar *current-form-number*)
 
+;;; Report as we try each transform? (either source or IR)
+;;; Bind to T to see the ones that didn't abort,
+;;; or :ALL if you want to see each attempted transform.
+(defvar *show-transforms-p* nil)
+
 (declaim (inline source-form-has-path-p))
 (defun source-form-has-path-p (form)
   (not (typep form '(or symbol fixnum character))))
@@ -84,8 +89,6 @@
   to optimize code which uses those definitions? Setting this true
   gives non-ANSI, early-CMU-CL behavior. It can be useful for improving
   the efficiency of stable code.")
-
-(defvar *post-binding-variable-lexenv* nil)
 
 ;;;; namespace management utilities
 
@@ -96,9 +99,9 @@
   (let ((answer
          (typecase env
            (null nil)
-           #!+(and sb-fasteval (host-feature sb-xc))
-           (sb!interpreter:basic-env
-            (sb!interpreter::fun-lexically-notinline-p name env))
+           #+(and sb-fasteval (not sb-xc-host))
+           (sb-interpreter:basic-env
+            (sb-interpreter::fun-lexically-notinline-p name env))
            (t
             (let ((fun (cdr (assoc name (lexenv-funs env) :test #'equal))))
               ;; FIXME: this seems to omit FUNCTIONAL
@@ -124,7 +127,7 @@
   (let ((where (info :function :where-from name)))
     (when (and (eq where :assumed)
                ;; Slot accessors are defined just-in-time, if not already.
-               (not (typep name '(cons (eql sb!pcl::slot-accessor))))
+               (not (typep name '(cons (eql sb-pcl::slot-accessor))))
                ;; In the ordinary target Lisp, it's silly to report
                ;; undefinedness when the function is defined in the
                ;; running Lisp. But at cross-compile time, the current
@@ -137,7 +140,7 @@
                ;; complain about undefined functions.
                (not latep))
       (note-undefined-reference name :function))
-    (let ((ftype (proclaimed-ftype name))
+    (let ((ftype (global-ftype name))
           (notinline (fun-lexically-notinline-p name)))
       (make-global-var
        :kind :global-function
@@ -235,7 +238,7 @@
                           :type (if (and (eq inlinep :notinline)
                                          (neq where :declared))
                                     (specifier-type 'function)
-                                    (proclaimed-ftype name))))
+                                    (global-ftype name))))
                        (find-global-fun name nil)))))))))
 
 ;;; Return the LEAF structure for the lexically apparent function
@@ -285,7 +288,7 @@
                 (:alien
                  (info :variable :alien-info name))
                 ;; FIXME: The return value in this case should really be
-                ;; of type SB!C::LEAF.  I don't feel too badly about it,
+                ;; of type SB-C::LEAF.  I don't feel too badly about it,
                 ;; because the MACRO idiom is scattered throughout this
                 ;; file, but it should be cleaned up so we're not
                 ;; throwing random conses around.  --njf 2002-03-23
@@ -294,20 +297,12 @@
                        (type (type-specifier (info :variable :type name))))
                    `(macro . (the ,type ,expansion))))
                 (:constant
-                 (let ((value (symbol-value name)))
-                   ;; Override the values of standard symbols in XC,
-                   ;; since we can't redefine them.
-                   #+sb-xc-host
-                   (when (eql (find-symbol (symbol-name name) :cl) name)
-                     (multiple-value-bind (xc-value foundp)
-                         (xc-constant-value name)
-                       (cond (foundp
-                              (setf value xc-value))
-                             ((not (eq value name))
-                              (compiler-warn
-                               "Using cross-compilation host's definition of ~S: ~A~%"
-                               name (symbol-value name))))))
-                   (find-constant value name)))
+                 #+sb-xc-host ; check that we never reference host constants
+                 (unless (member name '(nil t)) ; other than these 2
+                   (when (eq (find-symbol (string name) "XC-STRICT-CL")
+                             name)
+                     (error "Using a constant from the host ~s" name)))
+                 (find-constant (symbol-value name) name))
                 (t
                  (make-global-var :kind kind
                                   :%source-name name
@@ -321,10 +316,11 @@
 (defun maybe-emit-make-load-forms (constant &optional (name nil namep))
   (let ((xset (alloc-xset)))
     (labels ((trivialp (value)
-               (typep value
+               (sb-xc:typep value
                       '(or
                         #-sb-xc-host
-                        (or unboxed-array #!+sb-simd-pack simd-pack)
+                        (or unboxed-array #+sb-simd-pack simd-pack
+                                          #+sb-simd-pack-256 simd-pack-256)
                         #+sb-xc-host
                         (and array (not (array t)))
                         symbol
@@ -358,7 +354,10 @@
                    ((array t)
                     (dotimes (i (array-total-size value))
                       (grovel (row-major-aref value i))))
-                   (#+sb-xc-host structure!object
+                   ;; We use ordinary host packages to model the target packages.
+                   ;; Those objects are not in "our" type system, but we can check
+                   ;; for them and make them legal to dump out.
+                   (#+sb-xc-host (or structure!object package)
                     #-sb-xc-host instance
                     ;; In the target SBCL, we can dump any instance, but
                     ;; in the cross-compilation host, %INSTANCE-FOO
@@ -374,12 +373,12 @@
                     (when (emit-make-load-form value)
                       #+sb-xc-host
                       (aver (eql (layout-bitmap (%instance-layout value))
-                                 sb!kernel::+layout-all-tagged+))
+                                 sb-kernel::+layout-all-tagged+))
                       (do-instance-tagged-slot (i value)
                         (grovel (%instance-ref value i)))))
                    (t
                     (compiler-error
-                      "Objects of type ~/sb!impl:print-type-specifier/ ~
+                      "Objects of type ~/sb-impl:print-type-specifier/ ~
                        can't be dumped into fasl files."
                      (type-of value)))))))
       ;; Dump all non-trivial named constants using the name.
@@ -407,7 +406,7 @@
 ;;; determine what is evaluated next. If the ctran has no block, then
 ;;; we make it be in the block that the node is in. If the ctran heads
 ;;; its block, we end our block and link it to that block.
-#!-sb-fluid (declaim (inline use-ctran))
+#-sb-fluid (declaim (inline use-ctran))
 (defun use-ctran (node ctran)
   (declare (type node node) (type ctran ctran))
   (if (eq (ctran-kind ctran) :unused)
@@ -460,7 +459,7 @@
            (setf (lvar-uses lvar) (list node (lvar-uses lvar)))))
     (reoptimize-lvar lvar)))
 
-#!-sb-fluid(declaim (inline use-continuation))
+#-sb-fluid(declaim (inline use-continuation))
 (defun use-continuation (node ctran lvar)
   (use-ctran node ctran)
   (use-lvar node lvar))
@@ -734,17 +733,17 @@
   (flet ((legal-cm-name-p (name)
            (and (legal-fun-name-p name)
                 (or (not (symbolp name))
-                    (not (sb!xc:macro-function name *lexenv*))))))
+                    (not (sb-xc:macro-function name *lexenv*))))))
     (if (eq opname 'funcall)
         (let ((fun-form (cadr form)))
           (cond ((and (consp fun-form) (eq 'function (car fun-form))
                       (not (cddr fun-form)))
                  (let ((real-fun (cadr fun-form)))
                    (if (legal-cm-name-p real-fun)
-                       (values (sb!xc:compiler-macro-function real-fun *lexenv*)
+                       (values (sb-xc:compiler-macro-function real-fun *lexenv*)
                                real-fun)
                        (values nil nil))))
-                ((sb!xc:constantp fun-form *lexenv*)
+                ((sb-xc:constantp fun-form *lexenv*)
                  (let ((fun (constant-form-value fun-form *lexenv*)))
                    (if (legal-cm-name-p fun)
                        ;; CLHS tells us that local functions must shadow
@@ -756,12 +755,12 @@
                        ;; a list (function name)", that means that
                        ;; (funcall 'name) that gets here doesn't fit the
                        ;; definition.
-                       (values (sb!xc:compiler-macro-function fun nil) fun)
+                       (values (sb-xc:compiler-macro-function fun nil) fun)
                        (values nil nil))))
                 (t
                  (values nil nil))))
         (if (legal-fun-name-p opname)
-            (values (sb!xc:compiler-macro-function opname *lexenv*) opname)
+            (values (sb-xc:compiler-macro-function opname *lexenv*) opname)
             (values nil nil)))))
 
 ;;; If FORM has a usable compiler macro, use it; otherwise return FORM itself.
@@ -885,7 +884,7 @@
                    ;; system to work through them, and this code does so, by
                    ;; crudely suppressing all warnings in cross-compilation
                    ;; macroexpansion. -- WHN 19990412
-                   #+(and cmu sb-xc-host)
+                   #+host-quirks-cmu
                    (warning (lambda (c)
                               (compiler-notify
                                "~@<~A~:@_~
@@ -944,18 +943,6 @@
 
 ;;;; code coverage
 
-;;; Used as the CDR of the code coverage instrumentation records
-;;; (instead of NIL) to ensure that any well-behaving user code will
-;;; not have constants EQUAL to that record. This avoids problems with
-;;; the records getting coalesced with non-record conses, which then
-;;; get mutated when the instrumentation runs. Note that it's
-;;; important for multiple records for the same location to be
-;;; coalesced. -- JES, 2008-01-02
-;;; Use of #. mandates :COMPILE-TOPLEVEL for several Lisps
-;;; even though for us it's immediately accessible to EVAL.
-(eval-when (:compile-toplevel :load-toplevel :execute)
-  (defconstant +code-coverage-unmarked+ '%code-coverage-unmarked%))
-
 ;;; Check the policy for whether we should generate code coverage
 ;;; instrumentation. If not, just return the original START
 ;;; ctran. Otherwise insert code coverage instrumentation after
@@ -984,9 +971,13 @@
                                    (cons path +code-coverage-unmarked+)))
                   (next (make-ctran))
                   (*allow-instrumenting* nil))
+              #+(or x86-64 x86) (declare (ignore store)) ; eval'd for side-effect only
               (push (ctran-block start)
                     (gethash path (code-coverage-blocks metadata)))
               (ir1-convert start next nil
+                           #+(or x86-64 x86)
+                           `(%primitive mark-covered ',path)
+                           #-(or x86-64 x86)
                            `(locally
                                 (declare (optimize speed
                                                    (safety 0)
@@ -1013,26 +1004,6 @@
           (when *current-path*
             (instrument-coverage start nil form))))
       start))
-
-(defun record-code-coverage (info cc)
-  (setf (gethash info *code-coverage-info*) cc))
-
-(defun clear-code-coverage ()
-  (clrhash *code-coverage-info*))
-
-(defun reset-code-coverage ()
-  (maphash (lambda (info cc)
-             (declare (ignore info))
-             (dolist (cc-entry cc)
-               (setf (cdr cc-entry) +code-coverage-unmarked+)))
-           *code-coverage-info*))
-
-(defun code-coverage-record-marked (record)
-  (aver (consp record))
-  (ecase (cdr record)
-    ((#.+code-coverage-unmarked+) nil)
-    ((t) t)))
-
 
 ;;;; converting combinations
 
@@ -1046,12 +1017,12 @@
            ;; if the value is unused.
            (if (member symbol '(symeval sym-global-val))
                (not (constantp (second form)))
-               (not (member (symbol-package symbol)
+               (not (member (sb-xc:symbol-package symbol)
                             (load-time-value
                               ;; KLUDGE: packages we're not interested in
                               ;; stepping.
-                              (mapcar #'find-package '(sb!c sb!int sb!impl
-                                                       sb!kernel sb!pcl))
+                              (mapcar #'find-package '(sb-c sb-int sb-impl
+                                                       sb-kernel sb-pcl))
                              t))))))
     (and *allow-instrumenting*
          (policy *lexenv* (= insert-step-conditions 3))
@@ -1116,6 +1087,10 @@
         (setf (combination-args node) (arg-lvars))))
     node))
 
+(defun show-transform (kind name new-form)
+  (let ((*print-right-margin* 100))
+    (format *trace-output* "~&xform (~a) ~S~% -> ~S~%"
+            kind name new-form)))
 ;;; Convert a call to a global function. If not :NOTINLINE, then we do
 ;;; source transforms and try out any inline expansion. If there is no
 ;;; expansion, but is :INLINE, then give an efficiency note (unless a
@@ -1129,26 +1104,20 @@
                    (defined-fun-inlinep var))))
     (if (eq inlinep :notinline)
         (ir1-convert-combination start next result form var)
-        (let* ((transform (info :function :source-transform name)))
+        (let ((transform (info :function :source-transform name)))
           (if transform
               (multiple-value-bind (transformed pass)
                   (if (functionp transform)
                       (funcall transform form *lexenv*)
-                      (let ((result
-                             (if (eq (cdr transform) :predicate)
-                                 (and (singleton-p (cdr form))
-                                      `(%instance-typep
-                                        ,(cadr form)
-                                        ',(dd-name (car transform))))
-                                 (slot-access-transform
-                                  (if (consp name) :write :read)
-                                  (cdr form) transform))))
-                        (values result (null result))))
+                      (struct-fun-transform transform form name))
+                ;; Note that "pass" means fail. Gotta love it.
                 (cond (pass
                        (ir1-convert-maybe-predicate start next result form var))
                       (t
                        (unless (policy *lexenv* (zerop store-xref-data))
                          (record-call name (ctran-block start) *current-path*))
+                       (when *show-transforms-p*
+                         (show-transform "src" name transformed))
                        (ir1-convert start next result transformed))))
               (ir1-convert-maybe-predicate start next result form var))))))
 
@@ -1278,8 +1247,8 @@
                                     'type-warning
                                     :format-control
                                     "The type declarations ~
-                                  ~/sb!impl:print-type/ and ~
-                                  ~/sb!impl:print-type/ for ~
+                                  ~/sb-impl:print-type/ and ~
+                                  ~/sb-impl:print-type/ for ~
                                   ~S conflict."
                                     :format-arguments
                                     (list old-type type var-name))))
@@ -1349,9 +1318,14 @@
 
 ;;; Process a special declaration, returning a new LEXENV. A non-bound
 ;;; special declaration is instantiated by throwing a special variable
-;;; into the variables if BINDING-FORM-P is NIL, or otherwise into
-;;; *POST-BINDING-VARIABLE-LEXENV*.
-(defun process-special-decl (spec res vars binding-form-p context)
+;;; into the variables if POST-BINDING-LEXENV is NIL,
+;;; or by mutating POST-BINDING-LEXENV if non-nil.
+;;; Note that POST-BINDING-LEXENV is not actually a lexenv - it is only
+;;; the contents of the 'vars' slot of a LEXENV, and moreover, as supplied
+;;; to this function, it has a dummy cons in front which is later removed.
+;;; The dummy cons serves a double purpose - as an indicator that this decl
+;;; occurs in a LET/LET* form, and facilitating pass by reference of the list.
+(defun process-special-decl (spec res vars post-binding-lexenv context)
   (declare (list spec vars) (type lexenv res))
   (collect ((new-venv nil cons))
     (dolist (name (cdr spec))
@@ -1388,9 +1362,9 @@
           (null
            (unless (or (assoc name (new-venv) :test #'eq))
              (new-venv (cons name (specvar-for-binding name))))))))
-    (cond (binding-form-p
-           (setf *post-binding-variable-lexenv*
-                 (append (new-venv) *post-binding-variable-lexenv*))
+    (cond (post-binding-lexenv
+           (setf (cdr post-binding-lexenv)
+                 (append (new-venv) (cdr post-binding-lexenv)))
            res)
           ((new-venv)
            (make-lexenv :default res :vars (new-venv)))
@@ -1571,9 +1545,9 @@
                (etypecase fun
                  (leaf
                   (if bound-fun
-                      #!+stack-allocatable-closures
+                      #+stack-allocatable-closures
                       (setf (leaf-extent bound-fun) extent)
-                      #!-stack-allocatable-closures
+                      #-stack-allocatable-closures
                       (compiler-notify
                        "Ignoring DYNAMIC-EXTENT declaration on function ~S ~
                         (not supported on this platform)." fname)
@@ -1600,7 +1574,7 @@
 ;;; Process a single declaration spec, augmenting the specified LEXENV
 ;;; RES. Return RES and result type. VARS and FVARS are as described
 ;;; PROCESS-DECLS.
-(defun process-1-decl (raw-spec res vars fvars binding-form-p context)
+(defun process-1-decl (raw-spec res vars fvars post-binding-lexenv context)
   (declare (type list raw-spec vars fvars))
   (declare (type lexenv res))
   (let ((spec (canonized-decl-spec raw-spec))
@@ -1617,7 +1591,8 @@
        ((ignore ignorable)
         (process-ignore-decl spec vars fvars res)
         res)
-       (special (process-special-decl spec res vars binding-form-p context))
+       (special
+        (process-special-decl spec res vars post-binding-lexenv context))
        (ftype
         (unless (cdr spec)
           (compiler-error "no type specified in FTYPE declaration: ~S" spec))
@@ -1683,7 +1658,7 @@
         (allow-explicit-check allow-lambda-list)
         (lambda-list (if allow-lambda-list :unspecified nil))
         (optimize-qualities)
-        (*post-binding-variable-lexenv* nil))
+        (post-binding-lexenv (if binding-form-p (list nil)))) ; dummy cell
     (flet ((process-it (spec decl)
              (cond ((atom spec)
                     (compiler-error "malformed declaration specifier ~S in ~S"
@@ -1721,10 +1696,11 @@
                                       it))
                     (setq explicit-check (or (cdr spec) t)
                           allow-explicit-check nil)) ; at most one of this decl
+                   ((equal spec '(top-level-form))) ; ignore
                    (t
                     (multiple-value-bind (new-env new-qualities)
                         (process-1-decl spec lexenv vars fvars
-                                        binding-form-p context)
+                                        post-binding-lexenv context)
                       (setq lexenv new-env
                             optimize-qualities
                             (nconc new-qualities optimize-qualities)))))))
@@ -1736,7 +1712,7 @@
               ;; Kludge: EVAL calls this function to deal with LOCALLY.
               (process-it spec decl)))))
     (warn-repeated-optimize-qualities (lexenv-policy lexenv) optimize-qualities)
-    (values lexenv result-type *post-binding-variable-lexenv*
+    (values lexenv result-type (cdr post-binding-lexenv)
             lambda-list explicit-check)))
 
 (defun %processing-decls (decls vars fvars ctran lvar binding-form-p fun)
@@ -1758,10 +1734,9 @@
 (defmacro processing-decls ((decls vars fvars ctran lvar
                                    &optional post-binding-lexenv)
                             &body forms)
-  (check-type ctran symbol)
-  (check-type lvar symbol)
+  (declare (symbol ctran lvar))
   (let ((post-binding-lexenv-p (not (null post-binding-lexenv)))
-        (post-binding-lexenv (or post-binding-lexenv (sb!xc:gensym "LEXENV"))))
+        (post-binding-lexenv (or post-binding-lexenv (sb-xc:gensym "LEXENV"))))
     `(%processing-decls ,decls ,vars ,fvars ,ctran ,lvar
                         ,post-binding-lexenv-p
                         (lambda (,ctran ,lvar ,post-binding-lexenv)
