@@ -7,7 +7,17 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
+
+(defconstant sb-assem:assem-scheduler-p t)
+(defconstant sb-assem:+inst-alignment-bytes+ 4)
+(defconstant sb-assem:+assem-max-locations+ 68)
+
+(defconstant +backend-fasl-file-implementation+ :mips)
+
+  ;; The o32 ABI specifies 4k-64k as page size. We have to pick the
+  ;; maximum since mprotect() works only with page granularity.
+(defconstant +backend-page-bytes+ 65536)
 
 ;;;; Machine Architecture parameters:
 (eval-when (:compile-toplevel :load-toplevel :execute)
@@ -64,7 +74,7 @@
 
 ;;;; Description of the target address space.
 
-#!+irix
+#+irix
 (progn
   ;; Where to put the different spaces.
   ;; Old definitions, might be still relevant for an IRIX port.
@@ -75,12 +85,10 @@
   (defconstant static-space-start    #x06000000)
   (defconstant static-space-end      #x08000000)
 
-  (defconstant dynamic-0-space-start #x08000000)
-  (defconstant dynamic-0-space-end   #x0c000000)
-  (defconstant dynamic-1-space-start #x0c000000)
-  (defconstant dynamic-1-space-end   #x10000000))
+  (defparameter dynamic-0-space-start #x08000000)
+  (defparameter dynamic-0-space-end   #x0c000000))
 
-#!+linux
+#+linux
 (progn
   ;; Where to put the address spaces on Linux.
   ;;
@@ -92,10 +100,8 @@
   (defconstant static-space-end      #x0fff0000)
   ;; C runtime read/write segment starts at 0x10000000, heap and DSOs
   ;; start at 0x2a000000
-  (defconstant dynamic-0-space-start #x30000000)
-  (defconstant dynamic-0-space-end   #x4fff0000)
-  (defconstant dynamic-1-space-start #x50000000)
-  (defconstant dynamic-1-space-end   #x6fff0000)
+  (defparameter dynamic-0-space-start #x30000000)
+  (defparameter dynamic-0-space-end   #x4fff0000)
 
   (defconstant linkage-table-space-start #x70000000)
   (defconstant linkage-table-space-end   #x71000000)
@@ -116,13 +122,13 @@
 (defenum (:start 8)
   halt-trap
   pending-interrupt-trap
-  error-trap
   cerror-trap
   breakpoint-trap
   fun-end-breakpoint-trap
   after-breakpoint-trap
   single-step-around-trap
-  single-step-before-trap)
+  single-step-before-trap
+  error-trap)
 
 ;;;; Static symbols.
 
@@ -134,12 +140,11 @@
 ;;; space directly after the static symbols.  That way, the raw-addr
 ;;; can be loaded directly out of them by indirecting relative to NIL.
 (defconstant-eqx +static-symbols+
-  `(,@+common-static-symbols+
-    ,@+c-callable-static-symbols+)
-  #'equal)
+  `#(,@+common-static-symbols+)
+  #'equalp)
 
 (defconstant-eqx +static-fdefns+
-  '(two-arg-+
+  #(two-arg-+
     two-arg--
     two-arg-*
     two-arg-/
@@ -157,4 +162,4 @@
     length
     two-arg-gcd
     two-arg-lcm)
-  #'equal)
+  #'equalp)
