@@ -10,7 +10,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!C")
+(in-package "SB-C")
 
 (declaim (type fixnum *compiler-sset-counter*))
 (defvar *compiler-sset-counter* 0)
@@ -63,7 +63,9 @@
   (use nil :type (or node null))
   ;; the basic block this continuation is in. This is null only in
   ;; :UNUSED continuations.
-  (block nil :type (or cblock null)))
+  (block nil :type (or cblock null))
+  ;; Entries created by the BLOCK special operator
+  (entries nil :type list))
 
 (defmethod print-object ((x ctran) stream)
   (print-unreadable-object (x stream :type t :identity t)
@@ -87,49 +89,94 @@
   ;; the optimizer for this node type doesn't care, it can elect not
   ;; to clear this flag.
   (reoptimize t :type boolean)
-  ;; Cached type which is checked by DEST. If NIL, then this must be
-  ;; recomputed: see LVAR-EXTERNALLY-CHECKABLE-TYPE.
-  (%externally-checkable-type nil :type (or null ctype))
   ;; if the LVAR value is DYNAMIC-EXTENT, CLEANUP protecting it.
   (dynamic-extent nil :type (or null cleanup))
   ;; something or other that the back end annotates this lvar with
   (info nil)
-  (dependent-casts nil))
+  (dependent-casts nil)
+  (annotations nil)
+  (dependent-annotations nil))
 (!set-load-form-method lvar (:xc :target) :ignore-it)
 
+;;; These are used for annottating a LVAR with information that can't
+;;; be expressed using types.
+;;; Right now it's basically used for tracking constants and checking
+;;; them for things like proper sequence, or valid type specifier.
+(defstruct lvar-annotation
+  (source-path nil :type list)
+  lexenv
+  fired)
+
+(defprinter (lvar-annotation)
+  fired)
+
+(defstruct (lvar-modified-annotation
+            (:include lvar-annotation)
+            (:copier nil))
+  caller)
+
+(defstruct (lvar-hook
+            (:include lvar-annotation)
+            (:copier nil))
+  (hook #'missing-arg :type function))
+
+(defstruct (lvar-type-spec-annotation
+            (:include lvar-hook)
+            (:copier nil)))
+
+(defstruct (lvar-proper-sequence-annotation
+            (:include lvar-annotation)
+            (:copier nil))
+  (kind 'proper-sequence :type (member proper-list proper-sequence
+                                       proper-or-circular-list proper-or-dotted-list)))
+
+(defstruct (lvar-dependent-annotation
+             (:include lvar-annotation)
+             (:copier nil))
+    (deps nil :type list))
+
+(defstruct (lvar-function-designator-annotation
+             (:include lvar-dependent-annotation)
+             (:copier nil))
+  (caller nil :type symbol)
+  (arg-specs nil :type list)
+  (result-specs nil :type list)
+  type)
+
+(defstruct (lvar-function-annotation
+             (:include lvar-annotation)
+             (:copier nil))
+  type
+  context)
+
 (defmethod print-object ((x lvar) stream)
   (print-unreadable-object (x stream :type t :identity t)
     (when (boundp '*compiler-ir-obj-map*)
       (format stream "~D" (cont-num x)))))
 
-#!-sb-fluid (declaim (inline lvar-has-single-use-p))
+#-sb-fluid (declaim (inline lvar-has-single-use-p))
 (defun lvar-has-single-use-p (lvar)
   (typep (lvar-uses lvar) '(not list)))
 
 ;;; Return the unique node, delivering a value to LVAR.
-#!-sb-fluid (declaim (inline lvar-use))
+#-sb-fluid (declaim (inline lvar-use))
 (defun lvar-use (lvar)
   (the (not list) (lvar-uses lvar)))
 
-#!-sb-fluid (declaim (inline lvar-derived-type))
+#-sb-fluid (declaim (inline lvar-derived-type))
 (defun lvar-derived-type (lvar)
   (declare (type lvar lvar))
   (or (lvar-%derived-type lvar)
       (setf (lvar-%derived-type lvar)
             (%lvar-derived-type lvar))))
 
-#!-sb-fluid(declaim (inline flush-lvar-externally-checkable-type))
-(defun flush-lvar-externally-checkable-type (lvar)
-  (declare (type lvar lvar))
-  (setf (lvar-%externally-checkable-type lvar) nil))
-
 (def!struct (node (:constructor nil)
                   (:include sset-element
                             (number (unless (eql *compiler-sset-counter* 0)
                                       (incf *compiler-sset-counter*))))
                   (:copier nil))
   ;; unique ID for debugging
-  #!+sb-show (id (new-object-id) :read-only t)
+  #+sb-show (id (new-object-id) :read-only t)
   ;; True if this node needs to be optimized. This is set to true
   ;; whenever something changes about the value of an lvar whose DEST
   ;; is this node.
@@ -177,7 +224,7 @@
   ;; can null out this slot.
   (tail-p nil :type boolean))
 
-#!-sb-fluid (declaim (inline node-block))
+#-sb-fluid (declaim (inline node-block))
 (defun node-block (node)
   (ctran-block (node-prev node)))
 
@@ -200,7 +247,7 @@
   ;; the value is unused.
   (lvar nil :type (or lvar null)))
 
-#!-sb-fluid (declaim (inline node-dest))
+#-sb-fluid (declaim (inline node-dest))
 (defun node-dest (node)
   (awhen (node-lvar node) (lvar-dest it)))
 
@@ -289,13 +336,13 @@
                     (:copier nil)
                     (:conc-name block-)
                     (:predicate block-p))
-  ;; a list of all the blocks that are predecessors/successors of this
-  ;; block. In well-formed IR1, most blocks will have one successor.
-  ;; The only exceptions are:
-  ;;  1. component head blocks (any number)
-  ;;  2. blocks ending in an IF (1 or 2)
-  ;;  3. blocks with DELETE-P set (zero)
-  (pred nil :type list)
+    ;; a list of all the blocks that are predecessors/successors of this
+    ;; block. In well-formed IR1, most blocks will have one successor.
+    ;; The only exceptions are:
+    ;;  1. component head blocks (any number)
+    ;;  2. blocks ending in an IF (1 or 2)
+    ;;  3. blocks with DELETE-P set (zero)
+    (pred nil :type list)
   (succ nil :type list)
   ;; the ctran which heads this block (a :BLOCK-START), or NIL when we
   ;; haven't made the start ctran yet (and in the dummy component head
@@ -383,8 +430,7 @@
   ;; the INFO slot as :type (or (satisfies ir2-component-p) ...)
   ;; During make-host-2, the solution to this is the same hack
   ;; as for everything else: use DEF!STRUCT for IR2-COMPONENT.
-  #!+(and (host-feature sb-xc-host) (host-feature sbcl))
-  (declare (sb-ext:muffle-conditions style-warning))
+  #+host-quirks-sbcl (declare (host-sb-ext:muffle-conditions style-warning))
 (def!struct (component (:copier nil)
                        (:constructor
                         make-component
@@ -393,7 +439,11 @@
                          (last-block tail)
                          (outer-loop (make-loop :kind :outer :head head)))))
   ;; unique ID for debugging
-  #!+sb-show (id (new-object-id) :read-only t)
+  #+sb-show (id (new-object-id) :read-only t)
+  ;; space where this component will be allocated in
+  ;; :auto won't make any codegen optimizations pertinent to immobile space,
+  ;; but will place the code there given sufficient available space.
+  (%mem-space nil :type (member nil :dynamic :immobile :auto))
   ;; the kind of component
   ;;
   ;; (The terminology here is left over from before
@@ -498,7 +548,7 @@
   (sset-number 0 :type fixnum)))
 (defprinter (component :identity t)
   name
-  #!+sb-show id
+  #+sb-show id
   (reanalyze :test reanalyze))
 
 (declaim (inline reoptimize-component))
@@ -537,7 +587,7 @@
   (kind (missing-arg)
         :type (member :special-bind :catch :unwind-protect
                       :block :tagbody :dynamic-extent
-                      #!-c-stack-is-control-stack :restore-nsp))
+                      #-c-stack-is-control-stack :restore-nsp))
   ;; the node that messes things up. This is the last node in the
   ;; non-messed-up environment. Null only temporarily. This could be
   ;; deleted due to unreachability.
@@ -685,7 +735,7 @@
                   (:copier nil)
                   (:constructor nil))
   ;; unique ID for debugging
-  #!+sb-show (id (new-object-id) :read-only t)
+  #+sb-show (id (new-object-id) :read-only t)
   ;; (For public access to this slot, use LEAF-SOURCE-NAME.)
   ;;
   ;; the name of LEAF as it appears in the source, e.g. 'FOO or '(SETF
@@ -774,7 +824,7 @@
              :pretty-ir-printer
              (pretty-print-global-var structure stream))
   %source-name
-  #!+sb-show id
+  #+sb-show id
   (type :test (not (eq type *universal-type*)))
   (defined-type :test (not (eq defined-type *universal-type*)))
   (where-from :test (not (eq where-from :assumed)))
@@ -783,9 +833,9 @@
 (defun fun-locally-defined-p (name env)
   (typecase env
     (null nil)
-    #!+(and sb-fasteval (host-feature sb-xc))
-    (sb!interpreter:basic-env
-     (values (sb!interpreter:find-lexical-fun env name)))
+    #+(and sb-fasteval (not sb-xc-host))
+    (sb-interpreter:basic-env
+     (values (sb-interpreter:find-lexical-fun env name)))
     (t
      (let ((fun (cdr (assoc name (lexenv-funs env) :test #'equal))))
        (and fun (not (global-var-p fun)))))))
@@ -811,7 +861,7 @@
 (defprinter (defined-fun :identity t
              :pretty-ir-printer (pretty-print-global-var structure stream))
   %source-name
-  #!+sb-show id
+  #+sb-show id
   inlinep
   (functionals :test functionals))
 
@@ -969,7 +1019,9 @@
   (xref () :type list)
   ;; True if this functional was created from an inline expansion. This
   ;; is either T, or the GLOBAL-VAR for which it is an expansion.
-  (inline-expanded nil))
+  (inline-expanded nil)
+  ;; Is it coming from a top-level NAMED-LAMBDA?
+  (top-level-defun-p nil))
 
 (defun pretty-print-functional (functional stream)
   (let ((name (functional-debug-name functional)))
@@ -983,7 +1035,7 @@
              :pretty-ir-printer (pretty-print-functional structure stream))
   %source-name
   %debug-name
-  #!+sb-show id)
+  #+sb-show id)
 
 (defun leaf-debug-name (leaf)
   (if (functional-p leaf)
@@ -1040,7 +1092,7 @@
                      (:constructor make-lambda)
                      (:copier nil))
   ;; list of LAMBDA-VAR descriptors for arguments
-  (vars nil :type list :read-only t)
+  (vars nil :type list)
   ;; If this function was ever a :OPTIONAL function (an entry-point
   ;; for an OPTIONAL-DISPATCH), then this is that OPTIONAL-DISPATCH.
   ;; The optional dispatch will be :DELETED if this function is no
@@ -1104,7 +1156,7 @@
              :pretty-ir-printer (pretty-print-functional structure stream))
   %source-name
   %debug-name
-  #!+sb-show id
+  #+sb-show id
   kind
   (type :test (not (eq type *universal-type*)))
   (where-from :test (not (eq where-from :assumed)))
@@ -1186,7 +1238,7 @@
              :pretty-ir-printer (pretty-print-functional structure stream))
   %source-name
   %debug-name
-  #!+sb-show id
+  #+sb-show id
   (type :test (not (eq type *universal-type*)))
   (where-from :test (not (eq where-from :assumed)))
   arglist
@@ -1289,14 +1341,12 @@
   (eql-var-constraints     nil :type (or null (array t 1)))
   (inheritable-constraints nil :type (or null (array t 1)))
   (private-constraints     nil :type (or null (array t 1)))
-  ;; Initial type of a LET variable as last seen by PROPAGATE-FROM-SETS.
-  (last-initial-type *universal-type* :type ctype)
   ;; The FOP handle of the lexical variable represented by LAMBDA-VAR
   ;; in the fopcompiler.
   (fop-value nil))
 (defprinter (lambda-var :identity t)
   %source-name
-  #!+sb-show id
+  #+sb-show id
   (type :test (not (eq type *universal-type*)))
   (where-from :test (not (eq where-from :assumed)))
   (flags :test (not (zerop flags))
@@ -1331,9 +1381,11 @@
   ;; CONSTANT nodes are always anonymous, since we wish to coalesce named and
   ;; unnamed constants that are equivalent, we need to keep track of the
   ;; reference name for XREF.
-  (%source-name (missing-arg) :type symbol :read-only t))
+  (%source-name (missing-arg) :type symbol :read-only t)
+  ;; Constraints that cannot be expressed as NODE-DERIVED-TYPE
+  constraints)
 (defprinter (ref :identity t)
-  #!+sb-show id
+  #+sb-show id
   (%source-name :test (neq %source-name '.anonymous.))
   leaf)
 
@@ -1411,7 +1463,7 @@
                          (:constructor make-combination (fun))
                          (:copier nil)))
 (defprinter (combination :identity t)
-  #!+sb-show id
+  #+sb-show id
   (fun :prin1 (lvar-uses fun))
   (args :prin1 (mapcar (lambda (x)
                          (if x
@@ -1487,7 +1539,7 @@
   (context nil)
   ;; Avoid compile time type conflict warnings.
   ;; Used by things that expand into ETYPECASE.
-  (silent-conflict nil :type boolean))
+  (silent-conflict nil :type (or boolean (eql :style-warning))))
 (defprinter (cast :identity t)
   %type-check
   value
@@ -1517,32 +1569,8 @@
   (array (missing-arg) :type lvar)
   (bound (missing-arg) :type lvar))
 
-(def!struct (dependent-cast (:include cast) (:copier nil))
-  ;; Either LVARs or LEAFs
-  (deps nil :type list))
-
-(def!struct (function-designator-cast (:include dependent-cast)
-                                      (:copier nil))
-  (caller nil :type symbol)
-  (arg-specs nil :type list)
-  (result-specs nil :type list))
-(defprinter (function-designator-cast :identity t)
-  %type-check
-  value
-  asserted-type
-  (deps :test deps)
-  (caller :test caller)
-  (arg-specs :test arg-specs)
-  (result-specs :test result-specs))
-
-;;; Something will modify this value, warn if it's a constant
-(def!struct (modifying-cast (:include cast) (:copier nil))
-  ;; NIL after a warning has been issued
-  caller)
-
-;;; The hook is called when the value becomes constant
-(def!struct (cast-with-hook (:include cast) (:copier nil))
-  (hook #'missing-arg :type (or null function)))
+;;; Inserted by ARRAY-CALL-TYPE-DERIVER so that it can be later deleted
+(def!struct (array-index-cast (:include cast) (:copier nil)))
 
 ;;;; non-local exit support
 ;;;;
@@ -1559,7 +1587,7 @@
   ;; The cleanup for this entry. NULL only temporarily.
   (cleanup nil :type (or cleanup null)))
 (defprinter (entry :identity t)
-  #!+sb-show id)
+  #+sb-show id)
 
 ;;; The EXIT node marks the place at which exit code would be emitted,
 ;;; if necessary. This is interposed between the uses of the exit
@@ -1580,7 +1608,7 @@
   (value nil :type (or lvar null))
   (nlx-info nil :type (or nlx-info null)))
 (defprinter (exit :identity t)
-  #!+sb-show id
+  #+sb-show id
   (entry :test entry)
   (value :test value))
 
@@ -1600,8 +1628,8 @@
 (defun %coerce-to-policy (thing)
   (typecase thing
     (policy thing)
-    #!+(and sb-fasteval (host-feature sb-xc))
-    (sb!interpreter:basic-env (sb!interpreter:env-policy thing))
+    #+(and sb-fasteval (not sb-xc-host))
+    (sb-interpreter:basic-env (sb-interpreter:env-policy thing))
     (null **baseline-policy**)
     (t (lexenv-policy (etypecase thing
                         (lexenv thing)
@@ -1613,7 +1641,7 @@
 ;; FIXME: the frozen-ness can't actually help optimize anything
 ;; until this file is compiled by the cross-compiler.
 ;; Anything compiled prior to then uses the non-frozen classoid as existed
-;; at load-time of the cross-compiler. SB!XC:PROCLAIM would likely work here.
-#!-sb-fluid
+;; at load-time of the cross-compiler. SB-XC:PROCLAIM would likely work here.
+#-sb-fluid
 (declaim (freeze-type node lexenv ctran lvar cblock component cleanup
                       physenv tail-set nlx-info))
