@@ -9,7 +9,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 
 ;;;; LIST and LIST*
 (define-vop (list-or-list*)
@@ -25,7 +25,7 @@
   (:variant-vars star)
   (:policy :safe)
   (:node-var node)
-  #!-gencgc (:ignore alloc-temp)
+  #-gencgc (:ignore alloc-temp)
   (:generator 0
     (cond ((zerop num)
            (move result null-tn))
@@ -78,33 +78,6 @@
 
 
 ;;;; Special purpose inline allocators.
-#!-gencgc
-(define-vop (allocate-code-object)
-  (:args (boxed-arg :scs (any-reg))
-         (unboxed-arg :scs (any-reg)))
-  (:results (result :scs (descriptor-reg)))
-  (:temporary (:scs (non-descriptor-reg)) ndescr)
-  (:temporary (:scs (non-descriptor-reg)) size)
-  (:temporary (:scs (any-reg) :from (:argument 0)) boxed)
-  (:temporary (:scs (non-descriptor-reg)) unboxed)
-  (:temporary (:sc non-descriptor-reg :offset nl3-offset) pa-flag)
-  (:generator 100
-    (inst addi boxed boxed-arg (fixnumize (1+ code-constants-offset)))
-    (inst clrrwi boxed boxed n-lowtag-bits)
-    (inst srwi unboxed unboxed-arg word-shift)
-    (inst addi unboxed unboxed lowtag-mask)
-    (inst clrrwi unboxed unboxed n-lowtag-bits)
-    (pseudo-atomic (pa-flag)
-      ;; Note: we don't have to subtract off the 4 that was added by
-      ;; pseudo-atomic, because oring in other-pointer-lowtag just adds
-      ;; it right back.
-      (inst add size boxed unboxed)
-      (allocation result size other-pointer-lowtag :temp-tn ndescr :flag-tn pa-flag)
-      (inst slwi ndescr boxed (- n-widetag-bits word-shift))
-      (inst ori ndescr ndescr code-header-widetag)
-      (storew ndescr result 0 other-pointer-lowtag)
-      (storew unboxed-arg result code-code-size-slot other-pointer-lowtag)
-      (storew null-tn result code-debug-info-slot other-pointer-lowtag))))
 
 (define-vop (make-fdefn)
   (:args (name :scs (descriptor-reg) :to :eval))
@@ -191,7 +164,7 @@
   (:args (extra :scs (any-reg)))
   (:arg-types positive-fixnum)
   (:info name words type lowtag)
-  (:ignore name #!-gencgc temp)
+  (:ignore name #-gencgc temp)
   (:results (result :scs (descriptor-reg)))
   (:temporary (:scs (any-reg)) bytes)
   (:temporary (:scs (non-descriptor-reg)) header)
@@ -200,8 +173,13 @@
   (:generator 6
     (inst addi bytes extra (* (1+ words) n-word-bytes))
     (inst slwi header bytes (- n-widetag-bits n-fixnum-tag-bits))
-    (inst addi header header (+ (ash -2 n-widetag-bits) type))
-    (inst clrrwi bytes bytes n-lowtag-bits)
+    ;; The specified EXTRA value is the exact value placed in the header
+    ;; as the word count when allocating code.
+    (cond ((= type code-header-widetag)
+           (inst addi header header type))
+          (t
+           (inst addi header header (+ (ash -2 n-widetag-bits) type))
+           (inst clrrwi bytes bytes n-lowtag-bits)))
     (pseudo-atomic (pa-flag)
       (allocation result bytes lowtag :temp-tn temp :flag-tn pa-flag)
       (storew header result 0 lowtag))))
