@@ -9,7 +9,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 
 ;;; Instruction-like macros.
 
@@ -42,7 +42,7 @@
                                           "SYMBOL-"
                                           (string slot)
                                           "-SLOT")
-                             (find-package "SB!VM"))))
+                             (find-package "SB-VM"))))
          `(progn
             (defmacro ,loader (reg symbol)
               `(inst lwz ,reg null-tn
@@ -62,7 +62,7 @@
   ;; cross-compile time so we can just use a fixed offset within the
   ;; TLS block instead of mucking about with the extra memory access
   ;; (and temp register, for stores)?
-  #!+sb-thread
+  #+sb-thread
   (defmacro load-tl-symbol-value (reg symbol)
     `(progn
        (inst lwz ,reg null-tn
@@ -70,11 +70,11 @@
                 (ash symbol-tls-index-slot word-shift)
                 (- other-pointer-lowtag)))
        (inst lwzx ,reg thread-base-tn ,reg)))
-  #!-sb-thread
+  #-sb-thread
   (defmacro load-tl-symbol-value (reg symbol)
     `(load-symbol-value ,reg ,symbol))
 
-  #!+sb-thread
+  #+sb-thread
   (defmacro store-tl-symbol-value (reg symbol temp)
     `(progn
        (inst lwz ,temp null-tn
@@ -82,7 +82,7 @@
                 (ash symbol-tls-index-slot word-shift)
                 (- other-pointer-lowtag)))
        (inst stwx ,reg thread-base-tn ,temp)))
-  #!-sb-thread
+  #-sb-thread
   (defmacro store-tl-symbol-value (reg symbol temp)
     (declare (ignore temp))
     `(store-symbol-value ,reg ,symbol)))
@@ -186,9 +186,9 @@
   ;; then or in the lowtag.
   ;; Normal allocation to the heap.
   (declare (ignore stack-p node)
-           #!-gencgc
+           #-gencgc
            (ignore temp-tn flag-tn))
-    #!-gencgc
+    #-gencgc
     (let ((alloc-size (gensym)))
       `(let ((,alloc-size ,size))
          (if (logbitp (1- n-lowtag-bits) ,lowtag)
@@ -200,7 +200,7 @@
          (if (numberp ,alloc-size)
              (inst addi alloc-tn alloc-tn ,alloc-size)
              (inst add alloc-tn alloc-tn ,alloc-size))))
-    #!+gencgc
+    #+gencgc
     `(progn
        ;; Make temp-tn be the size
        (cond ((numberp ,size)
@@ -208,11 +208,11 @@
              (t
               (move ,temp-tn ,size)))
 
-       #!-sb-thread
+       #-sb-thread
        (inst lr ,flag-tn (make-fixup "gc_alloc_region" :foreign))
-       #!-sb-thread
+       #-sb-thread
        (inst lwz ,result-tn ,flag-tn 0)
-       #!+sb-thread
+       #+sb-thread
        (inst lwz ,result-tn thread-base-tn (* thread-alloc-region-slot
                                               n-word-bytes))
 
@@ -220,9 +220,9 @@
        ;; it working
        ;; (inst lr ,flag-tn (make-fixup "gc_alloc_region" :foreign 4))
        ;; (inst lwz ,flag-tn ,flag-tn 0)
-       #!-sb-thread
+       #-sb-thread
        (inst lwz ,flag-tn ,flag-tn 4)
-       #!+sb-thread
+       #+sb-thread
        (inst lwz ,flag-tn thread-base-tn (* (1+ thread-alloc-region-slot)
                                             n-word-bytes))
 
@@ -244,13 +244,13 @@
          (inst tw :lge ,result-tn ,flag-tn)
 
          ;; The C code depends on this instruction sequence taking up
-         ;; #!-sb-thread three #!+sb-thread one machine instruction.
+         ;; #-sb-thread three #+sb-thread one machine instruction.
          ;; The lr of a fixup counts as two instructions.
-         #!-sb-thread
+         #-sb-thread
          (inst lr ,flag-tn (make-fixup "gc_alloc_region" :foreign))
-         #!-sb-thread
+         #-sb-thread
          (inst stw ,result-tn ,flag-tn 0)
-         #!+sb-thread
+         #+sb-thread
          (inst stw ,result-tn thread-base-tn (* thread-alloc-region-slot
                                                 n-word-bytes)))
 
@@ -312,7 +312,7 @@
 (defun generate-error-code (vop error-code &rest values)
   "Generate-Error-Code Error-code Value*
   Emit code for an error with the specified Error-Code and context Values."
-  (assemble (*elsewhere*)
+  (assemble (:elsewhere)
     (let ((start-lab (gen-label)))
       (emit-label start-lab)
       (emit-error-break vop error-trap (error-number-or-lose error-code) values)
@@ -329,9 +329,9 @@
 ;;; trap if ALLOC-TN's negative (handling the deferred interrupt) and
 ;;; using FLAG-TN - minus the large constant - to correct ALLOC-TN.
 (defmacro pseudo-atomic ((flag-tn) &body forms)
-  #!+sb-safepoint-strictly
+  #+sb-safepoint-strictly
   `(progn ,flag-tn ,@forms (emit-safepoint))
-  #!-sb-safepoint-strictly
+  #-sb-safepoint-strictly
   `(progn
      (without-scheduling ()
        ;; Extra debugging stuff:
@@ -351,9 +351,9 @@
      (progn
        (inst andi. ,flag-tn alloc-tn lowtag-mask)
        (inst twi :ne ,flag-tn 0))
-     #!+sb-safepoint
+     #+sb-safepoint
      (emit-safepoint)))
 
-#!+sb-safepoint
+#+sb-safepoint
 (defun emit-safepoint ()
   (inst lwz zero-tn null-tn (- (+ gc-safepoint-trap-offset n-word-bytes other-pointer-lowtag))))
