@@ -11,7 +11,38 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
+
+; ok, so we're supposed to use our instruction scheduler, but we don't,
+; because of "needs a little more work in the assembler"
+(defconstant sb-assem:assem-scheduler-p nil)
+(defconstant sb-assem:+inst-alignment-bytes+ 4)
+;;; needs a little more work in the assembler, to realise that the
+;;; delays requested here are not mandatory, so that the assembler
+;;; shouldn't fill gaps with NOPs but with real instructions.  -- CSR,
+;;; 2003-09-08
+#+nil (defconstant sb-assem:+assem-max-locations+ 70)
+
+(defconstant +backend-fasl-file-implementation+ :ppc)
+  ;; On Linux, the ABI specifies the page size to be 4k-64k, use the
+  ;; maximum of that range. FIXME: it'd be great if somebody would
+  ;; find out whether using exact multiples of the page size actually
+  ;; matters in the few places where that's done, or whether we could
+  ;; just use 4k everywhere.
+(defconstant +backend-page-bytes+ #+linux 65536 #-linux 4096)
+
+;;; The size in bytes of GENCGC cards, i.e. the granularity at which
+;;; writes to old generations are logged.  With mprotect-based write
+;;; barriers, this must be a multiple of the OS page size.
+(defconstant gencgc-card-bytes +backend-page-bytes+)
+;;; The minimum size of new allocation regions.  While it doesn't
+;;; currently make a lot of sense to have a card size lower than
+;;; the alloc granularity, it will, once we are smarter about finding
+;;; the start of objects.
+(defconstant gencgc-alloc-granularity 0)
+;;; The minimum size at which we release address ranges to the OS.
+;;; This must be a multiple of the OS page size.
+(defconstant gencgc-release-granularity +backend-page-bytes+)
 
 ;;; number of bits per word where a word holds one lisp descriptor
 (defconstant n-word-bits 32)
@@ -84,7 +115,7 @@
 ;;;; Where to put the different spaces.
 
 ;;; On non-gencgc we need large dynamic and static spaces for PURIFY
-#!-gencgc
+#-gencgc
 (progn
   (defconstant read-only-space-start #x04000000)
   (defconstant read-only-space-end   #x07ff8000)
@@ -95,33 +126,29 @@
   (defconstant linkage-table-space-end   #x0b000000))
 
 ;;; While on gencgc we don't.
-#!+gencgc
+#+gencgc
 (!gencgc-space-setup #x04000000
                      :dynamic-space-start
-                     #!+linux   #x4f000000
-                     #!+netbsd  #x4f000000
-                     #!+openbsd #x4f000000
-                     #!+darwin  #x10000000)
+                     #+linux   #x4f000000
+                     #+netbsd  #x4f000000
+                     #+openbsd #x4f000000
+                     #+darwin  #x10000000)
 
 (defconstant linkage-table-entry-size 16)
 
-#!+linux
+#+linux
 (progn
-  #!-gencgc
+  #-gencgc
   (progn
-    (defconstant dynamic-0-space-start #x4f000000)
-    (defconstant dynamic-0-space-end   #x66fff000)
-    (defconstant dynamic-1-space-start #x67000000)
-    (defconstant dynamic-1-space-end   #x7efff000)))
+    (defparameter dynamic-0-space-start #x4f000000)
+    (defparameter dynamic-0-space-end   #x66fff000)))
 
-#!+netbsd
+#+netbsd
 (progn
-  #!-gencgc
+  #-gencgc
   (progn
-    (defconstant dynamic-0-space-start #x4f000000)
-    (defconstant dynamic-0-space-end   #x66fff000)
-    (defconstant dynamic-1-space-start #x67000000)
-    (defconstant dynamic-1-space-end   #x7efff000)))
+    (defparameter dynamic-0-space-start #x4f000000)
+    (defparameter dynamic-0-space-end   #x66fff000)))
 
 ;;; Text and data segments start at #x01800000.  Range for randomized
 ;;; malloc() starts #x20000000 (MAXDSIZ) after end of data seg and
@@ -130,37 +157,30 @@
 ;;; FIXME: MAXDSIZ is a kernel parameter, and can vary as high as 1GB.
 ;;; These parameters should probably be tested under such a configuration,
 ;;; as rare as it might or might not be.
-#!+openbsd
+#+openbsd
 (progn
-  #!-gencgc
+  #-gencgc
   (progn
-    (defconstant dynamic-0-space-start #x4f000000)
-    (defconstant dynamic-0-space-end   #x5cfff000)
-    (defconstant dynamic-1-space-start #x5f000000)
-    (defconstant dynamic-1-space-end   #x6cfff000)))
+    (defparameter dynamic-0-space-start #x4f000000)
+    (defparameter dynamic-0-space-end   #x5cfff000)))
 
-#!+darwin
+#+darwin
 (progn
-  #!-gencgc
+  #-gencgc
   (progn
-    (defconstant dynamic-0-space-start #x10000000)
-    (defconstant dynamic-0-space-end   #x3ffff000)
-
-    (defconstant dynamic-1-space-start #x40000000)
-    (defconstant dynamic-1-space-end   #x6ffff000)))
+    (defparameter dynamic-0-space-start #x10000000)
+    (defparameter dynamic-0-space-end   #x3ffff000)))
 
-;;;; Other miscellaneous constants.
-
 (defenum (:start 8)
   halt-trap
   pending-interrupt-trap
-  error-trap
   cerror-trap
   breakpoint-trap
   fun-end-breakpoint-trap
   after-breakpoint-trap
   single-step-around-trap
-  single-step-before-trap)
+  single-step-before-trap
+  error-trap)
 
 ;;;; Static symbols.
 
@@ -174,19 +194,11 @@
 ;;; can be loaded directly out of them by indirecting relative to NIL.
 ;;;
 (defconstant-eqx +static-symbols+
-  `(,@+common-static-symbols+
-    ,@+c-callable-static-symbols+
-     #!+gencgc *restart-lisp-function*
-
-     ;; CLH: 20060210 Taken from x86-64/parms.lisp per JES' suggestion
-     ;; Needed for callbacks to work across saving cores. see
-     ;; ALIEN-CALLBACK-ASSEMBLER-WRAPPER in c-call.lisp for gory
-     ;; details.
-     sb!alien::*enter-alien-callback*)
-  #'equal)
+ `#(,@+common-static-symbols+)
+  #'equalp)
 
 (defconstant-eqx +static-fdefns+
-  '(length
+  #(length
     two-arg-+
     two-arg--
     two-arg-*
@@ -205,4 +217,4 @@
     two-arg-eqv
     two-arg-gcd
     two-arg-lcm)
-  #'equal)
+  #'equalp)
