@@ -7,7 +7,27 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
+
+(defconstant sb-assem:assem-scheduler-p t)
+(defconstant sb-assem:+inst-alignment-bytes+ 4)
+(defconstant sb-assem:+assem-max-locations+ 100)
+
+(defconstant +backend-fasl-file-implementation+ :sparc)
+(defconstant +backend-page-bytes+ 8192)
+
+;;; The size in bytes of GENCGC cards, i.e. the granularity at which
+;;; writes to old generations are logged.  With mprotect-based write
+;;; barriers, this must be a multiple of the OS page size.
+(defconstant gencgc-card-bytes +backend-page-bytes+)
+;;; The minimum size of new allocation regions.  While it doesn't
+;;; currently make a lot of sense to have a card size lower than
+;;; the alloc granularity, it will, once we are smarter about finding
+;;; the start of objects.
+(defconstant gencgc-alloc-granularity 0)
+;;; The minimum size at which we release address ranges to the OS.
+;;; This must be a multiple of the OS page size.
+(defconstant gencgc-release-granularity +backend-page-bytes+)
 
 ;;;; Machine Architecture parameters:
 (eval-when (:compile-toplevel :load-toplevel :execute)
@@ -92,8 +112,11 @@
 
 ;;;; Description of the target address space.
 
+#+gencgc ; sensibly small read-only and static spaces
+(!gencgc-space-setup #x0f800000 :dynamic-space-start #x30000000)
+
 ;;; Where to put the different spaces.  Must match the C code!
-#!+(and linux cheneygc)
+#+(and linux cheneygc)
 (progn
   (defconstant linkage-table-space-start #x0f800000)
   (defconstant linkage-table-space-end   #x10000000)
@@ -104,27 +127,10 @@
   (defconstant static-space-start        #x28000000)
   (defconstant static-space-end          #x2c000000)
 
-  (defconstant dynamic-0-space-start #x30000000)
-  (defconstant dynamic-0-space-end   #x38000000)
+  (defparameter dynamic-0-space-start #x30000000)
+  (defparameter dynamic-0-space-end   #x38000000))
 
-  (defconstant dynamic-1-space-start #x40000000)
-  (defconstant dynamic-1-space-end   #x48000000))
-
-#!+(and linux gencgc) ; sensibly small read-only and static spaces
-(progn
-  (defconstant linkage-table-space-start #x0f800000)
-  (defconstant linkage-table-space-end   #x10000000)
-
-  (defconstant read-only-space-start     #x11000000)
-  (defconstant read-only-space-end       #x110ff000)
-
-  (defconstant static-space-start        #x11100000)
-  (defconstant static-space-end          #x111ff000)
-
-  (defconstant dynamic-space-start       #x30000000)
-  (defconstant dynamic-space-end         (!configure-dynamic-space-end)))
-
-#!+(and sunos cheneygc) ; might as well start by trying the same numbers
+#+(and sunos cheneygc) ; might as well start by trying the same numbers
 (progn
   (defconstant linkage-table-space-start #x0f800000)
   (defconstant linkage-table-space-end   #x10000000)
@@ -135,27 +141,10 @@
   (defconstant static-space-start        #x28000000)
   (defconstant static-space-end          #x2c000000)
 
-  (defconstant dynamic-0-space-start     #x30000000)
-  (defconstant dynamic-0-space-end       #x38000000)
+  (defparameter dynamic-0-space-start    #x30000000)
+  (defparameter dynamic-0-space-end      #x38000000))
 
-  (defconstant dynamic-1-space-start     #x40000000)
-  (defconstant dynamic-1-space-end       #x48000000))
-
-#!+(and sunos gencgc) ; sensibly small read-only and static spaces
-(progn
-  (defconstant linkage-table-space-start #x0f800000)
-  (defconstant linkage-table-space-end   #x10000000)
-
-  (defconstant read-only-space-start     #x11000000)
-  (defconstant read-only-space-end       #x110ff000)
-
-  (defconstant static-space-start        #x11100000)
-  (defconstant static-space-end          #x111ff000)
-
-  (defconstant dynamic-space-start       #x30000000)
-  (defconstant dynamic-space-end         (!configure-dynamic-space-end)))
-
-#!+netbsd ; Need a gap at 0x4000000 for shared libraries
+#+(and netbsd cheneygc) ; Need a gap at 0x4000000 for shared libraries
 (progn
   (defconstant linkage-table-space-start #x0f800000)
   (defconstant linkage-table-space-end   #x10000000)
@@ -166,30 +155,25 @@
   (defconstant static-space-start        #x18000000)
   (defconstant static-space-end          #x1c000000)
 
-  (defconstant dynamic-0-space-start     #x48000000)
-  (defconstant dynamic-0-space-end       #x5ffff000)
-
-  (defconstant dynamic-1-space-start     #x60000000)
-  (defconstant dynamic-1-space-end       #x77fff000))
+  (defparameter dynamic-0-space-start    #x48000000)
+  (defparameter dynamic-0-space-end      #x5ffff000))
 
 ;; Size of one linkage-table entry in bytes. See comment in
 ;; src/runtime/sparc-arch.c
 (defconstant linkage-table-entry-size 16)
 
 
-;;;; other random constants.
-
 (defenum (:start 8)
   halt-trap
   pending-interrupt-trap
-  error-trap
   cerror-trap
   breakpoint-trap
   fun-end-breakpoint-trap
   after-breakpoint-trap
   single-step-around-trap
   single-step-before-trap
-  #!+gencgc allocation-trap)
+  #+gencgc allocation-trap
+  error-trap)
 
 ;;;; static symbols.
 
@@ -202,18 +186,16 @@
 ;;; can be loaded directly out of them by indirecting relative to NIL.
 ;;;
 (defconstant-eqx +static-symbols+
-  `(,@+common-static-symbols+
-   ,@+c-callable-static-symbols+
-   #!+gencgc *restart-lisp-function*)
-  #'equal)
+  `#(,@+common-static-symbols+)
+  #'equalp)
 
 (defconstant-eqx +static-fdefns+
-  '(length
+  #(length
     two-arg-+ two-arg-- two-arg-* two-arg-/ two-arg-< two-arg-> two-arg-=
     two-arg-<= two-arg->= two-arg-/= eql %negate
     two-arg-and two-arg-ior two-arg-xor two-arg-eqv
     two-arg-gcd two-arg-lcm)
-  #'equal)
+  #'equalp)
 
 ;;;; Pseudo-atomic trap number
 
@@ -227,7 +209,7 @@
 ;;; allowing other architectures (which don't necessarily use traps
 ;;; for pseudo-atomic) to propagate a magic number to C land via
 ;;; sbcl.h.
-#!-linux
+#-linux
 (defconstant pseudo-atomic-trap #x10)
-#!+linux
+#+linux
 (defconstant pseudo-atomic-trap #x40)
