@@ -11,30 +11,22 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!C")
+(in-package "SB-C")
 
 ;;;; CL:COMPILE
 
-(defun get-lambda-to-compile (definition-designator)
-  (if (consp definition-designator)
-      definition-designator
-      (multiple-value-bind (definition env-p)
-                           (function-lambda-expression definition-designator)
-        (when env-p
-          (error "~S was defined in a non-null environment."
-                 definition-designator))
-        (unless definition
-          (error "can't find a definition for ~S" definition-designator))
-        definition)))
+;;; Handle the following:
+;;;  - CL:COMPILE when the argument is not already a compiled function.
+;;;  - %SIMPLE-EVAL in "pretend we don't have an interpreter" mode
+;;;    a/k/a "compile all the things"
+;;;  - SB-INTERPRETER::EVAL-IN-ENVIRONMENT when it can't just do that.
 
-;;; Handle the nontrivial case of CL:COMPILE.
-;;;
-;;; If ERRORP is true signals an error immediately -- otherwise returns
+;;; If ERORRP is true signals an error immediately -- otherwise returns
 ;;; a function that will signal the error.
-(defun actually-compile (name definition *lexenv* source-info tlf errorp)
+(defun compile-in-lexenv (form *lexenv* name source-info tlf ephemeral errorp)
   (let ((source-paths (when source-info *source-paths*)))
     (with-compilation-values
-      (sb!xc:with-compilation-unit ()
+      (sb-xc:with-compilation-unit ()
         ;; FIXME: These bindings were copied from SUB-COMPILE-FILE with
         ;; few changes. Once things are stable, the shared bindings
         ;; probably be merged back together into some shared utility
@@ -47,19 +39,18 @@
                   ;; also have a source-paths already set up -- so drop
                   ;; the ones from WITH-COMPILATION-VALUES.
                   (*source-paths* (or source-paths *source-paths*))
-                  (form (get-lambda-to-compile definition))
                   (*source-info* (or source-info
                                   (make-lisp-source-info
                                    form :parent *source-info*)))
                   (*toplevel-lambdas* ())
                   (*block-compile* nil)
                   (*allow-instrumenting* nil)
-                  (*code-coverage-records* nil)
-                  (*code-coverage-blocks* nil)
+                  (*compiler-coverage-metadata* nil)
+                  (*msan-unpoison*
+                   (and (member :msan *features*)
+                        (find-dynamic-foreign-symbol-address "__msan_unpoison")))
                   (*current-path* nil)
-                  (*last-format-string* nil)
-                  (*last-format-args* nil)
-                  (*last-message-count* 0)
+                  (*last-message-count* (list* 0 nil nil))
                   (*last-error-context* nil)
                   (*gensym-counter* 0)
                   ;; KLUDGE: This rebinding of policy is necessary so that
@@ -94,10 +85,9 @@
                          ;; the error came from, not how the compiler got there.
                          (go :error))))
                  (return
-                   (with-world-lock ()
-                     (%compile form (make-core-object)
+                     (%compile form (make-core-object ephemeral)
                                :name name
-                               :path `(original-source-start 0 ,tlf))))))
+                               :path `(original-source-start 0 ,tlf)))))
            :error
              ;; Either signal the error right away, or return a function that
              ;; will signal the corresponding COMPILED-PROGRAM-ERROR. This is so
@@ -118,35 +108,6 @@
                               :message message
                               :source source)))))))))))
 
-(defun compile-in-lexenv (name definition lexenv
-                          &optional source-info tlf errorp)
-  (dx-flet ((really-compile (definition lexenv)
-              (actually-compile
-               name definition lexenv source-info tlf errorp)))
-    (multiple-value-bind (compiled-definition warnings-p failure-p)
-        (typecase definition
-          #!+sb-fasteval
-          (sb!interpreter:interpreted-function
-           (multiple-value-call #'really-compile
-             (sb!interpreter:prepare-for-compile definition)))
-          #!+sb-eval
-          (sb!eval:interpreted-function
-           (multiple-value-call #'really-compile
-             (sb!eval:prepare-for-compile definition)))
-          (function
-           (values definition nil nil))
-          (t
-           (really-compile definition lexenv)))
-      (aver (typep compiled-definition 'compiled-function))
-      (let ((result (if (not name)
-                        compiled-definition
-                        (progn
-                          (if (and (symbolp name) (macro-function name))
-                              (setf (macro-function name) compiled-definition)
-                              (setf (fdefinition name) compiled-definition))
-                          name))))
-        (values result warnings-p failure-p)))))
-
 (defun compile (name &optional (definition (or (and (symbolp name)
                                                     (macro-function name))
                                                (fdefinition name))))
@@ -168,7 +129,34 @@ WARNING occur during the compilation, and NIL otherwise.
 Tertiary value is true if any conditions of type ERROR, or WARNING that are
 not STYLE-WARNINGs occur during compilation, and NIL otherwise.
 "
-  (compile-in-lexenv name definition (make-null-lexenv)))
+  (multiple-value-bind (compiled-definition warnings-p failure-p)
+      ;; TODO: generic functions with any interpreted methods
+      ;; should compile the methods and reinstall them.
+      (if (compiled-function-p definition)
+          (values definition nil nil)
+          (multiple-value-bind (sexpr lexenv)
+              (typecase definition
+                #+sb-fasteval
+                (sb-interpreter:interpreted-function
+                 (sb-interpreter:prepare-for-compile definition))
+                #+sb-eval
+                (sb-eval:interpreted-function
+                 (sb-eval:prepare-for-compile definition))
+                (t
+                 (values definition (make-null-lexenv))))
+            (multiple-value-bind (compiled-definition warnings-p failure-p)
+                (compile-in-lexenv (the cons sexpr) lexenv name nil nil nil nil)
+              (aver (typep compiled-definition 'compiled-function))
+              (values compiled-definition warnings-p failure-p))))
+    (values (cond (name
+                   (if (and (symbolp name) (macro-function name))
+                       (setf (macro-function name) compiled-definition)
+                       (setf (fdefinition name) compiled-definition))
+                   name)
+                  (t
+                   compiled-definition))
+            warnings-p
+            failure-p)))
 
 (defun make-form-tracking-stream-observer (file-info)
    (lambda (arg1 arg2 arg3)
@@ -337,3 +325,25 @@ not STYLE-WARNINGs occur during compilation, and NIL otherwise.
                      #'< :key 'caar))))
         (recurse path-to-find (cdaar list) (cdr list))))
     start-char))
+
+;;;; Coverage helpers
+
+(defun record-code-coverage (namestring cc)
+  (setf (gethash namestring (car *code-coverage-info*)) cc))
+
+(defun clear-code-coverage ()
+  (clrhash (car *code-coverage-info*))
+  (setf (cdr *code-coverage-info*) nil))
+
+(defun reset-code-coverage ()
+  (maphash (lambda (info cc)
+             (declare (ignore info))
+             (dolist (cc-entry cc)
+               (setf (cdr cc-entry) +code-coverage-unmarked+)))
+           (car *code-coverage-info*)))
+
+(defun code-coverage-record-marked (record)
+  (aver (consp record))
+  (ecase (cdr record)
+    ((#.+code-coverage-unmarked+) nil)
+    ((t) t)))
