@@ -10,87 +10,87 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!C")
+(in-package "SB-C")
 
 ;;; the largest number of TNs whose liveness changes that we can have
 ;;; in any block
 (defconstant local-tn-limit 64)
 
-(deftype local-tn-number () `(integer 0 (,local-tn-limit)))
+(def!type local-tn-number () `(integer 0 (,local-tn-limit)))
 (deftype local-tn-count () `(integer 0 ,local-tn-limit))
 (deftype local-tn-vector () `(simple-vector ,local-tn-limit))
-(deftype local-tn-bit-vector () `(simple-bit-vector ,local-tn-limit))
+(def!type local-tn-bit-vector () `(simple-bit-vector ,local-tn-limit))
 
 ;;; vectors indexed by SC numbers
-(deftype sc-vector () `(simple-vector ,sb!vm:sc-number-limit))
-(deftype sc-bit-vector () `(simple-bit-vector ,sb!vm:sc-number-limit))
+(def!type sc-vector () `(simple-vector ,sb-vm:sc-number-limit))
+(deftype sc-bit-vector () `(simple-bit-vector ,sb-vm:sc-number-limit))
 
 ;;; Bitset representation of a set of locations in a finite SC.
 (def!type sc-locations ()
-  `(unsigned-byte ,sb!vm:finite-sc-offset-limit))
+  `(unsigned-byte ,sb-vm:finite-sc-offset-limit))
 
 (defun make-sc-locations (locations)
   (reduce (lambda (value location)
-            (check-type location sb!vm:finite-sc-offset)
+            (check-type location sb-vm:finite-sc-offset)
             (dpb 1 (byte 1 location) value))
           locations :initial-value 0))
 
 (declaim (inline sc-offset-to-sc-locations)
-         (ftype (sfunction (sb!vm:finite-sc-offset) sc-locations)
+         (ftype (sfunction (sb-vm:finite-sc-offset) sc-locations)
                 sc-offset-to-sc-locations))
 (defun sc-offset-to-sc-locations (offset)
   (dpb 1 (byte 1 offset) 0))
 
 (declaim (inline sc-locations-count)
-         (ftype (sfunction (sc-locations) (integer 0 #.sb!vm:finite-sc-offset-limit))
+         (ftype (sfunction (sc-locations) (integer 0 #.sb-vm:finite-sc-offset-limit))
                 sc-locations-count))
 (defun sc-locations-count (locations)
   (logcount locations))
 
 (declaim (inline sc-locations-first)
-         (ftype (sfunction (sc-locations) sb!vm:finite-sc-offset)
+         (ftype (sfunction (sc-locations) sb-vm:finite-sc-offset)
                 sc-locations-first))
 (defun sc-locations-first (locations)
   (1- (integer-length (logxor locations (1- locations)))))
 
 (declaim (inline sc-locations-member)
-         (ftype (sfunction (sb!vm:finite-sc-offset sc-locations) boolean)
+         (ftype (sfunction (sb-vm:finite-sc-offset sc-locations) boolean)
                 sc-locations-member))
 (defun sc-locations-member (location locations)
   (logbitp location locations))
 
+;;; This used to have two local functions in it, but when it did,
+;;; using ABCL as the build host crashed thusly with no backtrace:
+;;; (funcall (macro-function 'do-sc-locations)
+;;;           '(do-sc-locations (el (sc-locations sc) nil (sc-element-size sc))
+;;;             (feep))
+;;;            nil)
+;;;  => Debugger invoked on condition of type TYPE-ERROR
+;;;     The value NIL is not of type STRUCTURE-OBJECT.
 (defmacro do-sc-locations ((location locations
-                            &optional result increment (limit nil limitp))
+                            &optional result increment (limit 'sb-vm:finite-sc-offset-limit))
                            &body body)
-  (let* ((limit (cond
-                  ((not limitp)
-                   sb!vm:finite-sc-offset-limit)
-                  ((not (constantp limit))
-                   limit)
-                  ((eval limit))
-                  (t
-                   sb!vm:finite-sc-offset-limit)))
-         (mid   (floor sb!vm:finite-sc-offset-limit 2)))
+  (let ((mid (floor sb-vm:finite-sc-offset-limit 2)))
     (once-only ((locations locations)
-                (increment `(the sb!vm:finite-sc-offset ,(or increment 1))))
-      (labels ((make-block (start end)
-                 `(loop named #:noname
-                        for ,location of-type (integer 0 ,sb!vm:finite-sc-offset-limit)
-                        from ,start below ,end by ,increment
-                        when (logbitp ,location ,locations)
-                          do ,@body))
-               (make-guarded-block (start end)
-                 (unless (and (numberp limit) (<= limit start))
+                (increment `(the sb-vm:finite-sc-offset ,(or increment 1))))
+      (flet ((make-guarded-block (start end)
+                 (unless (and (integerp limit) (> start limit))
                    (let ((mask (dpb -1 (byte mid start) 0)))
                      `((when (logtest ,mask ,locations)
-                         ,(make-block start end )))))))
+                         (loop named #:noname
+                               for ,location
+                               from ,start below ,end by ,increment
+                               when (logbitp ,location ,locations)
+                                 do (locally (declare (type sb-vm:finite-sc-offset
+                                                            ,location))
+                                      ,@body))))))))
         `(block nil
            ,@(make-guarded-block 0   mid)
            ,@(make-guarded-block mid limit)
            ,result)))))
 
 ;;; the different policies we can use to determine the coding strategy
-(deftype ltn-policy ()
+(def!type ltn-policy ()
   '(member :safe :small :small-safe :fast :fast-safe))
 
 ;;;; PRIMITIVE-TYPEs
@@ -100,7 +100,7 @@
 ;;; done on the basis of the primitive types of the operands, and the
 ;;; primitive type of a value is used to constrain the possible
 ;;; representations of that value.
-(defstruct (primitive-type (:copier nil))
+(def!struct (primitive-type (:copier nil))
   ;; the name of this PRIMITIVE-TYPE
   (name nil :type symbol :read-only t)
   ;; a list of the SC numbers for all the SCs that a TN of this type
@@ -361,7 +361,7 @@
   ;; dynamic vop count info. This is needed by both ir2-convert and
   ;; setup-dynamic-count-info. (But only if we are generating code to
   ;; collect dynamic statistics.)
-  #!+sb-dyncount
+  #+sb-dyncount
   (dyncount-info nil :type (or null dyncount-info)))
 
 ;;; An ENTRY-INFO condenses all the information that the dumper needs
@@ -386,8 +386,8 @@
   ;; a function type specifier representing the arguments and results
   ;; of this function
   (type 'function :type (or list (member function)))
-  ;; docstring and/or xref information for the XEP
-  (info nil :type (or null simple-vector string (cons string simple-vector))))
+  ;; source form and/or docstring and/or xref information for the XEP
+  (form/doc/xrefs nil :type (or null simple-vector string cons)))
 
 ;;; An IR2-PHYSENV is used to annotate non-LET LAMBDAs with their
 ;;; passing locations. It is stored in the PHYSENV-INFO.
@@ -413,11 +413,11 @@
   (return-pc-pass (missing-arg) :type tn :read-only t)
   ;; a label that marks the first instruction after the RETURN-PC has
   ;; been moved from its passing location to its save location.
-  #!-fp-and-pc-standard-save
+  #-fp-and-pc-standard-save
   (lra-saved-pc nil :type (or label null))
   ;; a label that marks the first instruction after the OLD-FP has
   ;; been moved from its passing location to its save location.
-  #!-fp-and-pc-standard-save
+  #-fp-and-pc-standard-save
   (cfp-saved-pc nil :type (or label null))
   ;; True if this function has a frame on the number stack. This is
   ;; set by representation selection whenever it is possible that some
@@ -437,7 +437,7 @@
   ;; function as far as the debugger is concerned.
   (environment-start nil :type (or label null))
   (closure-save-tn nil :type (or tn null))
-  #!+unwind-to-frame-and-call-vop
+  #+unwind-to-frame-and-call-vop
   (bsp-save-tn nil :type (or tn null)))
 
 (defprinter (ir2-physenv)
@@ -461,7 +461,8 @@
   (count (missing-arg) :type (or index (member :unknown)))
   ;; If count isn't :UNKNOWN, then this is a list of the
   ;; primitive-types of each value.
-  (types () :type list)
+  (primitive-types () :type list)
+  (types nil :type list)
   ;; If kind is :FIXED, then this is the list of the TNs that we
   ;; return the values in.
   (locations () :type list))
@@ -478,10 +479,13 @@
   ;; unwind-block, so we leave this slot null.
   (home nil :type (or tn null))
   ;; the saved control stack pointer
-  (save-sp (missing-arg) :type tn)
+  (save-sp nil :type (or tn null))
   ;; the list of dynamic state save TNs
-  (dynamic-state (list* (make-stack-pointer-tn)
+
+  (dynamic-state #-x86-64
+                 (list* (make-stack-pointer-tn)
                         (make-dynamic-state-tns))
+                 #+x86-64 nil
                  :type list)
   ;; the target label for NLX entry
   (target (gen-label) :type label))
@@ -552,7 +556,9 @@
   ;; vice-versa.
   (target nil :type (or null tn-ref))
   ;; the load TN allocated for this operand, if any
-  (load-tn nil :type (or tn null)))
+  (load-tn nil :type (or tn null))
+  ;; The type of the LVAR the TN of this TN-REF is used for.
+  (type nil :type (or ctype null)))
 
 ;;; A TEMPLATE object represents a particular IR2 coding strategy for
 ;;; a known function.
@@ -761,7 +767,7 @@
 
 ;;; The SB structure represents the global information associated with
 ;;; a storage base.
-(defstruct (storage-base (:copier nil) (:conc-name sb-))
+(def!struct (storage-base (:copier nil) (:conc-name sb-))
   ;; name, for printing and reference
   (name nil :type symbol :read-only t)
   ;; the kind of storage base (which determines the packing
@@ -804,7 +810,6 @@
   ;; is set, then the location is in use somewhere in the block, and
   ;; thus has a conflict for always-live TNs.
   (always-live '#() :type simple-vector)
-  (always-live-count '#() :type simple-vector)
   ;; a vector containing the TN currently live in each location in the
   ;; SB, or NIL if the location is unused. This is used during load-tn pack.
   (live-tns '#() :type simple-vector)
@@ -812,7 +817,8 @@
   ;; might not be virgin, and thus must be reinitialized when PACK
   ;; starts. Less then the length of those vectors when not all of the
   ;; length was used on the previously packed component.
-  (last-block-count 0 :type index))
+  (last-block-count 0 :type index)
+  (wired-map 0 :type sb-vm:finite-sc-offset-map))
 (declaim (freeze-type storage-base finite-sb-template finite-sb))
 
 ;;; Give this a toplevel value so that it can be declaimed ALWAYS-BOUND.
@@ -836,8 +842,9 @@
   `(fsb-conflicts (svref *finite-sbs* (finite-sb-index ,sb))))
 (defmacro finite-sb-always-live (sb)
   `(fsb-always-live (svref *finite-sbs* (finite-sb-index ,sb))))
-(defmacro finite-sb-always-live-count (sb)
-  `(fsb-always-live-count (svref *finite-sbs* (finite-sb-index ,sb))))
+(defmacro finite-sb-wired-map (sb)
+  `(fsb-wired-map (svref *finite-sbs* (finite-sb-index ,sb))))
+
 (defmacro finite-sb-live-tns (sb)
   `(fsb-live-tns (svref *finite-sbs* (finite-sb-index ,sb))))
 (defmacro finite-sb-last-block-count (sb)
@@ -870,9 +877,9 @@
   ;; of the corresponding move functions. If loading is impossible,
   ;; then the entries are NIL. LOAD-COSTS is initialized to have a 0
   ;; for this SC.
-  (move-funs (make-array sb!vm:sc-number-limit :initial-element nil)
+  (move-funs (make-array sb-vm:sc-number-limit :initial-element nil)
              :type sc-vector)
-  (load-costs (make-array sb!vm:sc-number-limit :initial-element nil)
+  (load-costs (make-array sb-vm:sc-number-limit :initial-element nil)
               :type sc-vector)
   ;; a vector mapping from SC numbers to possibly
   ;; representation-specific move and coerce VOPs. Each entry is a
@@ -892,18 +899,18 @@
   ;; TNs wired in the standard argument registers, since there may
   ;; already be live TNs wired in those locations holding the values
   ;; that we are setting up for unknown-values return.
-  (move-vops (make-array sb!vm:sc-number-limit :initial-element nil)
+  (move-vops (make-array sb-vm:sc-number-limit :initial-element nil)
              :type sc-vector)
   ;; the costs corresponding to the MOVE-VOPS. Separate because this
   ;; info is needed at meta-compile time, while the MOVE-VOPs don't
   ;; exist till load time. If no move is defined, then the entry is
   ;; NIL.
-  (move-costs (make-array sb!vm:sc-number-limit :initial-element nil)
+  (move-costs (make-array sb-vm:sc-number-limit :initial-element nil)
               :type sc-vector)
   ;; similar to Move-VOPs, except that we only ever use the entries
   ;; for this SC and its alternates, since we never combine complex
   ;; representation conversion with argument passing.
-  (move-arg-vops (make-array sb!vm:sc-number-limit :initial-element nil)
+  (move-arg-vops (make-array sb-vm:sc-number-limit :initial-element nil)
                  :type sc-vector)
   ;; true if this SC or one of its alternates in in the NUMBER-STACK SB.
   (number-stack-p nil :type boolean)
@@ -984,11 +991,17 @@
         :type (member :normal :environment :debug-environment
                       :save :save-once  :load :constant
                       :component :alias :unused
-                      #!-fp-and-pc-standard-save :specified-save
+                      #-fp-and-pc-standard-save :specified-save
                       :arg-pass))
   ;; the primitive-type for this TN's value. Null in restricted or
   ;; wired TNs.
   (primitive-type nil :type (or primitive-type null))
+  ;; The type of the LVAR this TN is used for.
+  ;; Meaningless after copy-propagate or other optimizations are performed
+  ;; and TN-REF-TYPE should be used instead.
+  ;; The only purpose of this slot is for REFERENCE-TN to populate
+  ;; TN-REF-TYPE.
+  (type nil :type (or ctype null))
   ;; If this TN represents a variable or constant, then this is the
   ;; corresponding LEAF.
   (leaf nil :type (or leaf null))
@@ -1044,11 +1057,12 @@
   ;; If a :ENVIRONMENT or :DEBUG-ENVIRONMENT TN, this is the
   ;; physical environment that the TN is live throughout.
   (physenv nil :type (or physenv null))
-  ;; The depth of the deepest loop that this TN is used in.
-  (loop-depth 0 :type fixnum))
+  ;; Used by pack-iterative
+  vertex)
+
 (declaim (freeze-type tn))
 (defmethod print-object ((tn tn) stream)
-  (cond ((not (boundp 'sb!c::*compiler-ir-obj-map*))
+  (cond ((not (boundp 'sb-c::*compiler-ir-obj-map*))
          (print-unreadable-object (tn stream :type t :identity t)))
         (t
          (print-unreadable-object (tn stream :type t)
