@@ -9,7 +9,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 
 ;;;; allocation helpers
 
@@ -17,6 +17,11 @@
 ;;; from the C alloc() function by way of the alloc-tramp
 ;;; assembly routine.
 
+(defun tagify (result base lowtag)
+  (if (eql lowtag 0)
+      (inst mov result base)
+      (inst lea result (ea lowtag base))))
+
 (defun stack-allocation (alloc-tn size lowtag)
   (aver (not (location= alloc-tn rsp-tn)))
   (inst sub rsp-tn size)
@@ -28,7 +33,7 @@
   ;; - The real issue is that it's not obvious that the stack is
   ;;   16-byte-aligned at *all* times. Maybe it is, maybe it isn't.
   (inst and rsp-tn #.(lognot lowtag-mask))
-  (inst lea alloc-tn (make-ea :byte :base rsp-tn :disp lowtag))
+  (tagify alloc-tn rsp-tn lowtag)
   (values))
 
 (defun %alloc-tramp (node result-tn size lowtag)
@@ -46,18 +51,16 @@
     (invoke-asm-routine 'call (if to-r11 'alloc-tramp-r11 'alloc-tramp) node)
     (unless to-r11
       (inst pop result-tn)))
-
-  (when lowtag
-    (inst or (reg-in-size result-tn :byte) lowtag))
+  (unless (eql lowtag 0)
+    (inst or :byte result-tn lowtag))
   (values))
 
 ;;; Insert allocation profiler instrumentation
 (defun instrument-alloc (size node)
-  (when (policy node (> sb!c::instrument-consing 1))
+  (when (policy node (> sb-c::instrument-consing 1))
     (let ((skip-instrumentation (gen-label)))
       (inst mov temp-reg-tn
-            (make-ea :qword :base thread-base-tn
-                     :disp (* n-word-bytes thread-profile-data-slot)))
+            (ea (* n-word-bytes thread-profile-data-slot) thread-base-tn))
       (inst test temp-reg-tn temp-reg-tn)
       ;; This instruction is modified to "JMP :z" when profiling is
       ;; partially enabled. After the buffer is assigned, it becomes
@@ -71,13 +74,11 @@
                         'enable-alloc-counter
                         'enable-sized-alloc-counter)))
         (cond ((or (not node) ; assembly routine
-                   (sb!c::code-immobile-p node))
+                   (sb-c::code-immobile-p node))
                (inst call (make-fixup helper :assembly-routine)) ; 5 bytes
                (emit-alignment 3 :long-nop))
               (t
-               (inst call ; 7 bytes
-                     (make-ea :qword :disp
-                              (make-fixup helper :assembly-routine*)))
+               (inst call (ea (make-fixup helper :assembly-routine*))) ; 7 bytes
                (inst nop))) ; align
         (unless (integerp size)
           ;; This TEST instruction is never executed- it informs the profiler
@@ -98,56 +99,73 @@
   (aver (and (not (location= alloc-tn temp-reg-tn))
              (or (integerp size) (not (location= size temp-reg-tn)))))
 
-  #!+(and (not sb-thread) sb-dynamic-core)
+  #+(and (not sb-thread) sb-dynamic-core)
   ;; We'd need a spare reg in which to load boxed_region from the linkage table.
   ;; Could push/pop any random register on the stack and own it temporarily,
   ;; but seeing as nobody cared about this, just punt.
   (%alloc-tramp node alloc-tn size lowtag)
 
-  #!-(and (not sb-thread) sb-dynamic-core)
+  #-(and (not sb-thread) sb-dynamic-core)
   ;; Otherwise do the normal inline allocation thing
   (let ((NOT-INLINE (gen-label))
         (DONE (gen-label))
         ;; Yuck.
-        (in-elsewhere (sb!assem::assembling-to-elsewhere-p))
+        (in-elsewhere (sb-assem::assembling-to-elsewhere-p))
         ;; thread->alloc_region.free_pointer
         (free-pointer
-         #!+sb-thread (thread-slot-ea thread-alloc-region-slot)
-         #!-sb-thread
-         (make-ea :qword :disp (make-fixup "gc_alloc_region" :foreign)))
+         #+sb-thread (thread-slot-ea thread-alloc-region-slot)
+         #-sb-thread (ea (make-fixup "gc_alloc_region" :foreign)))
         ;; thread->alloc_region.end_addr
         (end-addr
-         #!+sb-thread (thread-slot-ea (1+ thread-alloc-region-slot))
-         #!-sb-thread
-         (make-ea :qword :disp (make-fixup "gc_alloc_region" :foreign 8))))
+         #+sb-thread (thread-slot-ea (1+ thread-alloc-region-slot))
+         #-sb-thread (ea (make-fixup "gc_alloc_region" :foreign 8))))
 
     (cond ((or in-elsewhere
                ;; large objects will never be made in a per-thread region
                (and (integerp size)
                     (>= size large-object-size)))
            (%alloc-tramp node alloc-tn size lowtag))
+          ((eql lowtag 0)
+           (cond ((and (tn-p size) (location= size alloc-tn))
+                  (inst mov temp-reg-tn free-pointer)
+                  ;; alloc-tn <- old free ptr and temp <- new free ptr
+                  (inst xadd temp-reg-tn alloc-tn))
+                 (t
+                  (inst mov alloc-tn free-pointer)
+                  (inst lea temp-reg-tn (ea size alloc-tn))))
+           (inst cmp temp-reg-tn end-addr)
+           (inst jmp :a NOT-INLINE)
+           (inst mov free-pointer temp-reg-tn)
+           (emit-label DONE)
+           (assemble (:elsewhere)
+             (emit-label NOT-INLINE)
+             (let ((size (cond ((and (tn-p size) (location= size alloc-tn))
+                                (inst sub temp-reg-tn alloc-tn)
+                                temp-reg-tn)
+                               (t
+                                size))))
+               (%alloc-tramp node alloc-tn size 0))
+             (inst jmp DONE)))
           (t
            (inst mov temp-reg-tn free-pointer)
            (cond ((integerp size)
-                  (inst lea alloc-tn (make-ea :qword :base temp-reg-tn :disp size)))
+                  (inst lea alloc-tn (ea size temp-reg-tn)))
                  ((location= alloc-tn size)
                   (inst add alloc-tn temp-reg-tn))
                  (t
-                  (inst lea alloc-tn (make-ea :qword :base temp-reg-tn :index size))))
+                  (inst lea alloc-tn (ea temp-reg-tn size))))
            (inst cmp alloc-tn end-addr)
            (inst jmp :a NOT-INLINE)
            (inst mov free-pointer alloc-tn)
            (emit-label DONE)
-           (if lowtag
-               (inst lea alloc-tn (make-ea :byte :base temp-reg-tn :disp lowtag))
-               (inst mov alloc-tn temp-reg-tn))
+           (tagify alloc-tn temp-reg-tn lowtag)
            (assemble (:elsewhere)
              (emit-label NOT-INLINE)
              (cond ((and (tn-p size) (location= size alloc-tn)) ; recover SIZE
                     (inst sub alloc-tn free-pointer)
-                    (%alloc-tramp node temp-reg-tn alloc-tn nil))
+                    (%alloc-tramp node temp-reg-tn alloc-tn 0))
                    (t ; SIZE is intact
-                    (%alloc-tramp node temp-reg-tn size nil)))
+                    (%alloc-tramp node temp-reg-tn size 0)))
              (inst jmp DONE))))
     (values)))
 
@@ -156,13 +174,16 @@
 ;;; RESULT-TN.
 (defun fixed-alloc (result-tn widetag size node &optional stack-allocate-p
                     &aux (bytes (pad-data-block size)))
-  (unless stack-allocate-p
-    (instrument-alloc bytes node))
-  (maybe-pseudo-atomic stack-allocate-p
-      (allocation result-tn bytes node stack-allocate-p other-pointer-lowtag)
-      (storew* (logior (ash (1- size) n-widetag-bits) widetag)
-               result-tn 0 other-pointer-lowtag
-               (not stack-allocate-p))))
+  (let ((header (logior (ash (1- size) n-widetag-bits) widetag)))
+    (cond (stack-allocate-p
+           (allocation result-tn bytes node t other-pointer-lowtag)
+           (storew header result-tn 0 other-pointer-lowtag))
+          (t
+           (instrument-alloc bytes node)
+           (pseudo-atomic ()
+             (allocation result-tn bytes node nil 0)
+             (storew* header result-tn 0 0 t)
+             (inst or :byte result-tn other-pointer-lowtag))))))
 
 ;;;; CONS, LIST and LIST*
 (define-vop (list-or-list*)
@@ -182,7 +203,8 @@
            (move result (tn-ref-tn things)))
           (t
            (macrolet
-               ((store-car (tn list &optional (slot cons-car-slot))
+               ((store-slot (tn list &optional (slot cons-car-slot)
+                                               (lowtag list-pointer-lowtag))
                   `(let ((reg
                           ;; FIXME: single-float gets placed in the boxed header
                           ;; rather than just doing an immediate store.
@@ -192,30 +214,50 @@
                              temp)
                             (t
                              (encode-value-if-immediate ,tn)))))
-                     (storew reg ,list ,slot list-pointer-lowtag))))
+                     (storew* reg ,list ,slot ,lowtag (not stack-allocate-p)))))
              (let* ((cons-cells (if star (1- num) num))
                     (stack-allocate-p (node-stack-allocate-p node))
                     (size (* (pad-data-block cons-size) cons-cells)))
                (unless stack-allocate-p
                  (instrument-alloc size node))
-               (maybe-pseudo-atomic stack-allocate-p
-                (allocation res size node stack-allocate-p list-pointer-lowtag)
-                (move ptr res)
-                (dotimes (i (1- cons-cells))
-                  (store-car (tn-ref-tn things) ptr)
-                  (setf things (tn-ref-across things))
-                  (inst add ptr (pad-data-block cons-size))
-                  (storew ptr ptr (- cons-cdr-slot cons-size)
-                          list-pointer-lowtag))
-                (store-car (tn-ref-tn things) ptr)
-                (cond (star
+               (pseudo-atomic (:elide-if stack-allocate-p)
+                (allocation res size node stack-allocate-p
+                            (if (= cons-cells 2) 0 list-pointer-lowtag))
+                (multiple-value-bind (last-base-reg lowtag car cdr)
+                    (cond
+                      ((= cons-cells 2)
+                       ;; Note that this does not use the 'ptr' register at all.
+                       ;; It would require a different vop to free that register up.
+                       (store-slot (tn-ref-tn things) res cons-car-slot 0)
                        (setf things (tn-ref-across things))
-                       (store-car (tn-ref-tn things) ptr cons-cdr-slot))
+                       (inst lea temp (ea (+ (* cons-size n-word-bytes) list-pointer-lowtag) res))
+                       (store-slot temp res cons-cdr-slot 0)
+                       (values res 0 (+ cons-size cons-car-slot) (+ cons-size cons-cdr-slot)))
+                      ;; 1 cons IR1-transforms to CONS which IR2-converts as FIXED-ALLOC.
+                      ((= cons-cells 1) (bug "Why?")) ; shoulda been CONS
                       (t
-                       (storew nil-value ptr cons-cdr-slot
-                               list-pointer-lowtag)))
-                (aver (null (tn-ref-across things)))))
-             (move result res))))))
+                       (move ptr res)
+                       (dotimes (i (1- cons-cells))
+                         (store-slot (tn-ref-tn things) ptr)
+                         (setf things (tn-ref-across things))
+                         (inst add ptr (pad-data-block cons-size))
+                         (storew ptr ptr (- cons-cdr-slot cons-size)
+                                 list-pointer-lowtag))
+                       (values ptr list-pointer-lowtag cons-car-slot cons-cdr-slot)))
+                  (store-slot (tn-ref-tn things) last-base-reg car lowtag)
+                  (cond (star
+                         (setf things (tn-ref-across things))
+                         (store-slot (tn-ref-tn things) last-base-reg cdr lowtag))
+                        (t
+                         (storew* nil-value last-base-reg cdr lowtag
+                                  (not stack-allocate-p))))
+                  (cond ((= cons-cells 2)
+                         (if (location= result res)
+                             (inst or :byte result list-pointer-lowtag)
+                             (inst lea result (ea list-pointer-lowtag res))))
+                        (t
+                         (move result res)))))))
+           (aver (null (tn-ref-across things)))))))
 
 (define-vop (list list-or-list*)
   (:variant nil))
@@ -227,29 +269,32 @@
 
 ;;; Special variant of 'storew' which might have a shorter encoding
 ;;; when storing to the heap (which starts out zero-filled).
+;;; This will always write 8 bytes if WORD is a negative number.
 (defun storew* (word object slot lowtag zeroed)
-  (if (or (not zeroed) (not (typep word '(signed-byte 32))))
-      (storew word object slot lowtag) ; Possibly use temp-reg-tn
-      (inst mov
-            (make-ea (cond ((typep word '(unsigned-byte 8)) :byte)
-                           ((and (not (logtest word #xff))
-                                 (typep (ash word -8) '(unsigned-byte 8)))
-                            ;; Array lengths 128 to 16384 which are multiples of 128
-                            (setq word (ash word -8))
-                            (decf lowtag 1) ; increment address by 1
-                            :byte)
-                           ((and (not (logtest word #xffff))
-                                 (typep (ash word -16) '(unsigned-byte 8)))
-                            ;; etc
-                            (setq word (ash word -16))
-                            (decf lowtag 2) ; increment address by 2
-                            :byte)
-                           ((typep word '(unsigned-byte 16)) :word)
-                           ;; Definitely a (signed-byte 32) due to pre-test.
-                           (t :dword))
-                     :base object
-                     :disp (- (* slot n-word-bytes) lowtag))
-            word)))
+  (cond
+   ((or (not zeroed) (not (typep word '(unsigned-byte 31))))
+    (storew word object slot lowtag)) ; Possibly use temp-reg-tn
+   ((/= word 0)
+    (let ((size
+           (cond ((typep word '(unsigned-byte 8))
+                  :byte)
+                 ((and (not (logtest word #xff))
+                       (typep (ash word -8) '(unsigned-byte 8)))
+                  ;; Array lengths 128 to 16384 which are multiples of 128
+                  (setq word (ash word -8))
+                  (decf lowtag 1) ; increment address by 1
+                  :byte)
+                 ((and (not (logtest word #xffff))
+                       (typep (ash word -16) '(unsigned-byte 8)))
+                  ;; etc
+                  (setq word (ash word -16))
+                  (decf lowtag 2) ; increment address by 2
+                  :byte)
+                 ((typep word '(unsigned-byte 16))
+                  :word)
+                 (t ; must be an (unsigned-byte 31)
+                  :dword))))
+      (inst mov size (ea (- (* slot n-word-bytes) lowtag) object) word)))))
 
 ;;; ALLOCATE-VECTOR
 (macrolet ((calc-size-in-bytes (n-words result-tn)
@@ -257,20 +302,17 @@
                      (pad-data-block (+ (tn-value ,n-words) vector-data-offset)))
                     (t
                      (inst lea ,result-tn
-                           (make-ea :byte :index ,n-words
-                                          :scale (ash 1 (- word-shift n-fixnum-tag-bits))
-                                          :disp (+ lowtag-mask
-                                                   (* vector-data-offset n-word-bytes))))
+                           (ea (+ lowtag-mask (* vector-data-offset n-word-bytes))
+                               nil ,n-words (ash 1 (- word-shift n-fixnum-tag-bits))))
                      (inst and ,result-tn (lognot lowtag-mask))
                      ,result-tn)))
-           (put-header (vector-tn type length zeroed)
+           (put-header (vector-tn lowtag type length zeroed)
              `(progn (storew* (if (sc-is ,type immediate) (tn-value ,type) ,type)
-                              ,vector-tn 0 other-pointer-lowtag ,zeroed)
+                              ,vector-tn 0 ,lowtag ,zeroed)
                      (storew* (if (sc-is ,length immediate)
                                   (fixnumize (tn-value ,length))
                                   ,length)
-                              ,vector-tn vector-length-slot other-pointer-lowtag
-                              ,zeroed))))
+                              ,vector-tn vector-length-slot ,lowtag ,zeroed))))
 
   (define-vop (allocate-vector-on-heap)
     (:args (type :scs (unsigned-reg immediate))
@@ -287,9 +329,10 @@
       ;; so don't move it inside.
       (let ((size (calc-size-in-bytes words result)))
         (instrument-alloc size node)
-        (pseudo-atomic
-         (allocation result size node nil other-pointer-lowtag)
-         (put-header result type length t)))))
+        (pseudo-atomic ()
+         (allocation result size node nil 0)
+         (put-header result 0 type length t)
+         (inst or :byte result other-pointer-lowtag)))))
 
   (define-vop (allocate-vector-on-stack)
     (:args (type :scs (unsigned-reg immediate))
@@ -307,20 +350,21 @@
     (:generator 100
       (let ((size (calc-size-in-bytes words result))
             (rax-zeroed))
+        ;; Compute tagged pointer sooner than later since access off RSP
+        ;; requires an extra byte in the encoding anyway.
         (stack-allocation result size other-pointer-lowtag)
-        (put-header result type length nil)
+        (put-header result other-pointer-lowtag type length nil)
         ;; FIXME: It would be good to check for stack overflow here.
         ;; It would also be good to skip zero-fill of specialized vectors
         ;; perhaps in a policy-dependent way. At worst you'd see random
         ;; bits, and CLHS says consequences are undefined.
-        (when sb!c::*msan-unpoison*
+        (when sb-c::*msan-unpoison*
           ;; Unpoison all DX vectors regardless of widetag.
           ;; Mark the header and length as valid, not just the payload.
-          #!+linux ; unimplemented for others
+          #+linux ; unimplemented for others
           (let ((words-savep
                  ;; 'words' might be co-located with any of the temps
-                 (or (location= words rdi) (location= words rcx) (location= words rax)))
-                (rax rax))
+                 (or (location= words rdi) (location= words rcx) (location= words rax))))
             (setq rax-zeroed (not (location= words rax)))
             (when words-savep ; use 'result' to save 'words'
               (inst mov result words))
@@ -328,11 +372,8 @@
                    (inst mov rcx (+ (tn-value words) vector-data-offset)))
                   (t
                    (inst lea rcx
-                         (make-ea :qword :base words
-                                  :disp (ash vector-data-offset n-fixnum-tag-bits)))
-                   (if (= n-fixnum-tag-bits 1)
-                       (setq rax (reg-in-size rax :dword)) ; don't bother shifting rcx
-                       (inst shr rcx n-fixnum-tag-bits))))
+                         (ea (ash vector-data-offset n-fixnum-tag-bits) words))
+                   (inst shr rcx n-fixnum-tag-bits)))
             (inst mov rdi msan-mem-to-shadow-xor-const)
             (inst xor rdi rsp-tn) ; compute shadow address
             (zeroize rax)
@@ -341,19 +382,18 @@
             (when words-savep
               (inst mov words result) ; restore 'words'
               (inst lea result ; recompute the tagged pointer
-                    (make-ea :byte :base rsp-tn :disp other-pointer-lowtag)))))
-        (unless (sb!c::vector-initialized-p node)
+                    (ea other-pointer-lowtag rsp-tn)))))
+        (unless (sb-c::vector-initialized-p node)
           (let ((data-addr
-                  (make-ea :qword :base result
-                                  :disp (- (* vector-data-offset n-word-bytes)
-                                           other-pointer-lowtag))))
+                  (ea (- (* vector-data-offset n-word-bytes) other-pointer-lowtag)
+                      result)))
             (block zero-fill
               (cond ((sc-is words immediate)
                      (let ((n (tn-value words)))
                        (cond ((> n 8)
                               (inst mov rcx (tn-value words)))
                              ((= n 1)
-                              (inst mov data-addr 0)
+                              (inst mov :qword data-addr 0)
                               (return-from zero-fill))
                              (t
                               (multiple-value-bind (double single) (truncate n 2)
@@ -361,9 +401,8 @@
                                 (dotimes (i double)
                                   (inst movapd data-addr zero)
                                   (setf data-addr
-                                        (make-ea :qword :base (ea-base data-addr)
-                                                        :disp (+ (ea-disp data-addr)
-                                                                 (* n-word-bytes 2)))))
+                                        (ea (+ (ea-disp data-addr) (* n-word-bytes 2))
+                                            (ea-base data-addr))))
                                 (unless (zerop single)
                                   (inst movaps data-addr zero))
                                 (return-from zero-fill))))))
@@ -385,21 +424,19 @@
                      (inst test ,length ,length)
                      (inst jmp :z done)
                      (inst lea ,answer
-                           (make-ea :byte :base nil :index ,length
-                                    :scale (ash 1 (1+ (- word-shift
-                                                         n-fixnum-tag-bits)))))
+                           (ea nil ,length
+                               (ash 1 (1+ (- word-shift n-fixnum-tag-bits)))))
                      ,answer)))
            (compute-end ()
              `(let ((size (cond ((or (not (fixnump size))
-                                     (immediate32-p size))
+                                     (plausible-signed-imm32-operand-p size))
                                  size)
                                 (t
                                  (inst mov limit size)
                                  limit))))
                 (inst lea limit
-                      (make-ea :qword :base result
-                                      :index (if (fixnump size) nil size)
-                                      :disp (if (fixnump size) size 0))))))
+                      (ea (if (fixnump size) size 0) result
+                          (if (fixnump size) nil size))))))
 
   (define-vop (allocate-list-on-stack)
     (:args (length :scs (any-reg immediate))
@@ -442,7 +479,7 @@
             (no-init
              (and (sc-is element immediate) (eql (tn-value element) 0))))
         (instrument-alloc size node)
-        (pseudo-atomic
+        (pseudo-atomic ()
          (allocation result size node nil list-pointer-lowtag)
          (compute-end)
          (inst mov next result)
@@ -459,7 +496,7 @@
         (storew nil-value tail cons-cdr-slot list-pointer-lowtag))
       done)))
 
-#!-immobile-space
+#-immobile-space
 (define-vop (make-fdefn)
   (:policy :fast-safe)
   (:translate make-fdefn)
@@ -485,22 +522,18 @@
           (header (logior (ash (1- words) n-widetag-bits) closure-widetag)))
      (unless stack-allocate-p
        (instrument-alloc bytes node))
-     (maybe-pseudo-atomic stack-allocate-p
+     (pseudo-atomic (:elide-if stack-allocate-p)
        (allocation result bytes node stack-allocate-p fun-pointer-lowtag)
-       (storew* #!-immobile-space header ; write the widetag and size
-                #!+immobile-space        ; ... plus the layout pointer
+       (storew* #-immobile-space header ; write the widetag and size
+                #+immobile-space        ; ... plus the layout pointer
                 (progn (inst mov temp header)
-                       (inst or temp #!-sb-thread (static-symbol-value-ea 'function-layout)
-                                     #!+sb-thread
+                       (inst or temp #-sb-thread (static-symbol-value-ea 'function-layout)
+                                     #+sb-thread
                                      (thread-slot-ea thread-function-layout-slot))
                        temp)
                 result 0 fun-pointer-lowtag (not stack-allocate-p)))
-     ;; These two instructions are within the scope of PSEUDO-ATOMIC.
-     ;; This is due to scav_closure() assuming that it can always subtract
-     ;; FUN_RAW_ADDR_OFFSET from closure->fun to obtain a Lisp object,
-     ;; without any precheck for whether that word is currently 0.
-     (inst lea temp (rip-relative-ea :qword label
-                                     (ash simple-fun-code-offset word-shift)))
+     ;; Done with pseudo-atomic
+     (inst lea temp (rip-relative-ea label (ash simple-fun-code-offset word-shift)))
      (storew temp result closure-fun-slot fun-pointer-lowtag))))
 
 ;;; The compiler likes to be able to directly make value cells.
@@ -527,8 +560,8 @@
   (:vop-var vop)
   (:generator 1
     (let ((tramp (make-fixup 'funcallable-instance-tramp :assembly-routine)))
-      (if (sb!c::code-immobile-p vop)
-          (inst lea result (make-ea :qword :base rip-tn :disp tramp))
+      (if (sb-c::code-immobile-p vop)
+          (inst lea result (ea tramp rip-tn))
           (inst mov result tramp)))))
 
 (define-vop (fixed-alloc)
@@ -537,27 +570,37 @@
   (:results (result :scs (descriptor-reg)))
   (:node-var node)
   (:generator 50
-   (let ((bytes (pad-data-block words)))
+   (let* ((instancep (typep type 'layout)) ; is this any instance?
+          (layoutp #+immobile-space
+                   (eq type #.(find-layout 'layout))) ; " a new LAYOUT instance?
+          (bytes (pad-data-block words)))
     (progn name) ; possibly not used
-    (unless stack-allocate-p
+    (unless (or stack-allocate-p layoutp)
       (instrument-alloc bytes node))
-    (maybe-pseudo-atomic stack-allocate-p
-     (allocation result bytes node stack-allocate-p lowtag)
-     (when type
-       (let* ((widetag (if (typep type 'layout) instance-widetag type))
-              (header (logior (ash (1- words) n-widetag-bits) widetag)))
-         (if (or #!+compact-instance-header
-                 (and (eq name '%make-structure-instance) stack-allocate-p))
-             ;; Write a :DWORD, not a :QWORD, because the high half will be
-             ;; filled in when the layout is stored. Can't use STOREW* though,
-             ;; because it tries to store as few bytes as possible,
-             ;; where this instruction must write exactly 4 bytes.
-             (inst mov (make-ea :dword :base result :disp (- lowtag)) header)
-             (storew* header result 0 lowtag (not stack-allocate-p)))
-         (unless (eq type widetag) ; TYPE is actually a LAYOUT
-           (inst mov (make-ea :dword :base result :disp (+ 4 (- lowtag)))
-                 ;; XXX: should layout fixups use a name, not a layout object?
-                 (make-fixup type :layout)))))))))
+    (pseudo-atomic (:elide-if stack-allocate-p)
+     (cond (layoutp
+            (invoke-asm-routine 'call 'alloc-layout node)
+            (inst mov result r11-tn))
+           (t
+            ;; If storing a header word, defer ORing in the lowtag until after
+            ;; the header is written so that displacement can be 0.
+            (allocation result bytes node stack-allocate-p (if type 0 lowtag))
+            (when type
+              (let* ((widetag (if instancep instance-widetag type))
+                     (header (logior (ash (1- words) n-widetag-bits) widetag)))
+                (if (or #+compact-instance-header
+                        (and (eq name '%make-structure-instance) stack-allocate-p))
+                    ;; Write a :DWORD, not a :QWORD, because the high half will be
+                    ;; filled in when the layout is stored. Can't use STOREW* though,
+                    ;; because it tries to store as few bytes as possible,
+                    ;; where this instruction must write exactly 4 bytes.
+                    (inst mov :dword (ea 0 result) header)
+                    (storew* header result 0 0 (not stack-allocate-p)))
+                (inst or :byte result lowtag))))))
+    (when instancep ; store its layout
+      (inst mov :dword (ea (+ 4 (- lowtag)) result)
+            ;; XXX: should layout fixups use a name, not a layout object?
+            (make-fixup type :layout))))))
 
 ;;; Allocate a non-vector variable-length object.
 ;;; Exactly 4 allocators are rendered via this vop:
@@ -578,35 +621,29 @@
   (:node-var node)
   (:generator 50
    ;; With the exception of bignums, these objects have effectively
-   ;; 32-bit headers because the high half contains other data.
-   (multiple-value-bind (bytes header extra)
-      (if (= type bignum-widetag)
-          (values bytes header extra)
-          (values (reg-in-size bytes  :dword)
-                  (reg-in-size header :dword)
-                  (reg-in-size extra  :dword)))
-    (inst lea bytes
-          (make-ea :qword :disp (* (1+ words) n-word-bytes) :index extra
-                   :scale (ash 1 (- word-shift n-fixnum-tag-bits))))
-    (inst mov header bytes)
-    (inst shl header (- n-widetag-bits word-shift)) ; w+1 to length field
-    (inst lea header                    ; (w-1 << 8) | type
-          (make-ea :qword :base header
-                   :disp (+ (ash -2 n-widetag-bits) type)))
-    (inst and bytes (lognot lowtag-mask)))
-    (instrument-alloc bytes node)
-    (pseudo-atomic
-     (allocation result bytes node nil lowtag)
-     (storew header result 0 lowtag))))
+   ;; 32-bit headers because the high 4 byes contain a layout pointer.
+   (let ((operand-size (if (= type bignum-widetag) :qword :dword)))
+      (inst lea operand-size bytes
+            (ea (* (1+ words) n-word-bytes) nil
+                extra (ash 1 (- word-shift n-fixnum-tag-bits))))
+      (inst mov operand-size header bytes)
+      (inst shl operand-size header (- n-widetag-bits word-shift)) ; w+1 to length field
+      (inst lea operand-size header                    ; (w-1 << 8) | type
+            (ea (+ (ash -2 n-widetag-bits) type) header))
+      (inst and operand-size bytes (lognot lowtag-mask)))
+      (instrument-alloc bytes node)
+      (pseudo-atomic ()
+       (allocation result bytes node nil lowtag)
+       (storew header result 0 lowtag))))
 
-#!+immobile-space
 (macrolet ((c-call (name)
              `(let ((c-fun (make-fixup ,name :foreign)))
-                (inst call (cond ((sb!c::code-immobile-p node) c-fun)
+                (inst call (cond ((sb-c::code-immobile-p node) c-fun)
                                  (t (progn (inst mov temp-reg-tn c-fun)
                                            temp-reg-tn)))))))
+#+immobile-space
 (define-vop (alloc-immobile-fixedobj)
-  (:info lowtag size word0 word1)
+  (:info lowtag size header)
   (:temporary (:sc unsigned-reg :to :eval :offset rdi-offset) c-arg1)
   (:temporary (:sc unsigned-reg :to :eval :offset rsi-offset) c-arg2)
   (:temporary (:sc unsigned-reg :from :eval :to (:result 0) :offset rax-offset)
@@ -615,31 +652,28 @@
   (:node-var node)
   (:generator 50
    (inst mov c-arg1 size)
-   (inst mov c-arg2 word0)
+   (inst mov c-arg2 header)
    ;; RSP needn't be restored because the allocators all return immediately
    ;; which has that effect
    (inst and rsp-tn -16)
-   (pseudo-atomic
+   (pseudo-atomic ()
      (c-call "alloc_fixedobj")
-     (inst lea result (make-ea :qword :base c-result :disp lowtag))
-     ;; If code, the next word must be set within the P-A
-     ;; otherwise the GC would compute the wrong object size.
-     (when word1
-       (inst mov (make-ea :qword :base result :disp (- n-word-bytes lowtag)) word1)))))
-(define-vop (alloc-immobile-layout)
-  (:args (slots :scs (descriptor-reg) :target c-arg1))
-  (:temporary (:sc unsigned-reg :from (:argument 0) :to :eval :offset rdi-offset)
-              c-arg1)
+     (inst lea result (ea lowtag c-result)))))
+
+(define-vop (alloc-dynamic-space-code)
+  (:args (total-words :scs (signed-reg) :target c-arg1))
+  (:temporary (:sc unsigned-reg :from (:argument 0) :to :eval :offset rdi-offset) c-arg1)
   (:temporary (:sc unsigned-reg :from :eval :to (:result 0) :offset rax-offset)
               c-result)
   (:results (result :scs (descriptor-reg)))
   (:node-var node)
   (:generator 50
-   (move c-arg1 slots)
+   (inst mov c-arg1 total-words)
    ;; RSP needn't be restored because the allocators all return immediately
    ;; which has that effect
    (inst and rsp-tn -16)
-   (pseudo-atomic (c-call "alloc_layout"))
-   (move result c-result)))
-) ; end MACROLET
+   (pseudo-atomic () (c-call "alloc_code_object"))
+   ;; C-RESULT is a tagged ptr. MOV doesn't need to be pseudo-atomic.
+   (inst mov result c-result)))
 
+) ; end MACROLET
