@@ -9,12 +9,12 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 
 
 ;; For use in constant indexing; we can't use INDEX since the displacement
 ;; field of an EA can't contain 64 bit values.
-(def!type low-index () '(signed-byte 29))
+(sb-xc:deftype low-index () '(signed-byte 29))
 
 ;;;; allocator for the array header
 
@@ -129,23 +129,43 @@
   (:variant t)
   (:vop-var vop)
   (:save-p :compute-only)
-  (:generator 5
+  (:generator 6
     (let ((error (generate-error-code vop 'invalid-array-index-error
                                       array bound index))
           (bound (if (sc-is bound immediate)
-                     (fixnumize (tn-value bound))
+                     (let ((value (tn-value bound)))
+                       (cond ((and %test-fixnum
+                                   (power-of-two-limit-p (1- value)))
+                              (lognot (fixnumize (1- value))))
+                             ((sc-is index any-reg descriptor-reg)
+                              (fixnumize value))
+                             (t
+                              value)))
                      bound))
           (index (if (sc-is index immediate)
-                     (fixnumize (tn-value index))
+                     (let ((value (tn-value index)))
+                       (if (sc-is bound any-reg descriptor-reg)
+                           (fixnumize value)
+                           value))
                      index)))
-      (when (and %test-fixnum (not (integerp index)))
-        (%test-fixnum index nil error t))
-      (cond ((integerp bound)
-             (inst cmp index bound)
-             (inst jmp :nb error))
+      (cond ((typep bound '(integer * -1))
+             ;; Power of two bound, can be checked for fixnumness at
+             ;; the same time as it always occupies a consecutive bit
+             ;; range, everything else, including the tag, has to be
+             ;; zero.
+             (inst test index (if (eql bound -1)
+                                  index ;; zero?
+                                  bound))
+             (inst jmp :ne error))
             (t
-             (inst cmp bound index)
-             (inst jmp :be error))))))
+             (when (and %test-fixnum (not (integerp index)))
+               (%test-fixnum index nil error t))
+             (cond ((integerp bound)
+                    (inst cmp index bound)
+                    (inst jmp :nb error))
+                   (t
+                    (inst cmp bound index)
+                    (inst jmp :be error))))))))
 (define-vop (check-bound/fast check-bound)
   (:policy :fast)
   (:variant nil)
@@ -158,6 +178,15 @@
   (:arg-types * * tagged-num)
   (:variant nil)
   (:variant-cost 4))
+
+(define-vop (check-bound/untagged check-bound)
+  (:args (array)
+         (bound :scs (unsigned-reg signed-reg))
+         (index :scs (unsigned-reg signed-reg)))
+  (:arg-types * (:or unsigned-num signed-num)
+                (:or unsigned-num signed-num))
+  (:variant nil)
+  (:variant-cost 5))
 
 ;;;; accessors/setters
 
@@ -786,7 +815,7 @@
     positive-fixnum unsigned-reg signed-reg)
   (define-data-vector-frobs simple-array-signed-byte-32 movsx :dword
     tagged-num signed-reg)
-  #!+sb-unicode
+  #+sb-unicode
   (define-data-vector-frobs simple-character-string movzx :dword
     character character-reg))
 
