@@ -9,7 +9,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 
 ;;;; instruction-like macros
 
@@ -19,31 +19,54 @@
 ;;; bloat is not justifiable.
 (defun move (dst src)
   "Move SRC into DST unless they are location=."
+  ;; The first case is for backward-compatibility. It's not necessary
+  ;; for any of our code, but it is for code that performs
+  ;;   (MOVE (REG-IN-SIZE blah :dword) (REG-IN-SIZE from :dword))
+  ;; Most of this garbage will go away because eventually I'd like to preserve
+  ;; all seemingly redundant moves, and then eliminate them before emission.
+  ;; This way we can track movement of TNs into the same physical reg in a
+  ;; different SC which will give useful information to a peephole optimizer.
+  (when (and (sb-x86-64-asm::register-p dst)
+             (sb-x86-64-asm::register-p src))
+    (let ((dst (sb-x86-64-asm::reg-id dst))
+          (src (sb-x86-64-asm::reg-id src)))
+      (aver (sb-x86-64-asm::is-gpr-id-p dst))
+      (aver (sb-x86-64-asm::is-gpr-id-p src))
+      (unless (= (sb-x86-64-asm::reg-id-num dst)
+                 (sb-x86-64-asm::reg-id-num src))
+        (inst mov dst src)))
+    (return-from move))
   (unless (location= dst src)
     (sc-case dst
       ((single-reg complex-single-reg)
-       (aver (xmm-register-p src))
+       (aver (xmm-tn-p src))
        (inst movaps dst src))
       ((double-reg complex-double-reg)
-       (aver (xmm-register-p src))
+       (aver (xmm-tn-p src))
        (inst movapd dst src))
-      #!+sb-simd-pack
+      #+sb-simd-pack
       ((int-sse-reg sse-reg)
-       (aver (xmm-register-p src))
+       (aver (xmm-tn-p src))
        (inst movdqa dst src))
-      #!+sb-simd-pack
+      #+sb-simd-pack
       ((single-sse-reg double-sse-reg)
-       (aver (xmm-register-p src))
+       (aver (xmm-tn-p src))
        (inst movaps dst src))
+      #+sb-simd-pack-256
+      ((int-avx2-reg avx2-reg)
+       (aver (xmm-tn-p src))
+       (inst vmovdqa dst src))
+      #+sb-simd-pack-256
+      ((single-avx2-reg double-avx2-reg)
+       (aver (xmm-tn-p src))
+       (inst vmovaps dst src))
       (t
        (inst mov dst src)))))
 
 (defmacro make-ea-for-object-slot (ptr slot lowtag)
-  `(make-ea :qword :base ,ptr :disp (- (* ,slot n-word-bytes) ,lowtag)))
-(defmacro make-ea-for-object-slot-half (ptr slot lowtag)
-  `(make-ea :dword :base ,ptr :disp (- (* ,slot n-word-bytes) ,lowtag)))
+  `(ea (- (* ,slot n-word-bytes) ,lowtag) ,ptr))
 (defmacro tls-index-of (sym)
-  `(make-ea :dword :base ,sym :disp (+ 4 (- other-pointer-lowtag))))
+  `(ea (+ 4 (- other-pointer-lowtag)) ,sym))
 
 (defmacro loadw (value ptr &optional (slot 0) (lowtag 0))
   `(inst mov ,value (make-ea-for-object-slot ,ptr ,slot ,lowtag)))
@@ -52,13 +75,14 @@
   (let* ((size (if (tn-p value)
                    (sc-operand-size (tn-sc value))
                    :qword))
-         (ea (make-ea size :base ptr :disp (- (* slot n-word-bytes) lowtag))))
+         (ea (ea (- (* slot n-word-bytes) lowtag) ptr)))
+    (aver (eq size :qword))
     (cond ((and (integerp value)
                 (not (typep value '(signed-byte 32))))
            (inst mov temp-reg-tn value)
            (inst mov ea temp-reg-tn))
           (t
-           (inst mov ea value)))))
+           (inst mov :qword ea value)))))
 
 (defmacro pushw (ptr &optional (slot 0) (lowtag 0))
   `(inst push (make-ea-for-object-slot ,ptr ,slot ,lowtag)))
@@ -72,38 +96,28 @@
 (defmacro load-symbol (reg symbol)
   `(inst mov ,reg (+ nil-value (static-symbol-offset ,symbol))))
 
-(defmacro make-ea-for-symbol-value (symbol)
-  `(make-ea :qword
-    :disp (+ nil-value
-           (static-symbol-offset ',symbol)
-           (ash symbol-value-slot word-shift)
-           (- other-pointer-lowtag))))
-
-(defmacro load-symbol-value (reg symbol)
-  `(inst mov ,reg (make-ea-for-symbol-value ,symbol)))
-
-(defmacro store-symbol-value (reg symbol)
-  `(inst mov (make-ea-for-symbol-value ,symbol) ,reg))
-
 ;; Return the effective address of the value slot of static SYMBOL.
 (defun static-symbol-value-ea (symbol)
-   (make-ea :qword
-            :disp (+ nil-value
-                     (static-symbol-offset symbol)
-                     (ash symbol-value-slot word-shift)
-                     (- other-pointer-lowtag))))
+   (ea (+ nil-value
+          (static-symbol-offset symbol)
+          (ash symbol-value-slot word-shift)
+          (- other-pointer-lowtag))))
 
-(defun thread-tls-ea (index &optional (size :qword))
-  (if (tn-p index)
-      ;; Due to an encoding peculiarity, flipping the base and index is better.
-      ;; Base of r13 is reg=5 in ModRegRM, so if mod were 0, it would imply
-      ;; RIP-relative addressing. (And attempting to encode an index is illegal)
-      ;; So the 'mod' bits must be nonzero, which mandates encoding of an
-      ;; explicit displacement of 0.  Using INDEX as base avoids the extra byte.
-      (make-ea size :base index :index thread-base-tn)
-      (make-ea size :base thread-base-tn :disp index)))
+(defun thread-tls-ea (index)
+  ;; Whether index is an an integer or a register, the EA constructor
+  ;; call is the same.
+  ;; Due to an encoding peculiarity, using thread-base-tn as the index register
+  ;; is better when index is non-constant.
+  ;; Base of r13 is reg=5 in ModRegRM, so if mod were 0, it would imply
+  ;; RIP-relative addressing. (And attempting to encode an index is illegal)
+  ;; So the 'mod' bits must be nonzero, which mandates encoding of an
+  ;; explicit displacement of 0.  Using INDEX as base avoids the extra byte.
+  (ea index thread-base-tn))
 
-#!+sb-thread
+(defun thread-slot-ea (slot-index)
+  (ea (ash slot-index word-shift) thread-base-tn))
+
+#+sb-thread
 (progn
   ;; Return an EA for the TLS of SYMBOL, or die.
   (defun symbol-known-tls-cell (symbol)
@@ -119,39 +133,23 @@
   (defmacro store-tl-symbol-value (reg symbol)
     `(inst mov (symbol-known-tls-cell ',symbol) ,reg)))
 
-#!-sb-thread
+#-sb-thread
 (progn
   (defmacro load-tl-symbol-value (reg symbol)
-    `(load-symbol-value ,reg ,symbol))
+    `(inst mov ,reg (static-symbol-value-ea ',symbol)))
   (defmacro store-tl-symbol-value (reg symbol)
-    `(store-symbol-value ,reg ,symbol)))
+    `(inst mov (static-symbol-value-ea ',symbol) ,reg)))
 
 (defmacro load-binding-stack-pointer (reg)
-  #!+sb-thread `(inst mov ,reg (symbol-known-tls-cell '*binding-stack-pointer*))
-  #!-sb-thread `(load-symbol-value ,reg *binding-stack-pointer*))
+  `(load-tl-symbol-value ,reg *binding-stack-pointer*))
 
 (defmacro store-binding-stack-pointer (reg)
-  #!+sb-thread `(inst mov (symbol-known-tls-cell '*binding-stack-pointer*) ,reg)
-  #!-sb-thread `(store-symbol-value ,reg *binding-stack-pointer*))
-
-(defmacro load-type (target source &optional (offset 0))
-  "Loads the type bits of a pointer into target independent of
-   byte-ordering issues."
-  `(inst movzx ,target (make-ea :byte :base ,source :disp ,offset)))
+  `(store-tl-symbol-value ,reg *binding-stack-pointer*))
 
 ;;;; error code
 (defun emit-error-break (vop kind code values)
   (assemble ()
-    #!-ud2-breakpoints
-    (inst int #!+int4-breakpoints 4 #!-int4-breakpoints 3)
-    ;; On Darwin, we need to use #x0b0f instead of int3 in order
-    ;; to generate a SIGILL instead of a SIGTRAP as darwin/x86
-    ;; doesn't seem to be reliably firing SIGTRAP
-    ;; handlers. Hopefully this will be fixed by Apple at a
-    ;; later date. Do the same on x86-64 as we do on x86 until this gets
-    ;; sorted out.
-    #!+ud2-breakpoints
-    (inst word #x0b0f)
+    (inst break)
     ;; The return PC points here; note the location for the debugger.
     (when vop
       (note-this-location vop :internal-error))
@@ -187,53 +185,42 @@
 ;;; place and there's no logical single place to attach documentation.
 ;;; grep (mostly in src/runtime) is your friend
 
-(defmacro maybe-pseudo-atomic (not-really-p &body body)
-  `(if ,not-really-p
-       (progn ,@body)
-       (pseudo-atomic ,@body)))
-
 ;;; Unsafely clear pa flags so that the image can properly lose in a
 ;;; pa section.
-#!+sb-thread
+#+sb-thread
 (defmacro %clear-pseudo-atomic ()
-  '(inst mov (thread-tls-ea (* n-word-bytes thread-pseudo-atomic-bits-slot)) 0))
+  '(inst mov :qword (thread-slot-ea thread-pseudo-atomic-bits-slot) 0))
 
-#!+sb-safepoint
+#+sb-safepoint
 (defun emit-safepoint ()
-  (inst test al-tn (make-ea :byte :disp
-                            (- nil-value n-word-bytes other-pointer-lowtag
-                               gc-safepoint-trap-offset))))
+  (inst test :byte rax-tn (ea (- nil-value n-word-bytes other-pointer-lowtag
+                                 gc-safepoint-trap-offset))))
 
-(defmacro pseudo-atomic (&rest forms)
-  #!+sb-safepoint-strictly
-  `(progn ,@forms (emit-safepoint))
-  #!-sb-safepoint-strictly
+(defmacro pseudo-atomic ((&key elide-if) &rest forms)
+  #+sb-safepoint-strictly
+  `(progn ,@forms (unless ,elide-if (emit-safepoint)))
+  #-sb-safepoint-strictly
   (with-unique-names (label pa-bits-ea)
     `(let ((,label (gen-label))
            (,pa-bits-ea
-            #!+sb-thread
-            (thread-tls-ea (* n-word-bytes thread-pseudo-atomic-bits-slot))
-            #!-sb-thread
-            (make-ea :qword
-                     :disp (+ nil-value
-                            (static-symbol-offset
-                             '*pseudo-atomic-bits*)
-                            (ash symbol-value-slot word-shift)
-                            (- other-pointer-lowtag)))))
-       (inst mov ,pa-bits-ea rbp-tn)
+            #+sb-thread (thread-slot-ea thread-pseudo-atomic-bits-slot)
+            #-sb-thread (static-symbol-value-ea '*pseudo-atomic-bits*)))
+       (unless ,elide-if
+         (inst mov ,pa-bits-ea rbp-tn))
        ,@forms
-       (inst xor ,pa-bits-ea rbp-tn)
-       (inst jmp :z ,label)
-       ;; if PAI was set, interrupts were disabled at the same time
-       ;; using the process signal mask.
-       (inst break pending-interrupt-trap)
-       (emit-label ,label)
-       #!+sb-safepoint
-       ;; In this case, when allocation thinks a GC should be done, it
-       ;; does not mark PA as interrupted, but schedules a safepoint
-       ;; trap instead.  Let's take the opportunity to trigger that
-       ;; safepoint right now.
-       (emit-safepoint))))
+       (unless ,elide-if
+         (inst xor ,pa-bits-ea rbp-tn)
+         (inst jmp :z ,label)
+         ;; if PAI was set, interrupts were disabled at the same time
+         ;; using the process signal mask.
+         (inst break pending-interrupt-trap)
+         (emit-label ,label)
+         #+sb-safepoint
+         ;; In this case, when allocation thinks a GC should be done, it
+         ;; does not mark PA as interrupted, but schedules a safepoint
+         ;; trap instead.  Let's take the opportunity to trigger that
+         ;; safepoint right now.
+         (emit-safepoint)))))
 
 ;;;; indexed references
 
@@ -257,11 +244,12 @@
        (:generator 5
          (move rax old-value)
          (inst cmpxchg
-               (make-ea :qword :base object
-                        :index  (unless (sc-is index immediate) index)
-                        :scale (ash 1 (- word-shift n-fixnum-tag-bits))
-                        :disp (- (* (+ (if (sc-is index immediate) (tn-value index) 0)
-                                       ,offset) n-word-bytes) ,lowtag))
+               (ea (- (* (+ (if (sc-is index immediate) (tn-value index) 0) ,offset)
+                         n-word-bytes)
+                      ,lowtag)
+                   object
+                   (unless (sc-is index immediate) index)
+                   (ash 1 (- word-shift n-fixnum-tag-bits)))
                new-value :lock)
          (move value rax)))))
 
@@ -277,10 +265,8 @@
        (:results (value :scs ,scs))
        (:result-types ,el-type)
        (:generator 3                    ; pw was 5
-         (inst mov value (make-ea :qword :base object :index index
-                                  :scale (ash 1 (- word-shift n-fixnum-tag-bits))
-                                  :disp (- (* ,offset n-word-bytes)
-                                           ,lowtag)))))
+         (inst mov value (ea (- (* ,offset n-word-bytes) ,lowtag)
+                             object index (ash 1 (- word-shift n-fixnum-tag-bits))))))
      (define-vop (,(symbolicate name "-C"))
        ,@(when translate
            `((:translate ,translate)))
@@ -293,9 +279,8 @@
        (:results (value :scs ,scs))
        (:result-types ,el-type)
        (:generator 2                    ; pw was 5
-         (inst mov value (make-ea :qword :base object
-                                  :disp (- (* (+ ,offset index) n-word-bytes)
-                                           ,lowtag)))))))
+         (inst mov value (ea (- (* (+ ,offset index) n-word-bytes) ,lowtag)
+                             object))))))
 
 (defmacro define-full-reffer+offset (name type offset lowtag scs el-type &optional translate)
   `(progn
@@ -312,10 +297,8 @@
        (:results (value :scs ,scs))
        (:result-types ,el-type)
        (:generator 3                    ; pw was 5
-         (inst mov value (make-ea :qword :base object :index index
-                                  :scale (ash 1 (- word-shift n-fixnum-tag-bits))
-                                  :disp (- (* (+ ,offset offset) n-word-bytes)
-                                           ,lowtag)))))
+         (inst mov value (ea (- (* (+ ,offset offset) n-word-bytes) ,lowtag)
+                             object index (ash 1 (- word-shift n-fixnum-tag-bits))))))
      (define-vop (,(symbolicate name "-C"))
        ,@(when translate
            `((:translate ,translate)))
@@ -330,11 +313,16 @@
        (:results (value :scs ,scs))
        (:result-types ,el-type)
        (:generator 2                    ; pw was 5
-         (inst mov value (make-ea :qword :base object
-                                  :disp (- (* (+ ,offset index offset) n-word-bytes)
-                                           ,lowtag)))))))
+         (inst mov value (ea (- (* (+ ,offset index offset) n-word-bytes) ,lowtag)
+                             object))))))
 
 (defmacro define-full-setter (name type offset lowtag scs el-type &optional translate)
+  (let ((want-both-variants
+         (cond ((symbolp name) t)
+               (t
+                (aver (typep name '(cons symbol (cons (eql :no-constant-variant) null))))
+                (setq name (car name))
+                nil))))
   `(progn
      (define-vop (,name)
        ,@(when translate
@@ -346,31 +334,44 @@
        (:arg-types ,type tagged-num ,el-type)
        (:results (result :scs ,scs))
        (:result-types ,el-type)
+       (:vop-var vop)
        (:generator 4                    ; was 5
-         (gen-cell-set
-                   (make-ea :qword :base object :index index
-                            :scale (ash 1 (- word-shift n-fixnum-tag-bits))
-                            :disp (- (* ,offset n-word-bytes) ,lowtag))
-                   value result)))
-     (define-vop (,(symbolicate name "-C"))
-       ,@(when translate
-           `((:translate ,translate)))
-       (:policy :fast-safe)
-       (:args (object :scs (descriptor-reg))
-              (value :scs ,scs :target result))
-       (:info index)
-       (:arg-types ,type
-                   (:constant (load/store-index ,n-word-bytes ,(eval lowtag)
-                                                ,(eval offset)))
-                   ,el-type)
-       (:results (result :scs ,scs))
-       (:result-types ,el-type)
-       (:generator 3                    ; was 5
-         (gen-cell-set
-                   (make-ea :qword :base object
-                            :disp (- (* (+ ,offset index) n-word-bytes)
-                                     ,lowtag))
-                   value result)))))
+         ,@(if (eq name 'code-header-set)
+               '((inst push value)
+                 ;; the asm routine wants a natural machine integer as the index,
+                 ;; but this macro declares the index arg as 'any-reg', so it has a tag bit,
+                 ;; so we'll push the arg and then shift right as the next instruction.
+                 (inst push index)
+                 (inst shr :qword (ea rsp-tn) n-fixnum-tag-bits)
+                 (inst push object)
+                 (invoke-asm-routine 'call 'code-header-set vop)
+                 (move result value))
+               `((gen-cell-set
+                   (ea (- (* ,offset n-word-bytes) ,lowtag)
+                       object index (ash 1 (- word-shift n-fixnum-tag-bits)))
+                   value result vop
+                   ,(eq name 'set-funcallable-instance-info))))))
+     ,@(when want-both-variants
+         `((define-vop (,(symbolicate name "-C"))
+            ,@(when translate
+                `((:translate ,translate)))
+            (:policy :fast-safe)
+            (:args (object :scs (descriptor-reg))
+                   (value :scs ,scs :target result))
+            (:info index)
+            (:arg-types ,type
+                        (:constant (load/store-index ,n-word-bytes ,(eval lowtag)
+                                                     ,(eval offset)))
+                        ,el-type)
+            (:results (result :scs ,scs))
+            (:result-types ,el-type)
+            (:vop-var vop)
+            (:generator 3                    ; was 5
+              (gen-cell-set
+                   (ea (- (* (+ ,offset index) n-word-bytes) ,lowtag)
+                       object)
+                   value result vop
+                   ,(eq name 'set-funcallable-instance-info)))))))))
 
 (defmacro define-full-setter+offset (name type offset lowtag scs el-type &optional translate)
   `(progn
@@ -391,9 +392,8 @@
        (:result-types ,el-type)
        (:generator 4                    ; was 5
          (gen-cell-set
-                   (make-ea :qword :base object :index index
-                            :scale (ash 1 (- word-shift n-fixnum-tag-bits))
-                            :disp (- (* (+ ,offset offset) n-word-bytes) ,lowtag))
+                   (ea (- (* (+ ,offset offset) n-word-bytes) ,lowtag)
+                       object index (ash 1 (- word-shift n-fixnum-tag-bits)))
                    value result)))
      (define-vop (,(symbolicate name "-C"))
        ,@(when translate
@@ -413,8 +413,7 @@
        (:result-types ,el-type)
        (:generator 3                    ; was 5
          (gen-cell-set
-                   (make-ea :qword :base object
-                            :disp (- (* (+ ,offset index offset) n-word-bytes)
-                                     ,lowtag))
+                   (ea (- (* (+ ,offset index offset) n-word-bytes) ,lowtag)
+                       object)
                    value result)))))
 
