@@ -9,7 +9,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 
 ;;;; register specs
 
@@ -38,11 +38,10 @@
                 ;; which is needed because of forms such as #.*qword-regs*
                 (eval-when (:compile-toplevel :load-toplevel :execute)
                   ,@(when want-offsets
-                      (let ((i 0))
+                      (let ((i -1))
                         (map 'list
                              (lambda (x)
-                               `(defconstant ,(symbolicate x "-OFFSET")
-                                  ,(prog1 (* i 2) (incf i))))
+                               `(defconstant ,(symbolicate x "-OFFSET") ,(incf i)))
                              array))))
                 (defglobal ,offsets-list
                     (remove-if (lambda (x)
@@ -50,7 +49,7 @@
                                              ,r13-offset ; thread base
                                              ,rsp-offset
                                              ,rbp-offset)))
-                               (loop for i below 16 collect (* i 2)))))))
+                               (loop for i below 16 collect i))))))
 
   (define-gprs t *qword-regs* +qword-register-names+
     #("RAX" "RCX" "RDX" "RBX" "RSP" "RBP" "RSI" "RDI"
@@ -61,20 +60,13 @@
   (define-gprs nil *word-regs* +word-register-names+
     #("AX"  "CX"  "DX"   "BX"   "SP"   "BP"   "SI"   "DI"
       "R8W" "R9W" "R10W" "R11W" "R12W" "R13W" "R14W" "R15W"))
-  ;; byte registers
-  ;;
-  ;; Note: the encoding here is different than that used by the chip.
-  ;; We use this encoding so that the compiler thinks that AX (and
-  ;; EAX) overlap AL and AH instead of AL and CL.
-  ;;
-  ;; High-byte are registers disabled on AMD64, since they can't be
-  ;; encoded for an op that has a REX-prefix and we don't want to
-  ;; add special cases into the code generation. The overlap doesn't
-  ;; therefore exist anymore, but the numbering hasn't been changed
-  ;; to reflect this.
+  ;; High-byte ("h") registers are not generally used on AMD64,
+  ;; since they can't be encoded in an instruction that has a REX-prefix,
+  ;; but we can sometimes use them.
   (define-gprs nil *byte-regs* +byte-register-names+
     #("AL"  "CL"  "DL"   "BL"   "SPL"  "BPL"  "SIL"  "DIL"
-      "R8B" "R9B" "R10B" "R11B" "R12B" "R13B" "R14B" "R15B"))
+      "R8B" "R9B" "R10B" "R11B" "R12B" "R13B" "R14B" "R15B"
+      "AH" "CH" "DH" "BH"))
 
   ;; floating point registers
   (defreg float0 0 :float)
@@ -104,21 +96,15 @@
   (eval-when (:compile-toplevel :load-toplevel :execute)
     (defparameter *register-arg-names* '(rdx rdi rsi)))
   (defregset    *register-arg-offsets* rdx rdi rsi)
-  #!-win32
+  #-win32
   (defregset    *c-call-register-arg-offsets* rdi rsi rdx rcx r8 r9)
-  #!+win32
+  #+win32
   (defregset    *c-call-register-arg-offsets* rcx rdx r8 r9))
 
 ;;;; SB definitions
 
-;;; There are 16 registers really, but we consider them 32 in order to
-;;; describe the overlap of byte registers. The only thing we need to
-;;; represent is what registers overlap. Therefore, we consider bytes
-;;; to take one unit, and [dq]?words to take two. We don't need to
-;;; tell the difference between [dq]?words, because you can't put two
-;;; words in a dword register.
 (!define-storage-bases
-(define-storage-base registers :finite :size 32)
+(define-storage-base registers :finite :size 16)
 
 (define-storage-base float-registers :finite :size 16)
 
@@ -146,10 +132,12 @@
   (fp-complex-single-immediate immediate-constant)
   (fp-complex-double-immediate immediate-constant)
 
-  #!+sb-simd-pack (int-sse-immediate immediate-constant)
-  #!+sb-simd-pack (double-sse-immediate immediate-constant)
-  #!+sb-simd-pack (single-sse-immediate immediate-constant)
-
+  #+sb-simd-pack (int-sse-immediate immediate-constant)
+  #+sb-simd-pack (double-sse-immediate immediate-constant)
+  #+sb-simd-pack (single-sse-immediate immediate-constant)
+  #+sb-simd-pack-256 (int-avx2-immediate immediate-constant)
+  #+sb-simd-pack-256 (double-avx2-immediate immediate-constant)
+  #+sb-simd-pack-256 (single-avx2-immediate immediate-constant)
   (immediate immediate-constant)
 
   ;;
@@ -160,7 +148,6 @@
   (control-stack stack)                 ; may be pointers, scanned by GC
 
   ;; the non-descriptor stacks
-  ;; XXX alpha backend has :element-size 2 :alignment 2 in these entries
   (signed-stack stack)                  ; (signed-byte 64)
   (unsigned-stack stack)                ; (unsigned-byte 64)
   (character-stack stack)               ; non-descriptor characters.
@@ -169,12 +156,18 @@
   (double-stack stack)
   (complex-single-stack stack)  ; complex-single-floats
   (complex-double-stack stack :element-size 2)  ; complex-double-floats
-  #!+sb-simd-pack
+  #+sb-simd-pack
   (int-sse-stack stack :element-size 2)
-  #!+sb-simd-pack
+  #+sb-simd-pack
   (double-sse-stack stack :element-size 2)
-  #!+sb-simd-pack
+  #+sb-simd-pack
   (single-sse-stack stack :element-size 2)
+  #+sb-simd-pack-256
+  (int-avx2-stack stack :element-size 4)
+  #+sb-simd-pack-256
+  (double-avx2-stack stack :element-size 4)
+  #+sb-simd-pack-256
+  (single-avx2-stack stack :element-size 4)
 
   ;;
   ;; magic SCs
@@ -195,7 +188,6 @@
   ;; bad will happen if they are. (fixnums, characters, header values, etc).
   (any-reg registers
            :locations #.*qword-regs*
-           :element-size 2 ; I think this is for the al/ah overlap thing
            :constant-scs (immediate)
            :save-p t
            :alternate-scs (control-stack))
@@ -203,20 +195,13 @@
   ;; pointer descriptor objects -- must be seen by GC
   (descriptor-reg registers
                   :locations #.*qword-regs*
-                  :element-size 2
-;                 :reserve-locations (#.eax-offset)
                   :constant-scs (constant immediate)
                   :save-p t
                   :alternate-scs (control-stack))
 
   ;; non-descriptor characters
   (character-reg registers
-                 :locations #!-sb-unicode #.*byte-regs*
-                            #!+sb-unicode #.*qword-regs*
-                 #!+sb-unicode #!+sb-unicode
-                 :element-size 2
-;                 #!-sb-unicode #!-sb-unicode
-;                 :reserve-locations (#.eax-offset)
+                 :locations #.*qword-regs*
                  :constant-scs (immediate)
                  :save-p t
                  :alternate-scs (character-stack))
@@ -224,8 +209,6 @@
   ;; non-descriptor SAPs (arbitrary pointers into address space)
   (sap-reg registers
            :locations #.*qword-regs*
-           :element-size 2
-;          :reserve-locations (#.eax-offset)
            :constant-scs (immediate)
            :save-p t
            :alternate-scs (sap-stack))
@@ -233,30 +216,15 @@
   ;; non-descriptor (signed or unsigned) numbers
   (signed-reg registers
               :locations #.*qword-regs*
-              :element-size 2
               :constant-scs (immediate)
               :save-p t
               :alternate-scs (signed-stack))
   (unsigned-reg registers
                 :locations #.*qword-regs*
-                :element-size 2
                 :constant-scs (immediate)
                 :save-p t
                 :alternate-scs (unsigned-stack))
 
-  ;; miscellaneous objects that must not be seen by GC. Used only as
-  ;; temporaries.
-  (word-reg registers
-            :locations #.*word-regs*
-            :element-size 2)
-  (dword-reg registers
-            :locations #.*dword-regs*
-            :element-size 2)
-  (byte-reg registers
-            :locations #.*byte-regs*)
-
-  ;; that can go in the floating point registers
-
   ;; non-descriptor SINGLE-FLOATs
   (single-reg float-registers
               :locations #.*float-regs*
@@ -284,40 +252,57 @@
                       :alternate-scs (complex-double-stack))
 
   ;; temporary only
-  #!+sb-simd-pack
+  #+sb-simd-pack
   (sse-reg float-registers
            :locations #.*float-regs*)
   ;; regular values
-  #!+sb-simd-pack
+  #+sb-simd-pack
   (int-sse-reg float-registers
                :locations #.*float-regs*
                :constant-scs (int-sse-immediate)
                :save-p t
                :alternate-scs (int-sse-stack))
-  #!+sb-simd-pack
+  #+sb-simd-pack
   (double-sse-reg float-registers
                   :locations #.*float-regs*
                   :constant-scs (double-sse-immediate)
                   :save-p t
                   :alternate-scs (double-sse-stack))
-  #!+sb-simd-pack
+  #+sb-simd-pack
   (single-sse-reg float-registers
                   :locations #.*float-regs*
                   :constant-scs (single-sse-immediate)
                   :save-p t
                   :alternate-scs (single-sse-stack))
+  #+avx2
+  (avx2-reg float-registers
+   :locations #.*float-regs*)
+  #+sb-simd-pack-256
+  (int-avx2-reg float-registers
+               :locations #.*float-regs*
+               :constant-scs (int-avx2-immediate)
+               :save-p t
+               :alternate-scs (int-avx2-stack))
+  #+sb-simd-pack-256
+  (double-avx2-reg float-registers
+                  :locations #.*float-regs*
+                  :constant-scs (double-avx2-immediate)
+                  :save-p t
+                  :alternate-scs (double-avx2-stack))
+  #+sb-simd-pack-256
+  (single-avx2-reg float-registers
+                  :locations #.*float-regs*
+                  :constant-scs (single-avx2-immediate)
+                  :save-p t
+                  :alternate-scs (single-avx2-stack))
 
   (catch-block stack :element-size catch-block-size)
   (unwind-block stack :element-size unwind-block-size)))
 
-(defparameter *byte-sc-names*
-  '(#!-sb-unicode character-reg byte-reg #!-sb-unicode character-stack))
-(defparameter *word-sc-names* '(word-reg))
-(defparameter *dword-sc-names* '(dword-reg))
 (defparameter *qword-sc-names*
   '(any-reg descriptor-reg sap-reg signed-reg unsigned-reg control-stack
     signed-stack unsigned-stack sap-stack single-stack
-    #!+sb-unicode character-reg #!+sb-unicode character-stack constant))
+    character-reg character-stack constant))
 ;;; added by jrd. I guess the right thing to do is to treat floats
 ;;; as a separate size...
 ;;;
@@ -326,36 +311,38 @@
 (defparameter *double-sc-names* '(double-reg double-stack))
 (defparameter *complex-sc-names* '(complex-single-reg complex-single-stack
                                    complex-double-reg complex-double-stack))
-#!+sb-simd-pack
-;;; FIXME: there is no SSE-STACK storage class
+#+sb-simd-pack
 (defparameter *oword-sc-names* '(sse-reg int-sse-reg single-sse-reg double-sse-reg
-                                 sse-stack int-sse-stack single-sse-stack double-sse-stack))
+                                 int-sse-stack single-sse-stack double-sse-stack))
+#+sb-simd-pack-256
+(defparameter *hword-sc-names* '(avx2-reg int-avx2-reg single-avx2-reg double-avx2-reg
+                                   int-avx2-stack single-avx2-stack double-avx2-stack))
 ) ; EVAL-WHEN
 (!define-storage-classes
   . #.(mapcar (lambda (class-spec)
                 (let ((size
-                       (case (car class-spec)
-                         (#.*oword-sc-names*   :oword)
-                         (#.*qword-sc-names*   :qword)
-                         (#.*dword-sc-names*   :dword)
-                         (#.*word-sc-names*    :word)
-                         (#.*byte-sc-names*    :byte)
-                         (#.*float-sc-names*   :float)
-                         (#.*double-sc-names*  :double)
-                         (#.*complex-sc-names* :complex))))
+                        (case (car class-spec)
+                          #+sb-simd-pack
+                          (#.*oword-sc-names*   :oword)
+                          #+sb-simd-pack-256
+                          (#.*hword-sc-names*   :hword)
+                          (#.*qword-sc-names*   :qword)
+                          (#.*float-sc-names*   :float)
+                          (#.*double-sc-names*  :double)
+                          (#.*complex-sc-names* :complex))))
                   (append class-spec (if size (list :operand-size size)))))
               *storage-class-defs*))
 
 ;;;; miscellaneous TNs for the various registers
 
-(macrolet ((def-gpr-tns (sc-name name-array &aux (i 0))
+(macrolet ((def-gpr-tns (sc-name name-array &aux (i -1))
              `(progn
                 ,@(map 'list
                        (lambda (reg-name)
                          `(define-load-time-global ,(symbolicate reg-name "-TN")
                               (make-random-tn :kind :normal
                                               :sc (sc-or-lose ',sc-name)
-                                              :offset ,(prog1 (* i 2) (incf i)))))
+                                              :offset ,(incf i))))
                        (symbol-value name-array))))
            (def-fpr-tns (sc-name &rest reg-names)
              (collect ((forms))
@@ -367,9 +354,6 @@
                                                :sc (sc-or-lose ',sc-name)
                                                :offset ,offset-name))))))))
   (def-gpr-tns unsigned-reg +qword-register-names+)
-  (def-gpr-tns dword-reg +dword-register-names+)
-  (def-gpr-tns word-reg +word-register-names+)
-  (def-gpr-tns byte-reg +byte-register-names+)
   ;; RIP is not an addressable register, but this global var acts as
   ;; a moniker for it in an effective address so that the EA structure
   ;; does not need to accept a symbol (such as :RIP) for the base reg.
@@ -380,22 +364,14 @@
       float0 float1 float2 float3 float4 float5 float6 float7
       float8 float9 float10 float11 float12 float13 float14 float15))
 
-(defun reg-in-size (tn size)
-  (make-random-tn :kind :normal
-                  :sc (sc-or-lose
-                       (ecase size
-                         (:byte 'byte-reg)
-                         (:word 'word-reg)
-                         (:dword 'dword-reg)
-                         (:qword 'unsigned-reg)
-                         ;; PC inside immobile code can fit into 32 bits
-                         (:immobile-code-pc
-                          (cond #!+immobile-code
-                                (sb!c::*code-is-immobile*
-                                 'dword-reg)
-                                (t
-                                 'unsigned-reg)))))
-                  :offset (tn-offset tn)))
+;;; Return true if THING is a general-purpose register TN.
+(defun gpr-tn-p (thing)
+  (and (tn-p thing)
+       (eq (sb-name (sc-sb (tn-sc thing))) 'registers)))
+;;; Return true if THING is an XMM register TN.
+(defun xmm-tn-p (thing)
+  (and (tn-p thing)
+       (eq (sb-name (sc-sb (tn-sc thing))) 'float-registers)))
 
 ;; A register that's never used by the code generator, and can therefore
 ;; be used as an assembly temporary in cases where a VOP :TEMPORARY can't
@@ -420,16 +396,18 @@
 ;;; If value can be represented as an immediate constant, then return
 ;;; the appropriate SC number, otherwise return NIL.
 (defun immediate-constant-sc (value)
+  ;; compiled TYPEP on ECL is wrong. See example in cross-typep.
+  #+host-quirks-ecl (declare (notinline typep))
   (typecase value
-    ((or (integer #.sb!xc:most-negative-fixnum #.sb!xc:most-positive-fixnum)
+    ((or (integer #.sb-xc:most-negative-fixnum #.sb-xc:most-positive-fixnum)
          character)
      immediate-sc-number)
     (symbol ; Symbols in static and immobile space are immediate
-     (when (or ;; With #!+immobile-symbols, all symbols are in immobile-space.
+     (when (or ;; With #+immobile-symbols, all symbols are in immobile-space.
                ;; And the cross-compiler always uses immobile-space if enabled.
-               #!+(or immobile-symbols (and immobile-space (host-feature sb-xc-host))) t
+               #+(or immobile-symbols (and immobile-space sb-xc-host)) t
 
-               ;; Otherwise, if #!-immobile-symbols, and the symbol was present
+               ;; Otherwise, if #-immobile-symbols, and the symbol was present
                ;; in the initial core image as indicated by the symbol header, then
                ;; it's in immobile-space. There is a way in which the bit can be wrong,
                ;; but it's highly unlikely - the symbol would have to be uninterned from
@@ -437,14 +415,14 @@
                ;; initial package. All without breaking anything. Hence, unlikely.
                ;; Also note that if compiling to memory, the symbol's current address
                ;; is used to determine whether it's immediate.
-               #!+(and (not (host-feature sb-xc-host)) immobile-space (not immobile-symbols))
+               #+(and (not sb-xc-host) immobile-space (not immobile-symbols))
                (or (logbitp +initial-core-symbol-bit+ (get-header-data value))
-                   (and (sb!c::core-object-p sb!c::*compile-object*)
+                   (and (sb-c::core-object-p sb-c::*compile-object*)
                         (immobile-space-obj-p value)))
 
                (static-symbol-p value))
        immediate-sc-number))
-    #!+immobile-space
+    #+immobile-space
     (layout
        immediate-sc-number)
     (single-float
@@ -459,12 +437,18 @@
        (if (eql value #c(0d0 0d0))
             fp-complex-double-zero-sc-number
             fp-complex-double-immediate-sc-number))
-    #!+(and sb-simd-pack (not (host-feature sb-xc-host)))
+    #+(and sb-simd-pack (not sb-xc-host))
     ((simd-pack double-float) double-sse-immediate-sc-number)
-    #!+(and sb-simd-pack (not (host-feature sb-xc-host)))
+    #+(and sb-simd-pack (not sb-xc-host))
     ((simd-pack single-float) single-sse-immediate-sc-number)
-    #!+(and sb-simd-pack (not (host-feature sb-xc-host)))
-    (simd-pack int-sse-immediate-sc-number)))
+    #+(and sb-simd-pack (not sb-xc-host))
+    (simd-pack int-sse-immediate-sc-number)
+    #+(and sb-simd-pack-256 (not sb-xc-host))
+    ((simd-pack-256 double-float) double-avx2-immediate-sc-number)
+    #+(and sb-simd-pack-256 (not sb-xc-host))
+    ((simd-pack-256 single-float) single-avx2-immediate-sc-number)
+    #+(and sb-simd-pack-256 (not sb-xc-host))
+    (simd-pack-256 int-avx2-immediate-sc-number)))
 
 (defun boxed-immediate-sc-p (sc)
   (eql sc immediate-sc-number))
@@ -477,7 +461,7 @@
           (symbol   (if (static-symbol-p val)
                         (+ nil-value (static-symbol-offset val))
                         (make-fixup val :immobile-object)))
-          #!+immobile-space
+          #+immobile-space
           (layout
            (make-fixup val :layout))
           (character (if tag
@@ -509,8 +493,6 @@
 (defun frame-byte-offset (index)
   (* (frame-word-offset index) n-word-bytes))
 
-(defconstant lra-save-offset return-pc-save-offset) ; ?
-
 ;;; This is used by the debugger.
 (defconstant single-value-return-byte-offset 3)
 
@@ -522,34 +504,23 @@
          (sb (sb-name (sc-sb sc)))
          (offset (tn-offset tn)))
     (ecase sb
-      (registers
-       (let ((index (ash offset -1))
-             (name-vec (case (sb!c:sc-operand-size sc)
-                         (:byte  +byte-register-names+)
-                         (:word  +word-register-names+)
-                         (:dword +dword-register-names+)
-                         (:qword +qword-register-names+))))
-         (or (and name-vec
-                  (evenp offset)
-                  (< -1 index (length name-vec))
-                  (svref name-vec index))
-             ;; FIXME: Shouldn't this be an ERROR?
-             (format nil "<unknown reg: off=~W, sc=~A>" offset (sc-name sc)))))
+      (registers (reg-name (tn-reg tn)))
       (float-registers (format nil "FLOAT~D" offset))
       (stack (format nil "S~D" offset))
       (constant (format nil "Const~D" offset))
       (immediate-constant "Immed")
       (noise (symbol-name (sc-name sc))))))
 
+(defconstant nargs-offset rcx-offset)
 (defconstant cfp-offset rbp-offset) ; pfw - needed by stuff in /code
 
 (defun combination-implementation-style (node)
-  (declare (type sb!c::combination node))
+  (declare (type sb-c::combination node))
   (flet ((valid-funtype (args result)
-           (sb!c::valid-fun-use node
-                                (sb!c::specifier-type
+           (sb-c::valid-fun-use node
+                                (sb-c::specifier-type
                                  `(function ,args ,result)))))
-    (case (sb!c::combination-fun-source-name node)
+    (case (sb-c::combination-fun-source-name node)
       (logtest
        (cond
          ((or (valid-funtype '(fixnum fixnum) '*)
@@ -564,8 +535,8 @@
        (cond
          ((or (and (valid-funtype '#.`((integer 0 ,(- 63 n-fixnum-tag-bits))
                                        fixnum) '*)
-                   (sb!c::constant-lvar-p
-                    (first (sb!c::basic-combination-args node))))
+                   (sb-c::constant-lvar-p
+                    (first (sb-c::basic-combination-args node))))
               (valid-funtype '((integer 0 63) (signed-byte 64)) '*)
               (valid-funtype '((integer 0 63) (unsigned-byte 64)) '*))
           (values :transform '(lambda (index integer)
