@@ -9,7 +9,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 
 ;;; We can load/store into fp registers through the top of stack
 ;;; %st(0) (fr0 here). Loads imply a push to an empty register which
@@ -96,10 +96,10 @@
 (defmacro store-symbol-value (reg symbol)
   `(inst mov (make-ea-for-symbol-value ,symbol) ,reg))
 
-#!+sb-thread
+#+sb-thread
 (progn
 (defmacro tls-index-of (symbol)
-  `(make-ea-for-object-slot ,symbol ,sb!vm:symbol-tls-index-slot
+  `(make-ea-for-object-slot ,symbol ,sb-vm:symbol-tls-index-slot
                             ,other-pointer-lowtag))
 (defmacro make-ea-for-symbol-tls-index (symbol)
   (declare (type symbol symbol))
@@ -121,7 +121,7 @@
                     :disp (make-ea-for-symbol-tls-index ,symbol))
      (inst mov EA ,reg :maybe-fs))))
 
-#!-sb-thread
+#-sb-thread
 (progn
 (defmacro load-tl-symbol-value (reg symbol) `(load-symbol-value ,reg ,symbol))
 (defmacro store-tl-symbol-value (reg symbol temp)
@@ -129,18 +129,18 @@
   `(store-symbol-value ,reg ,symbol)))
 
 (defmacro load-binding-stack-pointer (reg)
-  #!+sb-thread
+  #+sb-thread
   `(with-tls-ea (EA :base ,reg
                     :disp-type :constant
                     :disp (* 4 thread-binding-stack-pointer-slot))
      (inst mov ,reg EA :maybe-fs))
-  #!-sb-thread
+  #-sb-thread
   `(load-symbol-value ,reg *binding-stack-pointer*))
 
 (defmacro store-binding-stack-pointer (reg)
-  #!+sb-thread
+  #+sb-thread
   `(progn
-     #!+win32
+     #+win32
      (progn
        (inst push eax-tn)
        (inst push ,reg)
@@ -149,11 +149,11 @@
                         :disp (* 4 thread-binding-stack-pointer-slot))
          (inst pop EA))
        (inst pop eax-tn))
-     #!-win32
+     #-win32
      (with-tls-ea (EA :disp-type :constant
                       :disp (* 4 thread-binding-stack-pointer-slot))
        (inst mov EA ,reg :maybe-fs)))
-  #!-sb-thread
+  #-sb-thread
   `(store-symbol-value ,reg *binding-stack-pointer*))
 
 (defmacro load-type (target source &optional (offset 0))
@@ -164,16 +164,7 @@
 ;;;; error code
 (defun emit-error-break (vop kind code values)
   (assemble ()
-    #!-ud2-breakpoints
-    (inst int 3)                        ; i386 breakpoint instruction
-    ;; CLH 20060314
-    ;; On Darwin, we need to use #x0b0f instead of int3 in order
-    ;; to generate a SIGILL instead of a SIGTRAP as darwin/x86
-    ;; doesn't seem to be reliably firing SIGTRAP
-    ;; handlers. Hopefully this will be fixed by Apple at a
-    ;; later date.
-    #!+ud2-breakpoints
-    (inst word #x0b0f)
+    (inst break)
     ;; The return PC points here; note the location for the debugger.
     (when vop
       (note-this-location vop :internal-error))
@@ -197,55 +188,48 @@
 ;;; place and there's no logical single place to attach documentation.
 ;;; grep (mostly in src/runtime) is your friend
 
-;;; KLUDGE: since the stack on the x86 is treated conservatively, it
-;;; does not matter whether a signal occurs during construction of a
-;;; dynamic-extent object, as the half-finished construction of the
-;;; object will not cause any difficulty.  We can therefore elide
-(defmacro maybe-pseudo-atomic (not-really-p &body forms)
-  `(if ,not-really-p
-       (progn ,@forms)
-       (pseudo-atomic ,@forms)))
-
 ;;; Unsafely clear pa flags so that the image can properly lose in a
 ;;; pa section.
-#!+sb-thread
+#+sb-thread
 (defmacro %clear-pseudo-atomic ()
-  #!+win32
+  #+win32
   `(progn)
-  #!-win32
+  #-win32
   '(inst mov (make-ea :dword :disp (* 4 thread-pseudo-atomic-bits-slot)) 0 :fs))
 
-#!+sb-safepoint
+#+sb-safepoint
 (defun emit-safepoint ()
   (inst test eax-tn (make-ea :dword :disp
                              (- nil-value n-word-bytes other-pointer-lowtag
                                 gc-safepoint-trap-offset))))
 
-(defmacro pseudo-atomic (&rest forms)
-  #!+sb-safepoint-strictly
-  `(progn ,@forms (emit-safepoint))
-  #!-sb-safepoint-strictly
+(defmacro pseudo-atomic ((&key elide-if) &rest forms)
+  #+sb-safepoint-strictly
+  `(progn ,@forms (unless ,elide-if (emit-safepoint)))
+  #-sb-safepoint-strictly
   (with-unique-names (label pa-bits-ea)
     `(let ((,label (gen-label))
            (,pa-bits-ea
-            #!+sb-thread
+            #+sb-thread
             (make-ea :dword :disp (* 4 thread-pseudo-atomic-bits-slot))
-            #!-sb-thread
+            #-sb-thread
             (make-ea-for-symbol-value *pseudo-atomic-bits* :dword)))
-       (inst mov ,pa-bits-ea ebp-tn #!+sb-thread :fs)
+       (unless ,elide-if
+         (inst mov ,pa-bits-ea ebp-tn #+sb-thread :fs))
        ,@forms
-       (inst xor ,pa-bits-ea ebp-tn #!+sb-thread :fs)
-       (inst jmp :z ,label)
-       ;; if PAI was set, interrupts were disabled at the same time
-       ;; using the process signal mask.
-       (inst break pending-interrupt-trap)
-       (emit-label ,label)
-       #!+sb-safepoint
-       ;; In this case, when allocation thinks a GC should be done, it
-       ;; does not mark PA as interrupted, but schedules a safepoint
-       ;; trap instead.  Let's take the opportunity to trigger that
-       ;; safepoint right now.
-       (emit-safepoint))))
+       (unless ,elide-if
+         (inst xor ,pa-bits-ea ebp-tn #+sb-thread :fs)
+         (inst jmp :z ,label)
+         ;; if PAI was set, interrupts were disabled at the same time
+         ;; using the process signal mask.
+         (inst break pending-interrupt-trap)
+         (emit-label ,label)
+         #+sb-safepoint
+         ;; In this case, when allocation thinks a GC should be done, it
+         ;; does not mark PA as interrupted, but schedules a safepoint
+         ;; trap instead.  Let's take the opportunity to trigger that
+         ;; safepoint right now.
+         (emit-safepoint)))))
 
 ;;;; indexed references
 
@@ -432,7 +416,7 @@
    The value of the BASE register is undefined following the macro invocation."
   (check-type base-already-live-p boolean)
   (check-type disp-type (member :index :constant))
-  #!-(and win32 sb-thread)
+  #-(and win32 sb-thread)
   (let ((body (subst :fs :maybe-fs body)))
     (ecase disp-type
       (:constant
@@ -451,7 +435,7 @@
           ,@(subst `(make-ea :dword :base ,base)
                    ea-var
                    body)))))
-  #!+(and win32 sb-thread)
+  #+(and win32 sb-thread)
   ;; goes through a temporary register to add the thread address into it
   (multiple-value-bind (constant-disp ea-disp)
       (ecase disp-type
