@@ -9,7 +9,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!C")
+(in-package "SB-C")
 
 (declaim (type list *xref-kinds*))
 (defglobal *xref-kinds* '(:binds :calls :sets :references :macroexpands))
@@ -84,7 +84,7 @@
      ;; Record references to globals made using SYMBOL-VALUE.
      (let ((fun (principal-lvar-use (combination-fun node)))
            (arg (car (combination-args node))))
-       (when (and (ref-p fun) (eq 'symbol-value (leaf-%source-name (ref-leaf fun)))
+       (when (and (ref-p fun) (eq 'symeval (leaf-%source-name (ref-leaf fun)))
                   (constant-lvar-p arg) (symbolp (lvar-value arg)))
          (record-xref :references (lvar-value arg) context node nil))))
     (ref
@@ -138,24 +138,17 @@
     (list
      (every #'internal-name-p what))
     (symbol
-     #!+sb-xref-for-internals
-     (eq '.anonymous. what)
-     #!-sb-xref-for-internals
      (or (eq '.anonymous. what)
-         (member (symbol-package what)
+         #-sb-xref-for-internals
+         (member (sb-xc:symbol-package what)
                  (load-time-value
-                  (list* (find-package "COMMON-LISP")
-                         #+sb-xc-host (find-package "SB-XC")
-                         (remove-if-not
-                          (lambda (package)
-                            (= (mismatch #+sb-xc "SB-" #-sb-xc "SB!"
-                                         (package-name package))
-                               3))
-                          (list-all-packages))) t))
-         #+sb-xc-host   ; again, special case like in genesis and dump
-         (multiple-value-bind (cl-symbol cl-status)
-             (find-symbol (symbol-name what) sb!int:*cl-package*)
-           (and (eq what cl-symbol) (eq cl-status :external)))))
+                  (cons *cl-package*
+                        (remove-if-not
+                         (lambda (package)
+                           (= (mismatch "SB-" (package-name package))
+                              3))
+                         (list-all-packages)))
+                  t))))
     (t t)))
 
 (defun record-xref (kind what context node path)
