@@ -466,15 +466,15 @@
 (defmethod print-object ((obj interpreted-function) stream)
   ;; Do not try to directly print 'NAME-FOR-FUN', which returns OBJ
   ;; itself if it has no proper name.
-  (let ((name (fun-name obj)))
-    (if (eql name 0)
-        ;; To avoid an extra space between type and identity, the body must
-        ;; be empty, so we need two cases, because emptiness is compile-time
-        ;; determined, not based on whether the body actually printed anything.
-        (print-unreadable-object (obj stream :type t :identity t))
-        ;; show name whenever NAME it is not 0, even if not OBJ's proper name.
-        (print-unreadable-object (obj stream :type t)
-          (prin1 name stream)))))
+  (if (or (eql (interpreted-function-%proto-fn obj) 0)
+          (eql (fun-name obj) 0))
+      ;; To avoid an extra space between type and identity, the body must
+      ;; be empty, so we need two cases, because emptiness is compile-time
+      ;; determined, not based on whether the body actually printed anything.
+      (print-unreadable-object (obj stream :type t :identity t))
+      ;; show name whenever NAME it is not 0, even if not OBJ's proper name.
+      (print-unreadable-object (obj stream :type t)
+        (prin1 (fun-name obj) stream))))
 
 ;;; Return approximately a type specifier for LAMBDA-LIST.
 ;;; e.g. after doing (DEFUN FOO (A B) ...), you want (FUNCTION (T T) *)
@@ -517,7 +517,7 @@
     ;; Hooking all functions, makes them somewhat slower,
     ;; but allows for really nifty introspection,
     ;; such as discovering what calls are made by read-time evals.
-    (setf (funcallable-instance-fun function)
+    (setf (%funcallable-instance-fun function)
           (if *hook-all-functions-p*
               (lambda (&rest args)
                 (apply #'interpreter-hooked-trampoline function args))
@@ -618,7 +618,7 @@
   (declare (ignore env))
   (dolist (name symbols)
     (unless (symbolp name)
-      (ip-error "~A is not a symbol" name))
+      (%program-error "~A is not a symbol" name))
     ;; Same logic as SB-C::PROCESS-SPECIAL-DECL
     (let ((kind (info :variable :kind name)))
       (unless (member kind '(:special :unknown))
@@ -671,7 +671,7 @@
 
 (defun make-proto-fn (lambda-expression &optional (silent t))
   (multiple-value-bind (name lambda-list body)
-      (if (eq (car lambda-expression) 'named-lambda)
+      (if (memq (car lambda-expression) '(named-lambda))
           (with-subforms (name lambda-list . body) (cdr lambda-expression)
             (values name lambda-list body))
           (with-subforms (lambda-list . body) (cdr lambda-expression)
@@ -702,7 +702,7 @@
        (multiple-value-bind (definition index)
            (%find-position fname (the simple-vector (env-payload env))
                            nil 0 nil #'fname test)
-         (when definition
+         (when index
            (return (values (if (macro-env-p env) :macro :function)
                            definition (make-frame-ptr index level)))))))))
 
@@ -1202,7 +1202,7 @@
                       (let ((defined-fun
                              (sb-c::make-defined-fun
                               :%source-name fname
-                              :type (sb-int:proclaimed-ftype fname))))
+                              :type (sb-int:global-ftype fname))))
                         (setf (sb-c::defined-fun-inlinep defined-fun) inlinep)
                         (push (cons fname defined-fun) funs))))))))
             (ftype
