@@ -29,6 +29,7 @@
 #include "interr.h" /* for lose() */
 
 extern const char *widetag_names[];
+extern struct weak_pointer *weak_pointer_chain; /* in gc-common.c */
 
 /// Enable extra debug-only checks if DEBUG
 #ifdef DEBUG
@@ -82,49 +83,18 @@ do {                                                                   \
 #define FUN_SELF_FIXNUM_TAGGED 0
 #endif
 
-/* Code component trailer words:
- *                                                   v code size
- *      | fun_offset | fun_offset | .... | N-entries |
- *                                       ^
- *                 fun_table pointer ---/
- *
- * The fun_table pointer is aligned at a 4-byte boundary.
- */
-static inline unsigned int*
-code_fun_table(struct code* code) {
-  return (unsigned int*)((char*)code
-                         + code_header_words(code->header) * N_WORD_BYTES
-                         + fixnum_value(code->code_size) - sizeof (uint16_t));
-}
-static inline unsigned short
-code_n_funs(struct code* code) {
-    // immobile space filler objects appear to be code but have no simple-funs.
-    // Should probably consider changing the widetag to FILLER_WIDETAG.
-    return code_header_words(code->header)
-        ? *((unsigned short*)code_fun_table(code)) >> 4 : 0;
-}
+// Test for presence of a bit in vector's header.
+// As a special case, if 'val' is 0, then test for all bits clear.
+#define is_vector_subtype(header, val) \
+  (subtype_##val ? (HeaderValue(header) & subtype_##val) : \
+   !(HeaderValue(header) & 7))
 
-#define is_vector_subtype(header, val) ((HeaderValue(header) & 3) == subtype_##val)
-
-// Iterate over the native pointers to each function in 'code_var'
-// offsets are stored as the number of bytes into the instructions
-// portion of the code object at which the simple-fun object resides.
-// We use bytes, not words, because that's what the COMPUTE-FUN vop expects.
-#define for_each_simple_fun(index_var,fun_var,code_var,assertp,guts)        \
-  { int _nfuns_ = code_n_funs(code_var);                                    \
-    if (_nfuns_ > 0) {                                                      \
-      char *_insts_ = (char*)(code_var) +                                   \
-        (code_header_words((code_var)->header)<<WORD_SHIFT);                \
-      int index_var = 0;                                                    \
-      unsigned int* _offsets_ = code_fun_table(code_var) - 1;               \
-      do {                                                                  \
-       struct simple_fun* fun_var                                           \
-           = (struct simple_fun*)(_insts_ + _offsets_[-index_var]);         \
-       if (assertp)                                                         \
-         gc_assert(widetag_of(fun_var->header)==SIMPLE_FUN_WIDETAG);        \
-       guts ;                                                               \
-      } while (++index_var < _nfuns_);                                      \
-  }}
+// Mask out the fullcgc mark bit when asserting header validity
+#define UNSET_WEAK_VECTOR_VISITED(v) \
+  gc_assert((v->header & 0xffff) == \
+    (((subtype_VectorWeakVisited|subtype_VectorWeak) << N_WIDETAG_BITS) \
+     | SIMPLE_VECTOR_WIDETAG)); \
+  v->header ^= subtype_VectorWeakVisited << N_WIDETAG_BITS
 
 #define SIMPLE_FUN_SCAV_START(fun_ptr) &fun_ptr->name
 #define SIMPLE_FUN_SCAV_NWORDS(fun_ptr) ((lispobj*)fun_ptr->code - &fun_ptr->name)
@@ -142,7 +112,7 @@ code_n_funs(struct code* code) {
 
 extern sword_t (*sizetab[256])(lispobj *where);
 #define OBJECT_SIZE(header,where) \
-  (is_cons_half(header)?2:sizetab[widetag_of(header)](where))
+  (is_cons_half(header)?2:sizetab[header_widetag(header)](where))
 
 lispobj *gc_search_space3(void *pointer, lispobj *start, void *limit);
 static inline lispobj *gc_search_space(lispobj *start, void *pointer) {
@@ -223,9 +193,9 @@ extern lispobj virtual_fdefn_callee_lispobj(struct fdefn *fdefn,uword_t);
 
 static inline lispobj points_to_asm_routine_p(uword_t ptr) {
 # if defined(LISP_FEATURE_IMMOBILE_SPACE)
+    extern uword_t asm_routines_start, asm_routines_end;
     // Lisp assembly routines are in varyobj space, not readonly space
-    extern unsigned int asm_routines_end;
-    return ptr < (uword_t)asm_routines_end;
+    return asm_routines_start <= ptr && ptr < asm_routines_end;
 # else
     return READ_ONLY_SPACE_START <= ptr && ptr < READ_ONLY_SPACE_END;
 # endif
@@ -240,11 +210,10 @@ static inline lispobj fdefn_callee_lispobj(struct fdefn *fdefn) {
 #ifdef LISP_FEATURE_IMMOBILE_SPACE
 #include "genesis/layout.h"
 #define LAYOUT_SIZE (sizeof (struct layout)/N_WORD_BYTES)
-/// First 5 layouts: T, FUNCTION, STRUCTURE-OBJECT, LAYOUT, PACKAGE
+/// First 4 layouts: T, STRUCTURE-OBJECT, LAYOUT, FUNCTION
 /// (These #defines ought to be emitted by genesis)
-#define LAYOUT_OF_FUNCTION ((FIXEDOBJ_SPACE_START+1*LAYOUT_ALIGN)|INSTANCE_POINTER_LOWTAG)
-#define LAYOUT_OF_LAYOUT   ((FIXEDOBJ_SPACE_START+3*LAYOUT_ALIGN)|INSTANCE_POINTER_LOWTAG)
-#define LAYOUT_OF_PACKAGE  ((FIXEDOBJ_SPACE_START+4*LAYOUT_ALIGN)|INSTANCE_POINTER_LOWTAG)
+#define LAYOUT_OF_LAYOUT   ((FIXEDOBJ_SPACE_START+2*LAYOUT_ALIGN)|INSTANCE_POINTER_LOWTAG)
+#define LAYOUT_OF_FUNCTION ((FIXEDOBJ_SPACE_START+3*LAYOUT_ALIGN)|INSTANCE_POINTER_LOWTAG)
 #endif
 
 boolean valid_widetag_p(unsigned char widetag);
