@@ -12,16 +12,16 @@
 #ifndef _INCLUDED_GLOBALS_H_
 #define _INCLUDED_GLOBALS_H_
 
-#ifndef LANGUAGE_ASSEMBLY
+#ifndef __ASSEMBLER__
 # include <sys/types.h>
 # include <unistd.h>
 # include "runtime.h"
-# include "runtime-options.h"
+# include "os.h"
 #endif
 
 #include "sbcl.h"
 
-#ifndef LANGUAGE_ASSEMBLY
+#ifndef __ASSEMBLER__
 
 #ifdef LISP_FEATURE_SB_THREAD
 #define foreign_function_call_active_p(thread) \
@@ -35,7 +35,23 @@ extern int foreign_function_call_active;
 extern os_vm_size_t dynamic_space_size;
 extern os_vm_size_t thread_control_stack_size;
 
-extern struct runtime_options *runtime_options;
+#if defined(LISP_FEATURE_RELOCATABLE_HEAP)
+#ifdef LISP_FEATURE_CHENEYGC
+extern uword_t DYNAMIC_0_SPACE_START, DYNAMIC_1_SPACE_START;
+#else
+extern uword_t DYNAMIC_SPACE_START;
+#endif
+#endif
+#ifdef LISP_FEATURE_IMMOBILE_SPACE
+extern uword_t FIXEDOBJ_SPACE_START, VARYOBJ_SPACE_START;
+extern uword_t immobile_space_lower_bound, immobile_space_max_offset;
+extern uword_t immobile_range_1_max_offset, immobile_range_2_min_offset;
+extern unsigned int varyobj_space_size;
+#endif
+
+extern boolean alloc_profiling;
+extern os_vm_address_t alloc_profile_buffer;
+extern lispobj alloc_profile_data; // Lisp SIMPLE-VECTOR
 
 #ifdef LISP_FEATURE_WIN32
 #define ENVIRON _environ
@@ -58,7 +74,12 @@ extern lispobj *current_control_frame_pointer;
 extern lispobj *current_binding_stack_pointer;
 #endif
 
-#if !defined(LISP_FEATURE_X86) && !defined(LISP_FEATURE_X86_64)
+/* FIXME: the comment below this is obsolete. Most backends do want to use
+ * 'dynamic_space_free_pointer' contrary to the claim that it is only intended
+ * for cheneygc. The exact combinations of GC kind, architecture, and threads
+ * vs. no threads make it extremely confusing the figure out and document
+ * the state of things. Somebody should do that. */
+
 /* This is unused on X86 and X86_64, but is used as the global
  *  allocation pointer by the cheney GC, and, in some instances, as
  *  the global allocation pointer on PPC/GENCGC. This should probably
@@ -67,17 +88,29 @@ extern lispobj *current_binding_stack_pointer;
  *  bits, and is tightly coupled to reg_ALLOC by the assembly
  *  routines. */
 extern lispobj *dynamic_space_free_pointer;
+extern lispobj *read_only_space_free_pointer;
+extern lispobj *static_space_free_pointer;
+#ifdef LISP_FEATURE_IMMOBILE_SPACE
+extern lispobj *varyobj_free_pointer;
+extern lispobj *fixedobj_free_pointer;
 #endif
+extern os_vm_address_t anon_dynamic_space_start;
 
 # ifndef LISP_FEATURE_GENCGC
 extern lispobj *current_auto_gc_trigger;
 # endif
 
+#if defined(LISP_FEATURE_GENCGC)
+#define current_dynamic_space ((lispobj*)(DYNAMIC_SPACE_START))
+#elif defined(LISP_FEATURE_CHENEYGC)
 extern lispobj *current_dynamic_space;
+#else
+#error "Which GC?"
+#endif
 
 extern void globals_init(void);
 
-#else /* LANGUAGE_ASSEMBLY */
+#else /* __ASSEMBLER__ */
 
 # ifdef LISP_FEATURE_MIPS
 #  ifdef __linux__
@@ -101,7 +134,7 @@ extern void globals_init(void);
 #  endif
 # endif
 /**/
-# ifdef LISP_FEATURE_PPC
+# if defined(LISP_FEATURE_PPC) || defined(LISP_FEATURE_PPC64)
 #  ifdef LISP_FEATURE_DARWIN
 #   define EXTERN(name,bytes) .globl _ ## name
 #  else
@@ -113,7 +146,7 @@ extern void globals_init(void);
 #  define EXTERN(name,bytes) .global name
 # endif
 /**/
-# ifdef LISP_FEATURE_ARM
+# if defined(LISP_FEATURE_ARM) || defined(LISP_FEATURE_ARM64)
 #   define EXTERN(name,bytes) .global name
 # endif
 
@@ -138,6 +171,6 @@ EXTERN(current_binding_stack_pointer, POINTERSIZE)
 EXTERN(dynamic_space_free_pointer, POINTERSIZE)
 # endif
 
-#endif /* LANGUAGE_ASSEMBLY */
+#endif /* __ASSEMBLER__ */
 
 #endif /* _INCLUDED_GLOBALS_H_ */
