@@ -77,6 +77,7 @@
  * stack by the kernel, so copying a libc-sized sigset_t into it will
  * overflow and cause other data on the stack to be corrupted */
 /* FIXME: do not rely on NSIG being a multiple of 8 */
+/* See https://sourceware.org/bugzilla/show_bug.cgi?id=1780 */
 
 #ifdef LISP_FEATURE_WIN32
 # define REAL_SIGSET_SIZE_BYTES (4)
@@ -169,6 +170,34 @@ maybe_resignal_to_lisp_thread(int signal, os_context_t *context)
         return 0;
 }
 
+#if INSTALL_SIG_MEMORY_FAULT_HANDLER && defined(THREAD_SANITIZER)
+/* Under TSAN, every signal blocks every other signal regardless of the
+ * 'sa_mask' given to sigaction(). This is courtesy of an interceptor -
+ * https://github.com/llvm-mirror/compiler-rt/blob/bcc227ee4af1ef3e63033b35dcb1d5627a3b2941/lib/tsan/rtl/tsan_interceptors.cc#L1972
+ *
+ * So among other things, SIGSEGV is blocked on receipt of any random signal
+ * of interest (SIGPROF, SIGALRM, SIGPIPE, ...) that might call Lisp code.
+ * Therefore, if any handler re-enters Lisp, there is a high likelihood
+ * of SIGSEGV being delivered while blocked. Unfortunately, the OS treats
+ * blocked SIGSEGV exactly as if the specified disposition were SIG_DFL,
+ * which results in process termination and a core dump.
+ *
+ * It doesn't work to route all our signals through 'unblock_me_trampoline',
+ * because that only unblocks the specific signal that was just delivered,
+ * to work around the problem of SA_NODEFER not working. (Which says that
+ * a signal should not be blocked within in its own handler; it says nothing
+ * about all other signals.)
+ * Our trick is to unblock SIGSEGV early in every handler,
+ * so not to face sudden death if it happens to invoke Lisp.
+ */
+#  define UNBLOCK_SIGSEGV() \
+  { sigset_t mask; sigemptyset(&mask); \
+    sigaddset(&mask, SIG_MEMORY_FAULT); /* usually SIGSEGV */ \
+    thread_sigmask(SIG_UNBLOCK, &mask, 0); }
+#else
+#  define UNBLOCK_SIGSEGV() {}
+#endif
+
 /* These are to be used in signal handlers. Currently all handlers are
  * called from one of:
  *
@@ -190,6 +219,7 @@ maybe_resignal_to_lisp_thread(int signal, os_context_t *context)
 #define SAVE_ERRNO(signal,context,void_context)                 \
     {                                                           \
         int _saved_errno = errno;                               \
+        UNBLOCK_SIGSEGV();                                      \
         RESTORE_FP_CONTROL_WORD(context,void_context);          \
         if (!maybe_resignal_to_lisp_thread(signal, context))    \
         {
@@ -249,14 +279,22 @@ unblock_signals(sigset_t *what, sigset_t *where, sigset_t *old)
     }
 }
 
+// Stringify sigset into the supplied result buffer.
 static void
-print_sigset(sigset_t *sigset)
+sigset_tostring(sigset_t *sigset, char* result, int result_length)
 {
-  int i;
-  for(i = 1; i < NSIG; i++) {
-    if (sigismember(sigset, i))
-      fprintf(stderr, "Signal %d masked\n", i);
-  }
+    int i;
+    int len = 0;
+    for(i = 1; i < NSIG; i++)
+        if (sigismember(sigset, i)) {
+            // ensure room for (generously) 3 digits + comma + null, or give up
+            if (len > result_length - 5) {
+                strcpy(result, "too many to list");
+                return;
+            }
+            len += sprintf(result+len, "%s%d", len?",":"", i);
+        }
+    result[len] = 0;
 }
 
 /* Return 1 is all signals is sigset2 are masked in sigset, return 0
@@ -266,7 +304,6 @@ boolean
 all_signals_blocked_p(sigset_t *sigset, sigset_t *sigset2,
                                 const char *name)
 {
-#if !defined(LISP_FEATURE_WIN32) || defined(LISP_FEATURE_SB_THREAD)
     int i;
     boolean has_blocked = 0, has_unblocked = 0;
     sigset_t current;
@@ -283,14 +320,14 @@ all_signals_blocked_p(sigset_t *sigset, sigset_t *sigset2,
         }
     }
     if (has_blocked && has_unblocked) {
-        print_sigset(sigset);
-        lose("some %s signals blocked, some unblocked\n", name);
+        char buf[3*64]; // assuming worst case 64 signals present in sigset
+        sigset_tostring(sigset, buf, sizeof buf);
+        lose("%s signals partially blocked: {%s}\n", name, buf);
     }
     if (has_blocked)
         return 1;
     else
         return 0;
-#endif
 }
 
 
@@ -326,7 +363,7 @@ sigaddset_blockable(sigset_t *sigset)
 }
 
 void
-sigaddset_gc(sigset_t *sigset)
+sigaddset_gc(sigset_t __attribute__((unused)) *sigset)
 {
 #ifdef THREADS_USING_GCSIGNAL
     sigaddset(sigset,SIG_STOP_FOR_GC);
@@ -338,9 +375,6 @@ sigset_t deferrable_sigset;
 sigset_t blockable_sigset;
 sigset_t gc_sigset;
 
-#endif
-
-#if !defined(LISP_FEATURE_WIN32) || defined(LISP_FEATURE_SB_THREAD)
 boolean
 deferrables_blocked_p(sigset_t *sigset)
 {
@@ -501,8 +535,8 @@ they are not safe to interrupt at all, this is a pretty severe occurrence.\n");
 inline static void
 check_interrupts_enabled_or_lose(os_context_t *context)
 {
-    struct thread *thread=arch_os_get_current_thread();
-    if (SymbolValue(INTERRUPTS_ENABLED,thread) == NIL)
+    __attribute__((unused)) struct thread *thread = arch_os_get_current_thread();
+    if (read_TLS(INTERRUPTS_ENABLED,thread) == NIL)
         lose("interrupts not enabled\n");
     if (arch_pseudo_atomic_atomic(context))
         lose ("in pseudo atomic section\n");
@@ -558,14 +592,14 @@ maybe_save_gc_mask_and_block_deferrables(sigset_t *sigset)
  * enabled, without the protection of *GC-INHIBIT* T and there is gc
  * (or stop for gc) pending, but we haven't trapped yet? */
 int
-in_leaving_without_gcing_race_p(struct thread *thread)
+in_leaving_without_gcing_race_p(struct thread __attribute__((unused)) *thread)
 {
-    return ((SymbolValue(IN_WITHOUT_GCING,thread) != NIL) &&
-            (SymbolValue(INTERRUPTS_ENABLED,thread) != NIL) &&
-            (SymbolValue(GC_INHIBIT,thread) == NIL) &&
-            ((SymbolValue(GC_PENDING,thread) != NIL)
+    return ((read_TLS(IN_WITHOUT_GCING,thread) != NIL) &&
+            (read_TLS(INTERRUPTS_ENABLED,thread) != NIL) &&
+            (read_TLS(GC_INHIBIT,thread) == NIL) &&
+            ((read_TLS(GC_PENDING,thread) != NIL)
 #if defined(LISP_FEATURE_SB_THREAD)
-             || (SymbolValue(STOP_FOR_GC_PENDING,thread) != NIL)
+             || (read_TLS(STOP_FOR_GC_PENDING,thread) != NIL)
 #endif
              ));
 }
@@ -578,16 +612,29 @@ check_interrupt_context_or_lose(os_context_t *context)
     struct thread *thread = arch_os_get_current_thread();
     struct interrupt_data *data = thread->interrupt_data;
     int interrupt_deferred_p = (data->pending_handler != 0);
-    int interrupt_pending = (SymbolValue(INTERRUPT_PENDING,thread) != NIL);
+    int interrupt_pending = (read_TLS(INTERRUPT_PENDING,thread) != NIL);
     sigset_t *sigset = os_context_sigmask_addr(context);
     /* On PPC pseudo_atomic_interrupted is cleared when coming out of
      * handle_allocation_trap. */
-#if defined(LISP_FEATURE_GENCGC) && !defined(GENCGC_IS_PRECISE)
-    int interrupts_enabled = (SymbolValue(INTERRUPTS_ENABLED,thread) != NIL);
-    int gc_inhibit = (SymbolValue(GC_INHIBIT,thread) != NIL);
-    int gc_pending = (SymbolValue(GC_PENDING,thread) == T);
+#if defined(LISP_FEATURE_GENCGC) && !GENCGC_IS_PRECISE
+    int interrupts_enabled = (read_TLS(INTERRUPTS_ENABLED,thread) != NIL);
+    int gc_inhibit = (read_TLS(GC_INHIBIT,thread) != NIL);
+    int gc_pending = (read_TLS(GC_PENDING,thread) == T);
     int pseudo_atomic_interrupted = get_pseudo_atomic_interrupted(thread);
     int in_race_p = in_leaving_without_gcing_race_p(thread);
+    int safepoint_active = 0;
+#if defined(LISP_FEATURE_SB_SAFEPOINT)
+    /* Don't try to take the gc state lock if there's a chance that
+     * we're already holding it (thread_register_gc_trigger() is
+     * called from PA, gc_stop_the_world() and gc_start_the_world()
+     * are called from WITHOUT-GCING, all other takers of the lock
+     * have deferrables blocked). */
+    if (!(interrupt_pending || pseudo_atomic_interrupted || gc_inhibit)) {
+        WITH_GC_STATE_LOCK {
+            safepoint_active = gc_cycle_active();
+        }
+    }
+#endif
     /* In the time window between leaving the *INTERRUPTS-ENABLED* NIL
      * section and trapping, a SIG_STOP_FOR_GC would see the next
      * check fail, for this reason sig_stop_for_gc handler does not
@@ -597,14 +644,14 @@ check_interrupt_context_or_lose(os_context_t *context)
             lose("Stray deferred interrupt.\n");
     }
     if (gc_pending)
-        if (!(pseudo_atomic_interrupted || gc_inhibit || in_race_p))
+        if (!(pseudo_atomic_interrupted || gc_inhibit || in_race_p || safepoint_active))
             lose("GC_PENDING, but why?\n");
 #if defined(LISP_FEATURE_SB_THREAD)
     {
         int stop_for_gc_pending =
-            (SymbolValue(STOP_FOR_GC_PENDING,thread) != NIL);
+            (read_TLS(STOP_FOR_GC_PENDING,thread) != NIL);
         if (stop_for_gc_pending)
-            if (!(pseudo_atomic_interrupted || gc_inhibit || in_race_p))
+            if (!(pseudo_atomic_interrupted || gc_inhibit || in_race_p || safepoint_active))
                 lose("STOP_FOR_GC_PENDING, but why?\n");
         if (pseudo_atomic_interrupted)
             if (!(gc_pending || stop_for_gc_pending || interrupt_deferred_p))
@@ -641,7 +688,8 @@ check_interrupt_context_or_lose(os_context_t *context)
  */
 
 static void
-build_fake_control_stack_frames(struct thread *th,os_context_t *context)
+build_fake_control_stack_frames(struct thread __attribute__((unused)) *th,
+                                os_context_t __attribute__((unused)) *context)
 {
 #ifndef LISP_FEATURE_C_STACK_IS_CONTROL_STACK
 
@@ -658,8 +706,7 @@ build_fake_control_stack_frames(struct thread *th,os_context_t *context)
         == access_control_frame_pointer(th)) {
         /* There is a small window during call where the callee's
          * frame isn't built yet. */
-        if (lowtag_of(*os_context_register_addr(context, reg_CODE))
-            == FUN_POINTER_LOWTAG) {
+        if (functionp(*os_context_register_addr(context, reg_CODE))) {
             /* We have called, but not built the new frame, so
              * build it for them. */
             access_control_frame_pointer(th)[0] =
@@ -677,7 +724,7 @@ build_fake_control_stack_frames(struct thread *th,os_context_t *context)
         }
     } else
 #elif defined (LISP_FEATURE_ARM)
-        access_control_frame_pointer(th) =
+        access_control_frame_pointer(th) = (lispobj*)
             SymbolValue(CONTROL_STACK_POINTER, th);
 #elif defined (LISP_FEATURE_ARM64)
     access_control_frame_pointer(th) =
@@ -755,7 +802,7 @@ fake_foreign_function_call(os_context_t *context)
     /* Do dynamic binding of the active interrupt context index
      * and save the context in the context array. */
     context_index =
-        fixnum_value(SymbolValue(FREE_INTERRUPT_CONTEXT_INDEX,thread));
+        fixnum_value(read_TLS(FREE_INTERRUPT_CONTEXT_INDEX,thread));
 
     if (context_index >= MAX_INTERRUPTS) {
         lose("maximum interrupt nesting depth (%d) exceeded\n", MAX_INTERRUPTS);
@@ -764,7 +811,7 @@ fake_foreign_function_call(os_context_t *context)
     bind_variable(FREE_INTERRUPT_CONTEXT_INDEX,
                   make_fixnum(context_index + 1),thread);
 
-    thread->interrupt_contexts[context_index] = context;
+    nth_interrupt_context(context_index, thread) = context;
 
 #if !defined(LISP_FEATURE_X86) && !defined(LISP_FEATURE_X86_64)
     /* x86oid targets don't maintain the foreign function call flag at
@@ -778,7 +825,7 @@ fake_foreign_function_call(os_context_t *context)
  * the usual signal mask will be restored from the context when the handler
  * finishes.  Otherwise, be careful */
 void
-undo_fake_foreign_function_call(os_context_t *context)
+undo_fake_foreign_function_call(os_context_t __attribute__((unused)) *context)
 {
     struct thread *thread=arch_os_get_current_thread();
     /* Block all blockable signals. */
@@ -786,6 +833,12 @@ undo_fake_foreign_function_call(os_context_t *context)
 
     foreign_function_call_active_p(thread) = 0;
 
+#ifdef LISP_FEATURE_SB_SAFEPOINT
+    /* garbage_collect_generation may access it in parallel after
+       FREE_INTERRUPT_CONTEXT_INDEX has been updated, stuff a zero to
+       keep it from being confused. */
+    nth_interrupt_context(fixnum_value(read_TLS(FREE_INTERRUPT_CONTEXT_INDEX,thread)) - 1, thread) = NULL;
+#endif
     /* Undo dynamic binding of FREE_INTERRUPT_CONTEXT_INDEX */
     unbind(thread);
 
@@ -930,7 +983,7 @@ interrupt_handle_pending(os_context_t *context)
     if (data->gc_blocked_deferrables) {
         if (data->pending_handler)
             lose("GC blocked deferrables but still got a pending handler.");
-        if (SymbolValue(GC_INHIBIT,thread)!=NIL)
+        if (read_TLS(GC_INHIBIT,thread)!=NIL)
             lose("GC blocked deferrables while GC is inhibited.");
         /* Restore the saved signal mask from the original signal (the
          * one that interrupted us during the critical section) into
@@ -944,21 +997,24 @@ interrupt_handle_pending(os_context_t *context)
     }
 #endif
 
-    if (SymbolValue(GC_INHIBIT,thread)==NIL) {
+    if (read_TLS(GC_INHIBIT,thread)==NIL) {
         void *original_pending_handler = data->pending_handler;
 
-#ifdef LISP_FEATURE_SB_SAFEPOINT
+#if defined(LISP_FEATURE_SB_SAFEPOINT) && defined(LISP_FEATURE_SB_THREAD)
         /* handles the STOP_FOR_GC_PENDING case, plus THRUPTIONS */
-        if (SymbolValue(STOP_FOR_GC_PENDING,thread) != NIL
+        if (read_TLS(STOP_FOR_GC_PENDING,thread) != NIL
 # ifdef LISP_FEATURE_SB_THRUPTION
-             || (SymbolValue(THRUPTION_PENDING,thread) != NIL
-                 && SymbolValue(INTERRUPTS_ENABLED, thread) != NIL)
+             || (read_TLS(THRUPTION_PENDING,thread) != NIL
+                 && read_TLS(INTERRUPTS_ENABLED, thread) != NIL)
 # endif
-            )
+            ) {
             /* We ought to take this chance to do a pitstop now. */
+            fake_foreign_function_call(context);
             thread_in_lisp_raised(context);
+            undo_fake_foreign_function_call(context);
+        }
 #elif defined(LISP_FEATURE_SB_THREAD)
-        if (SymbolValue(STOP_FOR_GC_PENDING,thread) != NIL) {
+        if (read_TLS(STOP_FOR_GC_PENDING,thread) != NIL) {
             /* STOP_FOR_GC_PENDING and GC_PENDING are cleared by
              * the signal handler if it actually stops us. */
             arch_clear_pseudo_atomic_interrupted(context);
@@ -966,9 +1022,9 @@ interrupt_handle_pending(os_context_t *context)
         } else
 #endif
          /* Test for T and not for != NIL since the value :IN-PROGRESS
-          * is used in SUB-GC as part of the mechanism to supress
-          * recursive gcs.*/
-        if (SymbolValue(GC_PENDING,thread) == T) {
+          * used to be used in SUB-GC as part of the mechanism to
+          * supress recursive gcs.*/
+        if (read_TLS(GC_PENDING,thread) == T) {
 
             /* Two reasons for doing this. First, if there is a
              * pending handler we don't want to run. Second, we are
@@ -998,7 +1054,7 @@ interrupt_handle_pending(os_context_t *context)
                 unbind(thread);
                 unbind(thread);
             }
-        } else if (SymbolValue(GC_PENDING,thread) != NIL) {
+        } else if (read_TLS(GC_PENDING,thread) != NIL) {
             /* It's not NIL or T so GC_PENDING is :IN-PROGRESS. If
              * GC-PENDING is not NIL then we cannot trap on pseudo
              * atomic due to GC (see if(GC_PENDING) logic in
@@ -1012,10 +1068,10 @@ interrupt_handle_pending(os_context_t *context)
 
         /* No GC shall be lost. If SUB_GC triggers another GC then
          * that should be handled on the spot. */
-        if (SymbolValue(GC_PENDING,thread) != NIL)
+        if (read_TLS(GC_PENDING,thread) != NIL)
             lose("GC_PENDING after doing gc.");
 #ifdef THREADS_USING_GCSIGNAL
-        if (SymbolValue(STOP_FOR_GC_PENDING,thread) != NIL)
+        if (read_TLS(STOP_FOR_GC_PENDING,thread) != NIL)
             lose("STOP_FOR_GC_PENDING after doing gc.");
 #endif
         /* Check two things. First, that gc does not clobber a handler
@@ -1024,7 +1080,7 @@ interrupt_handle_pending(os_context_t *context)
          * an interrupt arrived during GC (POST-GC, really) it was
          * handled. */
         if (original_pending_handler != data->pending_handler)
-            lose("pending handler changed in gc: %x -> %x.",
+            lose("pending handler changed in gc: %p -> %p.",
                  original_pending_handler, data->pending_handler);
     }
 
@@ -1032,19 +1088,19 @@ interrupt_handle_pending(os_context_t *context)
     /* There may be no pending handler, because it was only a gc that
      * had to be executed or because Lisp is a bit too eager to call
      * DO-PENDING-INTERRUPT. */
-    if ((SymbolValue(INTERRUPTS_ENABLED,thread) != NIL) &&
+    if ((read_TLS(INTERRUPTS_ENABLED,thread) != NIL) &&
         (data->pending_handler))  {
         /* No matter how we ended up here, clear both
          * INTERRUPT_PENDING and pseudo atomic interrupted. It's safe
          * because we checked above that there is no GC pending. */
-        SetSymbolValue(INTERRUPT_PENDING, NIL, thread);
+        write_TLS(INTERRUPT_PENDING, NIL, thread);
         arch_clear_pseudo_atomic_interrupted(context);
         /* Restore the sigmask in the context. */
         sigcopyset(os_context_sigmask_addr(context), &data->pending_mask);
         run_deferred_handler(data, context);
     }
 #ifdef LISP_FEATURE_SB_THRUPTION
-    if (SymbolValue(THRUPTION_PENDING,thread)==T)
+    if (read_TLS(THRUPTION_PENDING,thread)==T)
         /* Special case for the following situation: There is a
          * thruption pending, but a signal had been deferred.  The
          * pitstop at the top of this function could only take care
@@ -1102,27 +1158,20 @@ interrupt_handle_now(int signal, siginfo_t *info, os_context_t *context)
          * support decides to pass on it. */
         lose("no handler for signal %d in interrupt_handle_now(..)\n", signal);
 
-    } else if (lowtag_of(handler.lisp) == FUN_POINTER_LOWTAG) {
+    } else if (functionp(handler.lisp)) {
         /* Once we've decided what to do about contexts in a
          * return-elsewhere world (the original context will no longer
          * be available; should we copy it or was nobody using it anyway?)
          * then we should convert this to return-elsewhere */
 
-        /* CMUCL comment said "Allocate the SAPs while the interrupts
-         * are still disabled.".  I (dan, 2003.08.21) assume this is
-         * because we're not in pseudoatomic and allocation shouldn't
-         * be interrupted.  In which case it's no longer an issue as
-         * all our allocation from C now goes through a PA wrapper,
-         * but still, doesn't hurt.
-         *
-         * Yeah, but non-gencgc platforms don't really wrap allocation
-         * in PA. MG - 2005-08-29  */
-
-
 #ifndef LISP_FEATURE_SB_SAFEPOINT
         /* Leave deferrable signals blocked, the handler itself will
          * allow signals again when it sees fit. */
+#ifdef LISP_FEATURE_C_STACK_IS_CONTROL_STACK
+        /* handler.lisp will hide from the GC, will be enabled in the handler itself.
+         * Not a problem for the conservative GC. */
         unblock_gc_signals(0, 0);
+#endif
 #else
         WITH_GC_AT_SAFEPOINTS_ONLY()
 #endif
@@ -1188,7 +1237,7 @@ maybe_defer_handler(void *handler, struct interrupt_data *data,
 
     check_blockables_blocked_or_lose(0);
 
-    if (SymbolValue(INTERRUPT_PENDING,thread) != NIL)
+    if (read_TLS(INTERRUPT_PENDING,thread) != NIL)
         lose("interrupt already pending\n");
     if (thread->interrupt_data->pending_handler)
         lose("there is a pending handler already (PA)\n");
@@ -1203,14 +1252,14 @@ maybe_defer_handler(void *handler, struct interrupt_data *data,
      * interrupt_handle_pending is going to be called soon, so
      * stashing the signal away is safe.
      */
-    if ((SymbolValue(INTERRUPTS_ENABLED,thread) == NIL) ||
+    if ((read_TLS(INTERRUPTS_ENABLED,thread) == NIL) ||
         in_leaving_without_gcing_race_p(thread)) {
         FSHOW_SIGNAL((stderr,
                       "/maybe_defer_handler(%x,%d): deferred (RACE=%d)\n",
                       (unsigned int)handler,signal,
                       in_leaving_without_gcing_race_p(thread)));
         store_signal_data_for_later(data,handler,signal,info,context);
-        SetSymbolValue(INTERRUPT_PENDING, T,thread);
+        write_TLS(INTERRUPT_PENDING, T,thread);
         check_interrupt_context_or_lose(context);
         return 1;
     }
@@ -1241,7 +1290,7 @@ store_signal_data_for_later (struct interrupt_data *data, void *handler,
                              siginfo_t *info, os_context_t *context)
 {
     if (data->pending_handler)
-        lose("tried to overwrite pending interrupt handler %x with %x\n",
+        lose("tried to overwrite pending interrupt handler %p with %p",
              data->pending_handler, handler);
     if (!handler)
         lose("tried to defer null interrupt handler\n");
@@ -1284,7 +1333,6 @@ low_level_interrupt_handle_now(int signal, siginfo_t *info,
     check_blockables_blocked_or_lose(0);
     check_interrupts_enabled_or_lose(context);
     (*interrupt_low_level_handlers[signal])(signal, info, context);
-    /* No Darwin context fixage needed, caller does that. */
 }
 
 static void
@@ -1305,20 +1353,22 @@ low_level_maybe_now_maybe_later(int signal, siginfo_t *info, void *void_context)
 
 /* This function must not cons, because that may trigger a GC. */
 void
-sig_stop_for_gc_handler(int signal, siginfo_t *info, os_context_t *context)
+sig_stop_for_gc_handler(int __attribute__((unused)) signal,
+                        siginfo_t __attribute__((unused)) *info,
+                        os_context_t *context)
 {
     struct thread *thread=arch_os_get_current_thread();
     boolean was_in_lisp;
 
     /* Test for GC_INHIBIT _first_, else we'd trap on every single
      * pseudo atomic until gc is finally allowed. */
-    if (SymbolValue(GC_INHIBIT,thread) != NIL) {
+    if (read_TLS(GC_INHIBIT,thread) != NIL) {
         FSHOW_SIGNAL((stderr, "sig_stop_for_gc deferred (*GC-INHIBIT*)\n"));
-        SetSymbolValue(STOP_FOR_GC_PENDING,T,thread);
+        write_TLS(STOP_FOR_GC_PENDING,T,thread);
         return;
     } else if (arch_pseudo_atomic_atomic(context)) {
         FSHOW_SIGNAL((stderr,"sig_stop_for_gc deferred (PA)\n"));
-        SetSymbolValue(STOP_FOR_GC_PENDING,T,thread);
+        write_TLS(STOP_FOR_GC_PENDING,T,thread);
         arch_set_pseudo_atomic_interrupted(context);
         maybe_save_gc_mask_and_block_deferrables
             (os_context_sigmask_addr(context));
@@ -1337,8 +1387,8 @@ sig_stop_for_gc_handler(int signal, siginfo_t *info, os_context_t *context)
     }
 
     /* Not pending anymore. */
-    SetSymbolValue(GC_PENDING,NIL,thread);
-    SetSymbolValue(STOP_FOR_GC_PENDING,NIL,thread);
+    write_TLS(GC_PENDING,NIL,thread);
+    write_TLS(STOP_FOR_GC_PENDING,NIL,thread);
 
     /* Consider this: in a PA section GC is requested: GC_PENDING,
      * pseudo_atomic_interrupted and gc_blocked_deferrables are set,
@@ -1358,8 +1408,10 @@ sig_stop_for_gc_handler(int signal, siginfo_t *info, os_context_t *context)
     }
 
     if(thread_state(thread)!=STATE_RUNNING) {
-        lose("sig_stop_for_gc_handler: wrong thread state: %ld\n",
-             fixnum_value(thread->state));
+        // macOS warns if OBJ_FMTX is used to format a 'sword_t'
+        // which fixnum_value() returns.
+        lose("sig_stop_for_gc_handler: wrong thread state: %"OBJ_FMTX,
+             (lispobj)fixnum_value(thread->state));
     }
 
     set_thread_state(thread,STATE_STOPPED);
@@ -1367,7 +1419,7 @@ sig_stop_for_gc_handler(int signal, siginfo_t *info, os_context_t *context)
 
     /* While waiting for gc to finish occupy ourselves with zeroing
      * the unused portion of the control stack to reduce conservatism.
-     * On hypothetic platforms with threads and exact gc it is
+     * On the platforms with threads and exact gc it is
      * actually a must. */
     scrub_control_stack();
 
@@ -1375,8 +1427,8 @@ sig_stop_for_gc_handler(int signal, siginfo_t *info, os_context_t *context)
     FSHOW_SIGNAL((stderr,"resumed\n"));
 
     if(thread_state(thread)!=STATE_RUNNING) {
-        lose("sig_stop_for_gc_handler: wrong thread state on wakeup: %ld\n",
-             fixnum_value(thread_state(thread)));
+        lose("sig_stop_for_gc_handler: wrong thread state on wakeup: %"OBJ_FMTX,
+             (lispobj)fixnum_value(thread_state(thread)));
     }
 
     if (was_in_lisp) {
@@ -1476,7 +1528,7 @@ arrange_return_to_c_function(os_context_t *context,
 #endif
 
 #if defined(LISP_FEATURE_DARWIN)
-    u32 *register_save_area = (u32 *)os_validate(0, 0x40);
+    u32 *register_save_area = (u32 *)os_allocate(0x40);
 
     FSHOW_SIGNAL((stderr, "/arrange_return_to_lisp_function: preparing to go to function %x, sp: %x\n", function,
                   *os_context_register_addr(context,reg_ESP)));
@@ -1660,11 +1712,6 @@ void reset_control_stack_guard_page(void)
     }
 }
 
-void lower_control_stack_guard_page(void)
-{
-    lower_thread_control_stack_guard_page(arch_os_get_current_thread());
-}
-
 boolean
 handle_guard_page_triggered(os_context_t *context,os_vm_address_t addr)
 {
@@ -1672,8 +1719,9 @@ handle_guard_page_triggered(os_context_t *context,os_vm_address_t addr)
 
     if(addr >= CONTROL_STACK_HARD_GUARD_PAGE(th) &&
        addr < CONTROL_STACK_HARD_GUARD_PAGE(th) + os_vm_page_size) {
+        fake_foreign_function_call(context);
         lose("Control stack exhausted, fault: %p, PC: %p",
-             addr, *os_context_pc_addr(context));
+             addr, (void*)*os_context_pc_addr(context));
     }
     else if(addr >= CONTROL_STACK_GUARD_PAGE(th) &&
             addr < CONTROL_STACK_GUARD_PAGE(th) + os_vm_page_size) {
@@ -1681,9 +1729,14 @@ handle_guard_page_triggered(os_context_t *context,os_vm_address_t addr)
          * protection so the error handler has some headroom, protect the
          * previous page so that we can catch returns from the guard page
          * and restore it. */
+        if (lose_on_corruption_p || gc_active_p) {
+            fake_foreign_function_call(context);
+            lose("Control stack exhausted, fault: %p, PC: %p",
+                 addr, (void*)*os_context_pc_addr(context));
+        }
         if (th->control_stack_guard_page_protected == NIL)
             lose("control_stack_guard_page_protected NIL");
-        lower_control_stack_guard_page();
+        lower_thread_control_stack_guard_page(th);
 #ifdef LISP_FEATURE_C_STACK_IS_CONTROL_STACK
         /* For the unfortunate case, when the control stack is
          * exhausted in a signal handler. */
@@ -1712,6 +1765,12 @@ handle_guard_page_triggered(os_context_t *context,os_vm_address_t addr)
             addr < BINDING_STACK_GUARD_PAGE(th) + os_vm_page_size) {
         protect_binding_stack_guard_page(0, NULL);
         protect_binding_stack_return_guard_page(1, NULL);
+
+        if (lose_on_corruption_p) {
+            fake_foreign_function_call(context);
+            lose("Binding stack exhausted");
+        }
+
         fprintf(stderr, "INFO: Binding stack guard page unprotected\n");
 
         /* For the unfortunate case, when the binding stack is
@@ -1785,7 +1844,9 @@ static volatile int sigaction_nodefer_works = -1;
 #define SA_NODEFER_TEST_KILL_SIGNAL SIGUSR1
 
 static void
-sigaction_nodefer_test_handler(int signal, siginfo_t *info, void *void_context)
+sigaction_nodefer_test_handler(int signal,
+                               siginfo_t __attribute__((unused)) *info,
+                               void __attribute__((unused)) *void_context)
 {
     sigset_t current;
     int i;
@@ -1976,7 +2037,7 @@ undoably_install_low_level_interrupt_handler (int signal,
         sa.sa_sigaction = low_level_handle_now_handler;
 #endif
 
-    sigcopyset(&sa.sa_mask, &blockable_sigset);
+    sa.sa_mask = blockable_sigset;
     sa.sa_flags = SA_SIGINFO | SA_RESTART
         | (sigaction_nodefer_works ? SA_NODEFER : 0);
 #if defined(LISP_FEATURE_C_STACK_IS_CONTROL_STACK)
@@ -1998,7 +2059,7 @@ undoably_install_low_level_interrupt_handler (int signal,
 /* This is called from Lisp. */
 uword_t
 install_handler(int signal, void handler(int, siginfo_t*, os_context_t*),
-                int synchronous)
+                int __attribute__((unused)) synchronous)
 {
 #ifndef LISP_FEATURE_WIN32
     struct sigaction sa;
@@ -2029,7 +2090,7 @@ install_handler(int signal, void handler(int, siginfo_t*, os_context_t*),
         else
             sa.sa_sigaction = interrupt_handle_now_handler;
 
-        sigcopyset(&sa.sa_mask, &blockable_sigset);
+        sa.sa_mask = blockable_sigset;
         sa.sa_flags = SA_SIGINFO | SA_RESTART |
             (sigaction_nodefer_works ? SA_NODEFER : 0);
         sigaction(signal, &sa, NULL);
@@ -2052,7 +2113,9 @@ install_handler(int signal, void handler(int, siginfo_t*, os_context_t*),
 /* This must not go through lisp as it's allowed anytime, even when on
  * the altstack. */
 void
-sigabrt_handler(int signal, siginfo_t *info, os_context_t *context)
+sigabrt_handler(int __attribute__((unused)) signal,
+                siginfo_t __attribute__((unused)) *info,
+                os_context_t *context)
 {
     /* Save the interrupt context. No need to undo it, since lose()
      * shouldn't return. */
@@ -2099,43 +2162,88 @@ siginfo_code(siginfo_t *info)
 {
     return info->si_code;
 }
-os_vm_address_t current_memory_fault_address;
 
 void
 lisp_memory_fault_error(os_context_t *context, os_vm_address_t addr)
 {
-   /* FIXME: This is lossy: if we get another memory fault (eg. from
-    * another thread) before lisp has read this, we lose the information.
-    * However, since this is mostly informative, we'll live with that for
-    * now -- some address is better then no address in this case.
-    */
-    current_memory_fault_address = addr;
-
     /* If we lose on corruption, provide LDB with debugging information. */
     fake_foreign_function_call(context);
 
     /* To allow debugging memory faults in signal handlers and such. */
-    corruption_warning_and_maybe_lose("Memory fault at %p (pc=%p, sp=%p)",
+#ifdef ARCH_HAS_STACK_POINTER
+    corruption_warning_and_maybe_lose("Memory fault at %p (pc=%p, fp=%p, sp=%p) tid %#lx",
                                       addr,
                                       *os_context_pc_addr(context),
-#ifdef ARCH_HAS_STACK_POINTER
-                                      *os_context_sp_addr(context)
+                                      os_context_frame_pointer(context),
+                                      *os_context_sp_addr(context),
+                                      thread_self()); // = 0 if -sb-thread
 #else
-                                      0
+    corruption_warning_and_maybe_lose("Memory fault at %p (pc=%p)",
+                                      addr, *os_context_pc_addr(context));
 #endif
-                                      );
+
 #ifdef LISP_FEATURE_C_STACK_IS_CONTROL_STACK
+#  if !(defined(LISP_FEATURE_X86) || defined(LISP_FEATURE_X86_64))
+#    error memory fault emulation needs validating for this architecture
+#  endif
+    /* We're on the altstack, and don't want to run Lisp code here, so
+     * we need to return from this signal handler.  But when we get to
+     * Lisp we'd like to have a signal context (with correct values in
+     * it) to present to the debugger, along with knowledge of what
+     * the faulting address was.  To get a signal context on the main
+     * stack, we arrange to return to a trap instruction.  To get the
+     * correct program counter in the context, we save it on the stack
+     * here and restore it to the context in the trap handler.  To
+     * pass the fault address, we save it on the stack here and pick
+     * it up in the trap handler.  And the stack pointer manipulation
+     * works as long as the on-stack side only pops items in its trap
+     * handler. */
+    extern void memory_fault_emulation_trap(void);
     undo_fake_foreign_function_call(context);
-    unblock_signals_in_context_and_maybe_warn(context);
-    arrange_return_to_lisp_function(context,
-                                    StaticSymbolFunction(MEMORY_FAULT_ERROR));
-#else
-    unblock_gc_signals(0, 0);
-    funcall0(StaticSymbolFunction(MEMORY_FAULT_ERROR));
-    undo_fake_foreign_function_call(context);
-#endif
+    void **sp = (void **)*os_context_sp_addr(context);
+    *--sp = (void *)*os_context_pc_addr(context);
+    *--sp = addr;
+#  ifdef LISP_FEATURE_X86
+    /* KLUDGE: x86-linux sp_addr doesn't affect the CPU on return */
+    *((void **)os_context_register_addr(context, reg_ESP)) = sp;
+#  else
+    *((void **)os_context_sp_addr(context)) = sp;
+#  endif
+    *os_context_pc_addr(context) =
+        (os_context_register_t)memory_fault_emulation_trap;
+    /* We exit here, letting the signal handler return, picking up at
+     * memory_fault_emulation_trap (in target-assem.S), which will
+     * trap, and the handler calls the function below, where we
+     * restore our state to parallel what a non-x86oid would have, and
+     * then run the common code for handling the error in Lisp. */
 }
+
+void
+handle_memory_fault_emulation_trap(os_context_t *context)
+{
+    void **sp = (void **)*os_context_sp_addr(context);
+    void *addr = *sp++;
+    *os_context_pc_addr(context) = (os_context_register_t)*sp++;
+#  ifdef LISP_FEATURE_X86
+    /* KLUDGE: x86-linux sp_addr doesn't affect the CPU on return */
+    *((void **)os_context_register_addr(context, reg_ESP)) = sp;
+#  else
+    *os_context_sp_addr(context) = (os_context_register_t)sp;
+#  endif
+    fake_foreign_function_call(context);
+#endif /* C_STACK_IS_CONTROL_STACK */
+    /* On x86oids, we're in handle_memory_fault_emulation_trap().
+     * On real computers, we're still in lisp_memory_fault_error(). */
+#ifndef LISP_FEATURE_SB_SAFEPOINT
+    unblock_gc_signals(0, 0);
 #endif
+    DX_ALLOC_SAP(context_sap, context);
+    DX_ALLOC_SAP(fault_address_sap, addr);
+    funcall2(StaticSymbolFunction(MEMORY_FAULT_ERROR),
+             context_sap, fault_address_sap);
+    undo_fake_foreign_function_call(context);
+}
+#endif /* !LISP_FEATURE_WIN32 */
 
 static void
 unhandled_trap_error(os_context_t *context)
@@ -2159,6 +2267,9 @@ unhandled_trap_error(os_context_t *context)
 void
 handle_trap(os_context_t *context, int trap)
 {
+    if (trap >= trap_Error) {
+        trap = trap_Error;
+    }
     switch(trap) {
 #if !(defined(LISP_FEATURE_WIN32) && defined(LISP_FEATURE_SB_THREAD))
     case trap_PendingInterrupt:
@@ -2211,6 +2322,11 @@ handle_trap(os_context_t *context, int trap)
         arch_handle_allocation_trap(context);
         arch_skip_instruction(context);
         break;
+#endif
+#if defined(LISP_FEATURE_C_STACK_IS_CONTROL_STACK) && !defined(LISP_FEATURE_WIN32)
+    case trap_MemoryFaultEmulation:
+        handle_memory_fault_emulation_trap(context);
+        break;
 #endif
     case trap_Halt:
         fake_foreign_function_call(context);
