@@ -58,17 +58,12 @@
 #include <thread.h>
 #endif
 
-#ifdef LISP_FEATURE_WIN32
+#if defined(LISP_FEATURE_WIN32) || defined(OS_THREAD_STACK)
 # define IMMEDIATE_POST_MORTEM
+#else
+static struct thread *postmortem_thread;
 #endif
 
-struct thread_post_mortem {
-    os_thread_t os_thread;
-    pthread_attr_t *os_attr;
-    os_vm_address_t os_address;
-};
-
-static struct thread_post_mortem * volatile pending_thread_post_mortem = 0;
 #endif
 
 int dynamic_values_bytes=TLS_SIZE*sizeof(lispobj);  /* same for all threads */
@@ -81,9 +76,11 @@ static pthread_mutex_t create_thread_lock = PTHREAD_MUTEX_INITIALIZER;
 
 #ifdef LISP_FEATURE_GCC_TLS
 __thread struct thread *current_thread;
-#endif
+__thread int is_lisp_thread;
+#else
 pthread_key_t lisp_thread = 0;
 #endif
+#endif
 
 #if defined(LISP_FEATURE_X86) || defined(LISP_FEATURE_X86_64)
 extern lispobj call_into_lisp_first_time(lispobj fun, lispobj *args, int nargs)
@@ -176,7 +173,7 @@ wait_for_thread_state_change(struct thread *thread, lispobj state)
             thread->state_not_stopped_waitcount++;
             break;
         default:
-            lose("Invalid state in wait_for_thread_state_change: "OBJ_FMTX"\n", state);
+            lose("Invalid state in wait_for_thread_state_change: %"OBJ_FMTX, state);
         }
     } else {
         wait_sem = NULL;
@@ -199,7 +196,11 @@ initial_thread_trampoline(struct thread *th)
     lispobj *args = NULL;
 #endif
 #ifdef LISP_FEATURE_SB_THREAD
+# ifdef LISP_FEATURE_GCC_TLS
+    is_lisp_thread = 1;
+# else
     pthread_setspecific(lisp_thread, (void *)1);
+# endif
 #endif
 #if defined(THREADS_USING_GCSIGNAL) && (defined(LISP_FEATURE_PPC) || defined(LISP_FEATURE_ARM64))
     /* SIG_STOP_FOR_GC defaults to blocked on PPC? */
@@ -237,6 +238,24 @@ initial_thread_trampoline(struct thread *th)
 
 #ifdef LISP_FEATURE_SB_THREAD
 
+/* THREAD POST MORTEM CLEANUP
+ *
+ * Memory allocated for the thread stacks cannot be reclaimed while
+ * the thread is still alive, so we need a mechanism for post mortem
+ * cleanups. FIXME: We actually have two, for historical reasons as
+ * the saying goes. Do we really need two? Nikodemus guesses that
+ * not anymore, now that we properly call pthread_attr_destroy before
+ * freeing the stack. */
+
+static void free_thread_struct(struct thread *th)
+{
+#if defined(LISP_FEATURE_WIN32)
+    os_invalidate_free((os_vm_address_t) th->os_address, THREAD_STRUCT_SIZE);
+#else
+    os_invalidate((os_vm_address_t) th->os_address, THREAD_STRUCT_SIZE);
+#endif
+}
+
 # if defined(IMMEDIATE_POST_MORTEM)
 
 /*
@@ -247,51 +266,15 @@ static void
 schedule_thread_post_mortem(struct thread *corpse)
 {
     pthread_detach(pthread_self());
-    int result = pthread_attr_destroy(corpse->os_attr);
-    gc_assert(!result);
-#if defined(LISP_FEATURE_WIN32)
-    os_invalidate_free(corpse->os_address, THREAD_STRUCT_SIZE);
-#else
-    os_invalidate(corpse->os_address, THREAD_STRUCT_SIZE);
-#endif
+    free_thread_struct(corpse);
 }
 
 # else
 
-/* THREAD POST MORTEM CLEANUP
- *
- * Memory allocated for the thread stacks cannot be reclaimed while
- * the thread is still alive, so we need a mechanism for post mortem
- * cleanups. FIXME: We actually have three, for historical reasons as
- * the saying goes. Do we really need three? Nikodemus guesses that
- * not anymore, now that we properly call pthread_attr_destroy before
- * freeing the stack. */
-
-static struct thread_post_mortem *
-plan_thread_post_mortem(struct thread *corpse)
-{
-    if (corpse) {
-        struct thread_post_mortem *post_mortem = malloc(sizeof(struct thread_post_mortem));
-        gc_assert(post_mortem);
-        post_mortem->os_thread = corpse->os_thread;
-        post_mortem->os_attr = corpse->os_attr;
-        post_mortem->os_address = corpse->os_address;
-
-        return post_mortem;
-    } else {
-        /* FIXME: When does this happen? */
-        return NULL;
-    }
-}
-
 static void
-perform_thread_post_mortem(struct thread_post_mortem *post_mortem)
+perform_thread_post_mortem(struct thread *post_mortem)
 {
-#ifdef CREATE_POST_MORTEM_THREAD
-    pthread_detach(pthread_self());
-#endif
-    int result;
-    if (post_mortem) {
+    gc_assert(post_mortem);
         /* The thread may exit before pthread_create() has finished
            initialization and it may write into already unmapped
            memory. This lock doesn't actually need to protect
@@ -301,42 +284,59 @@ perform_thread_post_mortem(struct thread_post_mortem *post_mortem)
            Possible improvements: stash the address of the thread
            struct for which a pthread is being created and don't lock
            here if it's not the one being terminated. */
-        result = pthread_mutex_lock(&create_thread_lock);
-        gc_assert(result == 0);
-        result = pthread_mutex_unlock(&create_thread_lock);
-        gc_assert(result == 0);
+    int result = pthread_mutex_lock(&create_thread_lock);
+    gc_assert(result == 0);
+    result = pthread_mutex_unlock(&create_thread_lock);
+    gc_assert(result == 0);
 
-        if ((result = pthread_join(post_mortem->os_thread, NULL))) {
-            lose("Error calling pthread_join in perform_thread_post_mortem:\n%s",
-                 strerror(result));
-        }
-        if ((result = pthread_attr_destroy(post_mortem->os_attr))) {
-            lose("Error calling pthread_attr_destroy in perform_thread_post_mortem:\n%s",
-                 strerror(result));
-        }
-        os_invalidate(post_mortem->os_address, THREAD_STRUCT_SIZE);
-        free(post_mortem);
+    if ((result = pthread_join(post_mortem->os_thread, NULL))) {
+        lose("Error calling pthread_join in perform_thread_post_mortem:\n%s",
+             strerror(result));
     }
+    free_thread_struct(post_mortem);
+}
+
+static inline struct thread*
+fifo_buffer_shift(struct thread** dest, struct thread* value)
+{
+#ifdef __ATOMIC_SEQ_CST
+    return __atomic_exchange_n(dest, value, __ATOMIC_SEQ_CST);
+#elif defined(LISP_FEATURE_X86) || defined(LISP_FEATURE_X86_64)
+    /* I'd like to remove this case but I don't know whether MSVC has
+     * either of __atomic_exchange_n() or __sync_val_compare_and_swap() */
+    lispobj old_value;
+    asm volatile
+        ("lock xchg %0,(%1)"
+         : "=r" (old_value)
+         : "r" (dest), "0" (value)
+         : "memory");
+    return old_value;
+#else
+    // We don't want a compare-and-swap, but if that's all we have,
+    // then build an atomic exchange out of compare-and-swap
+    lispobj old =  *dest;
+    while (1) {
+        lispobj actual = __sync_val_compare_and_swap(dest, old, value);
+        if (actual == old)
+            return old;
+        old = actual;
+    }
+#endif
 }
 
 static void
 schedule_thread_post_mortem(struct thread *corpse)
 {
-    struct thread_post_mortem *post_mortem = NULL;
-    if (corpse) {
-        post_mortem = plan_thread_post_mortem(corpse);
-
-#ifdef CREATE_POST_MORTEM_THREAD
-        pthread_t thread;
-        int result = pthread_create(&thread, NULL, perform_thread_post_mortem, post_mortem);
-        gc_assert(!result);
-#else
-        post_mortem = (struct thread_post_mortem *)
-            swap_lispobjs((lispobj *)(void *)&pending_thread_post_mortem,
-                          (lispobj)post_mortem);
-        perform_thread_post_mortem(post_mortem);
-#endif
-    }
+    gc_assert(corpse);
+    // This strange little mechanism is a FIFO buffer of capacity 1.
+    // By stuffing one new thing in and reading the old one out, we ensure
+    // that at least one pthread_create() has completed by this point.
+    // In particular, the thread we're post-morteming must have completed
+    // because thread creation is completely serialized (which is
+    // somewhat unfortunate).
+    struct thread* previous = fifo_buffer_shift(&postmortem_thread, corpse);
+    if (previous) // any random thread which pre-deceased me
+        perform_thread_post_mortem(previous);
 }
 
 # endif /* !IMMEDIATE_POST_MORTEM */
@@ -349,7 +349,11 @@ init_new_thread(struct thread *th,
 {
     int lock_ret;
 
+#ifdef LISP_FEATURE_GCC_TLS
+    is_lisp_thread = 1;
+#else
     pthread_setspecific(lisp_thread, (void *)1);
+#endif
     if(arch_os_thread_init(th)==0) {
         /* FIXME: handle error */
         lose("arch_os_thread_init failed\n");
@@ -480,14 +484,32 @@ void* new_thread_trampoline(void* arg)
     result = funcall0(function);
     undo_init_new_thread(th, &scribble);
 
+#ifndef OS_THREAD_STACK
     schedule_thread_post_mortem(th);
+#endif
 
     FSHOW((stderr,"/exiting thread %p\n", thread_self()));
     return (void*)(uintptr_t)result;
 }
 
+#ifdef OS_THREAD_STACK
+extern void* funcall1_switching_stack(void*, void *(*fun)(void *))
+# ifdef LISP_FEATURE_X86_64
+    __attribute__((sysv_abi))
+# endif
+    ;
+
+void* new_thread_trampoline_switch_stack(void* arg) {
+    struct thread *th = (struct thread *)arg;
+    void* ret = funcall1_switching_stack(arg,  new_thread_trampoline);
+
+    schedule_thread_post_mortem(th);
+    return ret;
+
+}
+#endif
+
 static struct thread *create_thread_struct(lispobj);
-static void free_thread_struct(struct thread *th);
 
 void
 attach_os_thread(init_thread_data *scribble)
@@ -505,11 +527,6 @@ attach_os_thread(init_thread_data *scribble)
     unblock_gc_signals(0, 0);
 #endif
 
-    /* We don't actually want a pthread_attr here, but rather than add
-     * `if's to the post-mostem, let's just keep that code happy by
-     * keeping it initialized: */
-    pthread_attr_init(th->os_attr);
-
 #if !defined(LISP_FEATURE_WIN32) && defined(LISP_FEATURE_C_STACK_IS_CONTROL_STACK)
     /* On windows, arch_os_thread_init will take care of finding the
      * stack. */
@@ -527,7 +544,7 @@ attach_os_thread(init_thread_data *scribble)
   stack_addr = (void*)((size_t)stack.ss_sp - stack_size);
 #elif defined(LISP_FEATURE_DARWIN)
     stack_size = pthread_get_stacksize_np(os);
-    stack_addr = pthread_get_stackaddr_np(os) - stack_size;
+    stack_addr = (char*)pthread_get_stackaddr_np(os) - stack_size;
 #else
     pthread_attr_t attr;
 #ifdef LISP_FEATURE_FREEBSD
@@ -561,7 +578,11 @@ detach_os_thread(init_thread_data *scribble)
     undo_init_new_thread(th, scribble);
 
     odxprint(misc, "deattach_os_thread: detached");
+#ifdef LISP_FEATURE_GCC_TLS
+    is_lisp_thread = 0;
+#else
     pthread_setspecific(lisp_thread, (void *)0);
+#endif
     thread_sigmask(SIG_SETMASK, &scribble->oldset, 0);
     free_thread_struct(th);
 }
@@ -620,16 +641,6 @@ callback_wrapper_trampoline(
 }
 #endif /* LISP_FEATURE_SB_THREAD */
 
-static void __attribute__((unused))
-free_thread_struct(struct thread *th)
-{
-#if defined(LISP_FEATURE_WIN32)
-    os_invalidate_free((os_vm_address_t) th->os_address, THREAD_STRUCT_SIZE);
-#else
-    os_invalidate((os_vm_address_t) th->os_address, THREAD_STRUCT_SIZE);
-#endif
-}
-
 /* this is called from any other thread to create the new one, and
  * initialize all parts of it that can be initialized from another
  * thread
@@ -754,7 +765,6 @@ create_thread_struct(lispobj initial_function) {
     th->interrupt_data = &nonpointer_data->interrupt_data;
 
 #ifdef LISP_FEATURE_SB_THREAD
-    th->os_attr = &nonpointer_data->os_attr;
 # ifndef LISP_FEATURE_SB_SAFEPOINT
     th->state_sem = &nonpointer_data->state_sem;
     th->state_not_running_sem = &nonpointer_data->state_not_running_sem;
@@ -870,7 +880,7 @@ create_thread_struct(lispobj initial_function) {
 
 void create_initial_thread(lispobj initial_function) {
     struct thread *th = create_thread_struct(initial_function);
-#ifdef LISP_FEATURE_SB_THREAD
+#if defined(LISP_FEATURE_SB_THREAD) && !defined(LISP_FEATURE_GCC_TLS)
     pthread_key_create(&lisp_thread, 0);
 #endif
     if(th) {
@@ -885,56 +895,56 @@ extern int pthread_attr_setstack (pthread_attr_t *__attr, void *__stackaddr,
                                   size_t __stacksize);
 #endif
 
+/* Call pthread_create() and return 1 for success, 0 for failure */
 boolean create_os_thread(struct thread *th,os_thread_t *kid_tid)
 {
     /* The new thread inherits the restrictive signal mask set here,
      * and enables signals again when it is set up properly. */
     sigset_t oldset;
-    boolean r=1;
-    int retcode = 0, initcode;
-
-    FSHOW_SIGNAL((stderr,"/create_os_thread: creating new thread\n"));
+    int retcode = 0;
+    boolean success = 0;
 
     /* Blocking deferrable signals is enough, no need to block
      * SIG_STOP_FOR_GC because the child process is not linked onto
      * all_threads until it's ready. */
     block_deferrable_signals(&oldset);
 
+    pthread_attr_t attr;
+    if (pthread_attr_init(&attr) == 0) {
+
     /* See perform_thread_post_mortem for at least one reason why this lock is neccessary */
-    retcode = pthread_mutex_lock(&create_thread_lock);
-    gc_assert(retcode == 0);
-    FSHOW_SIGNAL((stderr,"/create_os_thread: got lock\n"));
+        retcode = pthread_mutex_lock(&create_thread_lock);
+        gc_assert(retcode == 0);
 
-    if((initcode = pthread_attr_init(th->os_attr)) ||
        /* call_into_lisp_first_time switches the stack for the initial
         * thread. For the others, we use this. */
-#if defined(LISP_FEATURE_WIN32)
-       (pthread_attr_setstacksize(th->os_attr, thread_control_stack_size)) ||
+        if (
+#ifdef OS_THREAD_STACK
+            pthread_attr_setstacksize(&attr, PTHREAD_STACK_MIN) ||
+            (retcode = pthread_create(kid_tid, &attr, new_thread_trampoline_switch_stack, th))
 #else
-# if defined(LISP_FEATURE_C_STACK_IS_CONTROL_STACK)
-       (pthread_attr_setstack(th->os_attr,th->control_stack_start,
-                              thread_control_stack_size)) ||
+
+# if defined(LISP_FEATURE_WIN32)
+            pthread_attr_setstacksize(&attr, thread_control_stack_size) ||
+# elif defined(LISP_FEATURE_C_STACK_IS_CONTROL_STACK)
+            pthread_attr_setstack(&attr, th->control_stack_start, thread_control_stack_size) ||
 # else
-       (pthread_attr_setstack(th->os_attr,th->alien_stack_start,
-                              ALIEN_STACK_SIZE)) ||
+            pthread_attr_setstack(&attr, th->alien_stack_start, ALIEN_STACK_SIZE) ||
 # endif
+            (retcode = pthread_create(kid_tid, &attr, new_thread_trampoline, th))
 #endif
-       (retcode = pthread_create(kid_tid, th->os_attr, new_thread_trampoline, th))) {
-        FSHOW_SIGNAL((stderr, "init = %d\n", initcode));
-        FSHOW_SIGNAL((stderr, "pthread_create returned %d, errno %d\n",
-                      retcode, errno));
-        if(retcode < 0) {
-            perror("create_os_thread");
+            ) {
+          perror("create_os_thread");
+        } else {
+          success = 1;
         }
-        r=0;
+        retcode = pthread_mutex_unlock(&create_thread_lock);
+        gc_assert(retcode == 0);
+        pthread_attr_destroy(&attr);
     }
 
-    retcode = pthread_mutex_unlock(&create_thread_lock);
-    gc_assert(retcode == 0);
-    FSHOW_SIGNAL((stderr,"/create_os_thread: released lock\n"));
-
     thread_sigmask(SIG_SETMASK,&oldset,0);
-    return r;
+    return success;
 }
 
 os_thread_t create_thread(lispobj initial_function) {
@@ -1001,8 +1011,10 @@ void gc_stop_the_world()
                 /* This thread has exited. */
                 gc_assert(thread_state(p)==STATE_DEAD);
             } else if (status) {
-                lose("cannot send suspend thread=%lu: %d, %s\n",
-                     p->os_thread,status,strerror(status));
+                lose("cannot send suspend thread=%lx: %d, %s",
+                     // KLUDGE: assume that os_thread can be cast as long.
+                     // See comment in 'interr.h' about that.
+                     (long)p->os_thread,status,strerror(status));
             }
         }
     }
@@ -1036,8 +1048,8 @@ void gc_start_the_world()
             lispobj state = thread_state(p);
             if (state != STATE_DEAD) {
                 if(state != STATE_STOPPED) {
-                    lose("gc_start_the_world: wrong thread state is %d\n",
-                         fixnum_value(state));
+                    lose("gc_start_the_world: wrong thread state is %"OBJ_FMTX,
+                         (lispobj)fixnum_value(state));
                 }
                 FSHOW_SIGNAL((stderr, "/gc_start_the_world: resuming %lu\n",
                               p->os_thread));
@@ -1116,7 +1128,10 @@ kill_safely(os_thread_t os_thread, int signal)
          * would go wrong.  Why are we running interruptions while
          * stopping the world though?  Test case is (:ASYNC-UNWIND
          * :SPECIALS), especially with s/10/100/ in both loops. */
-        if (os_thread == pthread_self()) {
+        /* From the linux man page on pthread_self() -
+         * "variables  of  type  pthread_t  can't  portably be compared using
+         *  the C equality operator (==); use pthread_equal(3) instead." */
+        if (thread_equal(os_thread, pthread_self())) {
             pthread_kill(os_thread, signal);
 #ifdef LISP_FEATURE_WIN32
             check_pending_thruptions(NULL);
