@@ -5,6 +5,7 @@
 #include "lispregs.h"
 #include "gc-internal.h"
 #include "gc-private.h"
+#include "code.h"
 #include "genesis/closure.h"
 #include "genesis/cons.h"
 #include "genesis/constants.h"
@@ -16,6 +17,7 @@
 #include "genesis/vector.h"
 #include "pseudo-atomic.h" // for get_alloc_pointer()
 #include "search.h"
+#include "genesis/avlnode.h"
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -152,6 +154,7 @@ static int find_ref(lispobj* source, lispobj target)
         bitmap = layout ? LAYOUT(layout)->bitmap : make_fixnum(-1);
         for(i=1; i<scan_limit; ++i)
             if (layout_bitmap_logbitp(i-1, bitmap)) check_ptr(i, source[i]);
+        // FIXME: check CUSTOM_GC_SCAVENGE in the header
         return -1;
 #if FUN_SELF_FIXNUM_TAGGED
     case CLOSURE_WIDETAG:
@@ -163,7 +166,7 @@ static int find_ref(lispobj* source, lispobj target)
             int wordindex = &function_ptr->name - source;
             for (j=0; j<4; ++j) check_ptr(wordindex+j, source[wordindex+j]);
         })
-        scan_limit = code_header_words(header);
+        scan_limit = code_header_words((struct code*)source);
         break;
     case FDEFN_WIDETAG:
         check_ptr(3, fdefn_callee_lispobj((struct fdefn*)source));
@@ -373,23 +376,39 @@ static struct node* find_node(struct layer* layer, lispobj ptr)
 static inline uint32_t encode_pointer(lispobj pointer)
 {
     uword_t encoding;
-    if (pointer >= DYNAMIC_SPACE_START) {
+    if (find_page_index((void*)pointer) >= 0) {
         // A dynamic space pointer is stored as a count in doublewords
         // from the heap base address. A 32GB range is representable.
         encoding = (pointer - DYNAMIC_SPACE_START) / (2*N_WORD_BYTES);
         gc_assert(encoding <= 0x7FFFFFFF);
-        return (encoding<<1) | 1; // Low bit signifies compressed ptr.
-    } else {
-        // Non-dynamic-space pointers are stored as-is.
-        gc_assert(pointer <= 0xFFFFFFFF && !(pointer & 1));
-        return pointer; // Low bit 0 signifies literal pointer
+        // Low bit of 1 signifies dynamic space compressed ptr.
+        return (encoding<<1) | 1;
+    } else
+#ifdef LISP_FEATURE_IMMOBILE_SPACE
+    if (find_varyobj_page_index((void*)pointer) >= 0) {
+        // A varyobj space pointer is stored as a count in doublewords
+        // from the base address.
+        encoding = (pointer - VARYOBJ_SPACE_START) / (2*N_WORD_BYTES);
+        gc_assert(encoding <= 0x3FFFFFFF);
+        // bit pattern #b10 signifies varyobj space compressed ptr.
+        return (encoding<<2) | 2;
+    } else
+#endif
+    {
+        // Everything else is stored as-is.
+        gc_assert(pointer <= 0xFFFFFFFF && !(pointer & 3));
+        return pointer; // bit pattern #b00 signifies uncompressed ptr
     }
 }
 
 static inline lispobj decode_pointer(uint32_t encoding)
 {
-    if (encoding & 1)  // Compressed ptr
+    if (encoding & 1)  // Compressed ptr to dynamic space
         return (encoding>>1)*(2*N_WORD_BYTES) + DYNAMIC_SPACE_START;
+#ifdef LISP_FEATURE_IMMOBILE_SPACE
+    else if ((encoding & 3) == 2) // Compressed ptr to varyobj space
+        return (encoding>>2)*(2*N_WORD_BYTES) + VARYOBJ_SPACE_START;
+#endif
     else
         return encoding; // Literal pointer
 }
@@ -702,6 +721,7 @@ static uword_t build_refs(lispobj* where, lispobj* end,
             check_ptr(layout);
             // Partially initialized instance can't have nonzero words yet
             bitmap = layout ? LAYOUT(layout)->bitmap : make_fixnum(-1);
+            // FIXME: check CUSTOM_GC_SCAVENGE in the header
             // If no raw slots, just scan without use of the bitmap.
             if (bitmap == make_fixnum(-1)) break;
             for(i=1; i<scan_limit; ++i)
@@ -717,7 +737,7 @@ static uword_t build_refs(lispobj* where, lispobj* end,
                 int wordindex = &function_ptr->name - where;
                 for (j=0; j<4; ++j) check_ptr(where[wordindex+j]);
             })
-            scan_limit = code_header_words(header);
+            scan_limit = code_header_words((struct code*)where);
             break;
         case FDEFN_WIDETAG:
             check_ptr(fdefn_callee_lispobj((struct fdefn*)where));
@@ -861,12 +881,13 @@ static int trace_paths(void (*context_scanner)(),
         lispobj car = CONS(weak_pointers)->car;
         lispobj value = ((struct weak_pointer*)native_pointer(car))->value;
         weak_pointers = CONS(weak_pointers)->cdr;
-        if (value != UNBOUND_MARKER_WIDETAG) {
+        if (is_lisp_pointer(value)) {
             if (heap_trace_verbose)
                 fprintf(stderr, "Target=%p (%s)\n", (void*)value, classify_obj(value));
             hopscotch_reset(&visited);
             hopscotch_reset(&targets);
-            n_found += trace1(value, &targets, &visited,
+            n_found += trace1(canonical_obj(value),
+                              &targets, &visited,
                               &inverted_heap, &scratchpad,
                               n_pins, pins, context_scanner, criterion);
         }
@@ -885,24 +906,31 @@ int gc_prove_liveness(void(*context_scanner)(),
                       int n_pins, uword_t* pins,
                       int criterion)
 {
-    int n_watched = 0, n_live = 0, n_bad = 0;
+    int n_watched = 0, n_live = 0, n_bad = 0, n_imm = 0;
     lispobj list;
     for (list = objects ; list != NIL && listp(list) ; list = CONS(list)->cdr) {
         ++n_watched;
         lispobj car = CONS(list)->car;
         if ((lowtag_of(car) != OTHER_POINTER_LOWTAG ||
-             widetag_of(native_pointer(car)) != WEAK_POINTER_WIDETAG))
+             widetag_of(native_pointer(car)) != WEAK_POINTER_WIDETAG)) {
             ++n_bad;
-        else
-            n_live += ((struct weak_pointer*)native_pointer(car))->value
-                != UNBOUND_MARKER_WIDETAG;
+            continue;
+        }
+        lispobj wpval = ((struct weak_pointer*)native_pointer(car))->value;
+        if (is_lisp_pointer(wpval))
+            ++n_live;
+        else if (wpval != UNBOUND_MARKER_WIDETAG)
+            ++n_imm;
     }
     if (!listp(list) || n_bad) {
         fprintf(stderr, "; Bad value in liveness tracker\n");
         return -1;
     }
-    fprintf(stderr, "; Liveness tracking: %d/%d live watched objects\n",
+    fprintf(stderr, "; Liveness tracking: %d/%d live watched objects",
             n_live, n_watched);
+    if (n_imm)
+        fprintf(stderr, " (ignored %d non-pointers)", n_imm);
+    putc('\n', stderr);
     if (!n_live)
         return 0;
     // Put back lowtags on pinned objects, since wipe_nonpinned_words() removed
@@ -932,16 +960,31 @@ int prove_liveness(lispobj objects, int criterion)
 #define LISP_THREAD_NAME_SLOT INSTANCE_DATA_START+0
 #define LISP_THREAD_OS_THREAD_SLOT INSTANCE_DATA_START+3
 
+/* Perform exhaustive search on *ALL-THREADS* looking for a specific thread.
+ * Efficiency is not a concern.
+ */
+static struct instance* find_thread(os_thread_t os_thread, lispobj tree)
+{
+    if (tree == NIL) return 0;
+    struct avlnode* node = (struct avlnode*)native_pointer(tree);
+    struct instance* lisp_thread = (struct instance*)native_pointer(node->data);
+    if ((os_thread_t)lisp_thread->slots[LISP_THREAD_OS_THREAD_SLOT] == os_thread)
+        return lisp_thread;
+    if ((lisp_thread = find_thread(os_thread, node->left)) != NULL)
+        return lisp_thread;
+    if ((lisp_thread = find_thread(os_thread, node->right)) != NULL)
+        return lisp_thread;
+    return 0;
+}
+
 struct vector* lisp_thread_name(os_thread_t os_thread)
 {
     static unsigned int hint;
     lispobj* sym = find_symbol("*ALL-THREADS*", "SB-THREAD", &hint);
-    lispobj list = sym ? ((struct symbol*)sym)->value : NIL;
-    while (list != NIL) {
-        struct instance* lisp_thread = (struct instance*)native_pointer(CONS(list)->car);
-        list = CONS(list)->cdr;
-        if ((os_thread_t)lisp_thread->slots[LISP_THREAD_OS_THREAD_SLOT]
-            == os_thread)
+    if (sym) {
+        struct instance* lisp_thread =
+            find_thread(os_thread, ((struct symbol*)sym)->value);
+        if (lisp_thread)
             return VECTOR(lisp_thread->slots[LISP_THREAD_NAME_SLOT]);
     }
     return 0;
