@@ -4,8 +4,15 @@
            #:really-invoke-debugger
            #:*break-on-failure* #:*break-on-expected-failure*
 
+           ;; type tools
+           #:type-evidently-=
+           #:ctype=
+           #:assert-tri-eq
+
            ;; thread tools
            #:make-kill-thread #:make-join-thread
+           ;; cause tests to run in multiple threads
+           #:enable-test-parallelism
 
            ;; MAP-OPTIMIZATION-*
            #:map-optimization-quality-combinations
@@ -16,7 +23,7 @@
            #:checked-compile-capturing-source-paths
            #:checked-compile-condition-source-paths
 
-           #:runtime #:split-string #:shuffle))
+           #:runtime #:split-string #:integer-sequence #:shuffle))
 
 (in-package :test-util)
 
@@ -29,11 +36,51 @@
 (defvar *threads-to-kill*)
 (defvar *threads-to-join*)
 
-(eval-when (:compile-toplevel :load-toplevel :execute)
-  (require :sb-posix))
+(defun setenv (name value)
+  #-win32
+  (let ((r (sb-alien:alien-funcall
+            (sb-alien:extern-alien
+             "setenv" (function sb-alien:int (sb-alien:c-string :not-null t)
+                                (sb-alien:c-string :not-null t) sb-alien:int))
+                          name value 1)))
+    (if (minusp r)
+        (error "setenv: ~a" (sb-int:strerror))
+        r))
+  #+win32
+  (let ((r (sb-alien:alien-funcall
+            (sb-alien:extern-alien "_putenv" (function sb-alien:int (sb-alien:c-string :not-null t)))
+                          (format nil "~A=~A" name value))))
+    (if (minusp r)
+        (error "putenv: ~a" (sb-int:strerror))
+        r)))
 
-(sb-posix:putenv (format nil "SBCL_MACHINE_TYPE=~A" (machine-type)))
-(sb-posix:putenv (format nil "SBCL_SOFTWARE_TYPE=~A" (software-type)))
+(setenv "SBCL_MACHINE_TYPE" (machine-type))
+(setenv "SBCL_SOFTWARE_TYPE" (software-type))
+
+
+;;; Type tools
+
+(defun type-evidently-= (x y)
+  (and (subtypep x y) (subtypep y x)))
+
+(defun ctype= (left right)
+  (let ((a (sb-kernel:specifier-type left)))
+    ;; SPECIFIER-TYPE is a memoized function, and TYPE= is a trivial
+    ;; operation if A and B are EQ.
+    ;; To actually exercise the type operation, remove the memoized parse.
+    (sb-int:drop-all-hash-caches)
+    (let ((b (sb-kernel:specifier-type right)))
+      (assert (not (eq a b)))
+      (sb-kernel:type= a b))))
+
+(defmacro assert-tri-eq (expected-result expected-certainp form)
+  (sb-int:with-unique-names (result certainp)
+    `(multiple-value-bind (,result ,certainp) ,form
+       (assert (eq ,expected-result ,result))
+       (assert (eq ,expected-certainp ,certainp)))))
+
+
+;;; Thread tools
 
 #+sb-thread
 (defun make-kill-thread (&rest args)
@@ -50,13 +97,13 @@
       (push thread *threads-to-join*))
     thread))
 
-(defun log-msg (&rest args)
-  (apply #'format *trace-output* "~&::: ~@?~%" args)
-  (force-output *trace-output*))
+(defun log-msg (stream &rest args)
+  (prog1 (apply #'format stream "~&::: ~@?~%" args)
+    (force-output stream)))
 
-(defun log-msg/non-pretty (&rest args)
+(defun log-msg/non-pretty (stream &rest args)
   (let ((*print-pretty* nil))
-    (apply #'log-msg args)))
+    (apply #'log-msg stream args)))
 
 (defun run-test (test-function name fails-on)
   (start-test)
@@ -69,7 +116,7 @@
                                 (fail-test :unexpected-failure name error))
                             (return-from run-test))))
       ;; Non-pretty is for cases like (with-test (:name (let ...)) ...
-      (log-msg/non-pretty "Running ~S" name)
+      (log-msg/non-pretty *trace-output* "Running ~S" name)
       (funcall test-function)
       #+sb-thread
       (let ((any-leftover nil))
@@ -99,9 +146,40 @@
       (if (expected-failure-p fails-on)
           (fail-test :unexpected-success name nil)
           ;; Non-pretty is for cases like (with-test (:name (let ...)) ...
-          (log-msg/non-pretty "Success ~S" name)))))
+          (log-msg/non-pretty *trace-output* "Success ~S" name)))))
 
-(defmacro with-test ((&key fails-on broken-on skipped-on name)
+;;; Like RUN-TEST but do not perform any of the automated thread management.
+;;; Since multiple threads are executing tests, there is no reason to kill
+;;; unrecognized threads.
+(sb-ext:define-load-time-global *output-mutex* (sb-thread:make-mutex))
+(defun run-test-concurrently (test-spec)
+  (destructuring-bind (test-body . name) test-spec
+    (sb-thread:with-mutex (*output-mutex*)
+      (log-msg/non-pretty *trace-output* "Running ~S" name))
+    (let ((stream (make-string-output-stream)))
+      (let ((*standard-output* stream)
+            (*error-output* stream))
+        (let ((f (compile nil `(lambda () ,@test-body))))
+          (funcall f))
+        (let ((string (get-output-stream-string stream)))
+          (sb-thread:with-mutex (*output-mutex*)
+            (when (plusp (length string))
+              (log-msg/non-pretty *trace-output* "Output from ~S" name)
+              (write-string string *trace-output*))
+            (log-msg/non-pretty *trace-output* "Success ~S" name)))))))
+
+(defvar *deferred-test-forms*)
+(defun enable-test-parallelism ()
+  (let ((n (sb-ext:posix-getenv "SBCL_TEST_PARALLEL")))
+    (when n
+      (setq *deferred-test-forms* (vector (parse-integer n) nil nil)))))
+
+;;; Tests which are not broken in any way and do not mandate sequential
+;;; execution are pushed on a worklist to execute in multiple threads.
+;;; The purpose of running tests in parallel is to exercise the compiler
+;;; to show that it works without acquiring the world lock,
+;;; but the nice side effect is that the tests finish quicker.
+(defmacro with-test ((&key fails-on broken-on skipped-on name serial slow)
                      &body body)
   (flet ((name-ok (x y)
            (declare (ignore y))
@@ -123,9 +201,17 @@
      `(progn
         (start-test)
         (fail-test :skipped-disabled ',name "Test disabled for this combination of platform and features")))
+    ((and (boundp '*deferred-test-forms*)
+          (not serial)
+          (or (not fails-on)
+              (not (expected-failure-p fails-on))))
+     ;; To effectively parallelize calls to COMPILE, we must defer compilation
+     ;; until a worker thread has picked off the test from shared worklist.
+     ;; Thus we push only the form to be compiled, not a lambda.
+     `(push (cons ',body ',name)
+            (elt *deferred-test-forms* ,(if slow 1 2))))
     (t
-     `(run-test (lambda ()
-                  ,@body)
+     `(run-test (lambda () ,@body)
                 ',name
                 ',fails-on))))
 
@@ -150,9 +236,9 @@
 
 (defun fail-test (type test-name condition)
   (if (stringp condition)
-      (log-msg "~@<~A ~S ~:_~A~:>"
+      (log-msg *trace-output* "~@<~A ~S ~:_~A~:>"
                type test-name condition)
-      (log-msg "~@<~A ~S ~:_due to ~S: ~4I~:_\"~A\"~:>"
+      (log-msg *trace-output* "~@<~A ~S ~:_due to ~S: ~4I~:_\"~A\"~:>"
                type test-name (type-of condition) condition))
   (push (list type *test-file* (or test-name *test-count*))
         *failures*)
@@ -695,6 +781,9 @@
         collect (subseq string begin end)
         while end))
 
+(defun integer-sequence (n)
+  (loop for i below n collect i))
+
 (defun shuffle (sequence)
   (typecase sequence
     (list
