@@ -15,8 +15,8 @@
 ;;;; WHITE-BOX TESTS
 
 (shadowing-import 'assertoid:assert-error)
-(use-package :sb-thread)
-(use-package :sb-sys)
+(use-package "SB-THREAD")
+(use-package "SB-SYS")
 
 (setf sb-unix::*on-dangerous-wait* :error)
 
@@ -145,13 +145,16 @@
 
 (with-test (:name (:threads :more-trivia))
   (let ((old-threads (list-all-threads))
-        (thread (make-thread (lambda ()
-                               (assert (find *current-thread*
-                                             sb-thread::*all-threads*))
-                               (sleep 2))))
+        (thread (make-thread
+                 (lambda ()
+                   (assert (sb-thread::avl-find
+                            (sb-kernel:get-lisp-obj-address sb-vm:*control-stack-start*)
+                            sb-thread::*all-threads*))
+                   (sleep 2))))
         (new-threads (list-all-threads)))
     (assert (thread-alive-p thread))
-    (assert (eq thread (first new-threads)))
+    ;; there is no order guarantee
+    ;;(assert (eq thread (first new-threads)))
     (assert (= (1+ (length old-threads)) (length new-threads)))
     (sleep 3)
     (assert (not (thread-alive-p thread)))))
@@ -1308,10 +1311,10 @@
   ;; (because the code and lexenv were set separately).
   (let ((fun (sb-kernel:%make-funcallable-instance 0))
         (condition nil))
-    (setf (sb-kernel:funcallable-instance-fun fun) #'closure-one)
+    (setf (sb-kernel:%funcallable-instance-fun fun) #'closure-one)
     (flet ((changer ()
-             (loop (setf (sb-kernel:funcallable-instance-fun fun) #'closure-one)
-                   (setf (sb-kernel:funcallable-instance-fun fun) #'closure-two)))
+             (loop (setf (sb-kernel:%funcallable-instance-fun fun) #'closure-one)
+                   (setf (sb-kernel:%funcallable-instance-fun fun) #'closure-two)))
            (test ()
              (handler-case (loop (funcall fun))
                (serious-condition (c) (setf condition c)))))
